package ch.ige.adr.server;

import java.sql.SQLException;
import java.util.ArrayList;

import javax.ejb.EJBException;

import ch.ige.adr.server.dao.RegadrDAO;
import ch.ige.adr.vo.RegadrSelKritVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.vo.RegadrVO;
import ch.ige.util.vo.VersadrVO;
/**
 * Business Object für Registeradressen
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: RegadrBO.java,v 1.40 2007/07/03 15:20:54 nydegger Exp $
 */

public class RegadrBO
{
  private AdresseXxIdentNumberLocal xxIdentNumberLocal = null;

  /**
   * Constructor
   * @param xxIdentNumberLocal
   */
  public RegadrBO(AdresseXxIdentNumberLocal xxIdentNumberLocal)
  {
    this.xxIdentNumberLocal = xxIdentNumberLocal;
  }

  /**
   * Kreiert eine Registeradresse.
   * @param regadrVO Value Object der Registeradresse
   * @return Registeradressnummer
   */
  public int createRegadr(RegadrVO regadrVO)
  {
    return createRegadr(regadrVO, null);
  }

  /**
   * Kreiert eine Registeradresse und eine Versandadresse.
   * @param regadrVO Value Object der Registeradresse
   * @param versadrVO Value Object der Versandadresse
   * @return Registeradressnummer
   */
  public int createRegadr(RegadrVO regadrVO, VersadrVO versadrVO)
  {
    // Versandadresse speichern
    if (versadrVO != null)
    {
      regadrVO.setAdrNr(new AdresseBO(xxIdentNumberLocal).createVersadr(versadrVO));
    }
    // Primärschlüssel lösen
    try
    {
      regadrVO.setRegadrNr(xxIdentNumberLocal.getNextId(AdrConstants.REGADR_NR));
    }
    catch (Exception ex)
    {
      LogLevelHelper.log(ex, RegadrBO.class);
      throw new EJBException(ex);
    }
    regadrVO.setUpVerzName(regadrVO.getUpVerzNameOrEmptyString().toUpperCase());
    regadrVO.setUpRegOrt(regadrVO.getRegOrtOrEmptyString().toUpperCase());
    // Registeradresse speichern
    try
    {
      new RegadrDAO().insert(regadrVO);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrBO.class);
      throw new EJBException(ex);
    }
    return regadrVO.getRegadrNr();
  }

  /**
   * Gibt das Value Object der Registeradresse zurück.
   * @param regadrNr Registeradressnummer
   * @return Value Object der Registeradresse
   */
  public RegadrVO getRegadr(int regadrNr)
  {
    try
    {
      RegadrVO regadrVO = new RegadrDAO().select(regadrNr);
      if (regadrVO == null)
      {
        throw new EJBException("MNR=1303, getRegadr");
      }
      return regadrVO;
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Setzt die Registeradresse auf den Wert des Value Objects.
   * @param regadrVO Registeradresse
   */
  public void setRegadr(RegadrVO regadrVO)
  {
    regadrVO.setUpVerzName(regadrVO.getUpVerzNameOrEmptyString().toUpperCase());
    regadrVO.setUpRegOrt(regadrVO.getRegOrtOrEmptyString().toUpperCase());
    regadrVO.setVersion(regadrVO.getVersion() + 1);
    try
    {
      if (new RegadrDAO().update(regadrVO) != 1)
      {
        throw new EJBException("MNR=1303, setRegadr");
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Erstellt eine Versandadresse und setzt die Adressnummer in der
   * Registeradresse ein und setzt die Registeradresse auf den Wert
   * des Value Objects.
   * @param regadrVO Registeradresse (wird mutiert!)
   * @param versadrVO Versandadresse (wird erstellt!)
   */
  public void setRegadr(RegadrVO regadrVO, VersadrVO versadrVO)
  {
    regadrVO.setAdrNr(new AdresseBO(xxIdentNumberLocal).createVersadr(versadrVO));
    setRegadr(regadrVO);
  }


  /**
   * Löscht die Registeradresse
   * @param regadrNr Registeradressnummer
   */
  public void removeRegadr(int regadrNr)
  {
    try
    {
      if (new RegadrDAO().delete(regadrNr) != 1)
      {
        throw new EJBException("MNR=1303, removeRegadr");
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrBO.class);
      if (ex.getMessage().indexOf("ORA-02292: integrity constraint (CRESCENDO.FK_REGADR_SCHUTIT_REG_1)") > -1)
      {
        throw new EJBException("MNR=1251, removeRegadr");
      }
      else
      {
         throw new EJBException(ex);
      }
    }
  }

  /**
   * Findet alle Registeradressen, auf welche die Selektionskriterien zutreffen.
   * @param regadrSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit RegadrAVO-Objekten mit den gefundenen Registeradresse
   */
  public ArrayList findRegadr(RegadrSelKritVO regadrSelKritVO)
  {
    try
    {
      if (!regadrSelKritVO.suchmodusVolltext)
      {
        // bei Index-Suche regName und regOrt normalisieren
        regadrSelKritVO.regName = PostOrtNorm.getOrtNorm(regadrSelKritVO.regName, true);
        regadrSelKritVO.regOrt = PostOrtNorm.getOrtNorm(regadrSelKritVO.regOrt, true);
        if (!(regadrSelKritVO.regName.equals("") && regadrSelKritVO.regOrt.equals("")) &&
            regadrSelKritVO.adrNr == 0 && regadrSelKritVO.regadrNr == 0 &&
            regadrSelKritVO.upVerzName.equals("") &&
            regadrSelKritVO.regName.equals("") && regadrSelKritVO.regOrt.equals(""))
        {
          // Es wurden keine Suchworte eingegeben
          throw new EJBException("MNR=1232, findRegadr");
        }
      }
      return new RegadrDAO().select(regadrSelKritVO);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrBO.class);
      throw new EJBException(ex);
    }
  }
}
