package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;

import ch.ige.art.vo.ArtpreisVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für artpreis
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtpreisDAO.java,v 1.11 2008/06/25 13:44:29 ruegsegger Exp $
 */

public class ArtpreisDAO
{
  /**
   * Selektiert einen Datensatz aus der Tabelle artpreis
   *
   * @param  artNr         ArtikelNr
   * @param  mengeinh     Mengeneinheit
   * @param  artpreisartCd Artikelpreisart
   * @param  gueltigAb     Datum Gueltig ab
   * @return ArtbezVO      Value Object
   * @throws SQLException  Unerwarteter SQL Fehler
   */
  public ArtpreisVO select( String artNr, String mengeinh, String artpreisartCd, Date gueltigAb ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArtpreisVO vo = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "trim(mengeinh) as mengeinh, " + "trim(artpreisart_cd) as artpreisart_cd, " + "gueltig_ab, " + "preis, pro, "
          + "trim(kennzeichen) as kennzeichen, " + "erst_dat, " + "trim(erst_ben) as erst_ben, " + "geaend_dat, " + "trim(geaend_ben) as geaend_ben, "
          + "u_version " + "FROM artpreis WHERE art_nr = RPAD(?, 18,' ') " + "AND mengeinh = RPAD(?, 3, ' ') " + "AND artpreisart_cd = RPAD(?, 3, ' ') "
          + "AND gueltig_ab = ? ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      ps.setString( 2, mengeinh );
      ps.setString( 3, artpreisartCd );
      ps.setDate( 4, new java.sql.Date( gueltigAb.getTime() ) );

      rs = ps.executeQuery();

      if ( rs.next() )
      {
        vo = new ArtpreisVO();
        vo.artNr = rs.getString( "art_nr" );
        vo.mengeinh = rs.getString( "mengeinh" );
        vo.artpreisartCd = rs.getString( "artpreisart_cd" );
        vo.gueltigAb = rs.getDate( "gueltig_ab" );
        vo.preis = rs.getBigDecimal( "preis" );
        vo.pro = new Integer( rs.getInt( "pro" ) );
        if ( rs.wasNull() )
        {
          vo.pro = null;
        }
        vo.kennzeichen = rs.getString( "kennzeichen" );
        vo.erstDat = rs.getDate( "erst_dat" );
        vo.erstBen = rs.getString( "erst_ben" );
        vo.geaendDat = rs.getDate( "geaend_dat" );
        vo.geaendBen = rs.getString( "geaend_ben" );
        vo.uVersion = rs.getString( "u_version" );
      }
      return vo;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtpreisDAO.class, query + " artNr: " + artNr + " mengeinh: " + mengeinh + " artpreisartCd: " + artpreisartCd + " gueltigAb: "
          + gueltigAb );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Preise eines Artikels aus der Tabelle artpreis
   *
   * @param  artNr         Artikel Nummer
   * @return ArrayList     ArrayList mit Value Object
   * @throws SQLException  Unerwarteter SQL Fehler
   */
  public ArrayList<ArtpreisVO> select( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<ArtpreisVO> al = new ArrayList<ArtpreisVO>();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "trim(mengeinh) as mengeinh, " + "trim(artpreisart_cd) as artpreisart_cd, " + "gueltig_ab, preis, pro, "
          + "trim(kennzeichen) as kennzeichen, " + "erst_dat, erst_ben, geaend_dat, geaend_ben, u_version "
          + "FROM artpreis WHERE art_nr = RPAD(?, 18,' ') ORDER BY gueltig_ab";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        ArtpreisVO artpreisVO = new ArtpreisVO();
        artpreisVO.artNr = rs.getString( "art_nr" );
        artpreisVO.mengeinh = rs.getString( "mengeinh" );
        artpreisVO.artpreisartCd = rs.getString( "artpreisart_cd" );
        artpreisVO.gueltigAb = rs.getDate( "gueltig_ab" );
        artpreisVO.preis = rs.getBigDecimal( "preis" );
        artpreisVO.pro = new Integer( rs.getInt( "pro" ) );
        if ( rs.wasNull() )
        {
          artpreisVO.pro = null;
        }
        artpreisVO.kennzeichen = rs.getString( "kennzeichen" );
        artpreisVO.erstDat = rs.getDate( "erst_dat" );
        artpreisVO.erstBen = rs.getString( "erst_ben" );
        artpreisVO.geaendDat = rs.getDate( "geaend_dat" );
        artpreisVO.geaendBen = rs.getString( "geaend_ben" );
        artpreisVO.uVersion = rs.getString( "u_version" );
        al.add( artpreisVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtpreisDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle artpreis ein
   *
   * @param vo  ValueObject      ArtpreisVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( ArtpreisVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO artpreis(" + "art_nr, " + "mengeinh, " + "artpreisart_cd, " + "gueltig_ab, " + "preis, " + "pro, " + "kennzeichen, " + "erst_dat, "
          + "erst_ben, " + "geaend_dat, " + "geaend_ben, " + "u_version) " + "VALUES( ?,?,?,?,?,?,?,?,?,?,?,?) ";

      ps = con.prepareStatement( query );
      ps.setString( 1, vo.artNr );
      ps.setString( 2, vo.mengeinh );
      ps.setString( 3, vo.artpreisartCd );
      if ( vo.gueltigAb == null )
      {
        ps.setNull( 4, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 4, new java.sql.Date( vo.gueltigAb.getTime() ) );
      }
      ps.setBigDecimal( 5, vo.preis );
      if ( vo.pro == null )
      {
        ps.setNull( 6, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 6, vo.pro.intValue() );
      }
      ps.setString( 7, vo.kennzeichen );
      if ( vo.erstDat == null )
      {
        ps.setNull( 8, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 8, new java.sql.Date( vo.erstDat.getTime() ) );
      }
      ps.setString( 9, vo.erstBen );
      if ( vo.geaendDat == null )
      {
        ps.setNull( 10, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 10, new java.sql.Date( vo.geaendDat.getTime() ) );
      }
      ps.setString( 11, vo.geaendBen );
      ps.setString( 12, vo.uVersion );

      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtpreisDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle artbez
   *
   * @param     vo            ArtpreisVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update( ArtpreisVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE artpreis SET " + "preis = ?, " + "pro = ?, " + "geaend_ben = ?, " + "geaend_dat = ?, " + "u_version = ? "
          + "WHERE art_nr = RPAD(?, 18,' ') " + "AND mengeinh = RPAD(?, 3, ' ') " + "AND artpreisart_cd = RPAD(?, 3, ' ') " + "AND gueltig_ab = ? ";

      ps = con.prepareStatement( query );
      ps.setBigDecimal( 1, vo.preis );
      if ( vo.pro == null )
      {
        ps.setNull( 2, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 2, vo.pro.intValue() );
      }
      if ( vo.geaendDat == null )
      {
        ps.setNull( 3, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 3, new java.sql.Date( vo.geaendDat.getTime() ) );
      }
      ps.setString( 4, vo.geaendBen );
      ps.setString( 5, vo.uVersion );

      ps.setString( 6, vo.artNr );
      ps.setString( 7, vo.mengeinh );
      ps.setString( 8, vo.artpreisartCd );
      ps.setDate( 9, new java.sql.Date( vo.gueltigAb.getTime() ) );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtpreisDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle artpreis
   *
   * @param     artNr          ArtikelNr      Primary Key
   * @param     mengeinh      Mengeneinheit  Primary Key
   * @param     artpreisartCd  Preisart       Primary Key
   * @param     gueltigAb      Datum          Primary Key
   * @return    int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int delete( String artNr, String mengeinh, String artpreisartCd, Date gueltigAb ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "DELETE FROM artpreis " + "WHERE art_nr = RPAD(?, 18,' ') " + "AND mengeinh = RPAD(?, 3, ' ') " + "AND artpreisart_cd = RPAD(?, 3, ' ') "
          + "AND gueltig_ab = ? ";
      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      ps.setString( 2, mengeinh );
      ps.setString( 3, artpreisartCd );
      ps.setDate( 4, new java.sql.Date( gueltigAb.getTime() ) );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtpreisDAO.class, query );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
