package ch.ige.art.xml;

import java.util.*;

import ch.ige.art.vo.*;
import ch.ige.chdomain.*;
import ch.ige.chdomain.impl.*;
import ch.ige.xmlutil.bind.*;

/**
 * Repräsentiert ein CH-Domain Document
 * Zur Nachführung der Artikeltexte auf Swissreg wird die domain mit dem Namen "artnr" verwendet.
 * Bei der domain artnr muss die view angegeben werden, dies kann durch das Schema nicht sichergestellt werden.
 * Falls es sich um einen Publikationstext (ARTTEXTART_CD = PBL) handelt wird die view auf "publication" gesetzt.
 * Falls es sich um einen Registertext (ARTTEXTART_CD = REG) handelt wird die view auf "register" gesetzt.
 *
 * Bei der Nachführung der Artikeltexte ist das domain-Element immer gleich und beinhaltet 0-n Entry-Elemente:
 * <domain name="artnr" scope="ALL" type="string">
 * Pro XML werden nur Aktionen eines einzelnen Artikels übertragen, der key ist demzufolge pro XML immer gleich.
 * Die Aktion bezieht sich immer auf eine Textart und deren Übersetzung.
 * Anhand des XML ist nicht ersichtlich, ob der entsprechende Artikel auf Bagis-DB neu erfasst oder mutiert wurde,
 * somit ist nicht bekannt, ob der Artikel auf Swissreg bereits existiert oder neu erfasst werden muss.
 *
 * Da sich die Aktion eines Artikeltextes sowohl auf die Textart wie auch auf die Sprache bezieht,
 * wird jede Übersetzung in einem separaten Entry Element geliefert.
 * Ein Entry-Element beinhaltet somit genau 1 Value-Element.
 * Das XML kann dadurch gewisse Redundanzen aufweisen, die Generierung ist jedoch erheblich einfacher
 * wenn die Gruppierungen nicht berücksichtigt werden muss.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * @author A. Rueegsegger
 * @version $Id: DomainDocument.java,v 1.10 2006/12/13 10:08:31 ruegsegger Exp $
 */
public class DomainDocument
{
  private static final BindingType BINDING_TYPE = Bindings.CHXML_CHDOMAIN;

  private String artNr;
  private Domains domains;
  private DomainType domain;

  public DomainDocument( String artNr )
  {
    this.artNr = artNr;

    domains = new DomainsImpl();
    domains.setExtractdate( Calendar.getInstance() );

    domain = new DomainTypeImpl();
    domain.setName( "artnr" );
    domain.setScope( "ALL" );
    domain.setType("string");

    domains.getDomain().add( domain );
  }


  public void addPublicationEntry(  List registerValues )
  {
    addEntry( "publication", registerValues);
  }

  public void addRegisterEntry( List publicationValues  )
  {
    addEntry( "register", publicationValues);
  }

  private void addEntry( String view, List values )
  {
    if ( !values.isEmpty() )
    {
      EntryType entry = new EntryTypeImpl();
      entry.setAction( "update" );
      entry.setKey( artNr );
      entry.setView( view );

      for (Iterator iter = values.iterator(); iter.hasNext(); )
      {
        ArtextVO artextVO = (ArtextVO)iter.next();
        Value value = createValue( artextVO );
        entry.getValue().add( value );
      }

      domain.getEntry().add( entry );
    }
  }


  public Value createValue( ArtextVO artextVO )
  {
    Value value = new ValueImpl();
    value.setLanguage( artextVO.getLanguage() );
    value.setValue( artextVO.text != null ?  artextVO.text : "" );

    return value;
  }


  public String validateAndNormalize() throws Exception
  {
    Bindings.validate( BINDING_TYPE, domains );
    return getContent();
  }


  public String getContent() throws Exception
  {
    return Bindings.marshalToString( BINDING_TYPE, domains );
  }


  public boolean hasEntries()
  {
    return ( domain.getEntry().size() > 0 );
  }


}
