package ch.ige.epband.clientneu;

import java.util.List;

import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreignisVO;
import ch.ige.epband.vo.PatVO;
import ch.ige.epband.vo.SchutitVO;
import ch.ige.util.Pruefziffer;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: SGML-Parsen des übergebenen StringBuffers<br>
 * und aufbereiten von ValueObject
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author M. Trachsel
 * @version $Id: ParseSGML.java,v 1.28 2008/05/13 12:02:08 wild Exp $
 */

public class ParseSGML
{

  private EpbandVO epbandVO;

  private XMLDoc xml;

  private Ereignis ereignis = new Ereignis();

  private Schutit schutit = new Schutit();

  private Pat pat;

  private Prioritaet prioritaet;

  private RegAdr regAdr;

  /**
   * Initialisierung der Klasse
   * 
   * @param blandWork ArrayList mit allen Bländer
   * @throws Exception
   */
  public ParseSGML( List<String> blandWork ) throws Exception
  {
    schutit = new Schutit();
    pat = new Pat();
    prioritaet = new Prioritaet( blandWork );
    regAdr = new RegAdr( blandWork );
  }

  /**
   * Setzen der Objektvariablen
   * 
   * @param epbandVO
   * @param xml
   * @throws ProcessingException
   */

  public void setData( EpbandVO epbandVO, XMLDoc xml ) throws ProcessingException
  {
    this.epbandVO = epbandVO;
    this.xml = xml;

    epbandVO.schutitVO = new SchutitVO();
    epbandVO.patVO = new PatVO();

    // Schutit-Nr aufbereiten damit im Fehlerfall ID mitgegeben werden kann
    // Bei ungültiger Schutit-Nr wird Document übersprungen (ProcessingException)
    prepSchutitGesuchNr();
    Util.schutitNr = epbandVO.schutitVO.schutitNr;
    Util.gesuchNr = epbandVO.schutitVO.gesuchNr;
  }

  /**
   * Verarbeitung aufgrund Patentart WIPO: Ereignis 30060 generieren<br>
   * CH-Patent: Patent weiterbearbeiten<br>
   * CH nicht benannt: auf CH-Zurücknahme testen.<br>
   * CH-Zurücknahme: Ereignis 20250 schreiben<br>
   * 
   * @throws ProcessingException
   * @throws Exception
   */
  public void doParseSGML() throws ProcessingException, Exception
  {
    if ( isWipo() ) // Falls kein WIPO
    {
      processWipo();
    }
    else if ( isCHPatent() ) // Falls CH benannt
    {
      processCHPatent();
    }
    else
    {
      testZuruecknahme();
    }
  }

  /**
   * Organisation ermitteln welche das Dokument momentan publiziert EP = European Patent Office WO = WIPO (Word
   * Intellectual Property Organisation)
   * 
   * @return true falls es sich um ein Patent der WIPO handelt
   */
  private boolean isWipo()
  {
    return ( xml.getValueOf( "B190" ).equalsIgnoreCase( "WO" ) );
  }

  /**
   * Test ob CH für das Patent benannt ist
   * 
   * @return boolean true falls CH für das Patent benannt
   */
  public boolean isCHPatent()
  {
    List<String> list = xml.getValuesOf( "B840", "ctry" );
    return ( list.contains( "CH" ) || list.contains( "LI" ) );
  }

  /**
   * Schutit-Nr und Gesuchs-Nr aufbereiten damit im Fehlerfall mitgegeben werden kann
   * 
   * @throws ProcessingException Verarbeitungsfehler - Patent überspringen Schutit-Nr: Wird mit führender Null auf acht
   *           Stellen erweitert und am Schluss mit einer Prüfziffer ergänzt <B110>0842640</B110> => schutitNr =
   *           008426405 <B110>0000000</B110> => schutitNr = ""
   */
  private void prepSchutitGesuchNr() throws ProcessingException
  {
    String schutitNr = "";
    String gesuchNr = "";
    schutitNr = xml.getValueOf( "B110" );

    /** TODO notwendigkeit hilfsschutit abklären */
    String hilfsSchutitNr = "";
    for ( int i = 0; i < schutitNr.length(); i++ )
    {

      if ( schutitNr.charAt( i ) != '0' )
      {
        hilfsSchutitNr = schutitNr;
        break;
      }
    }
    schutitNr = hilfsSchutitNr;

    if ( !schutitNr.equals( "" ) )
    {
      while ( schutitNr.length() < 8 )
      {
        schutitNr = "0" + schutitNr;
      }

      /** TODO Pruefziffer berechnen in Projekt nehmen */
      schutitNr = Pruefziffer.modulo10_ep( schutitNr );
      if ( schutitNr.equals( "" ) )
      {
        throw new ProcessingException( "Ungültige Schutitnummer" );
      }
    }

    gesuchNr = xml.getValueOf( "B210" );
    if ( !gesuchNr.equals( "" ) )
    {
      int pos = gesuchNr.indexOf( "." );

      gesuchNr = gesuchNr.substring( 0, pos ) + gesuchNr.substring( pos + 1 );
    }
    epbandVO.schutitVO.schutitNr = schutitNr;
    epbandVO.schutitVO.gesuchNr = gesuchNr;
  }

  /**
   * Falls es sich um ein Patent der WIPO (PCT) handelt, das in die europäische Phase eintritt wird Ereignis 30060 für
   * PCT generieren. B006EP = 9121 => Eintreten in europäische Phase. Andernfalls muss nichts unternommen werden.
   * 
   * @throws ProcessingException Verarbeitungsfehler - Patent überspringen
   */
  private void processWipo() throws ProcessingException
  {
    EreignisVO ereignisVO = ereignis.getEreignisWipo();

    // Zuerst Schutit-Nr aufbereiten damit im Fehlerfall ID
    // mitgegeben werden kann
    epbandVO.schutitVO.schutitNr = Util.prepWONr( xml.getValueOf( "B110" ) );
    if ( epbandVO.schutitVO.schutitNr == null || epbandVO.schutitVO.schutitNr.equals( "" ) )
    {
      String message = "Keine oder ungültige schutit-nr gefunden (B110), " + "WIPO nicht bearbeitet.";
      throw new ProcessingException( message );
    }

    // ereigIdat bestimmen
    if ( !xml.getValueOf( "B006EP" ).equals( "9121" ) )
    {
      throw new ProcessingException();
    }
    else
    {
      String str = xml.getValueOf( "B237", "date" );
      if ( !str.equals( "" ) )
      {
        ereignisVO.ereigIdat = Util.calculateIdat( str );
      }
      else
      {
        str = xml.getValueOf( "B235", "date" );
        if ( !str.equals( "" ) )
        {
          ereignisVO.ereigIdat = Util.calculateIdat( str );
        }
        else
        {
          str = xml.getValueOf( "B236", "date" );
          if ( !str.equals( "" ) )
          {
            ereignisVO.ereigIdat = Util.calculateIdat( str );
          }
        }
      }

      if ( ereignisVO.ereigIdat == 0 )
      {
        String message = "Bei PCT " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Ereignis 30060 nicht generiert"
            + "\r\nERROR \t Kein Datum gefunden in B235, B236, B237";
        throw new ProcessingException( message );
      }

      epbandVO.ereigAL.add( ereignisVO );

      // Vermerken, dass es sich um ein PCT handelt
      epbandVO.wipo = true;
    }
  }

  /**
   * Patentart: A1, A2, A3 = EP Patentgesuche und Euro-PCT's, welche noch in keiner der drei Sprachen publiziert wurde.
   * B1, B2 = EP-Patente (beim EP-Bulletin in unterschiedlichen Sektionen aufgelistet Änderungsstatus N = New R =
   * Replacement C = Correction (not yet used) D = Deletion Automatisch übernommen werden: EP-Gesuche (A1..A9) und neue
   * EP-Patente (B1 und Status = "N) Bei Replace wird das in der DB vorhandene EP gelöscht und mit den neuen
   * aufbereiteten Daten neu erstellt Bei eingetreagenen EP ("B1....B9) und Status = "R" oder "C" wird die Änderung
   * durch einen Mitarbeiten vorgenommen (Listen). evtl. wird das Ereignis "Kein Einspruch" generiert.
   * 
   * @throws ProcessingException Verarbeitungsfehler - Patent überspringen
   */
  private void processCHPatent() throws ProcessingException, Exception
  {
    // Patentart und Änderungsstatus ermitteln
    epbandVO.epArt = xml.getAttribute( "ep-bulletin", "kind" );
    epbandVO.epStatus = xml.getAttribute( "ep-bulletin", "status" );

    if ( !epbandVO.epStatus.equalsIgnoreCase( "N" ) && !epbandVO.epStatus.equalsIgnoreCase( "R" ) && !epbandVO.epStatus.equalsIgnoreCase( "C" )
        && !epbandVO.epStatus.equalsIgnoreCase( "D" ) )
    {
      String message = "Unbekannter Status (" + epbandVO.epStatus + ") in" + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr;
      throw new ProcessingException( message );
    }

    // Mutationen bei eingetragenen Patenten
    if ( epbandVO.isPatent() && !epbandVO.epArt.equals( "B8" ) && !epbandVO.epArt.equals( "B9" )
        && ( epbandVO.epStatus.equalsIgnoreCase( "R" ) || epbandVO.epStatus.equalsIgnoreCase( "C" ) ) )
    {
      keinEinspruch();
    }
    else
    {
      // alle mit B2 ... B9 nicht weiter bearbeiten
      // aber eine Log-Meldung schreiben. 
      if ( epbandVO.isPatent() && !epbandVO.epArt.equals( "B1" ) )
      {
        if ( epbandVO.epArt.equals( "B2" ) )
        {
          throw new ProcessingException();
        }
        else
        {
          String message = "Keine Verarbeitung für  " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + ": Kind Code: " + epbandVO.epArt
              + " Kindcode wird nicht berücksichtig. ";
          throw new ProcessingException( message );
        }
      }

      // Prüfen ob Anmededatum vorhanden ist und diverse Aufbereitungen
      epbandVO.anmeldedateString = xml.getValueOf( "B220", "date" );
      epbandVO.anmeldeIdat = Util.calculateIdat( epbandVO.anmeldedateString );
      if ( epbandVO.anmeldeIdat == 0 )
      {
        String message = "Kein Anmeldedatum für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " gefunden. EP nicht übernommen";
        throw new ProcessingException( message );
      }
      epbandVO.anmeldeDate = Util.createDate( epbandVO.anmeldedateString );
      if ( epbandVO.anmeldeDate == null )
      {
        String message = "Anmededatum für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " ist nicht plausibel. EP nicht übernommen";
        throw new ProcessingException( message );
      }

      // Die einzelnen Entitäten aufbereiten
      // Die Reihenfolge ist massgebend, da einige Attribute bei nachfolgenden
      // Entitäten vorausgesetzt werden. (Bsp. pat.a_publ_sprache wird bei
      // ereignis benutzt).

      schutit.parseSGML( xml, epbandVO );
      pat.parseSGML( xml, epbandVO );
      ereignis.parseSGML( xml, epbandVO );
      prioritaet.parseSGML( xml, epbandVO );
      pat.genpatIpc();
      regAdr.parseSGML( xml, epbandVO );
    }
  }

  /**
   * Ereignis keinEinspruch 20450 generieren
   * 
   * @throws ProcessingException Verarbeitungsfehler - Patent überspringen
   */
  private void keinEinspruch() throws ProcessingException
  {
    EreignisVO ereignisVO = ereignis.getEreignisKeinEinspruch();

    String ereigIdat = xml.getValueOf( "B078EP", "date" );

    if ( !ereigIdat.equals( "" ) )
    {
      ereignisVO.ereigIdat = Util.calculateIdat( ereigIdat );
      if ( ereignisVO.ereigIdat == 0 )
      {
        String message = "Für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " falsches Datum für Ereignis "
            + "\"Kein Einspruch\". Ereignis 20450 nicht erstellt";
        throw new ProcessingException( message );
      }

      String ereigIdatErteilung = xml.getValueOf( "B450", "date" );
      if ( !ereigIdatErteilung.equals( "" ) )
      {
        int grantIdat = Util.calculateIdat( ereigIdatErteilung );
        if ( grantIdat <= ereignisVO.ereigIdat )
        {
          ereignisVO.publStat = EreignisVO.PUBLSTAT_READY_PUBLICATE;
        }
        else
        {
          ereignisVO.publStat = EreignisVO.PUBLSTAT_WAITLIST;
          ereignisVO.publDat = Util.createDate( Integer.toString( grantIdat / 100 ) );
        }
      }
      else
      {
        String message = "Für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " kein Erteilungsdatum vorhanden. "
            + " Ereignis 20450 nicht erstellt";
        throw new ProcessingException( message );
      }

      epbandVO.ereigAL.add( ereignisVO );
      epbandVO.keinEinspruch = true;
    }
    else
    {
      // Falls Tag nicht gefunden muss nichts unternommen werden
      throw new ProcessingException();
    }
  }

  /**
   * Für das Gesuch das nicht für CH/LI benannt ist muss das Ereignis 20250 vorhanden sein.<b> Falls nicht, dieses
   * aufbereiten und schreiben lassen.
   * 
   * @throws ProcessingException Verarbeitungsfehler - Patent überspringen
   */
  private void testZuruecknahme() throws ProcessingException
  {

    // Patentart und Änderungsstatus ermitteln
    epbandVO.epArt = xml.getAttribute( "ep-bulletin", "kind" );
    epbandVO.epStatus = xml.getAttribute( "ep-bulletin", "status" );

    // Aufbereiten der benötigten Daten, Ob das Ereignis geschrieben werden
    // muss oder nicht, wir in EpbandBO entschieden
    EreignisVO ereignisVO = ereignis.getEreignisZuruecknahme();

    // Publikationsdatum in B405
    ereignisVO.ereigIdat = Util.calculateIdat( xml.getValueOf( "B405", "date" ) );
    if ( ereignisVO.ereigIdat == -1 )
    {
      String message = "Für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " ist das Publikationsdatum nicht numerisch. "
          + "Verarbeitung für den Schutztitel abgebrochen";
      throw new ProcessingException( message );
    }
    epbandVO.ereigAL.add( ereignisVO );

    // schutit
    epbandVO.schutitVO.schutitTyp = SchutitVO.SCHUTIT_TYP_PCT;

    // publdat in loschdat
    epbandVO.schutitVO.loeschDat = Util.createDate( Integer.toString( ereignisVO.ereigIdat / 100 ) );
    epbandVO.schutitVO.mutMa = new Integer( EpbandVO.MA_NR );

    // Vermerken, dass es sich um ein CH Zurücknahme handelt
    epbandVO.chzuruecknahme = true;
  }
}
