package ch.ige.image;

import java.awt.image.RenderedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel.MapMode;
import java.sql.Blob;
import java.sql.SQLException;

/**
 * ImageFactory which delegates all the work to loadBytes.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: AbstractImageFactory.java,v 1.14 2009/04/16 13:07:39 locher Exp $
 */
public abstract class AbstractImageFactory implements ImageFactory
{

  private static final int BLOB_DIRECT_BUFFER_LIMIT = 204800;

  public Image wrapImage( java.awt.Image image )
  {
    MetadataBean metadata = new MetadataBean( image );
    metadata.setDPI( 72 );
    return this.wrapImage( image, metadata );
  }

  public Image wrapImage( java.awt.Image image, Metadata metadata )
  {
    return BufferedImageWrapper.constructFromAWTImage( image, metadata );
  }

  public Image wrapRenderedImage( RenderedImage image, Metadata metadata )
  {
    return new BufferedImageWrapper( image, metadata );
  }

  /**
   * Delegates to {@link #loadBytes(ByteBuffer)}. 
   */
  public EncodedImage loadFile( File image ) throws IOException, ImageException
  {
    try
    {
      final boolean writable = image.canWrite();
      final boolean toHeapBuffer = ( !writable && image.length() < BLOB_DIRECT_BUFFER_LIMIT );
      final MapMode mode = writable ? MapMode.PRIVATE : MapMode.READ_ONLY;
      return this.loadBytes( IOUtil.ensureWritable( IOUtil.asByteBuffer( image, !toHeapBuffer, mode ) ) );
    }
    catch ( FileNotFoundException e )
    {
      //hmm #canWrite lied to us...
      return this.loadBytes( IOUtil.ensureWritable( IOUtil.asByteBuffer( image, false, MapMode.READ_ONLY ) ) );
    }
  }

  /**
   * Delegates to {@link #loadStream(InputStream)}.
   */
  public EncodedImage loadURL( java.net.URL image ) throws IOException, ImageException
  {
    return this.loadStream( image.openStream() );
  }

  /**
   * Delegates to {@link #loadBytes(ByteBuffer)}. 
   */
  public EncodedImage loadStream( InputStream imageData ) throws IOException, ImageException
  {
    return this.loadBytes( IOUtil.toBuffer( imageData, true ) );
  }

  /**
   * Delegates to {@link #loadBytes(ByteBuffer)}. 
   */
  public EncodedImage loadBlob( Blob image ) throws IOException, ImageException, SQLException
  {
    int size = (int) image.length();
    boolean direct = ( size > BLOB_DIRECT_BUFFER_LIMIT );
    return this.loadBytes( IOUtil.toBuffer( image.getBinaryStream(), direct, size ) );
  }

  public EncodedImage reload( EncodedImage image ) throws ImageException
  {
    return this.loadBytes( image.getDataBuffer() );
  }

  @Deprecated
  public EncodedImage loadBytes( byte[] imageData ) throws ImageException
  {
    return loadBytes( ByteBuffer.wrap( imageData ) );
  }

  public abstract EncodedImage loadBytes( ByteBuffer imageData ) throws ImageException;

}
