package ch.ige.image;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.color.ColorSpace;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.awt.image.ColorModel;
import java.awt.image.RenderedImage;
import java.awt.image.WritableRaster;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.WindowConstants;

import ch.ige.image.color.GrayScale;
import ch.ige.image.color.GrayScale.Algorithm;
import ch.ige.image.gui.ImageDisplay;

/**
 * Baseclass for bitmap images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: BitmapImage.java,v 1.32 2009/04/14 11:38:04 locher Exp $
 */
public abstract class BitmapImage implements Image, Externalizable, Cloneable
{
  private Metadata metadata;

  /** For deserialization only. */
  protected BitmapImage()
  {
    super();
  }

  @Override
  protected Object clone() throws CloneNotSupportedException
  {
    BitmapImage image = (BitmapImage) super.clone();
    image.metadata = new MetadataBean( this.metadata );
    return image;
  }

  @Override
  public String toString()
  {
    return "Image " + new MetadataBean( this.metadata ).toString();
  }

  public Metadata getMetadata()
  {
    return this.metadata;
  }

  public WritableMetadata getWritableMetadata()
  {
    if ( this.metadata instanceof WritableMetadata )
    {
      return (WritableMetadata) this.metadata;
    }
    WritableMetadata converted = new MetadataBean( this.metadata );
    this.metadata = converted;
    return converted;
  }

  protected void setMetadata( Metadata metadata )
  {
    this.metadata = metadata;
  }

  abstract public RenderedImage asRenderedImage();

  public BufferedImage asBufferedImage()
  {
    return BitmapImage.asBufferedImage( this.asRenderedImage() );
  }

  protected static BufferedImage asBufferedImage( RenderedImage source )
  {
    if ( source instanceof BufferedImage )
    {
      return (BufferedImage) source;
    }
    ColorModel cm = source.getColorModel();
    WritableRaster raster = source.getData().createCompatibleWritableRaster();
    BufferedImage result = new BufferedImage( cm, raster, cm.isAlphaPremultiplied(), null );
    Graphics2D g = result.createGraphics();
    g.drawRenderedImage( source, new AffineTransform() );
    g.dispose();
    return result;
  }

  public Image grayscale()
  {
    return this.grayscale( GrayScale.DEFAULT_ALGORITHM );
  }

  // @see ch.ige.image.Image#grayscale(ch.ige.image.color.GrayScale.Algorithm)
  public Image grayscale( Algorithm algorithm )
  {
    return new BufferedImageWrapper( GrayScale.grayscale( algorithm, this.asBufferedImage() ), this.metadata );
  }

  public Image grayscaleAndEliminateAlpha()
  {
    return new BufferedImageWrapper( GrayScale.grayscale( Algorithm.LINEAR, this.withBackdropColor( Color.WHITE ) ), this.metadata );
  }

  public Image asRGB()
  {
    return new BufferedImageWrapper( BitmapImage.toRGB( this.asBufferedImage() ), this.metadata );
  }

  /** Convert image to sRGB. */
  private static BufferedImage toRGB( BufferedImage source )
  {
    BufferedImage target = new BufferedImage( source.getWidth(), source.getHeight(), source.getColorModel().hasAlpha() ? BufferedImage.TYPE_INT_ARGB
        : BufferedImage.TYPE_INT_RGB );
    BufferedImageOp op = new ColorConvertOp( ColorSpace.getInstance( ColorSpace.CS_sRGB ), null );
    target = op.filter( source, target );
    return target;
  }

  public BufferedImageWrapper resize( Dimension targetSize )
  {
    final RenderedImage source = this.asRenderedImage();
    final BufferedImage target = createResizeTarget( source, targetSize );
    final Graphics2D g = target.createGraphics();

    final double sx = targetSize.getWidth() / source.getWidth();
    final double sy = targetSize.getHeight() / source.getHeight();
    final Object interpolationMode = ( sx >= 1.0 || sy > 1.0 ) ? RenderingHints.VALUE_INTERPOLATION_BICUBIC : RenderingHints.VALUE_INTERPOLATION_BILINEAR;

    g.setRenderingHint( RenderingHints.KEY_INTERPOLATION, interpolationMode );
    g.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
    g.setRenderingHint( RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY );
    g.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
    g.setRenderingHint( RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY );
    g.drawRenderedImage( source, AffineTransform.getScaleInstance( sx, sy ) );
    g.dispose();
    return new BufferedImageWrapper( target, this.metadata );
  }

  private BufferedImage createResizeTarget( RenderedImage source, Dimension targetSize )
  {
    final ColorModel cm = source.getColorModel();
    if ( TransparentGrayscaleProblem.hasProblem( source ) )
    {
      int imageType = cm.hasAlpha() ? BufferedImage.TYPE_INT_ARGB : BufferedImage.TYPE_BYTE_GRAY;
      return new BufferedImage( targetSize.width, targetSize.height, imageType );
    }
    else
    {
      WritableRaster raster = source.getData().createCompatibleWritableRaster( targetSize.width, targetSize.height );
      return new BufferedImage( cm, raster, cm.isAlphaPremultiplied(), null );
    }
  }

  public BufferedImageWrapper resizeKeepAspect( Dimension maxSize )
  {
    return this.resize( ResizeUtil.trimHeight( ResizeUtil.stretchWidth( new Dimension( this.getMetadata().getSize() ), maxSize ), maxSize ) );
  }

  private BufferedImage withBackdropColor( Color color )
  {
    return BitmapImage.paintBackdropColor( this.asRenderedImage(), color );
  }

  /**
   * Replaces the alpha channel of an image with a solid color.
   * 
   * @param image the image
   * @param color the replacement color
   * @return the resulting image
   */
  public static BufferedImage paintBackdropColor( RenderedImage image, Color color )
  {
    boolean needed = image.getColorModel().getTransparency() != Transparency.OPAQUE;
    if ( !needed && image instanceof BufferedImage )
    {
      return (BufferedImage) image;
    }
    int width = image.getWidth();
    int height = image.getHeight();
    boolean opaque = ( color.getAlpha() == 255 );
    BufferedImage result = new BufferedImage( width, height, opaque ? BufferedImage.TYPE_INT_RGB : BufferedImage.TYPE_INT_ARGB );
    Graphics2D g2d = (Graphics2D) result.getGraphics();
    g2d.setColor( color );
    g2d.fillRect( 0, 0, width, height );
    g2d.drawRenderedImage( image, new AffineTransform() );
    g2d.dispose();
    return result;
  }

  private JComponent getDisplayPanel()
  {
    return new ImageDisplay( this );
  }

  public JFrame show()
  {
    JFrame frame = new JFrame( "Image" );
    frame.setDefaultCloseOperation( WindowConstants.DISPOSE_ON_CLOSE );
    frame.getContentPane().add( this.getDisplayPanel(), BorderLayout.CENTER );
    frame.pack();
    frame.setLocationRelativeTo( null );
    frame.setVisible( true );
    return frame;
  }

  /**
   * Calculates the metric size at a given DPI resolution.
   * 
   * @param dim the size in pixels
   * @param dpi the resolution
   * @return the size in millimeters
   */
  public static Dimension convertPixelToMillimeter( Dimension dim, int dpi )
  {
    final double MM_PER_INCH = 25.4;
    final double PIXELSIZE = MM_PER_INCH / dpi;
    Dimension result = new Dimension();
    result.setSize( dim.getWidth() * PIXELSIZE, dim.getHeight() * PIXELSIZE );
    return result;
  }

  /**
   * Creates a copy of a BufferedImage.
   * 
   * @param source the image
   * @return the copy
   */
  public static BufferedImage copyBufferedImage( BufferedImage source )
  {
    if ( source != null )
    {
      WritableRaster data = source.getRaster().createCompatibleWritableRaster();
      data.setRect( source.getData() );
      return new BufferedImage( source.getColorModel(), data, source.isAlphaPremultiplied(), null );
    }
    return null;
  }

  //-------------------------

  // @see java.io.Externalizable#readExternal(java.io.ObjectInput)
  public void readExternal( ObjectInput in ) throws IOException, ClassNotFoundException
  {
    this.metadata = (Metadata) in.readObject();
  }

  // @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
  public void writeExternal( ObjectOutput out ) throws IOException
  {
    out.writeObject( this.metadata );
  }

}
