package ch.ige.image;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.Externalizable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.WritableByteChannel;

import javax.swing.JFrame;

import ch.ige.image.color.GrayScale.Algorithm;

/**
 * Wrapper around the byte representation of an image. Access to metadata is not guaranteed and all image manipulation
 * will fail.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.proxy.ProxyingImageHandler
 * @author Michael Locher
 * @version $Id: EncodedImageWrapper.java,v 1.16 2009/03/19 13:26:22 locher Exp $
 */
public class EncodedImageWrapper implements EncodedImage, Externalizable
{
  private static final long serialVersionUID = 1L;

  private static final String NOT_SUPPORTED_MESSAGE = "not supported by this EncodedImageWrapper";

  private transient ByteBuffer data;

  private WritableMetadata metadata;

  /** For deserialization only. */
  public EncodedImageWrapper()
  {
    super();
  }

  public EncodedImageWrapper( InputStream is, WritableMetadata metadata ) throws IOException
  {
    this( IOUtil.toBuffer( is, false ), metadata );
  }

  @Deprecated
  public EncodedImageWrapper( byte[] data, WritableMetadata metadata )
  {
    this( ByteBuffer.wrap( data ), metadata );
  }

  public EncodedImageWrapper( ByteBuffer data, WritableMetadata metadata )
  {
    this.data = data;
    this.metadata = metadata;
  }

  @Override
  protected void finalize() throws Throwable
  {
    this.dispose();
    super.finalize();
  }

  public void dispose()
  {
    this.data = null;
  }

  public void flushCache()
  {
    // ignore
  }

  public void setMetadata( WritableMetadata metadata )
  {
    this.metadata = metadata;
  }

  public byte[] getData()
  {
    if ( this.data.isDirect() )
    {
      byte[] d = new byte[this.data.remaining()];
      this.data.duplicate().get( d );
      return d;
    }
    return this.data.array();
  }

  public ByteBuffer getDataBuffer()
  {
    return this.data.duplicate();
  }

  public String getDigest()
  {
    return EncodedBitmapImage.imageDigest( this.getDataBuffer() );
  }

  public int getFilesize()
  {
    return this.data.remaining();
  }

  /**
   * @see ch.ige.image.Image#getMetadata()
   * @throws UnsupportedOperationException if no metadata has been provided to constructor
   */
  public WritableMetadata getMetadata()
  {
    return this.getWritableMetadata();
  }

  /**
   * @see ch.ige.image.Image#getWritableMetadata()
   * @throws UnsupportedOperationException if no metadata has been provided to constructor
   */
  public WritableMetadata getWritableMetadata()
  {
    if ( this.metadata == null )
    {
      throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
    }
    return this.metadata;
  }

  public Image cloneImage() throws CloneNotSupportedException
  {
    final int size = this.data.remaining();
    final ByteBuffer copy = this.data.isDirect() ? ByteBuffer.allocateDirect( size ) : ByteBuffer.allocate( size );
    copy.put( this.data.duplicate() );
    copy.flip();
    return new EncodedImageWrapper( copy, this.metadata );
  }

  /**
   * @see ch.ige.image.Image#grayscale()
   * @throws UnsupportedOperationException when invoked
   */
  public Image grayscale()
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#grayscale(ch.ige.image.color.GrayScale.Algorithm)
   * @throws UnsupportedOperationException when invoked
   */
  public Image grayscale( Algorithm algorithm )
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#grayscaleAndEliminateAlpha()
   * @throws UnsupportedOperationException when invoked
   */
  public Image grayscaleAndEliminateAlpha()
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#asRGB()
   * @throws UnsupportedOperationException when invoked
   */
  public Image asRGB()
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#resize(java.awt.Dimension)
   * @throws UnsupportedOperationException when invoked
   */
  public Image resize( Dimension dimension )
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#resizeKeepAspect(java.awt.Dimension)
   * @throws UnsupportedOperationException when invoked
   */
  public Image resizeKeepAspect( Dimension maxDimension )
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#show()
   * @throws UnsupportedOperationException when invoked
   */
  public JFrame show()
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#asRenderedImage()
   * @throws UnsupportedOperationException when invoked
   */
  public RenderedImage asRenderedImage()
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  /**
   * @see ch.ige.image.Image#asBufferedImage()
   * @throws UnsupportedOperationException when invoked
   */
  public BufferedImage asBufferedImage()
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  public void write( File destFile ) throws IOException
  {
    EncodedBitmapImage.write( destFile, this.data );
  }

  public void write( OutputStream os ) throws IOException
  {
    WritableByteChannel out = Channels.newChannel( os );
    ByteBuffer outData = data.duplicate();
    do
    {
      out.write( outData );
    } while ( outData.hasRemaining() );
    os.flush();
  }

  /**
   * @see ch.ige.image.EncodedImage#writeSVG(java.io.OutputStream)
   * @throws UnsupportedOperationException when invoked
   */
  public void writeSVG( OutputStream os ) throws IOException
  {
    throw new UnsupportedOperationException( NOT_SUPPORTED_MESSAGE );
  }

  public void writeBase64( OutputStream os ) throws IOException
  {
    new sun.misc.BASE64Encoder().encodeBuffer( this.data, os );
    os.flush();
  }

  // @see java.io.Externalizable#readExternal(java.io.ObjectInput)
  public void readExternal( ObjectInput in ) throws IOException
  {
    this.data = IOUtil.readBuffer( in );
  }

  // @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
  public void writeExternal( ObjectOutput out ) throws IOException
  {
    IOUtil.writeBuffer( out, this.data );
  }

}
