package ch.ige.image;

import java.nio.ByteBuffer;

/**
 * Default Image Factory. Tries to create images with javax.imageio and uses JAI
 * as fallback.
 * <p>
 * DiskCache is disabled by default and native libraries are used if available.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see #setUseDiskCache(boolean)
 * 
 * @author Michael Locher
 * @version $Id: ImageFactoryImpl.java,v 1.9 2009/03/19 13:26:22 locher Exp $
 */
public class ImageFactoryImpl extends AbstractImageFactory implements ImageFactory
{

  static
  {
    ImageFactoryImpl.setUseDiskCache( false );
  }

  /**
   * Sets the usage of the disk cache.
   * 
   * @see ImageIOImage#setUseDiskCache(boolean)
   * 
   * @param useDisk
   *          whether disk cache should be used.
   */
  public static void setUseDiskCache( boolean useDisk )
  {
    ImageIOImage.setUseDiskCache( useDisk );
  }

  /**
   * Suppress usage of the native libraries.
   * 
   * <pre>
   *     JAI (mediaLib):
   *     System.setProperty(&quot;com.sun.media.jai.disableMediaLib&quot;, &quot;true&quot;);
   * 
   *     JAI-IMAGEIO (codecLib):
   *     System.setProperty(&quot;com.sun.media.imageio.disableCodecLib&quot;, &quot;true&quot;);
   * </pre>
   * 
   * <blockquote>Brian Burkhalter (main developer of JAI): "To suppress the
   * native libraries you also have these options"</blockquote>
   * 
   * see http://www.javadesktop.org/forums/thread.jspa?messageID=94137
   */
  public static void setUseNativeLibs( boolean useNative )
  {
    String value = "false";
    if ( useNative )
    {
      value = "true";
    }
    System.setProperty( "com.sun.media.jai.disableMediaLib", value );
    System.setProperty( "com.sun.media.imageio.disableCodecLib", value );
  }

  @Override
  public EncodedImage loadBytes( ByteBuffer imageData ) throws ImageException
  {
    try
    {
      return ImageIOImage.parse( imageData );
    }
    catch ( Exception e )
    {
      try
      {
        return JAIImage.parse( imageData );
      }
      catch ( Exception ex )
      {
        throw new ImageException( "Image not parsable", ex );
      }
    }
  }

}
