package ch.ige.image.format;

import java.awt.image.RenderedImage;
import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;
import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.event.IIOWriteWarningListener;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.stream.ImageOutputStream;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactoryImpl;
import ch.ige.image.MetadataException;
import ch.ige.image.IOUtil.WritableByteBuffer;

/**
 * An image format.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: Format.java,v 1.33 2009/03/19 13:26:20 locher Exp $
 */
public abstract class Format implements Serializable
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.33 $".hashCode();

  private static final Logger LOG = Logger.getLogger( Format.class );

  private String identifier;

  private String compressionType = null;

  private String formatVersion = null;

  private boolean lossless = false;

  private boolean losslessInformationAvailable = false;

  private String signatureXQuery = null;

  private String xmlRootTag = null;

  private static ThreadLocal<ConversionContext> convertContext = new ThreadLocal<ConversionContext>();

  private static final Map<String, Class<? extends Format>> SUPPORTED_FORMATS = new HashMap<String, Class<? extends Format>>();

  private static final IIOWriteWarningListener WRITE_WARNING_HANDLER = new IIOWriteWarningListener()
  {
    public void warningOccurred( ImageWriter source, int imageIndex, String warning )
    {
      LOG.warn( source.getClass().getName() + ": " + warning );
    }
  };

  public static final String KEYWORD_ORIGINAL_METADATA = "origMetadata";

  public static final String KEYWORD_ORIGINAL_RULE_NAME = "ruleName";

  public static final String KEYWORD_MAC = "mac";

  static
  {
    SUPPORTED_FORMATS.put( PNGFormat.IDENTIFIER, PNGFormat.class );
    SUPPORTED_FORMATS.put( JPEGFormat.IDENTIFIER, JPEGFormat.class );
    SUPPORTED_FORMATS.put( TIFFFormat.IDENTIFIER, TIFFFormat.class );
    SUPPORTED_FORMATS.put( TIFFFormat.IDENTIFIER2, TIFFFormat.class );
    SUPPORTED_FORMATS.put( BMPFormat.IDENTIFIER, BMPFormat.class );
    SUPPORTED_FORMATS.put( GIFFormat.IDENTIFIER, GIFFormat.class );
    SUPPORTED_FORMATS.put( JavaFormat.IDENTIFIER, JavaFormat.class );
  }

  protected Format( String identifier )
  {
    this.identifier = identifier;
    this.setSignatureXQuery( "/Text/TextEntry" );
    this.setXMLRootTag( "javax_imageio_1.0" );
  }

  /**
   * Factory for Format objects.
   * 
   * @see PNGFormat#IDENTIFIER
   * @see JPEGFormat#IDENTIFIER
   * @see TIFFFormat#IDENTIFIER
   * @see GIFFormat#IDENTIFIER
   * @see BMPFormat#IDENTIFIER
   * @param type a supported format identifier
   * @return a new Format instance of the requested type
   */
  public static Format fromTypeString( String type )
  {
    Class<? extends Format> clazz = SUPPORTED_FORMATS.get( type.toLowerCase().trim() );
    try
    {
      return clazz.newInstance();
    }
    catch ( Exception e )
    {
      throw new RuntimeException( type + ": unknown format" );
    }
  }

  /**
   * Gets the MIME type of this format.
   * 
   * @see <a href="http://www.ietf.org/rfc/rfc2045.txt">RFC 2045</a>
   * @return the MIME type
   */
  public abstract String getMIMEType();

  /**
   * Gets the name of this format.
   * 
   * @return the name of this format
   */
  public String getName()
  {
    return this.identifier;
  }

  /**
   * Tests if this format is a kind of a given format.
   * 
   * @param type the format to test for
   * @return true if this format is a kind of the given
   */
  public boolean isKindOf( String type )
  {
    return this.identifier.equals( Format.fromTypeString( type ).identifier );
  }

  /**
   * Tests if this format is a kind of a given format.
   * 
   * @param other the format to test for
   * @return true if this format is a kind of the given
   */
  public boolean isSameKind( Format other )
  {
    return isKindOf( other.identifier );
  }

  @Override
  public String toString()
  {
    StringBuffer sb = new StringBuffer( this.identifier );
    final boolean versionAvailable = this.formatVersion != null;
    final boolean compressionAvailable = this.compressionType != null;

    if ( versionAvailable || compressionAvailable || this.losslessInformationAvailable )
    {
      sb.append( " [" );

      if ( versionAvailable )
      {
        sb.append( this.formatVersion );
        sb.append( " - " );
      }

      if ( compressionAvailable )
      {
        sb.append( this.compressionType );
        sb.append( " - " );
      }

      if ( this.losslessInformationAvailable )
      {
        sb.append( this.lossless ? "lossless" : "lossy" );
      }
      sb.append( "]" );
    }

    return sb.toString();
  }

  @Override
  public boolean equals( Object obj )
  {
    if ( obj instanceof Format )
    {
      Format other = (Format) obj;
      return this.identifier.equals( other.identifier ) && this.formatVersion == null ? other.formatVersion == null : this.formatVersion
          .equals( other.formatVersion )
          && this.compressionType == null ? other.compressionType == null : this.compressionType.equals( other.compressionType )
          && this.lossless == other.lossless && this.losslessInformationAvailable == other.losslessInformationAvailable;
    }
    return false;
  }

  @Override
  public int hashCode()
  {
    return this.identifier.hashCode() | ( this.formatVersion == null ? 0 : this.formatVersion.hashCode() )
        | ( this.compressionType == null ? 0 : this.compressionType.hashCode() ) | ( this.lossless ? 2 : 0 ) | ( this.losslessInformationAvailable ? 4 : 0 );
  }

  /**
   * Gets the used compression scheme.
   * 
   * @return the scheme
   * @throws MetadataException it the information is not available
   */
  public String getCompressionType() throws MetadataException
  {
    if ( this.compressionType == null )
    {
      throw new MetadataException( "CompressionType Metadata not available" );
    }
    return compressionType;
  }

  /**
   * Specifies the used compression scheme.
   * 
   * @param compressionType the compression scheme
   */
  public void setCompressionType( String compressionType )
  {
    this.compressionType = compressionType;
  }

  /**
   * Gets the used compression scheme.
   * 
   * @return the scheme
   * @throws MetadataException if the information is not available
   */
  public String getFormatVersion() throws MetadataException
  {
    if ( this.formatVersion == null )
    {
      throw new MetadataException( "FormatVersion Metadata not available" );
    }
    return formatVersion;
  }

  /**
   * Specifies the format version.
   * 
   * @param formatVersion the version of this format
   */
  public void setFormatVersion( String formatVersion )
  {
    this.formatVersion = formatVersion;
  }

  /**
   * Tests whether this format is lossless.
   * 
   * @return true if this format is lossless
   * @throws MetadataException if the information is not available
   */
  public boolean isLossless() throws MetadataException
  {
    if ( !this.losslessInformationAvailable )
    {
      throw new MetadataException( "Lossless Metadata not available" );
    }
    return lossless;
  }

  /**
   * Specifies whether this format is lossless.
   * 
   * @param lossless true means that this format is lossless
   */
  public void setLossless( boolean lossless )
  {
    this.losslessInformationAvailable = true;
    this.lossless = lossless;
  }

  //-----------------------------------------------------------------
  // Conversion context

  /**
   * Sets the conversion context.
   * @param rulesName the name of the rule
   * @param origMetadata the metadata of the input image
   */
  public static void setConversionContext( String rulesName, String origMetadata )
  {
    Format.convertContext.set( new ConversionContext( rulesName, origMetadata ) );
  }

  private static ConversionContext getConversionContext()
  {
    return Format.convertContext.get();
  }

  /**
   * Resets the conversion context.
   */
  public static void resetConversionContext()
  {
    Format.convertContext.remove();
  }

  //-----------------------------------------------------------------

  /**
   * Encodes the given image with javax.imageio into this format.
   * 
   * @param image the image to encode
   * @return the encoded image (independant of the input image)
   * @throws ImageException it the encoding fails
   */
  public EncodedImage encode( Image image ) throws IOException, ImageException
  {
    return this.encode( image, false );
  }

  /**
   * Encodes the given image with javax.imageio into this format.
   * 
   * @param image the image to encode
   * @param sign if true, the embedded MAC tags are signed
   * @return the encoded image (independant of the input image)
   * @throws ImageException it the encoding fails
   */
  public EncodedImage encode( Image image, boolean sign ) throws IOException, ImageException
  {
    EncodingAdapter adapter = this.getEncodingAdapter( image );
    RenderedImage input = adapter.getPreparedImage( image.asRenderedImage() );
    ImageTypeSpecifier type = new ImageTypeSpecifier( input );

    Iterator<ImageWriter> imageWriters = ImageIO.getImageWriters( type, this.getName() );
    if ( !imageWriters.hasNext() )
    {
      LOG.error( "no writer found for image type: " + image.getMetadata() );
      if ( !input.getColorModel().getColorSpace().isCS_sRGB() )
      {
        LOG.info( "try to convert to sRGB" );
        Image converted = image.asRGB();
        adapter = this.getEncodingAdapter( converted );
        input = adapter.getPreparedImage( converted.asRenderedImage() );
        type = new ImageTypeSpecifier( input );
        imageWriters = ImageIO.getImageWriters( type, this.getName() );
        if ( imageWriters.hasNext() )
        {
          LOG.info( "conversion worked, using sRGB" );
        }
        else
        {
          LOG.info( "conversion did not help, giving up" );
          throw new ImageException( "no image writer available" );
        }
      }
      else
      {
        throw new ImageException( "no image writer available" );
      }
    }
    ImageWriter writer = imageWriters.next();
    writer.addIIOWriteWarningListener( WRITE_WARNING_HANDLER );

    EncodedImage encoded = null;
    try
    {
      do
      {
        WritableByteBuffer result = new WritableByteBuffer( true );
        ImageOutputStream ios = ImageIO.createImageOutputStream( result );
        writer.setOutput( ios );
        try
        {
          ImageWriteParam params = writer.getDefaultWriteParam();
          this.applyCompressionType( params );
          if ( adapter != null )
          {
            adapter.adaptWriteParams( params );
          }

          IIOMetadata streamMetadata = writer.getDefaultStreamMetadata( params );
          IIOMetadata imgMetadata = writer.getDefaultImageMetadata( type, params );
          if ( adapter != null )
          {
            adapter.adaptMetadata( Format.getConversionContext(), streamMetadata, imgMetadata );
          }
          writer.write( streamMetadata, new IIOImage( input, null, imgMetadata ), params );
        }
        finally
        {
          ios.flush();
          ios.close();
        }
        if ( encoded != null )
        {
          encoded.dispose();
        }
        ByteBuffer converted = result.getBuffer();
        if ( sign )
        {
          converted = signImage( converted );
        }
        encoded = new ImageFactoryImpl().loadBytes( converted );
      } while ( adapter != null && !adapter.resultIsOK( encoded ) );
    }
    finally
    {
      writer.removeIIOWriteWarningListener( WRITE_WARNING_HANDLER );
      writer.dispose();
    }
    return encoded;
  }

  /**
   * signs the given image data.
   * @param converted the data to be signed
   */
  protected ByteBuffer signImage( ByteBuffer converted )
  {
    return EmbeddedHashes.sign( converted );
  }

  protected void applyCompressionType( ImageWriteParam params )
  {
    if ( this.compressionType != null )
    {
      params.setCompressionMode( ImageWriteParam.MODE_EXPLICIT );
      params.setCompressionType( this.compressionType );
    }
  }

  public EncodingAdapter getEncodingAdapter( Image image )
  {
    return new EncodingAdapter()
    {
      // return a NO_OP default Implementation
    };
  }

  /**
   * @return the xpath to find the signature
   */
  public String getSignatureXQuery()
  {
    return this.signatureXQuery;
  }

  protected void setSignatureXQuery( String xQuery )
  {
    this.signatureXQuery = xQuery;
  }

  /**
   * @return the metadata xml root tag
   */
  public String getXMLRootTag()
  {
    return this.xmlRootTag;
  }

  protected void setXMLRootTag( String rootTag )
  {
    this.xmlRootTag = rootTag;
  }

  public abstract class EncodingAdapter
  {

    public RenderedImage getPreparedImage( RenderedImage image )
    {
      return image;
    }

    public void adaptWriteParams( ImageWriteParam params )
    {
    }

    /** The image will be encoded in a loop until this methods returns true. */
    public boolean resultIsOK( EncodedImage encoded )
    {
      return true;
    }

    public void adaptMetadata( ConversionContext conversionContext, IIOMetadata streamMetadata, IIOMetadata imageMetadata )
    {
    }

    protected void setPixelSizeViaDefaultMetadataSchema( IIOMetadata metadata, float value ) throws SAXException, IOException, ParserConfigurationException
    {
      String val = Float.toString( value );
      String mergeDoc = "<javax_imageio_1.0><Dimension><HorizontalPixelSize value='" + val + "'/><VerticalPixelSize value='" + val
          + "'/></Dimension></javax_imageio_1.0>";
      this.mergeMetadata( metadata, "javax_imageio_1.0", mergeDoc );
    }

    protected void mergeMetadata( IIOMetadata metadata, String format, String mergeData ) throws SAXException, IOException, ParserConfigurationException
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      Document mergeDoc = factory.newDocumentBuilder().parse( new InputSource( new StringReader( mergeData ) ) );
      // IIOMetadataUtil.serializeDOM(mergeDoc.getDocumentElement(), new
      // PrintWriter(System.out));
      metadata.mergeTree( format, mergeDoc.getDocumentElement() );
    }

  } // inner-class

  public static class ConversionContext implements Serializable
  {
    /** The serialization version. */
    private static final long serialVersionUID = "$Revision: 1.33 $".hashCode();

    private String rulesName;

    private String originalMetadata;

    /**
     * Data container, holds 
     * @param rulesName
     * @param originalMetadata
     */
    public ConversionContext( String rulesName, String originalMetadata )
    {
      super();
      this.rulesName = rulesName;
      this.originalMetadata = originalMetadata;
    }

    /**
     * Gets the rulesName.
     * @return the rulesName
     */
    public String getRulesName()
    {
      return this.rulesName;
    }

    /**
     * Gets the originalMetadata.
     * @return the originalMetadata
     */
    public String getOriginalMetadata()
    {
      return this.originalMetadata;
    }

  } //inner-class

  /**
   * Gets a header line for the supported CSV format.
   * 
   * @see #getCSV()
   * @return the header line
   */
  public static String getCSVHeader()
  {
    return "format;formatVersion;compressionType;lossless";
  }

  /**
   * Gets the format information in CSV format.
   * 
   * @return a CSV line
   */
  public String getCSV()
  {
    StringBuffer sb = new StringBuffer( this.identifier );
    sb.append( ';' );
    sb.append( ( this.formatVersion != null ) ? this.formatVersion : "?" );
    sb.append( ';' );
    sb.append( ( this.compressionType != null ) ? this.compressionType : "?" );
    sb.append( ';' );
    sb.append( this.losslessInformationAvailable ? ( this.lossless ? "yes" : "no" ) : "?" );
    return sb.toString();
  }

}
