package ch.ige.image.format;

import java.awt.image.RenderedImage;
import java.nio.ByteBuffer;

import javax.imageio.ImageWriteParam;
import javax.imageio.metadata.IIOMetadata;

import org.apache.log4j.Logger;

import ch.ige.image.Image;
import ch.ige.image.MetadataException;
import ch.ige.imagehandler.BuildInfo;

/**
 * PNG Format.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: PNGFormat.java,v 1.25 2009/05/06 09:57:40 locher Exp $
 */
public class PNGFormat extends Format
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.25 $".hashCode();

  private static final Logger LOG = Logger.getLogger( PNGFormat.class );

  /**
   * Switch to control whether sRGB chunks are written for sRGB images. Default: true.
   */
  private static final boolean SWITCH_METADATA_SRGB = Boolean.parseBoolean( System.getProperty( "ch.ige.image.pdf.srgb", "true" ) );

  /**
   * Switch to control whether gAMA and cHRM chunks are written for sRGB images. Default: false.
   */
  private static final boolean SWITCH_METADATA_GAMA_CHRM = Boolean.parseBoolean( System.getProperty( "ch.ige.image.pdf.gama_chrm", "false" ) );

  public static final String IDENTIFIER = "png";

  public static final String MIME = "image/png";

  /**
   * PNG magic bytes.
   */
  public static final ByteBuffer MAGIC_HEADER = ByteBuffer.wrap( new byte[] { (byte) 0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A } ).asReadOnlyBuffer();

  private boolean progressive = true;

  public PNGFormat()
  {
    super( IDENTIFIER );
  }

  @Override
  public String getMIMEType()
  {
    return MIME;
  }

  /**
   * Defines if the image is encoded progressive.
   * 
   * @param progressive if true the encoding is progressive
   */
  public void setProgressive( boolean progressive )
  {
    this.progressive = progressive;
  }

  @Override
  public EncodingAdapter getEncodingAdapter( final Image image )
  {
    return new Format.EncodingAdapter()
    {
      private boolean sRGB = false;

      @Override
      public RenderedImage getPreparedImage( RenderedImage image )
      {
        try
        {
          this.sRGB = image.getColorModel().getColorSpace().isCS_sRGB();
        }
        catch ( Exception e )
        {
          LOG.warn( "could not determine if sRGB", e );
        }
        return super.getPreparedImage( image );
      }

      @Override
      public void adaptWriteParams( ImageWriteParam params )
      {
        if ( PNGFormat.this.progressive )
        {
          params.setProgressiveMode( ImageWriteParam.MODE_DEFAULT );
        }
        else
        {
          params.setProgressiveMode( ImageWriteParam.MODE_DISABLED );
        }

      }

      @Override
      public void adaptMetadata( ConversionContext conversionContext, IIOMetadata streamMetadata, IIOMetadata imageMetadata )
      {
        //---------------------------------------
        // Set DPI Information
        try
        {
          float dpi = image.getMetadata().getDPI();
          // Attention: there is a bug in the current PNG writer.
          // according to the spec the correct value should be (25.4f / dpi)
          // and not the value used below.
          this.setPixelSizeViaDefaultMetadataSchema( imageMetadata, dpi / 25.4f );
        }
        catch ( MetadataException e )
        {
          // no DPI information available... nothing to do
        }
        catch ( Exception e )
        {
          LOG.info( "Setting of Resolution Metadata failed", e );
        }

        //---------------------------------------
        // Set sRGB and generator tags
        try
        {
          StringBuilder mergeDoc = new StringBuilder( "<javax_imageio_png_1.0>" );
          if ( SWITCH_METADATA_SRGB && this.sRGB )
          {
            mergeDoc.append( "<sRGB renderingIntent='Perceptual'/>" );
            if ( SWITCH_METADATA_GAMA_CHRM )
            {
              // gAMA and cHRM chunks as recommended by the spec
              mergeDoc.append( "<gAMA value='45455'/>" );
              mergeDoc
                  .append( "<cHRM whitePointX='31270' whitePointY='32900' redX='64000' redY='33000' greenX='30000' greenY='60000' blueX='15000' blueY='6000' />" );
            }
          }
          mergeDoc.append( "<tEXt><tEXtEntry keyword='Software' value='ch.ige.image, " );
          mergeDoc.append( BuildInfo.CVS_TAG );
          mergeDoc.append( "'/></tEXt></javax_imageio_png_1.0>" );
          this.mergeMetadata( imageMetadata, "javax_imageio_png_1.0", mergeDoc.toString() );
        }
        catch ( Exception e )
        {
          LOG.info( "Setting of Metadata failed", e );
        }

        //---------------------------------------
        // Set conversion context data (original metadata, profile name, ...)
        if ( conversionContext != null )
        {
          try
          {
            this.mergeMetadata( imageMetadata, "javax_imageio_png_1.0", createConversionContext( conversionContext ) );
          }
          catch ( Exception e )
          {
            LOG.info( "merge of conversion metadata failed", e );
            throw new RuntimeException( e );
          }
        }
      }

      private String createConversionContext( ConversionContext conversionContext )
      {
        StringBuilder mergeDoc = new StringBuilder( "<javax_imageio_png_1.0><tEXt><tEXtEntry keyword='Source' value='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_METADATA );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getOriginalMetadata() );
        mergeDoc.append( "'/><tEXtEntry keyword='Description' value='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_RULE_NAME );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getRulesName() );
        mergeDoc.append( "'/></tEXt></javax_imageio_png_1.0>" );
        return mergeDoc.toString();
      }
    };
  }

  /**
   * Signs the given image data.
   * @param converted the data to be signed
   */
  @Override
  protected ByteBuffer signImage( ByteBuffer converted )
  {
    return EmbeddedHashes.signPNG( converted );
  }

  /**
   * Checks if the given data is an encoded png image.
   * @param data the data to be checked
   * @return <code>true</code> if this buffer data is a png image, otherwise <code>false</code> 
   */
  public static boolean isPNG( ByteBuffer data )
  {
    return MAGIC_HEADER.equals( data.duplicate().limit( data.position() + MAGIC_HEADER.capacity() ) );
  }
}