package ch.ige.image.format.png;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.IntBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CodingErrorAction;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.zip.CRC32;

import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.EncodedImageWrapper;
import ch.ige.image.WritableMetadata;
import ch.ige.image.IOUtil.ReadableByteBuffer;
import ch.ige.image.IOUtil.WritableByteBuffer;

/**
 * Filters chunks of an encoded png image.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: PNGChunkFilter.java,v 1.3 2009/04/07 11:31:18 locher Exp $
 */
public abstract class PNGChunkFilter
{
  private static final Logger LOG = Logger.getLogger( PNGChunkFilter.class );

  private static final ByteBuffer PNG_HEADER;

  static
  {
    ByteBuffer header = ByteBuffer.allocate( 8 );
    header.put( (byte) -119 );
    header.put( (byte) 80 );
    header.put( (byte) 78 );
    header.put( (byte) 71 );
    header.put( (byte) 13 );
    header.put( (byte) 10 );
    header.put( (byte) 26 );
    header.put( (byte) 10 );
    header.flip();
    PNG_HEADER = header.asReadOnlyBuffer();
  }

  public EncodedImage filter( EncodedImage img )
  {
    return new EncodedImageWrapper( filter( img.getDataBuffer() ), filterMetadata( img ) );
  }

  private WritableMetadata filterMetadata( EncodedImage img )
  {
    WritableMetadata metadataForResult = null;
    try
    {
      WritableMetadata metadataFromSource = img.getWritableMetadata();
      metadataForResult = this.filterMetadataObject( metadataFromSource );
    }
    catch ( UnsupportedOperationException ignore )
    {
      // could not handle the metadata
    }
    return metadataForResult;
  }

  public ByteBuffer filter( ByteBuffer img )
  {
    ReadableByteChannel in = Channels.newChannel( new ReadableByteBuffer( img ) );
    WritableByteBuffer out = new WritableByteBuffer( img.isDirect() );
    try
    {
      this.filter( in, Channels.newChannel( out ) );
    }
    catch ( IOException e )
    {
      throw new RuntimeException( e );
    }
    return out.getBuffer();
  }

  public void filter( ReadableByteChannel in, WritableByteChannel out ) throws IOException
  {
    int bufSize = 1024;
    ByteBuffer buf = ByteBuffer.allocate( bufSize );
    readHeader( buf, in );
    emitHeader( buf, out );

    //process IHDR
    EmitableChunk chunk = EmitableChunk.read( buf, in, out );
    handleChunk( chunk );

    //process rest
    boolean isEnd = false;
    while ( !isEnd )
    {
      chunk = chunk.readNext( in );
      isEnd = chunk.isIEND();
      if ( isEnd )
      {
        this.beforeLast( out );
      }
      this.handleChunk( chunk );
    }
  }

  private void readHeader( ByteBuffer buf, ReadableByteChannel in ) throws IllegalArgumentException, IOException
  {
    buf.limit( 8 );
    readRemaining( buf, in );
    if ( !PNG_HEADER.equals( buf ) )
    {
      throw new IllegalArgumentException( "invalid PNG header" );
    }
    buf.position( 0 );
  }

  protected void emitHeader( ByteBuffer buf, WritableByteChannel out ) throws IOException
  {
    writeRemaining( buf, out );
  }

  protected static void readRemaining( ByteBuffer buf, ReadableByteChannel in ) throws IOException
  {
    int count;
    do
    {
      count = in.read( buf );
    } while ( buf.hasRemaining() && count >= 0 );
    buf.flip();
  }

  protected static void writeRemaining( ByteBuffer buf, WritableByteChannel out ) throws IOException
  {
    while ( buf.hasRemaining() )
    {
      out.write( buf );
    }
  }

  //-----------------------------------

  protected void handleChunk( EmitableChunk in ) throws IOException
  {
    in.emit();
  }

  protected void beforeLast( WritableByteChannel out ) throws IOException
  {
    // hook
  }

  protected WritableMetadata filterMetadataObject( WritableMetadata metadata )
  {
    return metadata;
  }

  //---------------------------------------------------------------------------

  protected static ByteBuffer chunkTypeBytes( String type )
  {
    try
    {
      return ByteBuffer.wrap( type.getBytes( "ASCII" ) ).asReadOnlyBuffer();
    }
    catch ( UnsupportedEncodingException e )
    {
      throw new Error( e );
    }
  }

  public interface Chunk
  {
    public static final ByteBuffer IEND_TYPE = chunkTypeBytes( "IEND" );

    public static final ByteBuffer TEXT_TYPE = chunkTypeBytes( "tEXt" );

    public static final ByteBuffer TIME_TYPE = chunkTypeBytes( "tIME" );
  }

  protected static class EmitableChunk implements Chunk
  {

    private static final TimeZone GMT_ZONE = TimeZone.getTimeZone( "GMT" );

    private static final Charset TEXT_CHARSET = Charset.forName( "ISO-8859-1" );

    public static CharsetEncoder createEncoder()
    {
      CharsetEncoder encoder = TEXT_CHARSET.newEncoder();
      encoder.onMalformedInput( CodingErrorAction.REPORT );
      encoder.onUnmappableCharacter( CodingErrorAction.REPORT );
      return encoder;
    }

    public static CharsetDecoder createDecoder()
    {
      CharsetDecoder decoder = TEXT_CHARSET.newDecoder();
      decoder.onMalformedInput( CodingErrorAction.REPORT );
      decoder.onUnmappableCharacter( CodingErrorAction.REPORT );
      return decoder;
    }

    private final ByteBuffer data;

    private final WritableByteChannel out;

    protected EmitableChunk( ByteBuffer data, WritableByteChannel out )
    {
      this.data = data;
      this.out = out;
    }

    public static EmitableChunk createChunk( ByteBuffer typeBytes, ByteBuffer payload, WritableByteChannel out )
    {
      int dataLen = payload.remaining();
      int len = 12 + dataLen;
      ByteBuffer buf = ByteBuffer.allocate( len );
      buf.asIntBuffer().put( dataLen );
      buf.position( 4 );
      buf.put( typeBytes.duplicate() );
      buf.put( payload );
      assert buf.remaining() == 4;
      buf.position( 0 );
      EmitableChunk chunk = new EmitableChunk( buf, out );
      chunk.updateCRC();
      return chunk;
    }

    public static EmitableChunk createTextChunk( CharSequence keyword, CharSequence text, WritableByteChannel out )
    {
      CharsetEncoder encoder = createEncoder();
      ByteBuffer payload = ByteBuffer.allocate( keyword.length() + 1 + text.length() );
      encoder.encode( CharBuffer.wrap( keyword ), payload, true );
      payload.put( (byte) 0 );
      encoder.encode( CharBuffer.wrap( text ), payload, true );
      payload.flip();
      return createChunk( TEXT_TYPE, payload, out );
    }

    public CharSequence extractTextFromTextChunk()
    {
      if ( !this.isType( TEXT_TYPE ) )
      {
        throw new IllegalArgumentException( "tEXt Chunk expected" );
      }
      CharsetDecoder decoder = createDecoder();
      ByteBuffer payload = this.getDataArea();
      //keyword and payload are separated with a NULL byte
      boolean foundZero = false;
      while ( payload.hasRemaining() && !foundZero )
      {
        foundZero = ( payload.get() == 0 );
      }
      try
      {
        return decoder.decode( payload );
      }
      catch ( CharacterCodingException e )
      {
        return e.toString();
      }
    }

    public static EmitableChunk createTimeChunkNow( WritableByteChannel out )
    {
      return createTimeChunk( new GregorianCalendar( GMT_ZONE ), out );
    }

    public static EmitableChunk createTimeChunk( Calendar time, WritableByteChannel out )
    {
      ByteBuffer payload = ByteBuffer.allocate( 7 );
      payload.asShortBuffer().put( (short) time.get( Calendar.YEAR ) );
      payload.position( 2 );
      payload.put( (byte) ( 1 + time.get( Calendar.MONTH ) ) );
      payload.put( (byte) time.get( Calendar.DAY_OF_MONTH ) );
      payload.put( (byte) time.get( Calendar.HOUR_OF_DAY ) );
      payload.put( (byte) time.get( Calendar.MINUTE ) );
      payload.put( (byte) time.get( Calendar.SECOND ) );
      payload.flip();
      return createChunk( TIME_TYPE, payload, out );
    }

    public static EmitableChunk read( ByteBuffer reuse, ReadableByteChannel in, WritableByteChannel out ) throws IOException
    {
      reuse.position( 0 );
      reuse.limit( 4 );
      readRemaining( reuse, in );
      int length = reuse.asIntBuffer().get();
      int requiredCapacity = length + 12;
      ByteBuffer buf = reuse;
      if ( requiredCapacity > reuse.capacity() )
      {
        buf = ByteBuffer.allocate( requiredCapacity );
        buf.put( reuse );
      }
      else
      {
        buf.position( 4 );
      }
      buf.limit( requiredCapacity );
      readRemaining( buf, in );
      return new EmitableChunk( buf, out );
    }

    public EmitableChunk readNext( ReadableByteChannel in ) throws IOException
    {
      return EmitableChunk.read( this.data, in, out );
    }

    public void emit() throws IOException
    {
      emit( out );
    }

    public void emit( WritableByteChannel target ) throws IOException
    {
      writeRemaining( data, target );
    }

    // @see java.lang.Object#toString()
    @Override
    public String toString()
    {
      return this.getType() + "[" + this.getLength() + "," + ( this.hasCorrectCRC() ? "valid" : "invalid" ) + "]";
    }

    public int getLength()
    {
      return this.data.limit() - 12;
    }

    public String getType()
    {
      try
      {
        return new String( this.data.array(), 4, 4, "ASCII" );
      }
      catch ( UnsupportedEncodingException e )
      {
        throw new Error( e );
      }
    }

    protected ByteBuffer getTypeBytes()
    {
      ByteBuffer typeBytes = this.data.duplicate();
      typeBytes.position( 4 );
      typeBytes.limit( 8 );
      return typeBytes;
    }

    public boolean isType( ByteBuffer type )
    {
      return type.equals( getTypeBytes() );
    }

    public boolean isIEND()
    {
      return isType( Chunk.IEND_TYPE );
    }

    public ByteBuffer getDataArea()
    {
      ByteBuffer area = this.data.duplicate();
      area.position( 8 );
      area.limit( area.limit() - 4 );
      return area;
    }

    //-----------------------
    // CRC

    private IntBuffer getCRCArea()
    {
      ByteBuffer crcBytes = this.data.duplicate();
      crcBytes.position( crcBytes.limit() - 4 );
      return crcBytes.asIntBuffer();
    }

    private long calcCRC()
    {
      int offset = 4;
      int len = data.limit() - 8;
      byte[] dataArray = this.data.array();
      CRC32 calc = new CRC32();
      calc.reset();
      calc.update( dataArray, offset, len );
      return calc.getValue();
    }

    private long getCRC()
    {
      IntBuffer a = this.getCRCArea();
      int crc = a.get();
      return crc & 0xffffffffL;
    }

    private void setCRC( long crc )
    {
      this.getCRCArea().put( (int) crc );
    }

    public boolean hasCorrectCRC()
    {
      return this.getCRC() == this.calcCRC();
    }

    public void updateCRC()
    {
      this.setCRC( this.calcCRC() );
    }

  } //inner-class  

}
