package ch.ige.imagehandler;

import java.io.IOException;
import java.net.URL;

import org.xml.sax.SAXException;

import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactory;
import ch.ige.image.ImageFactoryImpl;
import ch.ige.image.MetadataException;
import ch.ige.image.WritableMetadata;
import ch.ige.image.format.Format;
import ch.ige.imagehandler.profile.Profile;
import ch.ige.imagehandler.profile.ProfileFactory;
import ch.ige.imagehandler.rule.RuleException;
import ch.ige.imagehandler.rule.RuleList;

/**
 * Standard Implementation of the Image Handler.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Priska, Steiner
 * @author Michael.Locher@ipi.ch
 * @version $Id: LocalImageHandler.java,v 1.13 2008/05/20 15:23:44 galli Exp $
 */
public class LocalImageHandler implements ImageHandler
{
  private Profile profile;

  /**
   * Constructs a LocalImageHandler configured with the rules located at the given URL.
   * 
   * @param profile the Profile to use.
   */
  public LocalImageHandler( Profile profile )
  {
    this.profile = profile;
  }

  /**
   * Constructs a LocalImageHandler configured with the rules located at the given URL.
   * 
   * @param profileURL the URL of an Profile XML.
   * @throws IOException if there are problems loading the configuration.
   * @throws SAXException if there are problems parsing the configuration.
   * @throws RuleException if there are problems with the rules in the configuration.
   */
  public LocalImageHandler( URL profileURL ) throws IOException, SAXException, RuleException
  {
    this.profile = ProfileFactory.createFromXML( profileURL );
  }

  public void setProfile( Profile profile )
  {
    this.profile = profile;
  }

  /**
   * Gives an AbstractImageFactory for image creation.
   * 
   * @return an ImageFactory for creating images.
   */
  public ImageFactory getImageFactory()
  {
    return new ImageFactoryImpl();
  }

  /**
   * Throws an exception which describes the reason for failing the check.
   * 
   * @param image the image to be checked.
   * @param rulesName name of the rule set of the profile.
   * @throws MetadataException is thrown if not all needed metadata is present.
   * @throws RuleException is thrown if the image does not conform to the rules.
   */
  public void checkWithFeedback( final Image image, String rulesName ) throws MetadataException, RuleException
  {
    RuleList rules = this.getRules( rulesName );
    this.applyDefaultMetadataToMissingMetadata( image );
    rules.checkImage( image );
  }

  /**
   * Checks if the image conforms to the given rules.
   * 
   * @param image the image to be checked.
   * @param rulesName name of the rule set of the profile.
   * @return true, if the image conforms to the rules.
   * @throws MetadataException is thrown if not all needed metadata is present.
   */
  public boolean check( Image image, String rulesName ) throws MetadataException
  {
    try
    {
      this.checkWithFeedback( image, rulesName );
    }
    catch ( RuleException e )
    {
      return false;
    }
    return true;
  }

  /**
   * Converts the image according to the choosen RuleList of the profile.
   * 
   * @param image the image to be converted.
   * @param rulesName name of the rule set of the profile.
   * @return a new image with the results of the conversion.
   * @throws RuleException if a problem with a rule is detected.
   * @throws ImageException if an image manipulation operation fails.
   */
  public Image convert( Image image, String rulesName ) throws RuleException, ImageException
  {
    RuleList rules = this.getRules( rulesName );
    this.applyDefaultMetadataToMissingMetadata( image );
    try
    {
      try
      {
        String origMetadata = image.getMetadata().toString().trim();
        Format.setConversionContext( rulesName, origMetadata );
      }
      catch ( Exception e )
      {
        throw new ImageException( "failed to extract original metadata", e );
      }
      return rules.convert( image );
    }
    finally
    {
      Format.resetConversionContext();
    }
  }

  // ------------------------------------------------------------------------

  /**
   * Complete image metadata with default values if thez are missing. e.g 
   * DPI values from GIF files must be completed.
   * 
   * @param image the image to complete.
   * @throws MetadataException if the metadata could not be attached to the image.
   */
  private void applyDefaultMetadataToMissingMetadata( Image image ) throws MetadataException
  {
    WritableMetadata fixed = image.getWritableMetadata();
    try
    {
      fixed.getDPI();
    }
    catch ( MetadataException e )
    {
      fixed.setDPI( Integer.parseInt( this.profile.getEnvironmentValue( Profile.DEFAULT_DPI_NAME ) ) );
    }
  }

  /**
   * Accessor for the named RuleList in the profile.
   * 
   * @param rulesName RuleList to retrieve.
   * @return the RuleList with the given name from the profile.
   */
  private RuleList getRules( String rulesName )
  {
    return this.profile.getRules( rulesName );
  }
}
