package ch.ige.imagehandler.batch;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;

import org.apache.log4j.BasicConfigurator;
import org.xml.sax.SAXException;

import ch.ige.image.ImageFactoryImpl;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.ImageHandlerFactory;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Executes an image batch.
 * 
 * Expects arguments: <code>['triage'|'review'] [sourceConfigURL] [destDir] [profileURL] [ruleName1] ... [ruleNameN]</code>
 * 
 * In <code>triage</code> mode the user decides interactively.
 * In <code>review</code> mode all images are converted without interaction.
 * 
 * <code>sourceConfigURL</code> is expected to be an UTF-8 encoded textfile with a path to an image file on each line.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: BatchConvertMain.java,v 1.4 2009/03/19 13:26:21 locher Exp $
 */
public class BatchConvertMain implements Callable<JobDescription>
{
  private static final boolean DEBUG = false;

  protected final String timeTag = new SimpleDateFormat( "yyyy-MM-dd'T'HHmmss" ).format( new Date() );

  protected final String[] setup;

  public static void main( String[] args ) throws Exception
  {
    BasicConfigurator.configure();
    ImageFactoryImpl.setUseNativeLibs( "true".equals( System.getProperty( "batchconvert.useNative", "true" ) ) );
    ImageFactoryImpl.setUseDiskCache( "true".equals( System.getProperty( "batchconvert.useDiskcache", "true" ) ) );

    String[] setup = ( args.length > 0 || !DEBUG ) ? args : new String[] { "triage", "test_img/batchSourceTestbilderMarken.txt", "build/debug",
        "c:/jp/masterbuild/config/ige/imageprofiles/ea2_profile.xml", "convert_highres_trademark", "convert_highres_trademark_grayscale" };

    new BatchConvertMain( setup ).call();
    System.exit( 0 );
  }

  public BatchConvertMain( String[] setup )
  {
    this.setup = setup.clone();
  }

  // @see java.util.concurrent.Callable#call()
  public JobDescription call() throws Exception
  {
    try
    {
      checkParams();
    }
    catch ( IllegalArgumentException e )
    {
      showUsage( e );
      throw e;
    }
    ImageHandler h = createHandler();
    JobDescription job = createJob( h );
    BatchConvert.runAndWait( BatchConvert.createBatch( h, job ) );
    job.getConsumer().shutdown();
    return job;
  }

  protected void showUsage( IllegalArgumentException e )
  {
    System.err.println( String.format( "Usage: %s ['triage'|'review'] [sourceConfigURL] [destDir] [profileURL] [ruleName1] ... [ruleNameN]", this.getClass()
        .getName() ) );
    if ( e != null )
    {
      System.err.println();
      System.err.println( e );
    }
  }

  protected int getMinNumArgs()
  {
    return 5;
  }

  protected void checkParams()
  {
    if ( setup.length < getMinNumArgs() )
    {
      throw new IllegalArgumentException( "arguments missing" );
    }
    isTriage();
    Collection<String> convs = getRequestedConversions();
    if ( convs.isEmpty() )
    {
      throw new IllegalArgumentException( "no conversion rules specified" );
    }
    try
    {
      URL srcConfig = this.getSourceConfig();
      checkReadableURL( srcConfig );

      URL profile = this.getProfileURL();
      checkReadableURL( profile );

      File destDir = this.getDestinationDir();
      if ( destDir.exists() )
      {
        throw new IllegalArgumentException( "destination directory already exists" );
      }
    }
    catch ( IOException e )
    {
      throw new IllegalArgumentException( e );
    }
  }

  protected void checkReadableURL( URL url )
  {
    InputStream is = null;
    try
    {
      is = url.openStream();
      is.available();
    }
    catch ( IOException e )
    {
      throw new IllegalArgumentException( url.toString() + " not readable" );
    }
    finally
    {
      if ( is != null )
      {
        try
        {
          is.close();
        }
        catch ( IOException e )
        {
          //ignore
        }
      }
    }
  }

  protected boolean isTriage()
  {
    if ( "convert".equalsIgnoreCase( this.setup[0] ) )
    {
      return false;
    }
    else if ( "triage".equalsIgnoreCase( this.setup[0] ) )
    {
      return true;
    }
    else
    {
      throw new IllegalArgumentException( "first argument must be 'triage' or 'convert'" );
    }
  }

  protected URL getSourceConfig() throws IOException
  {
    return fileOrURL( this.setup[1] );
  }

  @SuppressWarnings( "unused" )
  protected File getDestinationBaseDir() throws IOException
  {
    return new File( this.setup[2] );
  }

  protected File getDestinationDir() throws IOException
  {
    return new File( this.getDestinationBaseDir(), this.timeTag );
  }

  protected URL getProfileURL() throws MalformedURLException
  {
    return fileOrURL( this.setup[3] );
  }

  protected Collection<String> getRequestedConversions()
  {
    int start = getMinNumArgs() - 1;
    int cnt = this.setup.length - start;
    List<String> conversions = new ArrayList<String>( cnt );
    for ( int i = start; i < this.setup.length; i++ )
    {
      conversions.add( this.setup[i] );
    }
    return conversions;
  }

  protected static URL fileOrURL( String spec ) throws MalformedURLException
  {
    final File f = new File( spec );
    return f.exists() ? f.toURL() : new URL( spec );
  }

  //-----------------------------------------------------------------

  protected ImageHandler createHandler() throws MalformedURLException, IOException, SAXException, RuleException
  {
    return ImageHandlerFactory.createLocalInstance( getProfileURL() );
  }

  protected JobDescription createJob( ImageHandler h ) throws IOException, MalformedURLException
  {
    return new JobDescription( getRequestedConversions(), createSource( h ), createConsumer() );
  }

  protected ImageSource createSource( ImageHandler h ) throws IOException
  {
    return new FileListSource( 2, true, h.getImageFactory(), this.getSourceConfig() );
  }

  protected Consumer createConsumer() throws IOException
  {
    File destLocation = getDestinationDir();
    return isTriage() ? triageConsumer( destLocation ) : reviewConsumer( destLocation, true );
  }

  protected List<String> getTriageOptions()
  {
    return Arrays.asList( "YES", "NO", "MAYBE" );
  }

  protected Consumer triageConsumer( File destLocation ) throws IOException
  {
    Map<String, Consumer> processings = new LinkedHashMap<String, Consumer>();
    for ( String option: this.getTriageOptions() )
    {
      processings.put( option, reviewConsumer( new File( destLocation, option.toLowerCase() ), true ) );
    }
    return new DisplayConsumer( false, processings );
  }

  protected Consumer reviewConsumer( File destLocation, boolean withMetadata ) throws IOException
  {
    return new ConversionReview( destLocation, withMetadata );
  }

}
