package ch.ige.imagehandler.batch;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.EncodedImageWrapper;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactory;
import ch.ige.image.MetadataBean;

/**
 * 
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: FileListSource.java,v 1.7 2009/03/31 15:16:01 locher Exp $
 */
public class FileListSource implements ImageSource, Iterator<ImageWithContext<EncodedImage>>
{
  protected static final Logger LOG = Logger.getLogger( FileListSource.class );

  public static final String FILE_CONTEXT = "FILE";

  private final Fetcher fetcher;

  private final BlockingQueue<ImageWithContext<EncodedImage>> images;

  private ExecutorService fetchWorker;

  public FileListSource( int numPrefetch, boolean wrapUnparsable, ImageFactory factory, URL url )
  {
    this.images = new ArrayBlockingQueue<ImageWithContext<EncodedImage>>( numPrefetch > 0 ? numPrefetch : 1 );
    this.fetchWorker = Executors.newSingleThreadExecutor();
    this.fetcher = new Fetcher( images, url, factory, wrapUnparsable );
    this.fetchWorker.execute( fetcher );
    this.fetchWorker.shutdown();
  }

  // @see ch.ige.imagehandler.batch.ImageSource#abort()
  public void abort()
  {
    this.fetcher.abort();
    this.fetchWorker.shutdownNow();
  }

  // @see java.lang.Iterable#iterator()
  public Iterator<ImageWithContext<EncodedImage>> iterator()
  {
    return this;
  }

  // @see java.util.Iterator#hasNext()
  public synchronized boolean hasNext()
  {
    this.fetcher.awaitReady();
    return !this.fetcher.isFinished() || !this.images.isEmpty();
  }

  // @see java.util.Iterator#next()
  public synchronized ImageWithContext next()
  {
    if ( !this.hasNext() )
    {
      throw new NoSuchElementException();
    }
    try
    {
      while ( true )
      {
        ImageWithContext<EncodedImage> img = this.images.poll( 1, TimeUnit.SECONDS );
        if ( img == null )
        {
          if ( this.fetcher.isFinished() )
          {
            img = this.images.poll();
            if ( img == null )
            {
              throw new NoSuchElementException();
            }
            else
            {
              return img;
            }
          }
        }
        else
        {
          return img;
        }
      }
    }
    catch ( InterruptedException e )
    {
      throw new RuntimeException( e );
    }
  }

  // @see java.util.Iterator#remove()
  public void remove()
  {
    throw new UnsupportedOperationException();
  }

  // @see ch.ige.imagehandler.batch.ImageSource#writeManifestEntry(java.io.OutputStreamWriter, ch.ige.imagehandler.batch.ImageWithContext)
  public void writeManifestEntry( OutputStreamWriter manifest, ImageWithContext img ) throws IOException
  {
    File file = (File) img.getContextValue( FILE_CONTEXT );
    if ( file != null )
    {
      String path = file.getAbsolutePath();
      manifest.write( path );
      manifest.write( '\n' );
    }
  }

  //-----------------------------------

  private static class Fetcher implements Runnable
  {

    private final CountDownLatch ready = new CountDownLatch( 1 );

    private volatile boolean finished = false;

    private final URL imageList;

    private final ImageFactory factory;

    private final boolean wrapUnparsable;

    private BlockingQueue<ImageWithContext<EncodedImage>> dest;

    public Fetcher( BlockingQueue<ImageWithContext<EncodedImage>> dest, URL imageList, ImageFactory factory, boolean wrapUnparsable )
    {
      super();
      this.imageList = imageList;
      this.factory = factory;
      this.wrapUnparsable = wrapUnparsable;
      this.dest = dest;
    }

    public void abort()
    {
      this.finished = true;
    }

    public void awaitReady()
    {
      try
      {
        this.ready.await();
      }
      catch ( InterruptedException e )
      {
        throw new RuntimeException( e );
      }
    }

    public boolean isFinished()
    {
      return this.finished;
    }

    public void run()
    {
      try
      {
        InputStream is = this.imageList.openStream();
        InputStreamReader r = new InputStreamReader( is, Charset.forName( "UTF-8" ) );
        BufferedReader filenames = new BufferedReader( r );

        File next = this.scanForNextFile( filenames );
        while ( next != null )
        {
          this.dest.put( fetch( next ) );
          this.ready.countDown();
          next = this.scanForNextFile( filenames );
        }
        r.close();
      }
      catch ( IOException e )
      {
        throw new RuntimeException( e );
      }
      catch ( InterruptedException e )
      {
        throw new RuntimeException( e );
      }
      finally
      {
        this.finished = true;
        this.ready.countDown();
      }
    }

    private File scanForNextFile( BufferedReader filenames ) throws IOException
    {
      while ( true )
      {
        String line = filenames.readLine();
        if ( line == null )
        {
          return null;
        }
        else
        {
          File imageFile = new File( line );
          if ( imageFile.exists() && imageFile.isFile() && imageFile.canRead() )
          {
            return imageFile;
          }
          else
          {
            LOG.warn( imageFile + " not found or not readable" );
          }
        }
      }
    }

    private ImageWithContext fetch( File next )
    {
      try
      {
        LOG.debug( next );
        EncodedImage image;
        try
        {
          image = this.factory.loadFile( next );
        }
        catch ( ImageException e )
        {
          LOG.warn( "could not parse image", e );
          if ( this.wrapUnparsable )
          {
            FileInputStream is = new FileInputStream( next );
            try
            {
              //TODO use better image creation (wrap only factory and loadFile method)
              image = new EncodedImageWrapper( is, new MetadataBean( new Dimension( 1, 1 ) ) );
            }
            finally
            {
              is.close();
            }
          }
          else
          {
            throw new RuntimeException( e );
          }
        }
        ImageWithContext img = new ImageWithContext( image );
        img.putContext( FILE_CONTEXT, next );
        return img;
      }
      catch ( IOException e )
      {
        throw new RuntimeException( e );
      }
    }

  } //inner-class

}
