package ch.ige.imagehandler.profile;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;

import org.apache.commons.digester.AbstractObjectCreationFactory;
import org.apache.commons.digester.Digester;
import org.apache.commons.digester.RuleSetBase;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import ch.ige.imagehandler.rule.RuleException;

/**
 * Profile Factory.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: ProfileFactory.java,v 1.10 2008/02/26 08:07:14 locher Exp $
 */
public class ProfileFactory
{

  /**
   * Creates a Profile from a XML description.
   * 
   * @see "Uses Apache Jakarta Commons Digester"
   * @param xmlProfileDescription a string with the xml description.
   * @return the generated profile
   * @throws IOException if the XML can not be read
   * @throws SAXException if the XML can not be parsed
   * @throws RuleException if the rules are correctly specified
   */
  public static Profile createFromXML( String xmlProfileDescription ) throws IOException, SAXException, RuleException
  {
    return afterParsing( (ProfileImpl) createDigester().parse( new StringReader( xmlProfileDescription ) ) );
  }

  /**
   * Creates a Profile from a XML description.
   * 
   * @see "Uses Apache Jakarta Commons Digester"
   * @param xmlProfileDescription the location of the description.
   * @return the generated profile
   * @throws IOException if the XML can not be read
   * @throws SAXException if the XML can not be parsed
   * @throws RuleException if the rules are correctly specified
   */
  public static Profile createFromXML( URL xmlProfileDescription ) throws IOException, SAXException, RuleException
  {
    return afterParsing( (ProfileImpl) createDigester().parse( xmlProfileDescription.openStream() ) );
  }

  private static Profile afterParsing( ProfileImpl profile ) throws RuleException
  {
    profile.setRuleParents();
    profile.checkProfile();
    return profile;
  }

  // --------------------------------------------------------------------------------------------------

  private static Digester createDigester()
  {
    Digester digester = new Digester();
    digester.setUseContextClassLoader( true );
    digester.setValidating( false );
    digester.addRuleSet( new DigesterRuleset() );
    return digester;
  }

  /** @see "Jakarta Commons Digester Documentation" */
  static class DigesterRuleset extends RuleSetBase
  {

    @Override
    public void addRuleInstances( Digester digester )
    {

      digester.addObjectCreate( "profile", "ch.ige.imagehandler.profile.ProfileImpl" );
      digester.addSetProperties( "profile" );

      digester.addObjectCreate( "profile/environment", "ch.ige.imagehandler.rule.parameter.Environment" );
      digester.addObjectCreate( "profile/environment/element", "ch.ige.imagehandler.rule.parameter.SingleBinding" );
      digester.addSetProperties( "profile/environment/element" );
      digester.addCallMethod( "profile/environment/element", "setValue", 0 );
      digester.addSetNext( "profile/environment/element", "addBinding" );

      digester.addSetNext( "profile/environment", "setEnvironment" );

      digester.addObjectCreate( "profile/rules", "ch.ige.imagehandler.rule.ConjunctiveList" );
      digester.addSetProperties( "profile/rules" );

      digester.addObjectCreate( "*/or", "ch.ige.imagehandler.rule.DisjunctiveList" );
      digester.addSetProperties( "*/or" );
      digester.addSetNext( "*/or", "addRule" );

      digester.addObjectCreate( "*/if", "ch.ige.imagehandler.rule.IfRule" );
      digester.addSetProperties( "*/if" );
      digester.addObjectCreate( "*/if/test", "ch.ige.imagehandler.rule.ConjunctiveList" );
      digester.addSetProperties( "*/if/test" );
      digester.addSetNext( "*/if/test", "setTest" );
      digester.addObjectCreate( "*/if/then", "ch.ige.imagehandler.rule.ConjunctiveList" );
      digester.addSetProperties( "*/if/then" );
      digester.addSetNext( "*/if/then", "setThen" );
      digester.addObjectCreate( "*/if/else", "ch.ige.imagehandler.rule.ConjunctiveList" );
      digester.addSetProperties( "*/if/else" );
      digester.addSetNext( "*/if/else", "setElse" );
      digester.addSetNext( "*/if", "addRule" );

      digester.addFactoryCreate( "*/rule", new RuleFactory() );
      digester.addSetProperties( "*/rule" );

      digester.addObjectCreate( "*/rule/param", "ch.ige.imagehandler.rule.parameter.SingleBinding" );
      digester.addSetProperties( "*/rule/param" );
      digester.addCallMethod( "*/rule/param", "setValue", 0 );
      digester.addSetNext( "*/rule/param", "addParameter" );

      digester.addObjectCreate( "*/rule/paramlist", "ch.ige.imagehandler.rule.parameter.ListBinding" );
      digester.addSetProperties( "*/rule/paramlist" );
      digester.addCallMethod( "*/rule/paramlist/element", "addValue", 0 );
      digester.addSetNext( "*/rule/paramlist", "addParameter" );

      digester.addSetNext( "*/rule", "addRule" );

      digester.addSetNext( "profile/rules", "addRules" );
    }

  } // inner-class

  /** @see "Jakarta Commons Digester Documentation" */
  static class RuleFactory extends AbstractObjectCreationFactory
  {

    @Override
    public Object createObject( Attributes attributes ) throws Exception
    {
      String clazz = attributes.getValue( "class" );
      return Class.forName( clazz ).newInstance();
    }
  } // inner-class

}
