package ch.ige.imagehandler.proxy;

import java.awt.Dimension;
import java.io.IOException;
import java.net.SocketException;
import java.net.URL;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

import ch.ige.image.EncodedImage;
import ch.ige.image.EncodedImageWrapper;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactory;
import ch.ige.image.ImageFactoryWrapOnly;
import ch.ige.image.MetadataBean;
import ch.ige.image.MetadataException;
import ch.ige.image.WritableMetadata;
import ch.ige.image.format.Format;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Delegates Requests via HTTP to OldProxyServlet.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.proxy.ProxyServlet
 * @see ch.ige.imagehandler.ImageHandlerFactory#createProxiedInstance(URL, URL)
 * @see "Uses Apache Jakarta Commons HTTP Client"
 * @author Michael Locher
 * @version $Id: ProxyingImageHandler.java,v 1.16 2009/03/19 13:26:22 locher Exp $
 */
public class ProxyingImageHandler implements ImageHandler
{

  /**
   * todo The JDK must throw a SocketException with this detailed message if a connection is reset.
   */
  private static final String CONNECTION_RESET_EXCEPTION_MESSAGE = "Connection reset";

  private HttpClient client;

  private String proxy;

  private String profile;

  private HttpMethodRetryHandler retryHandler;

  public ProxyingImageHandler( URL profileURL, URL imageHandlerProxyURL )
  {
    this.client = new HttpClient();

    this.retryHandler = new DefaultHttpMethodRetryHandler( 10, true )
    {
      @Override
      public boolean retryMethod( final HttpMethod method, final IOException exception, int executionCount )
      {
        if ( exception instanceof SocketException && CONNECTION_RESET_EXCEPTION_MESSAGE.equals( exception.getMessage() ) )
        {
          return false;
        }
        try
        {
          int sleepTime = executionCount * 1000;
          Thread.sleep( sleepTime );
        }
        catch ( InterruptedException e )
        {
          // ignore
        }
        return super.retryMethod( method, exception, executionCount );
      }
    };

    this.profile = profileURL.toExternalForm();
    this.proxy = imageHandlerProxyURL.toExternalForm();
  }

  public ImageFactory getImageFactory()
  {
    return new ImageFactoryWrapOnly();
  }

  public void checkWithFeedback( Image image, String rulesName ) throws MetadataException, RuleException
  {
    HttpMethod request = this.createRequest( ProxyServletParameters.OP_CHECK, image, rulesName );
    try
    {
      this.execute( request );
      this.lookForExceptionsInResponse( request );
    }
    finally
    {
      request.releaseConnection();
    }
  }

  public boolean check( Image image, String rulesName ) throws MetadataException
  {
    try
    {
      this.checkWithFeedback( image, rulesName );
    }
    catch ( RuleException e )
    {
      return false;
    }
    catch ( RemoteImageException e )
    {
      return false;
    }
    return true;
  }

  public Image convert( Image image, String rulesName ) throws RuleException, ImageException
  {
    HttpMethod request = this.createRequest( ProxyServletParameters.OP_CONVERT, image, rulesName );
    try
    {
      this.execute( request );
      this.lookForExceptionsInResponse( request );
      try
      {
        EncodedImageWrapper result = (EncodedImageWrapper) new ImageFactoryWrapOnly().loadStream( request.getResponseBodyAsStream() );
        WritableMetadata metadata = this.extractMetadataFromResponse( request );
        if ( metadata != null )
        {
          result.setMetadata( metadata );
        }
        return result;
      }
      catch ( IOException e )
      {
        throw new RuntimeException( e );
      }
    }
    finally
    {
      request.releaseConnection();
    }
  }

  private HttpMethod createRequest( String operation, Image image, String rulesName )
  {
    byte[] data = null;
    if ( !( image instanceof EncodedImage ) )
    {
      throw new IllegalArgumentException( "proxying only available for encoded images" );
    }
    else
    {
      data = ( (EncodedImage) image ).getData();
    }
    PostMethod request = new PostMethod( proxy );
    request.getParams().setParameter( HttpMethodParams.RETRY_HANDLER, this.retryHandler );
    request.addRequestHeader( ProxyServletParameters.HEADER_PROFILE, this.profile );
    request.addRequestHeader( ProxyServletParameters.HEADER_RULES, rulesName );
    request.addRequestHeader( ProxyServletParameters.HEADER_OPERATION, operation );
    request.setRequestEntity( new ByteArrayRequestEntity( data ) );
    return request;
  }

  private synchronized void execute( HttpMethod method ) throws RuntimeException
  {
    int status = -1;
    try
    {
      status = this.client.executeMethod( method );
    }
    catch ( HttpException e )
    {
      throw new ProxingException( "HTTP Request failed", e.getCause() );
    }
    catch ( SocketException e )
    {
      throw new ProxingException( "Connection problem", e );

    }
    catch ( Exception e )
    {
      throw new RuntimeException( "HTTP Request failed", e );
    }
    if ( status != HttpStatus.SC_OK )
    {
      throw new ProxingException( "Method failed: " + method.getStatusLine() );
    }

  }

  /**
   * Extracts the Metadata from the HTTP Response
   * 
   * @return the metadata or null if the metadata could not be extracted
   */
  private WritableMetadata extractMetadataFromResponse( HttpMethod request )
  {
    MetadataBean result = null;
    try
    {

      Format format = Format.fromTypeString( request.getResponseHeader( ProxyServletParameters.HEADER_META_FORMAT ).getValue() );
      int width = Integer.parseInt( request.getResponseHeader( ProxyServletParameters.HEADER_META_PIXELSIZE_X ).getValue() );
      int height = Integer.parseInt( request.getResponseHeader( ProxyServletParameters.HEADER_META_PIXELSIZE_Y ).getValue() );
      result = new MetadataBean( new Dimension( width, height ) );
      result.setFormat( format );
      try
      {
        int dpi = Integer.parseInt( request.getResponseHeader( ProxyServletParameters.HEADER_META_DPI ).getValue() );
        result.setDPI( dpi );
      }
      catch ( Exception e )
      {
        // dpi was not available
      }
      try
      {
        String colorspace = request.getResponseHeader( ProxyServletParameters.HEADER_META_COLORSPACE ).getValue();
        result.setColorspaceType( colorspace );
      }
      catch ( Exception e )
      {
        // colorspace was not available
      }
    }
    catch ( Exception e )
    {
      // very unlikely that we arrive here...
      e.printStackTrace();
    }
    return result;
  }

  private void lookForExceptionsInResponse( HttpMethod request ) throws MetadataException, RuleException
  {
    if ( ProxyServletParameters.RESULT_EXCEPTION.equals( request.getResponseHeader( ProxyServletParameters.HEADER_RESULT ).getValue() ) )
    {
      try
      {
        String exceptionClass = request.getResponseHeader( ProxyServletParameters.HEADER_EXCEPTION_CLASS ).getValue();
        String msg = new String( request.getResponseBody() );
        if ( MetadataException.class.getName().equals( exceptionClass ) )
        {
          throw new MetadataException( msg );
        }
        else if ( ImageException.class.getName().equals( exceptionClass ) )
        {
          throw new RemoteImageException( msg );
        }
        else if ( RuleException.class.getName().equals( exceptionClass ) )
        {
          throw new RuleException( msg );
        }
        else
        {
          throw new RuntimeException( msg );
        }
      }
      catch ( IOException e )
      {
        throw new RuntimeException( e );
      }
    }
  }

  /**
   * Indicates that an ImageException was thrown at the Proxy.
   * <p>
   * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
   * 
   * @author Michael Locher
   * @version $Id: ProxyingImageHandler.java,v 1.16 2009/03/19 13:26:22 locher Exp $
   */
  public static class RemoteImageException extends RuntimeException
  {
    /** The serialization version. */
    private static final long serialVersionUID = "$Revision: 1.16 $".hashCode();

    public RemoteImageException( String msg )
    {
      super( msg );
    }
  }

  /**
   * Signals HTTP Problems with the Proxy.
   * <p>
   * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
   * 
   * @author Michael Locher
   * @version $Id: ProxyingImageHandler.java,v 1.16 2009/03/19 13:26:22 locher Exp $
   */
  public static class ProxingException extends RuntimeException
  {
    /** The serialization version. */
    private static final long serialVersionUID = "$Revision: 1.16 $".hashCode();

    public ProxingException( String msg )
    {
      super( msg );
    }

    public ProxingException( String msg, Throwable reason )
    {
      super( msg, reason );
    }
  }

  // @see ch.ige.imagehandler.ImageHandler#isConverted(ch.ige.image.Image)
  public boolean isConverted( Image image )
  {
    // TODO Auto-generated method stub
    return false;
  }

}
