package ch.ige.imagehandler.rule;

import ch.ige.image.Image;
import ch.ige.image.ImageException;

/**
 * Combines rules with the "AND" operator.
 * <p>
 * <em>Example</em><br>
 * ConjunctiveLists are implicitly created with <code>rules</code>-rulesets
 * in profiles.
 * 
 * <pre>
 *   &lt;rules name=&quot;foo&quot;&gt;
 *     &lt;rule class=&quot;ch.ige.imagehandler.rule.X&quot; /&gt;
 *     &lt;rule class=&quot;ch.ige.imagehandler.rule.Y&quot; /&gt;
 *     &lt;rule class=&quot;ch.ige.imagehandler.rule.Z&quot; /&gt;
 *   &lt;/rules&gt;
 * </pre>
 * 
 * <p>
 * <em>Example</em><br>
 * The test and the branches of the IfRule also implicitly create
 * ConjunctiveLists.
 * 
 * <pre>
 *   &lt;if&gt;
 *     &lt;test&gt;
 *       &lt;rule class=&quot;ch.ige.imagehandler.rule.FormatChecker&quot;&gt;
 *         &lt;paramlist name=&quot;allowedFormats&quot;&gt;
 *           &lt;element&gt;TIFF&lt;/element&gt;
 *         &lt;/paramlist&gt;
 *       &lt;/rule&gt;
 *       &lt;rule class=&quot;ch.ige.imagehandler.rule.DimensionChecker&quot;&gt;
 *         &lt;param name=&quot;maxWidth&quot;&gt;800&lt;/param&gt;
 *         &lt;param name=&quot;maxHeight&quot;&gt;800&lt;/param&gt;
 *         &lt;param name=&quot;unit&quot;&gt;mm&lt;/param&gt;
 *       &lt;/rule&gt;
 *     &lt;/test&gt;
 *     &lt;then&gt;
 *       &lt;!-- a rule --&gt;
 *     &lt;/then&gt;
 *   &lt;/if&gt;
 * </pre>
 * 
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.profile.Profile
 * @see ch.ige.imagehandler.rule.IfRule
 * 
 * @author Michael Locher
 * @version $Id: ConjunctiveList.java,v 1.9 2009/04/14 11:38:02 locher Exp $
 */
public class ConjunctiveList<I extends Image> extends RuleList<I>
{

  @Override
  protected String getConjunction()
  {
    return "AND";
  }

  /**
   * Consults all contained Checkers if the image is conforming.
   * 
   * @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image)
   */
  public void checkImage( I image ) throws RuleException
  {
    this.logMessage();
    for ( Rule rule: this.getRules() )
    {
      if ( rule instanceof Checker )
      {
        Checker<I> checker = (Checker<I>) rule;
        checker.checkImage( image );
      }
    }
  }

  /**
   * Passes the image through a conversion "pipe". Each contained converter
   * forms a stage in the pipe.
   * 
   * @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image)
   */
  public I convert( I source ) throws RuleException, ImageException
  {
    this.logMessage();
    I result = source;
    for ( Rule rule: this.getRules() )
    {
      if ( rule instanceof Converter )
      {
        Converter<I, I> converter = (Converter<I, I>) rule;
        result = converter.convert( result );
      }
    }
    return result;
  }

}
