package ch.ige.imagehandler.rule;

import java.awt.Dimension;
import java.nio.ByteBuffer;

import javax.media.jai.PlanarImage;

import ch.ige.image.BufferedImageWrapper;
import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.JAIImage;
import ch.ige.image.Metadata;
import ch.ige.image.MetadataBean;
import ch.ige.image.MetadataException;
import ch.ige.image.ResizeUtil;
import ch.ige.image.WritableMetadata;

/**
 * Scales a JAIImage
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>width (required)</td>
 * <td>width, positive integer</td>
 * </tr>
 * <tr>
 * <td>height (required)</td>
 * <td>height, positive integer</td>
 * </tr>
 * <tr>
 * <td>keepAspectRatio (required)</td>
 * <td>"true" or "false"</td>
 * </tr>
 * <tr>
 * <td>unit (required)</td>
 * <td>"mm" or "px"</td>
 * </tr>
 * <tr>
 * <td>resolution (required)</td>
 * <td>resolution, positive integer</td>
 * </tr>
 * </table>
 *
 * <p>
 * <em>Example</em>
 * <pre>
 *  &lt;rule class=&quot;ch.ige.imagehandler.rule.JAIScaleConverter&quot;&gt;
 *    &lt;param name=&quot;width&quot;&gt;240&lt;/param&gt;
 *    &lt;param name=&quot;height&quot;&gt;48&lt;/param&gt;
 *    &lt;param name=&quot;unit&quot;&gt;px&lt;/param&gt;
 *    &lt;param name=&quot;keepAspectRatio&quot;&gt;true&lt;/param&gt;
 *    &lt;param name=&quot;resolution&quot;&gt;72&lt;/param&gt;
 *  &lt;/rule&gt;
 * </pre>
 *
 * <p>
 * Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author Mirko Galli
 * @author Christian Heinzer
 * @version  $Id: JAIScaleConverter.java,v 1.1 2009/04/23 13:55:31 galli Exp $
 */
public class JAIScaleConverter<I extends EncodedImage> extends AbstractTrademarkRule<I>
{
  private static final double MM_PER_INCH = 25.4;

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "JAI Scale Converter";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters()
  {
    this.assertParameterAvailable( CONFIG_WIDTH );
    this.assertParameterAvailable( CONFIG_HEIGHT );
    this.assertParameterAvailable( CONFIG_KEEP_ASPECT_RATIO );
    this.assertParameterAvailable( CONFIG_RESOLUTION );
    this.assertAcceptedUnit();
    this.getSpecifiedSize();
    this.keepAspectRatio();
    this.getSpecifiedResolution();
  }

  private boolean keepAspectRatio()
  {
    return Boolean.valueOf( this.getParameterValue( CONFIG_KEEP_ASPECT_RATIO ) ).booleanValue();
  }

  private Dimension getSpecifiedSize()
  {
    return new Dimension( this.getWidth(), this.getHeight() );
  }

  private int getWidth()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_WIDTH ) );
  }

  private int getHeight()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_HEIGHT ) );
  }

  private int getSpecifiedResolution()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_RESOLUTION ) );
  }

  // --------------------------------------------------------------------------------------------
  // Convert

  /* @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image) */
  public Image convert( I source ) throws RuleException, ImageException
  {
    final JAIImage parsed;
    if ( !( source instanceof JAIImage ) )
    {
      try
      {
        ByteBuffer data = ( (EncodedImage) source ).getDataBuffer();
        parsed = JAIImage.parse( data );
      }
      catch ( Exception e )
      {
        throw new ImageException( "could not parse image with JAI", e );
      }
    }
    else
    {
      parsed = (JAIImage) source;
    }

    try
    {
      final Metadata metadata = source.getMetadata();
      final Dimension size = metadata.getSize();
      final Dimension targetSize = this.isUnitPixel() ? this.getSpecifiedSize() : this.getTargetSizeInPixel( this.getSpecifiedSize() );
      double xScale = targetSize.getWidth() / size.getWidth();
      double yScale = targetSize.getHeight() / size.getHeight();

      if ( this.keepAspectRatio() )
      {
        final Dimension newSize = ResizeUtil.trimHeight( ResizeUtil.stretchWidth( new Dimension( size ), targetSize ), targetSize );
        xScale = newSize.getWidth() / size.getWidth();
        yScale = newSize.getHeight() / size.getHeight();
      }

      // JAI pipeline
      PlanarImage pipe = jaiPipeline( parsed, xScale, yScale );

      final WritableMetadata resultMetadata = new MetadataBean( metadata );
      resultMetadata.setDPI( this.getSpecifiedResolution() );
      return new BufferedImageWrapper( pipe.getAsBufferedImage(), resultMetadata );
    }
    finally
    {
      parsed.dispose();
    }
  }

  /**
   * Calculates the target size in pixel.
   * 
   * @param specifiedSize the target {@link Dimension} in mm
   * @return the target {@link Dimension} in pixel
   * @throws MetadataException
   */
  private Dimension getTargetSizeInPixel( Dimension specifiedSize ) throws MetadataException
  {
    final double factor = this.getSpecifiedResolution() / MM_PER_INCH;
    Dimension result = new Dimension();
    result.setSize( factor * specifiedSize.width, factor * specifiedSize.height );
    return result;
  }
}
