package ch.ige.imagehandler.rule;

import java.util.ArrayList;
import java.util.List;

import ch.ige.image.Image;
import ch.ige.image.MetadataException;

/**
 * Checks the resolution of an image.
 * <p>
 * <em>Parameters, case 1</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>min (required)</td>
 * <td>minimum allowed resolution in DPI, positive integer</td>
 * </tr>
 * <tr>
 * <td>max (required)</td>
 * <td>maximum allowed resolution in DPI, positive integer</td>
 * </tr>
 * </table>
 * <p>
 * <em>Parameters, case 2</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>allowed (required)</td>
 * <td>list of allowed resolutions in DPI, positive integers</td>
 * </tr>
 * <tr> </table>
 * <p>
 * <em>Examples</em>
 * <pre>
 *   &lt;rule class=&quot;ch.ige.imagehandler.rule.ResolutionChecker&quot;&gt;
 *     &lt;param name=&quot;min&quot;&gt;300&lt;/param&gt;
 *     &lt;param name=&quot;max&quot;&gt;600&lt;/param&gt;
 *   &lt;/rule&gt;
 * </pre>
 * <pre>
 *   &lt;rule class=&quot;ch.ige.imagehandler.rule.ResolutionChecker&quot;&gt;
 *     &lt;paramlist name=&quot;allowed&quot;&gt;
 *       &lt;element&gt;72&lt;/element&gt;
 *       &lt;element&gt;300&lt;/element&gt;
 *     &lt;/paramlist&gt;
 *   &lt;/rule&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.Metadata#getDPI()
 * @author Michael Locher
 * @version $Id: ResolutionChecker.java,v 1.12 2009/04/14 11:38:03 locher Exp $
 */
public class ResolutionChecker<I extends Image> extends AbstractRule<I> implements Checker<I>
{

  private static final String CONFIG_ALLOWED_RESOLUTIONS = "allowed";

  private static final String CONFIG_MAX_RESOLUTION = "max";

  private static final String CONFIG_MIN_RESOLUTION = "min";

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "Resolution Checker";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
    boolean resolutionRangeSpecified = isResolutionRangeDefined();
    if ( this.hasParameter( CONFIG_ALLOWED_RESOLUTIONS ) )
    {
      this.assertParameterPredicate( "No resolution specified", !this.getAllowedResolutions().isEmpty() );
      this.assertParameterPredicate( "Resolution range not allowed, if resolutions are listed.", !resolutionRangeSpecified );
      return;
    }
    if ( resolutionRangeSpecified )
    {
      this.assertParameterPredicate( "Minimum range must not be greater than maximum range", this.getMinResolution() <= this.getMaxResolution() );
      return;
    }
    throw new IllegalArgumentException( "Either specify '" + CONFIG_ALLOWED_RESOLUTIONS + "' or a range with '" + CONFIG_MIN_RESOLUTION + "' AND '"
        + CONFIG_MAX_RESOLUTION + "'." );
  }

  private boolean isResolutionRangeDefined()
  {
    return this.hasParameter( CONFIG_MAX_RESOLUTION ) && this.hasParameter( CONFIG_MIN_RESOLUTION );
  }

  private List<Integer> getAllowedResolutions()
  {
    List<Integer> result = new ArrayList<Integer>();
    for ( String string: this.getParameterList( CONFIG_ALLOWED_RESOLUTIONS ) )
    {
      result.add( Integer.valueOf( this.parseResolutionParam( string ) ) );
    }
    return result;
  }

  private int getMinResolution()
  {
    return this.parseResolutionParam( this.getParameterValue( CONFIG_MIN_RESOLUTION ) );
  }

  private int getMaxResolution()
  {
    return this.parseResolutionParam( this.getParameterValue( CONFIG_MAX_RESOLUTION ) );
  }

  private int parseResolutionParam( String resolution )
  {
    int res = Integer.parseInt( resolution );
    this.assertParameterPredicate( "Resolution must be a positive integer", res > 0 );
    return res;
  }

  // --------------------------------------------------------------------------------------------
  // Check

  /* @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image) */
  public void checkImage( Image image ) throws RuleException
  {
    this.logCheck();
    try
    {
      int dpi = image.getMetadata().getDPI();
      if ( this.isResolutionRangeDefined() )
      {
        if ( dpi < this.getMinResolution() )
        {
          throw new RuleException( "Resolution too low" );
        }
        if ( dpi > this.getMaxResolution() )
        {
          throw new RuleException( "Resolution too big" );
        }
      }
      else
      {
        if ( !this.getAllowedResolutions().contains( Integer.valueOf( dpi ) ) )
        {
          throw new RuleException( "Resolution not allowed" );
        }
      }

    }
    catch ( MetadataException e )
    {
      throw new RuleException( "No resolution information in image" );
    }
  }

}
