package ch.ige.imagehandler.rule;

import java.awt.Dimension;

import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.MetadataException;

/**
 * Changes the resolution.
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>resolution (required)</td>
 * <td>resolution in DPI, positive integer</td>
 * </tr>
 * <tr>
 * <td>keepDimensions (required)</td>
 * <td>"true" or "false", if true preserve the dimensions of the image</td>
 * </tr> 
 * </table>
 * <p>
 * <em>Example</em>
 * <pre>
 *      &lt;rule class=&quot;ch.ige.imagehandler.rule.ResolutionConverter&quot;&gt;
 *        &lt;param name=&quot;resolution&quot;&gt;300&lt;/param&gt;
 *        &lt;param name=&quot;keepDimensions&quot;&gt;true&lt;/param&gt;        
 *      &lt;/rule&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.WritableMetadata#setDPI(int)
 * @author Michael Locher
 * @version $Id: ResolutionConverter.java,v 1.9 2009/04/23 13:56:55 galli Exp $
 */
public class ResolutionConverter<I extends Image> extends AbstractRule<I> implements Converter<I, Image>
{
  private static final String CONFIG_KEEP_DIMENSIONS = "keepDimensions";

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "Resolution Converter";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
    this.assertParameterAvailable( CONFIG_RESOLUTION );
    this.assertParameterAvailable( CONFIG_KEEP_DIMENSIONS );
    this.getSpecifiedResolution();
    this.getKeepDimensions();
  }

  private int getSpecifiedResolution()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_RESOLUTION ) );
  }

  private boolean getKeepDimensions()
  {
    return Boolean.valueOf( this.getParameterValue( CONFIG_KEEP_DIMENSIONS ) ).booleanValue();
  }

  // --------------------------------------------------------------------------------------------
  // Convert

  /* @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image) */
  public Image convert( Image source ) throws RuleException, ImageException
  {
    this.logConvert();
    Image result;
    try
    {
      result = source.cloneImage();
    }
    catch ( CloneNotSupportedException e )
    {
      throw new RuleException( "resolution change failed (image was not clonable)" );
    }
    int targetResolution = this.getSpecifiedResolution();
    result.getWritableMetadata().setDPI( targetResolution );

    if ( this.getKeepDimensions() )
    {
      //we should try to keep the dimension of the image
      //this is only possible if the resolution of the source image is known.
      try
      {
        int origResolution = source.getMetadata().getDPI();
        if ( origResolution != targetResolution )
        {
          double ratio = (double) targetResolution / (double) origResolution;
          Dimension origPixelSize = source.getMetadata().getSize();
          Dimension targetPixelSize = new Dimension();
          targetPixelSize.setSize( ratio * origPixelSize.width, ratio * origPixelSize.height );
          result = result.resize( targetPixelSize );
        }
      }
      catch ( MetadataException e )
      {
        // no metadata available for source image
      }
    }

    return result;
  }

}
