package ch.ige.imagehandler.rule;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.image.Image;
import ch.ige.imagehandler.profile.Profile;
import ch.ige.imagehandler.rule.parameter.Parameter;

/**
 * Base class for Conjunctive and Disjunctive Rulesets.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: RuleList.java,v 1.11 2009/04/14 13:28:42 locher Exp $
 */
public abstract class RuleList<I extends Image> implements FlowControl<I, Image>
{
  private static final Logger LOG = Logger.getLogger( RuleList.class );

  private final List<Rule<?>> rules;

  private String name;

  private String message = "";

  private Profile profile;

  public RuleList()
  {
    this.name = this.getConjunction();
    this.rules = new ArrayList<Rule<?>>();
  }

  protected abstract String getConjunction();

  // ------------------------------------------------------------------------------------------
  // Rules

  /**
   * Adds a rule to the list.
   * 
   * @param rule the rule
   */
  public void addRule( Rule<?> rule )
  {
    this.rules.add( rule );
  }

  protected List<Rule<?>> getRules()
  {
    return this.rules;
  }

  /**
   * Handles itself and then handles all its rules.
   * 
   * @see ch.ige.imagehandler.rule.Rule#accept(ch.ige.imagehandler.rule.RuleVisitor)
   */
  public void accept( RuleVisitor visitor ) throws RuleException
  {
    visitor.handleRule( this );
    for ( Rule<?> rule: this.rules )
    {
      rule.accept( visitor );
    }
  }

  // @see ch.ige.imagehandler.rule.FlowControl#isConverter()
  public boolean isConverter()
  {
    for ( Rule<?> rule: this.rules )
    {
      if ( rule instanceof FlowControl && !( (FlowControl<?, ?>) rule ).isConverter() )
      {
        return false;
      }
      else if ( !( rule instanceof Converter<?, ?> ) )
      {
        return false;
      }
    }
    return true;
  }

  // ------------------------------------------------------------------------------------------
  // Parameters

  /*
   * @see ch.ige.imagehandler.rule.Rule#checkParameters()
   */
  public void checkParameters()
  {
    // no parameters required for lists
  }

  /**
   * @see ch.ige.imagehandler.rule.Rule#addParameter(ch.ige.imagehandler.rule.parameter.Parameter)
   * @throws UnsupportedOperationException if invoked
   */
  public void addParameter( Parameter binding )
  {
    throw new UnsupportedOperationException( "no parameters allowed for lists" );
  }

  // ------------------------------------------------------------------------------------------
  // Profile

  /*
   * @see ch.ige.imagehandler.rule.Rule#setProfile(ch.ige.imagehandler.Profile)
   */
  public void setProfile( Profile profile )
  {
    this.profile = profile;
  }

  /*
   * @see ch.ige.imagehandler.rule.Rule#getProfile()
   */
  public Profile getProfile()
  {
    return this.profile;
  }

  // ------------------------------------------------------------------------------------------

  /**
   * Sets the name of this ruleset.
   * 
   * @param name the name
   */
  public void setName( String name )
  {
    this.name = name;
  }

  /**
   * Gives the name of this ruleset.
   * 
   * @return the name
   */
  public String getName()
  {
    return this.name;
  }

  /**
   * Sets the message parameter.
   * 
   * @param message the message.
   */
  public void setMessage( String message )
  {
    this.message = message;
  }

  protected void logMessage()
  {
    if ( !"".equals( this.message ) )
    {
      LOG.info( this.message );
    }
  }

  @Override
  public String toString()
  {
    final String START = "\n--------------------------------------------------\n";
    final String END = "==================================================\n";
    StringBuffer result = new StringBuffer( this.name + ( !"".equals( this.message ) ? " (" + this.message + ")" : "" ) + START );
    for ( Rule rule: this.rules )
    {
      result.append( rule.toString() );
      result.append( '\n' );
    }
    result.append( END );
    return result.toString();
  }

}
