package ch.ige.imagehandler.rule;

import java.awt.Dimension;

import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.MetadataException;

/**
 * Scales the image.
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>width (required)</td>
 * <td>width, positive integer</td>
 * </tr>
 * <tr>
 * <td>height (required)</td>
 * <td>height, positive integer</td>
 * </tr>
 * <tr>
 * <td>multipleOf (optional)</td>
 * <td>pixel sizes will be multiples of this positive integer</td>
 * </tr>
 * <tr>
 * <td>keepAspectRatio (required)</td>
 * <td>"true" or "false"</td>
 * </tr>
 * <tr>
 * <td>unit (required)</td>
 * <td>"mm" or "px"</td>
 * </tr>
 * </table>
 * 
 * <p>
 * <em>Example</em>
 * <pre>
 *  &lt;rule class=&quot;ch.ige.imagehandler.rule.ScaleConverter&quot;&gt;
 *    &lt;param name=&quot;width&quot;&gt;227&lt;/param&gt;
 *    &lt;param name=&quot;height&quot;&gt;43&lt;/param&gt;
 *    &lt;param name=&quot;unit&quot;&gt;px&lt;/param&gt;
 *    &lt;param name=&quot;multipleOf&quot;&gt;16&lt;/param&gt;
 *    &lt;param name=&quot;keepAspectRatio&quot;&gt;true&lt;/param&gt;
 *  &lt;/rule&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.Image#resize(Dimension)
 * @see ch.ige.image.Image#resizeKeepAspect(Dimension)
 * 
 * @author Michael Locher
 * @version $Id: ScaleConverter.java,v 1.10 2009/04/14 11:38:02 locher Exp $
 */
public class ScaleConverter<I extends Image> extends AbstractRule<I> implements Converter<I, Image>
{

  private static final String CONFIG_MULTIPLE_OF = "multipleOf";

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "Scale Converter";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
    this.assertParameterAvailable( CONFIG_WIDTH );
    this.assertParameterAvailable( CONFIG_HEIGHT );
    this.assertParameterAvailable( CONFIG_KEEP_ASPECT_RATIO );
    this.assertAcceptedUnit();
    this.getSpecifiedSize();
    this.keepAspectRatio();
    this.getMultipleOf();
  }

  private boolean keepAspectRatio()
  {
    return Boolean.valueOf( this.getParameterValue( CONFIG_KEEP_ASPECT_RATIO ) ).booleanValue();
  }

  private Dimension getSpecifiedSize()
  {
    return new Dimension( this.getWidth(), this.getHeight() );
  }

  private int getWidth()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_WIDTH ) );
  }

  private int getHeight()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_HEIGHT ) );
  }

  private int getMultipleOf()
  {
    if ( this.hasParameter( CONFIG_MULTIPLE_OF ) )
    {
      return this.parsePositiveInteger( this.getParameterValue( CONFIG_MULTIPLE_OF ) );
    }
    return 1;
  }

  private Dimension getTargetSizeInPixel( Image img ) throws MetadataException
  {
    double factor = 1.0;
    if ( this.isUnitMillimeter() )
    {
      int dpi = img.getMetadata().getDPI();
      factor = dpi / 25.4;
    }
    Dimension specSize = this.getSpecifiedSize();
    Dimension result = new Dimension();
    result.setSize( factor * specSize.width, factor * specSize.height );
    result.width -= result.width % this.getMultipleOf();
    result.height -= result.height % this.getMultipleOf();
    return result;
  }

  // --------------------------------------------------------------------------------------------
  // Convert

  /* @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image) */
  public Image convert( I source ) throws RuleException, ImageException
  {
    this.logConvert();
    Dimension targetSize = this.getTargetSizeInPixel( source );
    if ( this.keepAspectRatio() )
    {
      return source.resizeKeepAspect( targetSize );
    }
    else
    {
      return source.resize( targetSize );
    }
  }

}
