package ch.ige.imagehandler.rule;

import java.nio.ByteBuffer;
import java.security.SignatureException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import ch.ige.image.EncodedImage;
import ch.ige.image.format.EmbeddedHashes;

/**
 * Checks IGE specific signature of an image.
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>firstValidDate (required)</td>
 * <td>Date (yyyy-MM-dd)</td>
 * </tr>
 * </table>
 * <p>
 * <em>Example</em>
 * <pre>
 *     &lt;rule class=&quot;ch.ige.imagehandler.rule.SignatureChecker&quot;&gt;
 *       &lt;param name=&quot;firstValidDate&quot;&gt;2008-01-20&lt;/param&gt;
 *     &lt;/rule&gt;
 * </pre>
 * 
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   galli
 * @version  $Id: SignatureChecker.java,v 1.9 2009/04/15 07:23:36 locher Exp $
 */
public class SignatureChecker<I extends EncodedImage> extends AbstractRule<I> implements Checker<I>
{
  private static final String DATE_FORMAT = "yyyy-MM-dd";

  private static final String CONFIG_FIRST_VALID_DATE = "firstValidDate";

  // @see ch.ige.imagehandler.rule.AbstractRule#getRuleName()
  @Override
  protected String getRuleName()
  {
    return "Signature Checker";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
    this.assertParameterAvailable( CONFIG_FIRST_VALID_DATE );
    this.getFirstValidDate();
  }

  private Date getFirstValidDate() throws RuleException
  {
    try
    {
      return new SimpleDateFormat( DATE_FORMAT ).parse( this.getParameterValue( CONFIG_FIRST_VALID_DATE ) );
    }
    catch ( ParseException e )
    {
      throw new RuleException( CONFIG_FIRST_VALID_DATE + " is not a valid date" );
    }
  }

  // @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image)
  public void checkImage( I image ) throws RuleException
  {
    try
    {
      ByteBuffer buffer = ( (EncodedImage) image ).getDataBuffer();
      if ( EmbeddedHashes.verify( buffer, getFirstValidDate() ) )
      {
        return;
      }
    }
    catch ( SignatureException e )
    {
      throw new RuleException( "Image contains invalid signature" );
    }
    catch ( Exception e )
    {
      String msg = e.getMessage();
      throw new RuleException( "Could not verify signature: " + ( msg == null ? e.toString() : msg ) );
    }
    throw new RuleException( "Image contains no signature." );
  }
}
