package ch.ige.imagehandler.test;

import java.awt.Dimension;
import java.net.URL;
import java.nio.ByteBuffer;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.Metadata;
import ch.ige.image.format.PNGFormat;
import ch.ige.image.test.ImageFactoryTest;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.ImageHandlerFactory;
import ch.ige.imagehandler.proxy.ProxyingImageHandler;
import ch.ige.imagehandler.proxy.ProxyingImageHandler.RemoteImageException;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Test ImageHandler.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * see #imageHandlerHowto(BufferedImage)
 * @author Michael Locher
 * @version $Id: ImageHandlerTest.java,v 1.33 2009/04/23 13:57:37 galli Exp $
 */
public class ImageHandlerTest extends TestCase
{
  private ImageHandler imageHandler = null;

  public ImageHandlerTest( String name )
  {
    super( name );
  }

  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    // local handler
    imageHandler = ImageHandlerFactory.createLocalInstance( getProfile() );

    // proxy handler on team86 AppServer
    //imageHandler = ImageHandlerFactory.createProxiedInstance( new URL( "http://ipiebgs010.ipie.ch:18080/imagehandlerproxy/test_profile.xml" ), new URL(
    //    "http://ipiebgs010.ipie.ch:18080/imagehandlerproxy/proxy" ) );

    // proxy handler on localhost
    //imageHandler = ImageHandlerFactory.createProxiedInstance( new URL( "http://localhost:8080/imagehandlerproxy/test_profile.xml" ), new URL(
    //    "http://localhost:8080/imagehandlerproxy/proxy" ) );
  }

  private URL getProfile()
  {
    String profile = "test_profile.xml";
    return ImageHandlerTest.class.getResource( profile );
  }

  @Override
  protected void tearDown() throws Exception
  {
    imageHandler = null;
    super.tearDown();
  }

  private EncodedImage loadImage( String name ) throws Exception
  {
    assertNotNull( ImageFactoryTest.class.getResource( name ) );
    return this.imageHandler.getImageFactory().loadURL( ImageFactoryTest.class.getResource( name ) );
  }

  public void testCheck() throws Exception
  {
    EncodedImage img = this.loadImage( "circle.png" );

    this.imageHandler.checkWithFeedback( img, "checkingRules" );

    boolean isValid = this.imageHandler.check( img, "checkingRules" );
    assertTrue( isValid );
  }

  public void testCheckWithWrong() throws Exception
  {
    EncodedImage img = this.loadImage( "circle.bmp" );

    boolean ruleExceptionThrown = false;
    try
    {
      this.imageHandler.checkWithFeedback( img, "checkingRules" );
    }
    catch ( RuleException e )
    {
      ruleExceptionThrown = true;
    }
    assertTrue( ruleExceptionThrown );

    boolean isValid = this.imageHandler.check( img, "checkingRules" );
    assertTrue( !isValid );
  }

  public void testCheckWithTrash() throws Exception
  {
    EncodedImage img = null;
    try
    {
      img = this.imageHandler.getImageFactory().loadBytes( getTrash() );
    }
    catch ( ImageException e )
    {
      // ok, trash recognized. stop test here.
      return;
    }

    // if the test runs up to here, the handler might be a proxying handler...

    boolean runtimeException = false;
    try
    {
      this.imageHandler.checkWithFeedback( img, "checkingRules" );
    }
    catch ( RemoteImageException e )
    {
      runtimeException = true;
    }
    assertTrue( runtimeException );

    boolean isValid = this.imageHandler.check( img, "checkingRules" );
    assertTrue( !isValid );
  }

  /**
   * @return
   */
  private ByteBuffer getTrash()
  {
    ByteBuffer trash = ByteBuffer.wrap( new byte[] { 1, 2, 3, 42 } );
    return trash;
  }

  public void testConvert() throws Exception
  {
    EncodedImage img = this.loadImage( "circle.png" );
    this.imageHandler.convert( img, "checkingRules" );
  }

  public void testConvertWithTrash() throws Exception
  {
    EncodedImage img = null;
    try
    {
      img = this.imageHandler.getImageFactory().loadBytes( getTrash() );
    }
    catch ( ImageException e )
    {
      // ok, trash recognized. stop test here.
      return;
    }

    // if the test runs up to here, the handler might be a proxying handler...

    boolean runtimeException = false;
    try
    {
      this.imageHandler.convert( img, "checkingRules" );
    }
    catch ( RemoteImageException e )
    {
      runtimeException = true;
    }
    assertTrue( runtimeException );
  }

  public void testMultiples() throws Exception
  {
    EncodedImage img = this.loadImage( "circle.png" );
    Image result = this.imageHandler.convert( img, "multiples" );
    Metadata meta = result.getMetadata();

    assertEquals( "result needs to be 16px high", 16, (int) meta.getSize().getHeight() );
    assertEquals( "result needs to be 16px wide", 16, (int) meta.getSize().getWidth() );
  }

  public void testThumbail() throws Exception
  {
    EncodedImage img = this.loadImage( "a6_300.gif" );
    Image thumb = this.imageHandler.convert( img, "thumbnail" );
    Metadata meta = thumb.getMetadata();

    assertEquals( "thumb needs 72DPI", 72, meta.getDPI() );
    assertEquals( "thumb needs to be 48px high", 48, (int) meta.getSize().getHeight() );
    assertTrue( "tumb needs PNG format", meta.getFormat().isKindOf( PNGFormat.IDENTIFIER ) );
  }

  public void testJAIScaleConverter() throws Exception
  {
    EncodedImage img = this.loadImage( "a6_300.gif" );
    Image thumb = this.imageHandler.convert( img, "jai_thumbnail" );
    Metadata meta = thumb.getMetadata();

    assertEquals( "thumb needs 72DPI", 72, meta.getDPI() );
    assertEquals( "thumb needs to be 48px high", 48, (int) meta.getSize().getHeight() );
    assertTrue( "tumb needs PNG format", meta.getFormat().isKindOf( PNGFormat.IDENTIFIER ) );
  }

  public void testResolutionKeepingDimension() throws Exception
  {
    EncodedImage img = this.loadImage( "a6_300.gif" );
    Image result = this.imageHandler.convert( img, "resolutionKeepingDimension" );
    Metadata meta = result.getMetadata();

    final Dimension origDimension = img.getMetadata().getSize( Image.UNIT_MILLIMETER );
    final Dimension resultDimension = meta.getSize( Image.UNIT_MILLIMETER );
    int allowedDeltaMillimeters = 1;

    assertEquals( "needs 72DPI", 72, meta.getDPI() );
    assertTrue( "keeping width dimension", Math.abs( origDimension.getWidth() - resultDimension.getWidth() ) <= allowedDeltaMillimeters );
    assertTrue( "keeping height dimension", Math.abs( origDimension.getHeight() - resultDimension.getHeight() ) <= allowedDeltaMillimeters );
  }

  public void testThumbail2() throws Exception
  {
    EncodedImage img = this.loadImage( "a6_300.gif" );
    Image thumb = this.imageHandler.convert( img, "thumbnailManual" );
    Metadata meta = thumb.getMetadata();

    assertEquals( "thumb needs 72DPI", 72, meta.getDPI() );
    assertEquals( "thumb needs to be 48px high", 48, (int) meta.getSize().getHeight() );
    assertTrue( "tumb needs PNG format", meta.getFormat().isKindOf( PNGFormat.IDENTIFIER ) );
  }

  public void testJPEGTargetSize() throws Exception
  {
    EncodedImage img = this.loadImage( "a6_300.png" );
    EncodedImage result = (EncodedImage) this.imageHandler.convert( img, "smallFile" );
    final int TARGETSIZE = 30;
    final int TOLERANCE = 256;
    final int numBytesTooBig = result.getFilesize() - TARGETSIZE * 1024;
    assertTrue( "file too big", numBytesTooBig < 0 );
    assertTrue( "file too small", numBytesTooBig + TOLERANCE > 0 );
  }

  public void NORUNtestVMKiller() throws Exception
  {
    EncodedImage circle = this.loadImage( "circle.png" );
    EncodedImage killer = this.loadImage( "vmKiller.jpg" );
    try
    {
      this.imageHandler.convert( killer, "thumbnail" );
      fail( "killer image did not kill" );
    }
    catch ( ProxyingImageHandler.ProxingException e )
    {
      // should be thrown
    }
    this.imageHandler.convert( circle, "thumbnail" );
  }

  public static void main( String[] args )
  {
    BasicConfigurator.configure();
    junit.textui.TestRunner.run( ImageHandlerTest.class );
  }

}
