package ch.ige.ipcloadvalidity.client;

import java.io.File;
import java.io.FilenameFilter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ch.ige.ipcloadvalidity.util.ValidityProperties;


/**
 * Repräsentiert das Validity-File und das Verzeichnis, welches die Validity-Files beinhaltet
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: ValidityFile.java,v 1.13 2008/07/01 06:31:40 kuenzli Exp $
 */
public class ValidityFile extends File
{
  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );


  public ValidityFile( String pathname )
  {
    super( pathname );
    sdf.setLenient(false);
  }

  public ValidityFile( String parent, String child )
  {
    super( parent, child );
    sdf.setLenient(false);
  }


  /**
   * Gibt ein Array von Files zurück die dem Format ipcr_validity_(\\d{8})_core_(\\d{4}).xml entsprechen
   * @return ValidityFile[] Array von Validity-Files
   */
  public ValidityFile[] listValidityFiles()
  {
    return this.listValidityFiles( new ValidityFilter()  );
  }

  /**
   * Gibt ein Array von ValidityFiles zurück die dem Filter entsprechen
   * @param filter FilenameFilter Filter für Validity-Files
   * @return ValidityFile[] Array von Validity-Files
   */
  private ValidityFile[] listValidityFiles( FilenameFilter filter )
  {
    String ss[] = list();
    if ( ss == null )return null;
    ArrayList v = new ArrayList();
    for ( int i = 0; i < ss.length; i++ )
    {
      if ( ( filter == null ) || filter.accept( this, ss[i] ) )
      {
        v.add( new ValidityFile( this.getPath(), ss[i] ) );
      }
    }
    return ( ValidityFile[] ) ( v.toArray( new ValidityFile[0] ) );
  }

  /**
   * Gibt das validity-date-from zurück, anhand dem das File verarbeitet wird
   * Wird anhand des Verarbeitungsdatums ermittelt oder aus dem Config File gelesen
   * @throws Exception
   * @return String validity-date-from
   */
  public String getValidityDateFrom() throws Exception
  {
    String date = extractValidityDateFrom();
    Date dateFrom = sdf.parse( date );
    return sdf.format( dateFrom );
  }


  /**
   * Gibt das validity-date-to zurück, anhand dem das File verarbeitet wird
   * Wird berechnet anhand des Verarbeitungsdatums minus ein Tag oder aus dem Config File gelesen
   * @throws Exception
   * @return String validity-date-to
   */
  public Date getValidityDateToAsDate( ) throws Exception
  {
    String date = extractValidityDateTo();
    return sdf.parse(date);
  }


  /**
   * Gibt das validity-date-to zurück, anhand dem das File verarbeitet wird
   * Wird berechnet anhand des Verarbeitungsdatums minus ein Tag oder aus dem Config File gelesen
   * @throws Exception
   * @return String validity-date-to
   */
  public String getValidityDateToAsString( ) throws Exception
  {
    return extractValidityDateTo();

  }


  /**
   * Ermittelt das Gültig bis
   * Das Gültig bis  kann in der Config-Datei übersteuert werden,
   * ansonsten wird es anhand des ValidityFile-Name ermittelt
   * @return String Gültig von
   */
  private String extractValidityDateTo() throws Exception
  {
    String date = ValidityProperties.getInstance().getProperty( ValidityProperties.VALIDITY_DATE_TO );

    if ( date == null )
    {
      date = extractValidityDateFromFile();

      Date dateFrom = sdf.parse( date );

      Calendar cal = Calendar.getInstance();
      cal.setTime( dateFrom );

      // Bis-Datum berechnen
      cal.add( Calendar.DAY_OF_MONTH, -1 );

      Date dateTo = cal.getTime();
      date = sdf.format( dateTo );
    }
    return date;
  }

  /**
   * Ermittelt das Gültig von
   * Das Gültig von kann in der Config-Datei übersteuert werden,
   * ansonsten wird es anhand des ValidityFile-Name ermittelt
   * @return String Gültig von
   */
  private String extractValidityDateFrom()
  {
    String date = ValidityProperties.getInstance().getProperty( ValidityProperties.VALIDITY_DATE_FROM );

    if ( date == null )
    {
      date = extractValidityDateFromFile();
    }
    return date;
  }

  /**
   * Ermittelt das Verarbeitungsdatum anhand des ValidityFile-Name [ipcr_validity_yyyymmdd_core_yyyy.xml]
   * Das Verarbeitungsdatum kann in der Config-Datei übersteuert werden
   * @return String Verarbeitungsdatum
   */
  private String extractValidityDateFromFile()
  {
    Pattern p = Pattern.compile( "[0-9]{8}" );
    Matcher m = p.matcher( this.getName() );

    m.find();
    return m.group();
  }
}
