package ch.ige.ipcloadvalidity.client.batch;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

import org.apache.log4j.Logger;

import ch.ige.ipcloadvalidity.BuildInfo;

/**
 * Basisklasse für alle Batch Jobs.
 * <p>
 * Erstellt die Meldung ob der Batch Job erfolgreich war oder nicht und
 * generiert die Rückmeldung. Als Rückmeldung wird in der Regel eine Datei
 * erstellt mit der entsprechenden Meldung.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author S. Nydegger, T. Iten
 * @version $Id: BatchBasis.java,v 1.18 2007/01/24 14:44:30 ruegsegger Exp $
 */
public class BatchBasis
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(BatchBasis.class);

  private static final String   FILE_PATH     = "C:\\Data\\Jobs\\";
  private static final String   FILE_EXT_OK   = ".ok";
  private static final String   FILE_EXT_NOK  = ".nok";

  public static final  String   MSG_TYPE_OK   = "OK";
  public static final  String   MSG_TYPE_NOK  = "ERROR";

  private static final String   DEF_MSG_TYPE  = MSG_TYPE_OK;
  private static final String   DEF_MSG_NR    = "0";
  private static final String   DEF_MSG_BODY  = "BatchJob erfolgreich beendet";

  private static final String   DEF_SYS_TITLE = "System Informationen:";

  
  //
  // fields
  //
  private String    fJobName      = null;           // batch job name

  private boolean   fNotifyFile   = true;           // notify options
  private boolean   fNotifyEmail  = true;
  private boolean   fNotifyConsole= false;

  // notify fields
  // for file, email-body or consoloe output
  private String    fMsgType      = DEF_MSG_TYPE;   // message fields
  private String    fMsgNr        = DEF_MSG_NR;
  private String    fMsgTitle     = null;
  private String    fMsgBody      = DEF_MSG_BODY;

  private String    fSysTitle     = DEF_SYS_TITLE;  // system info



  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Erstellt BatchJob Basisklasse.
   * Der angegebene Job Name wird zur Erstellung des Dateinamens (in der
   * folgenden Form) verwendet: FILE_PATH + jobName + FILE_EXT
   *
   * @param jobName Name des aktuellen Batch Jobs
   */
  public BatchBasis(String jobName)
  {
    this.fJobName  = jobName;
    this.fMsgTitle = "Batch Job: " + jobName;
  }


  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public boolean getNotifyFile()
  {
    return fNotifyFile;
  }

  public void setNotifyFile(boolean state)
  {
    fNotifyFile = state;
  }


  public boolean getNotifyConsole()
  {
    return fNotifyConsole;
  }

  public void setNotifyConsole(boolean state)
  {
    fNotifyConsole = state;
  }


  public void setMessage (String msgType, String msgNr, String msgBody)
  {
    fMsgType  = msgType;
    fMsgNr    = msgNr;
    fMsgBody  = msgBody;
  }


  public void notifyMessage()
  {
    LOG.debug("notifyMessage - start...");

    if ( fNotifyFile )
    {
      LOG.debug("notifyMessage - notify file");
      writeFile(getFileName(), getContent());
    }

    if (fNotifyEmail)
    {
      MailMessage msg = new MailMessage( getSubject(), getBody(), getAttachment() );
      try
      {
        LOG.debug("notifyMessage - notify email");
        msg.send();
      }
      catch ( Exception ex )
      {
        LOG.error(ex);
        LOG.info( msg.toString() );
      }
    }

    if ( fNotifyConsole )
    {
      if ( fMsgType == MSG_TYPE_OK)
      {
        LOG.debug("notifyMessage - notify out-console");
        System.out.println(getContent());
      }
      else
      {
        LOG.debug("notifyMessage - notify err-console");
        System.err.println(getContent());
      }
    }

  }


  public void notifyMessage (String msgType, String msgNr, String msgBody)
  {
    setMessage (msgType,msgNr,msgBody);
    notifyMessage();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private String getFileName()
  {
    if ( fMsgType.equals(MSG_TYPE_OK) )
    {
      return FILE_PATH + fJobName + FILE_EXT_OK;
    }
    else
    {
      return FILE_PATH + fJobName + FILE_EXT_NOK;
    }
  }

  private String getSubject()
  {
      return BuildInfo.BUILD_ENV_NAME + " - " + fMsgTitle + " - " + fMsgType;
  }

  
  private String getBody()
  {
    StringBuffer buf  = new StringBuffer();
    
    buf.append(fMsgType  + " " + fMsgNr + "\n \n");
    buf.append(fMsgTitle + "\n");
    buf.append(fMsgBody  + "\n \n");

    buf.append("CVS Tag = " + BuildInfo.CVS_TAG + "\n");
    buf.append("Build Id = " + BuildInfo.BUILD_ID + "\n");

    return buf.toString();   
  }
  

  protected String getAttachment()
  {
    return getFileName();
  }

  protected String getContent()
  {
    StringBuffer buf  = new StringBuffer();
    Date         date = new Date();

    buf.append(fMsgType  + " " + fMsgNr + "\n \n");
    buf.append(fMsgTitle + "\n");
    buf.append(fMsgBody  + "\n \n");

    buf.append(fSysTitle + "\n");
    
    buf.append("  Umgebung = " + BuildInfo.BUILD_ENV_NAME + "\n");
    buf.append("  Datum = " + date.toString() + "\n");
    buf.append("  " + javax.naming.Context.PROVIDER_URL + " = " + getProviderUrl()+ "\n");
    buf.append("  CVS Tag = " + BuildInfo.CVS_TAG + "\n");
    buf.append("  Build Id = " + BuildInfo.BUILD_ID + "\n");

    return buf.toString();
  }
 
  
  /**
   * Get current used provider url. 
   * 
   * The provider url will be used by the ch.ige.util.ServiceLocator. 
   * If a corresponding system property is set, this provider url will be used, 
   * othewise the provider url delivered by the BuildInfo class.
   *  
   * @return provider url
   */
  private String getProviderUrl()
  {
    String result = System.getProperty( javax.naming.Context.PROVIDER_URL );
    
    if ( result == null )
    {
      result = ch.ige.util.BuildInfo.PROVIDER_URL;
    }
    
    return result;
  }  


  private boolean writeFile(String fileName, String content)
  {
    boolean ret = false;

    File file = new File( fileName );
    if ( !file.getParentFile().exists() )
    {
      file.getParentFile().mkdirs();
    }

    try
    {
      BufferedWriter  bufWriter = new BufferedWriter(new FileWriter(fileName));
      bufWriter.write(content);
      bufWriter.close();
      ret = true;
    }
    catch (IOException ex)
    {
      String err = "writeFile - Datei (" + fileName + ") konnte nicht angelegt werden, " +
                   "Exception: " + ex.getMessage();

      LOG.error(err);
      LOG.info( "content of file: " + content );
    }
    return ret;
  }

} // end class
