package ch.ige.ipcloadvalidity.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;

import org.apache.log4j.Logger;

import ch.ige.ipcloadvalidity.vo.IpcVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für die Tabelle IPC
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rüegsegger
 * @version $Id: IpcDAO.java,v 1.20 2007/01/09 10:30:32 ruegsegger Exp $
 */

public class IpcDAO
{
  private static final Logger LOG = Logger.getLogger( IpcDAO.class );


  /**
   * Fügt einen Datensatz in die Tabelle IPC ein
   *
   * @param     ipcVO            ValueObject IpcVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public int insert( IpcVO ipcVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO ipc(" +
          "ipc_symb, " +
          "zusatz, " +
          "gueltig_von, " +
          "gueltig_bis, " +
          "class_level, " +
          "core_pred, " +
          "version) " +
          "VALUES( ?,?,?,?,?,?,?)";

      ps = con.prepareStatement(query);
      ps.setString( 1, ipcVO.ipcSymb );
      ps.setString( 2, ipcVO.zusatz );

      if ( ipcVO.gueltigVon == null )
      {
        ps.setDate( 3, null );
      }
      else
      {
        ps.setDate( 3, new java.sql.Date( ipcVO.gueltigVon.getTime() ) );
      }

      if ( ipcVO.gueltigBis == null )
      {
        ps.setDate( 4, null );
      }
      else
      {
        ps.setDate( 4, new java.sql.Date( ipcVO.gueltigBis.getTime() ) );
      }

      ps.setString( 5, ipcVO.classLevel );
      ps.setString( 6, ipcVO.corePred );
      ps.setInt( 7, ipcVO.version );

      return ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, IpcDAO.class, query + ipcVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Attribute eines Datensatzes aus der Tabelle IPC
   *
   * @param  ipcSymb
   * @return IpcVO ValueObject
   *
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public IpcVO select( String ipcSymb, Date gueltigVon ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    IpcVO ipcVO = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "SELECT trim(ipc_symb) ipc_symb, zusatz, gueltig_von, gueltig_bis, class_level, trim(core_pred) core_pred, version " +
          "FROM ipc WHERE ipc_symb = RPAD(?,14,' ') AND gueltig_von = ?";

      ps = con.prepareStatement(query);
      ps.setString( 1, ipcSymb );
      if ( gueltigVon == null )
      {
        ps.setDate( 2, null);
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( gueltigVon.getTime() ) );
      }

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        ipcVO = new IpcVO();
        ipcVO.ipcSymb = rs.getString( "ipc_symb" );
        ipcVO.zusatz = rs.getString( "zusatz" );
        ipcVO.gueltigVon = rs.getDate( "gueltig_von" );
        ipcVO.gueltigBis = rs.getDate( "gueltig_bis" );
        ipcVO.classLevel = rs.getString( "class_level" );
        ipcVO.corePred = rs.getString( "core_pred" );
        ipcVO.version = rs.getInt( "version" );
      }

      return ipcVO;
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, IpcDAO.class, query + ipcSymb);
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Selektiert alle Attribute eines Datensatzes aus der Tabelle IPC
   *
   * @param  ipcSymb
   * @return IpcVO ValueObject
   *
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public IpcVO selectCurrent( String ipcSymb ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    IpcVO ipcVO = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "SELECT trim(ipc_symb) ipc_symb, zusatz, gueltig_von, gueltig_bis, class_level, trim(core_pred) core_pred, version " +
          "FROM ipc WHERE ipc_symb = RPAD(?,14,' ') AND gueltig_bis is null";

      ps = con.prepareStatement(query);
      ps.setString( 1, ipcSymb );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        ipcVO = new IpcVO();
        ipcVO.ipcSymb = rs.getString( "ipc_symb" );
        ipcVO.zusatz = rs.getString( "zusatz" );
        ipcVO.gueltigVon = rs.getDate( "gueltig_von" );
        ipcVO.gueltigBis = rs.getDate( "gueltig_bis" );
        ipcVO.classLevel = rs.getString( "class_level" );
        ipcVO.corePred = rs.getString( "core_pred" );
        ipcVO.version = rs.getInt( "version" );
      }

      return ipcVO;
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, IpcDAO.class, query + ipcSymb);
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Aktualisiert einen Datensatz in der Tabelle IPC
   *
   * @param     ipcVO            ValueObject IpcVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public int updateValidTo( IpcVO ipcVO, Date gueltigBis ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE ipc SET " +
          "gueltig_bis = ?, " +
          "version = version + 1 " +
          "WHERE ipc_symb = RPAD(?,14,' ') AND gueltig_von = ? AND (gueltig_bis is null or gueltig_bis = ?) ";
      ps = con.prepareStatement(query);

      if ( gueltigBis == null )
      {
        ps.setDate( 1, null );
        ps.setDate( 4, null );
      }
      else
      {
        ps.setDate( 1, new java.sql.Date( gueltigBis.getTime() ) );
        ps.setDate( 4, new java.sql.Date( gueltigBis.getTime() ) );
      }

      ps.setString( 2, ipcVO.ipcSymb );
      if ( ipcVO.gueltigVon == null )
      {
        ps.setDate( 3, null );
      }
      else
      {
        ps.setDate( 3, new java.sql.Date( ipcVO.gueltigVon.getTime() ) );
      }

      return ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, IpcDAO.class, query + ipcVO.toString() + " gueltig bis: " + gueltigBis.getTime());
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Prüft die Existenz eines IPC-Symbols
   *
   * @param     ipcSymb       zu prüfendes IPC-Symbol
   * @return    true          IPC-Symbol existiert.
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public boolean exists( String ipcSymb, Date gueltigVon ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT ipc_symb FROM ipc WHERE ipc_symb = RPAD(?,14,' ') AND gueltig_von = ?";
      ps = con.prepareStatement(query);
      ps.setString( 1, ipcSymb );
      if ( gueltigVon == null )
      {
        ps.setDate( 2, null);
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( gueltigVon.getTime() ) );
      }

      rs = ps.executeQuery();

      return rs.next();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, IpcDAO.class, query + ipcSymb);
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle IPC ein
   * @param     ipcSymb          IPC-Symbol
   * @exception SQLException     Unerwarteter SQL Fehler
   * @return int Anzahl gelöschte Datensätze
   */
  public int delete( String ipcSymb, Date gueltigVon ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "DELETE FROM ipc WHERE ipc_symb = RPAD(?, 14, ' ') AND gueltig_von = ?";
      ps = con.prepareStatement(query);
      ps.setString( 1, ipcSymb );
      if ( gueltigVon == null )
      {
        ps.setDate( 2, null);
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( gueltigVon.getTime()) );
      }

      return ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, IpcDAO.class, query + ipcSymb);
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }







}
