package ch.ige.jipps;

import java.rmi.RemoteException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.jipps.constants.StatusConstants;
import ch.ige.jipps.dao.AmvLaufDAO;
import ch.ige.jipps.pdf.PdfMerger;
import ch.ige.jipps.vo.AmvErrorVO;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.jipps.vo.AmvVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.constants.IpTypeConstants;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.printer.ReportPaper;
import ch.ige.util.enums.report.ReportType;

/**
 * AmvControl: Steuerungsklasse für das Erstellen der automatischen Markenverlängerungen. Im AmvControl werden die
 * automatischen Markenverlängerungen für die Erinnerungen erzeugt und in der JIPPS-Datenbank gespeichert.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author Marc Bouquet
 * @version $Id: AmvControl.java,v 1.65 2008/10/28 13:55:48 lang Exp $
 * @see AuszugControl
 * @see LetterControl
 */
public class AmvControl extends ReportControl
{
  private static final Logger LOG = Logger.getLogger( AmvControl.class );

  private static final String FORMTYPE_AMV = "amv1";

  public static final String FORMTYPE_AMV_TEXT = "AMV-Erinnerungen";

  // this is used as the name for the batchjob and is displayed in the printer-mailbox
  private static final String AMV_PRINTJOB_NAME = "AMVRemind";

  protected AmvTransactionManagerHome amvTransactionManagerHome = null;

  protected AmvTransactionManager amvTransactionManager = null;

  private int drucker = 0;

  private String path = "";

  private String isSave = "0"; // zusätzliche JMS-Property

  private boolean isFinish = false; // Variable für Abschluss des Laufes trotz möglichen Fehlern

  private boolean isBatch = false; // Variable für Batch-Lauf

  private java.util.Date heute;

  /**
   * Standard-Konstruktor
   */
  public AmvControl()
  {
    heute = new java.util.Date();
    LOG.debug( " *** AMV-Controller wurde erzeugt : " + heute );
  }

  /**
   * Startet die Verarbeitung der automatischen Markenverlängerung durch die Batch-Applikation.
   * 
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   *          einfügen)
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   *         {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  public int startAMVLauf( boolean hasCover )
  {
    this.isBatch = true;
    List<String> errorList = new ArrayList<String>();

    LOG.info( "Die Verarbeitung der  " + AmvControl.FORMTYPE_AMV_TEXT + " wurde gestartet." );

    // Holt via JIPPSPropertiesBean den Drucker und Pfad aus der Jipps-Property-Tabelle für die Erinnerungen
    getJIPPSProperties( AmvControl.FORMTYPE_AMV );

    try
    {
      AmvLaufVO amvLaufVO = getAmvTransactionManager().createAmvLauf( AmvControl.FORMTYPE_AMV );

      if ( amvLaufVO != null )
      {
        List amvList = getAmvTransactionManager().loadBagisDaten( amvLaufVO.amvLaufId, amvLaufVO.typCd );
        amvLaufVO.anzAmv = amvList.size();
        amvLaufVO.verarbStat = 1;
        amvLaufVO.batchId = 0;

        if ( amvLaufVO.anzAmv != 0 )
        {

          // Sobald der erste Methodenaufruf false zurück gibt wird abgebrochen.
          if ( getAmvTransactionManager().updateAmvLauf( amvLaufVO ) && getAmvTransactionManager().insertAmvDaten( amvList ) )
          {
            LOG.debug( "*** Laufdaten sind in der DB, createAMV kann gestartet werden!" );
            errorList = updateBFaktur( getAmvTransactionManager().selectAmvLaufBfaktIds( amvLaufVO.amvLaufId ), errorList );

            if ( createAMV( hasCover, amvList, amvLaufVO, errorList, true ) )
            {
              errorList = null;
              return StatusConstants.OK;
            }
            else
            {
              errorList = null;
              return StatusConstants.ERROR;
            }
          }
          else
          {
            amvLaufVO.verarbStat = 2;
            getAmvTransactionManager().updateAmvLauf( amvLaufVO );
            LOG.error( "Beim erstellen des AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT
                + ", laden oder speichern der BAGIS-Daten ist ein Fehler aufgetreten!" );
            errorList.add( "Beim erstellen des AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT
                + ", laden oder speichern der BAGIS-Daten ist ein Fehler aufgetreten!" );
            // Sendet via JIPPSPropertiesBean ein Bestätigungsmail an die AMV-Receiver mit der Meldung das der Lauf
            // abgebrochen wurde
            sendMail( false, 0, drucker, errorList, path, getAmvTransactionManager().selectAMVLaufToId( amvLaufVO.amvLaufId ) );
            errorList = null;
            return StatusConstants.ERROR;
          }
        }
        else
        {
          // Es hat keine Daten in Bagis für die Aufbereitung
          amvLaufVO.verarbStat = 4;
          amvLaufVO.batchId = 0;
          getAmvTransactionManager().updateAmvLauf( amvLaufVO );
          /*
           * Sendet via JIPPSPropertiesBean ein Bestätigungsmail an die AMV-Receiver mit der Meldung das keine
           * Erinnerungen in der Datenbank vorhanden wahren.
           */
          sendMail( false, 0, drucker, errorList, path, getAmvTransactionManager().selectAMVLaufToId( amvLaufVO.amvLaufId ) );
          errorList = null;
          return StatusConstants.OK;
        }
      }
      else
      {
        errorList.add( "Es kann kein AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT + " erstellt werden." );
        sendMail( false, 0, drucker, errorList, path, null );
        LOG.error( "Es kann kein AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT + " erstellt werden." );
        errorList = null;
        return StatusConstants.ERROR;
      }
    }
    catch ( RemoteException reex )
    {
      errorList.add( "Es ist ein Fehler auf dem Applikation-Server aufgetreten.\nRemoteException: " + reex.getMessage() );
      sendMail( false, 0, drucker, errorList, path, null );
      LOG.error( "RemoteException in startAMVLauf( boolean, String, String ) beim Zugriff auf AmvTransactionManagerBean", reex );
      errorList = null;
      return StatusConstants.ERROR;
    }
  }

  /**
   * Startet die Verarbeitung für die übergebene Lauf-Nummer ein weiteres mal für die automatischen Markenverlängerung
   * 
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   *          einfügen)
   * @param amvLaufId int Lauf-Nummer für die, die Aufbereitung nocheinmal gestartet wird
   * @param sendMail boolean true = Bestätigungs-Mail wird gesendet, false = Bestätigungs-Mail wird nicht gesendet
   * @param isFinish boolean true = Lauf wird abgeschlossen trotz möglichen Fehlern, false = Lauf wird nicht
   *          abgeschlossen bei möglichen Fehlern
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   *         {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  public int startAMVLauf( boolean hasCover, int amvLaufId, boolean sendMail, boolean isFinish )
  {
    this.isFinish = isFinish;
    List errorList = new ArrayList();

    LOG.info( "Die Wiederverarbeitung der  " + AmvControl.FORMTYPE_AMV_TEXT + " wurde gestartet." );

    // Holt via JIPPSPropertiesBean den Drucker und Pfad aus der Jipps-Property-Tabelle für die Erinnerungen
    getJIPPSProperties( AmvControl.FORMTYPE_AMV );

    try
    {
      AmvLaufVO amvLaufVO = getAmvTransactionManager().selectAMVLaufToId( amvLaufId );

      if ( amvLaufVO != null )
      {
        // Listen enthält nur die fehlerhaften Reports
        List amvList = getAmvTransactionManager().selectVerarbStatNok( amvLaufId );

        amvLaufVO.verarbStat = 6;
        amvLaufVO.batchId = 0;

        if ( amvLaufVO.anzAmv != 0 )
        {
          // Sobald der erste Methodenaufruf false zurück gibt wird abgebrochen.
          if ( getAmvTransactionManager().updateAmvLauf( amvLaufVO ) )
          {
            // Löscht die Error-Einträge in der Tabelle JIPPS_AMV_ERROR für die Wiederverabeitung
            getAmvTransactionManager().deleteErrorMsg( amvLaufId );

            LOG.debug( "*** Laufdaten wurden aus der DB gelesen, createAMV kann gestartet werden!" );

            if ( createAMV( hasCover, amvList, amvLaufVO, errorList, sendMail ) )
            {
              errorList = null;
              return StatusConstants.OK;
            }
            else
            {
              errorList = null;
              return StatusConstants.ERROR;
            }
          }
          else
          {
            amvLaufVO.verarbStat = 2;
            getAmvTransactionManager().updateAmvLauf( amvLaufVO );
            LOG.error( "Beim erstellen des AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT + ", laden der BAGIS-Daten ist ein Fehler aufgetreten!" );
            errorList.add( "Beim erstellen des AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT + ", laden der BAGIS-Daten ist ein Fehler aufgetreten!" );
            // Sendet via JIPPSPropertiesBean ein Bestätigungsmail an die AMV-Receiver mit der Meldung das der Lauf
            // abgebrochen wurde
            sendMail( false, 0, drucker, errorList, path, getAmvTransactionManager().selectAMVLaufToId( amvLaufVO.amvLaufId ) );
            errorList = null;
            return StatusConstants.ERROR;
          }
        }
        else
        {
          // Es hat keine Daten in Bagis für die Aufbereitung
          amvLaufVO.verarbStat = 4;
          amvLaufVO.batchId = 0;
          getAmvTransactionManager().updateAmvLauf( amvLaufVO );
          /*
           * Sendet via JIPPSPropertiesBean ein Bestätigungsmail an die AMV-Receiver mit der Meldung das keine
           * Erinnerungen in der Datenbank vorhanden wahren.
           */
          sendMail( false, 0, drucker, errorList, path, getAmvTransactionManager().selectAMVLaufToId( amvLaufVO.amvLaufId ) );
          errorList = null;
          return StatusConstants.OK;
        }
      }
      else
      {
        errorList.add( "Es kann kein AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT + " wiederhergestellt werden." );
        sendMail( false, 0, drucker, errorList, path, null );
        LOG.error( "Es kann kein AMV-Lauf für die " + AmvControl.FORMTYPE_AMV_TEXT + " wiederhergestellt werden." );
        errorList = null;
        return StatusConstants.ERROR;
      }
    }
    catch ( RemoteException reex )
    {
      errorList.add( "Es ist ein Fehler auf dem Applikation-Server aufgetreten.\nRemoteException: " + reex.getMessage() );
      sendMail( false, 0, drucker, errorList, path, null );
      LOG.error( "RemoteException in startAMVLauf( boolean , int, String , String , boolean ) beim Zugriff auf AmvTransactionManagerBean:" + reex.getMessage() );
      errorList = null;
      return StatusConstants.ERROR;
    }
  }

  /**
   * Holt via JIPPSPropertiesBean aus der Jipps_Property Tabelle den Drucker und den Pfad für die Erinnerungen
   * 
   * @param propertyKey String Property-Key für das Batch-Schreiben
   */
  private void getJIPPSProperties( String propertyKey )
  {
    try
    {
      JIPPSPropertiesServices jippsProperties = getJippsProperties();
      drucker = jippsProperties.getDrucker( propertyKey );
      LOG.debug( " *** Für die  " + AmvControl.FORMTYPE_AMV_TEXT + " wurde der folgende Drucker geladen: " + drucker );
      path = jippsProperties.getDirectoryPath( propertyKey );
      LOG.debug( " *** Für die  " + AmvControl.FORMTYPE_AMV_TEXT + " wurde der folgende Path geladen: " + path );
    }
    catch ( NamingException ex )
    {
      LOG.error( "NamingException beim Lookup auf jippsProperties", ex );
    }
    catch ( CreateException ex )
    {
      LOG.error( "CreateException beim Lookup auf jippsProperties", ex );
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception in getJIPPSProperties", ex );
    }
  }

  /**
   * Erzeugt das AmvTransactionManagerBean für die Verarbeitung der automatischen Markenverlängerungen.
   */
  private AmvTransactionManager getAmvTransactionManager()
  {
    if ( this.amvTransactionManager == null )
    {
      try
      {
        Context context = new InitialContext();
        this.amvTransactionManagerHome = (AmvTransactionManagerHome) context.lookup( AmvTransactionManagerHome.JNDI_NAME );
        this.amvTransactionManager = amvTransactionManagerHome.create();
      }
      catch ( NamingException ex )
      {
        LOG.error( "NamingException beim Lookup auf amvTransactionManager", ex );
      }
      catch ( CreateException ex )
      {
        LOG.error( "CreateException beim Lookup auf amvTransactionManager", ex );
      }
      catch ( Exception ex )
      {
        LOG.error( "Exception in getAmvTransactionManager", ex );
      }
    }
    return this.amvTransactionManager;
  }

  /**
   * Führt auf der JIPPS-DB einen Update über alle Bfakt-Id's aus, die in der übergebenen Liste enthalten sind. Somit
   * wird sicher gestellt, dass bei einem Problem mit diesem Lauf die Daten auf Bagis für den nächsten Lauf nicht mehr
   * gelesen werden können.
   * 
   * @param amvListDistinc List mit Value-Objects die aktualisiert werden
   * @param errorList List Liste mit den Fehlern
   * @throws RemoteException Unerwarteter Remote-Fehler
   * @return status des Updates auf Bfaktur true = Updates konnten gestartet werden, false = keine Daten für Update
   *         vorhanden
   */
  public List<String> updateBFaktur( List amvListDistinc, List<String> errorList ) throws RemoteException
  {
    if ( amvListDistinc.size() > 0 )
    {
      Iterator iter = amvListDistinc.iterator();

      while ( iter.hasNext() )
      {
        AmvVO amvVO = (AmvVO) iter.next();
        try
        {

          // TODO hier sollte mit batchUpdate gearbeitet werden!!
          getAmvTransactionManager().updateBFakturDruckstatus( amvVO.bfaktId, "1" );
          /*
           * 07.09.2004 btm: Work a Round bei Problemen mit Bfaktur amvTransactionManager.commit( amvVO.bfaktId, "1" );
           */
        }
        catch ( Exception ex )
        {
          LOG.error( ex );
          AmvErrorVO amvErrorVO = new AmvErrorVO();
          amvErrorVO.amvLaufId = amvVO.amvLaufId;
          amvErrorVO.adrNr = amvVO.adrNr;
          amvErrorVO.schutitKey = amvVO.schutitKey;
          // String auf Grösse des Datenfeldes in der Datenbank kürzen
          amvErrorVO.errorMsg = ex.getMessage().substring( 0, 250 );
          getAmvTransactionManager().insertErrorMsg( amvErrorVO );
          errorList.add( amvErrorVO.errorMsg );
        }
      }
    }
    else
    {
      errorList.add( "Es sind keine Daten in der Tabelle JIPPS_AMV für den Update auf BAGIS!" );
      LOG.error( "Es sind keine Daten in der Tabelle JIPPS_AMV für den Update auf BAGIS! ( amvListDistinc-Liste.size() < 0 )" );
    }
    return errorList;
  }

  /**
   * Startet die Aufbereitung für die automatischen Markenverlängerungen
   * 
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   *          einfügen)
   * @param amvList List Liste mit allen Daten die Verarbeitet werden
   * @param amvLaufVO AmvLaufVO Value-Object des Laufes
   * @param errorList List Liste mit Fehlern
   * @param sendMail boolean true = Bestätigungs-Mail wird gesendet, false = Bestätigungs-Mail wird nicht gesendet
   * @return int Status der Verarbeitung 1(true) = ok, 0(false) = Fehler
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  private boolean createAMV( boolean hasCover, List amvList, AmvLaufVO amvLaufVO, List errorList, boolean sendMail ) throws RemoteException
  {
    boolean isLaufOk = true;

    if ( amvList != null )
    {
      // Der Verarb-Stat wird in der Tabelle AMV-Lauf auf 6 = in Arbeit gesetzt,
      // damit die Verarbeitung eines Laufes nicht zweimal gestartet werden kann.
      /**
       * TODO: wenn die Verarbeitung nicht beendet werden kann, aufgrund eines Out-of-Memory bleibt der Status auf 6. Im
       * Fehlerfall sollte dieser aber auf 2 gesetzt werden, da aber nichts mehr läuft ist dies eher schwierig !!!!
       */
      amvLaufVO.verarbStat = 6;
      amvTransactionManager.updateAmvLauf( amvLaufVO );

      LOG.info( "*** Anzahl " + AmvControl.FORMTYPE_AMV_TEXT + ": " + amvList.size() );

      // Prüfen ob es überhaupt AMV-Erinnerungen hat
      if ( amvList.size() > 0 )
      {
        LOG.debug( "***   " + AmvControl.FORMTYPE_AMV_TEXT + " Prozess wird gestartet ..." );

        Iterator iter = amvList.iterator();

        while ( iter.hasNext() )
        {
          AmvVO amvVO = (AmvVO) iter.next();
          createAMVOne( errorList, amvVO );

          if ( hasByteArrayValidContent( amvVO.printJobFile ) )
          {
            amvTransactionManager.insertReportPdf( amvVO );
            LOG.debug( "*** Daten für AMV 1 gespeichert" );
          }
          else
          {
            LOG.debug( "*** Daten AMV 1 wurden nicht gespeichert, da die Liste keine oder ungültige Daten enthält." );
          }
        }

        // überprüft ob die Daten als Finish abgeschlossen werden oder alle Daten erfolgreich bearbeitet werden konnten.
        if ( isFinish || amvTransactionManager.selectCheckVerarbStat( amvLaufVO.amvLaufId ) )
        {
          if ( isFinish )
          {
            List amvListFinish = amvTransactionManager.selectVerarbStatOk( amvLaufVO.amvLaufId );

            if ( amvLaufVO.anzAmv != amvListFinish.size() )
            {
              amvLaufVO.verarbStat = 5;
            }
            else
            {
              amvLaufVO.verarbStat = 3;
            }
            Collections.sort( amvListFinish ); // Sortiert die Liste nach den geforderten Kriterien der FA
            amvLaufVO.batchId = insertPrintJobs( amvListFinish, hasCover, amvLaufVO.typCd );
            amvListFinish = null;
          }
          else
          {
            amvLaufVO.verarbStat = 3;
            Collections.sort( amvList ); // Sortiert die Liste nach den geforderten Kriterien der FA
            //TODO bei einem restart muss hier der gesamte Lauf übergeben werden, statt nur die fehlgeschlagenen!
            amvLaufVO.batchId = insertPrintJobs( amvList, hasCover, amvLaufVO.typCd );
            amvList = null;
          }

          amvTransactionManager.updateAmvLauf( amvLaufVO );

          sendBatchJob( amvLaufVO.batchId, null, drucker, true, AmvControl.AMV_PRINTJOB_NAME, path );

          StringBuffer bufLog = new StringBuffer();
          bufLog.append( "JMS-Nachricht (Batch) wurde gesendet: " );
          bufLog.append( "(PrintJobId) = 0" );
          bufLog.append( ", (DruckerId) = " );
          bufLog.append( Integer.toString( drucker ) );
          bufLog.append( ", (BatchId) = " );
          bufLog.append( Integer.toString( amvLaufVO.batchId ) );
          bufLog.append( ", (isSave) = " );
          bufLog.append( isSave );
          bufLog.append( ", (reportName) = " );
          bufLog.append( AmvControl.FORMTYPE_AMV_TEXT );
          LOG.info( bufLog.toString() );

          if ( sendMail )
          {
            // Sendet via JIPPSPropertiesBean das Bestätigungsmail an die AMV1-Receiver
            sendMail( true, amvLaufVO.batchId, drucker, errorList, path, amvTransactionManager.selectAMVLaufToId( amvLaufVO.amvLaufId ) );
          }
          isLaufOk = true;
        }
        else
        {
          amvLaufVO.verarbStat = 2;
          amvLaufVO.batchId = 0;
          amvTransactionManager.updateAmvLauf( amvLaufVO ); // Schliesst den AMV-Lauf mit einem Update ab

          if ( sendMail )
          {
            // Sendet via JIPPSPropertiesBean das Bestätigungsmail an die AMV1-Receiver
            sendMail( false, 0, drucker, errorList, path, amvTransactionManager.selectAMVLaufToId( amvLaufVO.amvLaufId ) );
          }
          isLaufOk = false;
        }
      }
      else
      {
        LOG.warn( "Es hat keine " + AmvControl.FORMTYPE_AMV_TEXT );
        amvLaufVO.verarbStat = 4;
        amvLaufVO.batchId = 0;
        amvTransactionManager.updateAmvLauf( amvLaufVO );
        /*
         * Sendet via JIPPSPropertiesBean ein Bestätigungsmail an die AMV-Receiver mit der Meldung das keine
         * Erinnerungen in der Datenbank vorhanden wahren.
         */
        sendMail( false, 0, drucker, errorList, path, amvTransactionManager.selectAMVLaufToId( amvLaufVO.amvLaufId ) );
        isLaufOk = true;
      }
    }
    else
    // wenn amvList == null
    {
      amvLaufVO.verarbStat = 2;
      amvLaufVO.batchId = 0;
      amvTransactionManager.updateAmvLauf( amvLaufVO );
      LOG.warn( "Die übergebene amvListe gibt null zurück." );
      isLaufOk = false;
    }
    LOG.debug( "*** Status der " + AmvControl.FORMTYPE_AMV_TEXT + " ist: " + isLaufOk );
    return isLaufOk;
  }

  /**
   * Diese Methode erstellt die PrintJobVO's für die AMV und fügt diese in die Tabelle Printjob.
   * 
   * @param amvResultList List Liste mit allen Report-Daten
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   *          einfügen)
   * @param typCd String Typ-Code des Reports ( typCd = amv1 oder amv2 )
   * @return int batchId des Laufes
   */
  private int insertPrintJobs( List amvResultList, boolean hasCover, String typCd )
  {
    int batchId = getNextBatchId(); // Erzeugt einen neue Batch-Id
    LOG.debug( "*** Batch-Id ist: " + batchId );

    Iterator iter = amvResultList.iterator();
    while ( iter.hasNext() )
    {
      AmvVO amvVO = (AmvVO) iter.next();
      storePdf( fillPrintJobVOAMV( batchId, amvVO.bfaktId, amvVO.printJobFile, hasCover, typCd ) );
    }
    return batchId;
  }

  /**
   * Erzeugt mit den übergebenen Werte ein Report für die AMV-Erinnerungen 1
   * 
   * @param errorList List Fehler-Liste
   * @param amvVO AmvVO Value-Object
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  private void createAMVOne( List errorList, AmvVO amvVO ) throws RemoteException
  {
    List amvResultList = new ArrayList();
    String errorMsg = "";
    boolean isLetterOk;
    boolean isAuszugOk;

    try
    {
      amvResultList.add( amvTransactionManager.createControl( "Letter", fillPrintJobVOLetter( amvVO.bfaktId, amvVO.adrNr ) ) );
      isLetterOk = hasByteArrayValidContent( (byte[]) amvResultList.get( 0 ) );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      isLetterOk = false;
      errorMsg = "Fehler im Schreiben: " + ex.getMessage();
    }

    try
    {
      byte[] auszugPdf = amvTransactionManager.createControl( "Auszug", fillPrintJobVOAuszug( 0, amvVO.schutitKey, 9 ) );
      amvResultList.add( auszugPdf );
      isAuszugOk = hasByteArrayValidContent( auszugPdf );
    }
    catch ( Exception ex )
    {
      isAuszugOk = false;

      if ( !isLetterOk )
      {
        errorMsg = errorMsg + ", Fehler im Auszug: " + ex.getMessage();
      }
      else
      {
        errorMsg = "Fehler im Auszug: " + ex.getMessage();
      }
      LOG.error( errorMsg, ex );
    }

    if ( isLetterOk && isAuszugOk )
    {
      byte[] pdfMerged = PdfMerger.mergePdfs( amvResultList );
      if ( pdfMerged != null )
      {
        amvVO.printJobFile = pdfMerged;
        amvVO.verarbStat = 3;
      }
      else
      {
        setErrorMsg( errorList, amvVO, "Pdf-Merge ist fehlgeschlagen!" );
      }
    }
    else
    {
      setErrorMsg( errorList, amvVO, errorMsg );
    }
    amvTransactionManager.updateAMV( amvVO );
    LOG.debug( "*** createAMVOne - amvVO :" + amvVO.toString() );
    errorMsg = null;
  }

  public void deleteAMVRunsOlderThanDate( Date date )
  {
    LOG.info( "delete AMVRuns older than " + date.toString() );
    AmvLaufDAO amvLaufDAO = new AmvLaufDAO();
    Integer updateCount = amvLaufDAO.deleteByDate( new java.sql.Date( date.getTime() ) );
    LOG.info( updateCount + " AMVRuns deleted." );
  }

  /**
   * Überprüft den byte[] auf seine Gültigkeit
   * 
   * @param arr byte[]
   * @return boolean true, wenn byte[] != null und Länge grösser 0
   */
  private boolean hasByteArrayValidContent( byte[] arr )
  {
    return ( arr != null && arr.length > 0 );
  }

  /**
   * Diese Methode fügt im Fehlerfall die Fehlermeldung in die Fehlerliste und setzt den Verarbeitungs-Status auf 2.
   * 
   * @param errorList List Fehlerliste
   * @param amvVO AmvVO Value-Object
   * @param errorMsg String Fehlermeldung
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  private void setErrorMsg( List errorList, AmvVO amvVO, String errorMsg ) throws RemoteException
  {
    AmvErrorVO amvErrorVO = new AmvErrorVO();
    amvErrorVO.amvLaufId = amvVO.amvLaufId;
    amvErrorVO.adrNr = amvVO.adrNr;
    amvErrorVO.schutitKey = amvVO.schutitKey;
    amvErrorVO.errorMsg = cutString( errorMsg );

    amvTransactionManager.insertErrorMsg( amvErrorVO );
    LOG
        .warn( "Fehler beim erzeugen der  " + AmvControl.FORMTYPE_AMV_TEXT + " ( " + amvVO.bfaktId + ", " + amvVO.schutitKey
            + ") siehe Tabelle JIPPS_AMV_ERROR" );
    errorList.add( "Adr-Nr.: " + amvVO.adrNr + ", Bfakt-Id: " + amvVO.bfaktId + ", Schutit-Key: " + amvVO.schutitKey );
    amvVO.verarbStat = 2;
  }

  /**
   * Diese Methode verkürzt den Übergebenen Text auf die Länge des Übergebenen Wertes.
   * 
   * @param text String Originaltext
   * @return String gekürzter Text
   */
  private String cutString( String text )
  {
    int maxLength = 250;
    // String auf Grösse des Datenfeldes in der Datenbank kürzen
    if ( text.length() > maxLength )
    {
      return text.substring( 0, maxLength - 1 );
    }
    else
    {
      return text;
    }
  }

  /**
   * Füllt ein PrintJobVO mit den übergebenen Werte für den Letter. In der LetterControl.makePdf() muss aufgrund der
   * Mehrfach-Schreiben eine Liste übergeben werden. Es kann sein das dieser Methode mehr als nur ein PrintJobVO
   * übergeben wird. In den AMV-Erinnerung wird jeweils nur ein PrintJobVO übergeben, da die Methode aber eine Liste
   * erwartet wird dieses Value-Object in eine Liste verpackt.
   * 
   * @param bfaktId int Bfaktur-Id
   * @param adrNr int Adress-Nummer
   * @return List printJobList
   */
  private List fillPrintJobVOLetter( int bfaktId, int adrNr )
  {
    List printJobList = new ArrayList();
    PrintJobVO printJobVO = new PrintJobVO();

    printJobVO.batchId = PrintJobVO.NO_BATCH_ID;
    printJobVO.adrNr = adrNr;
    printJobVO.formTyp = PrintJobVO.LETTER;
    printJobVO.druckerId = drucker;
    printJobVO.reportId = bfaktId;
    printJobVO.anzKopien = PrintJobVO.NO_COPIES;
    printJobVO.druckStatus = PrintStatusConstants.READY_FOR_PRINTING;
    printJobVO.sprache = PrintJobVO.USE_IP_LANGUAGE;
    printJobVO.hasPotZahl = false;
    printJobList.add( printJobVO );

    return printJobList;
  }

  /**
   * Füllt ein PrintJobVO mit den übergebenen Werte für den Auszug. In der AuszugControl.makePdf() muss aufgrund der
   * Mehrfach-Auszüge eine Liste übergeben werden. Es kann sein das dieser Methode mehr als nur ein PrintJobVO übergeben
   * wird. In den AMV-Erinnerung 1 wird jeweils nur ein PrintJobVO übergeben, da die Methode aber eine Liste erwartet
   * wird dieses Value-Object in eine Liste verpackt.
   * 
   * @param regdemId int Auftragsnummer
   * @param schutitKey int Schutit-Key
   * @param regdemStatus int Regdem-Status
   * @return List printJobList
   */
  private List fillPrintJobVOAuszug( int regdemId, int schutitKey, int regdemStatus )
  {
    List printJobList = new ArrayList();
    PrintJobVO printJobVO = new PrintJobVO();

    printJobVO.batchId = PrintJobVO.NO_BATCH_ID;
    printJobVO.formTyp = PrintJobVO.AUSZUG;
    printJobVO.druckerId = drucker;
    printJobVO.regauszTyp = ReportType.TRADEMARK_RENEWAL.toDB();
    printJobVO.registerKz = IpRegisterStateConstants.REGISTRATION_STATE;
    printJobVO.reportId = schutitKey;
    printJobVO.schutitTyp = IpTypeConstants.TRADEMARK_CH;
    printJobVO.druckStatus = PrintStatusConstants.READY_FOR_PRINTING;
    printJobVO.standDatum = this.heute;
    printJobVO.sprache = PrintJobVO.USE_IP_LANGUAGE;
    printJobList.add( printJobVO );

    return printJobList;
  }

  /**
   * Füllt das PrintJobVO für die AMV-Erinnerung
   * 
   * @param batchId int Batch-Id
   * @param bfaktId int Bfakt-Id
   * @param amvReport byte[] gerenderte Pdfs (Schreiben und Auszüge gemerged)
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   *          einfügen)
   * @param typCd String Typ-Code des Reports ( typCd = amv1 oder amv2 )
   * @return PrintJobVO Value-Object
   */
  private PrintJobVO fillPrintJobVOAMV( int batchId, int bfaktId, byte[] amvReport, boolean hasCover, String typCd )
  {
    PrintJobVO printJobVO = new PrintJobVO();

    printJobVO.printJobId = getNextPrintJobId();
    printJobVO.batchId = batchId;
    printJobVO.druckerId = drucker;
    printJobVO.paperMode = ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB();
    printJobVO.anzKopien = PrintJobVO.NO_COPIES;
    printJobVO.druckStatus = PrintStatusConstants.READY_FOR_PRINTING;
    printJobVO.senderUser = PrintJobVO.SYSTEM;
    printJobVO.senderApp = PrintJobVO.SYSTEM;
    printJobVO.erstDatum = new Timestamp( System.currentTimeMillis() );
    printJobVO.formTyp = typCd;
    printJobVO.reportId = bfaktId;
    printJobVO.printJobFile = amvReport;
    printJobVO.coverCd = hasCover;

    return printJobVO;
  }

  /**
   * Sendent via das JIPPSPropertiesBean das Bestätigungsmail an die Empfänger die in der Jipps_Property Tabelle
   * definiert sind
   * 
   * @param hasData boolean Flag zur Unterscheidung der Meldungsart
   * @param batchId int Batch-id
   * @param drucker int Drucker für die AMV-Erinnerungen
   * @param errorList List List mit den Bfaktur-Id's und Schutitkey's die nicht erstellet werden konnten
   * @param path String Pfad wo die PDF-Dateien gespeichert werden
   * @param amvLaufVO AmvLaufVO Value-Object mit den Laufdaten
   */
  private void sendMail( boolean hasData, int batchId, int drucker, List errorList, String path, AmvLaufVO amvLaufVO )
  {
    int fehler = 0;
    StringBuffer bufError = new StringBuffer();

    if ( errorList != null && errorList.size() > 0 )
    {
      fehler = errorList.size();
      bufError.append( "Folgende Fehlerliste wurde bei der Verarbeitung der " );
      bufError.append( AmvControl.FORMTYPE_AMV_TEXT );
      bufError.append( " erstellt:" );
      bufError.append( "\n\n" );

      Iterator it = errorList.iterator();
      while ( it.hasNext() )
      {
        bufError.append( (String) it.next() );
        bufError.append( "\n" );
      }
    }

    try
    {
      if ( hasData )
      {
        StringBuffer bufMsg = new StringBuffer();

        if ( !isBatch && isFinish )
        {
          bufMsg.append( "\nDie Wiederverarbeitung wurde mit der Aktion FINISH auf dem JIPPS-Admin gestartet! Die fehlerhaften " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " werden nicht ausgedruckt. Die Sortierreihenfolge zwischen den " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " und den SAP-Rechnungen sind nicht mehr identisch!\nDie fehlerhaften Bfakturen und Schutitkeys "
              + "müssen manuell weiterverarbeitet werden.\n\n" );
        }
        if ( !isBatch && !isFinish )
        {
          bufMsg.append( "\nDie Wiederverarbeitung wurde mit der Aktion RESTART auf dem JIPPS-Admin gestartet!\n\n" );
        }

        bufMsg.append( "Lauf-Nummer:\t\t\t\t" );
        bufMsg.append( amvLaufVO.laufNr );
        bufMsg.append( "\nAnzahl erstellte " );
        bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
        bufMsg.append( ":\t" );
        bufMsg.append( amvLaufVO.anzAmv );
        bufMsg.append( "\n- davon OK:\t\t\t\t\t" );
        bufMsg.append( amvLaufVO.anzAmv - fehler );
        bufMsg.append( "\n- davon NOK:\t\t\t\t" );
        bufMsg.append( fehler );
        bufMsg.append( "\n\nSysteminformationen:\n" + "--------------------\n" + "Verarbeitungs-Start:\t\t\t" );
        bufMsg.append( DateFormatter.mediumFormat( amvLaufVO.createdDat ) );
        bufMsg.append( "\nVerarbeitungs-Ende:\t\t\t" );
        bufMsg.append( DateFormatter.mediumFormat( amvLaufVO.verarbEndeDat ) );
        bufMsg.append( "\nAMV-Lauf-Key:\t\t\t\t" );
        bufMsg.append( amvLaufVO.amvLaufId );
        bufMsg.append( "\nVerarbeitungs-Status:\t\t\t" );
        bufMsg.append( amvLaufVO.verarbStat );
        bufMsg.append( " - " );
        bufMsg.append( getVerarbStatBez( amvLaufVO.verarbStat ) );
        bufMsg.append( "\nBatch-Nummer:\t\t\t\t" );
        bufMsg.append( batchId );
        bufMsg.append( "\nDrucker:\t\t\t\t\t" );
        bufMsg.append( drucker );
        bufMsg.append( "\nAblage-Verzeichnis:\t\t\t" );
        bufMsg.append( path );

        JIPPSPropertiesFactory.getInstance().doMail( batchId, AmvControl.FORMTYPE_AMV_TEXT, bufMsg.toString(), bufError.toString(), AmvControl.FORMTYPE_AMV,
            amvLaufVO.anzAmv, fehler, drucker, path );
        LOG.debug( "*** Meldung die an JIPPSProperties gesendet wird: " + bufMsg.toString() );
        LOG.debug( " *** Für die  " + AmvControl.FORMTYPE_AMV_TEXT + " wurde die Bestätigung gesendet." );
      }
      else
      {
        StringBuffer bufMsg = new StringBuffer();

        if ( !isBatch && isFinish )
        {
          bufMsg.append( "\nDie Wiederverarbeitung wurde mit der Aktion FINISH auf dem JIPPS-Admin gestartet! Die fehlerhaften " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " werden nicht ausgedruck. Die Sortierreihenfolge zwischen den " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " und den SAP-Rechnungen sind nicht mehr identisch!\nDie fehlerhaften Bfakturen und Schutitkeys "
              + "müssen manuell weiterverarbeitet werden.\n\n" );
        }
        if ( !isBatch && !isFinish )
        {
          bufMsg.append( "\nDie Wiederverarbeitung wurde mit der Aktion RESTART auf dem JIPPS-Admin gestartet!\n\n" );
        }

        if ( amvLaufVO == null )
        {
          bufMsg.append( "Die Verarbeitung für die " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " konnte nicht gestartet werden! Melden Sie sich beim Userdesk!\n\n" );
        }
        else if ( amvLaufVO.verarbStat == 2 )
        {
          bufMsg.append( "Lauf-Nummer:\t\t\t\t" );
          bufMsg.append( amvLaufVO.laufNr );
          bufMsg.append( "\nAnzahl erstellte " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( ":\t" );
          bufMsg.append( amvLaufVO.anzAmv );
          bufMsg.append( "\n- davon OK:\t\t\t\t\t" );
          bufMsg.append( amvLaufVO.anzAmv - fehler );
          bufMsg.append( "\n- davon NOK:\t\t\t\t" );
          bufMsg.append( fehler );
          bufMsg.append( "\n\nDie Verarbeitung der " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " wurde abgebrochen! Melden Sie sich beim Userdesk!\n\n" + "Systeminformationen:\n" + "--------------------\n"
              + "Verarbeitungs-Start:\t\t\t" );
          bufMsg.append( DateFormatter.mediumFormat( amvLaufVO.createdDat ) );
          bufMsg.append( "\nVerarbeitungs-Ende:\t\t\t" );
          bufMsg.append( DateFormatter.mediumFormat( amvLaufVO.verarbEndeDat ) );
          bufMsg.append( "\nAMV-Lauf-Key:\t\t\t\t" );
          bufMsg.append( amvLaufVO.amvLaufId );
          bufMsg.append( "\nVerarbeitungs-Status:\t\t\t" );
          bufMsg.append( amvLaufVO.verarbStat );
          bufMsg.append( " - " );
          bufMsg.append( getVerarbStatBez( amvLaufVO.verarbStat ) );
        }
        else
        {
          bufMsg.append( "Lauf-Nummer:\t\t\t\t" );
          bufMsg.append( amvLaufVO.laufNr );
          bufMsg.append( "\nAnzahl erstellte " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( ":\t" );
          bufMsg.append( amvLaufVO.anzAmv );
          bufMsg.append( "\n- davon OK:\t\t\t\t\t" );
          bufMsg.append( amvLaufVO.anzAmv - fehler );
          bufMsg.append( "\n- davon NOK:\t\t\t\t" );
          bufMsg.append( fehler );
          bufMsg.append( "\n\nIn der Datenbank sind keine Daten zur Verarbeitung vorhanden, aus diesem Grund konnten keine " );
          bufMsg.append( AmvControl.FORMTYPE_AMV_TEXT );
          bufMsg.append( " erstellt werden !\n\n" + "Systeminformationen:\n" + "--------------------\n" + "Verarbeitungs-Start:\t\t\t" );
          bufMsg.append( DateFormatter.mediumFormat( amvLaufVO.createdDat ) );
          bufMsg.append( "\nVerarbeitungs-Ende:\t\t\t" );
          bufMsg.append( DateFormatter.mediumFormat( amvLaufVO.verarbEndeDat ) );
          bufMsg.append( "\nAMV-Lauf-Key:\t\t\t\t" );
          bufMsg.append( amvLaufVO.amvLaufId );
          bufMsg.append( "\nVerarbeitungs-Status:\t\t\t" );
          bufMsg.append( amvLaufVO.verarbStat );
          bufMsg.append( " - " );
          bufMsg.append( getVerarbStatBez( amvLaufVO.verarbStat ) );
        }

        JIPPSPropertiesFactory.getInstance().doMailNoData( AmvControl.FORMTYPE_AMV_TEXT, bufMsg.toString(), bufError.toString(), AmvControl.FORMTYPE_AMV,
            fehler );
        LOG.debug( " *** Für die  " + AmvControl.FORMTYPE_AMV_TEXT + " sind keine Daten vorhanden die Bestätigung wird gesendet." );
        bufMsg = null;
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception in sendMail", ex );
    }
    bufError = null;
  }

  /**
   * Gibt die Bezeichnung des Verarbeitungs-Status zurück
   * 
   * @param verarbStat int Verarbeitungs-Status
   * @return String Bezechnung des Verarbeitungs-Status
   */
  private String getVerarbStatBez( int verarbStat )
  {
    String text = "";

    switch( verarbStat )
    {
      case 0:
        text = "Verarbeitung gestartet";
        break;
      case 1:
        text = "Daten bereit für Verarbeitung";
        break;
      case 2:
        text = "Verarbeitung mit Fehler gestoppt";
        break;
      case 3:
        text = "Verarbeitung erfolgreich beendet";
        break;
      case 4:
        text = "Keine Daten für die Verarbeitung";
        break;
      case 5:
        text = "Verarbeitung abgeschlossen mit Fehlern";
        break;
      case 6:
        text = "Daten werden Verarbeitet";
        break;
    }
    return text;
  }

}
