package ch.ige.jipps;

import java.sql.SQLException;
import java.util.Date;
import java.util.Locale;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.report.dao.RegAuszTypSTypDAO;
import ch.ige.ipobjects.report.dao.RegFormTextDAO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.ipobjects.report.vo.RegFormTextVO;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.jipps.currentstate.CurrentState;
import ch.ige.jipps.exception.DBException;
import ch.ige.jipps.header.Header;
import ch.ige.jipps.history.History;
import ch.ige.jipps.signature.Signature;
import ch.ige.jipps.snapshot.Snapshot;
import ch.ige.jipps.snapshot.SnapshotVO;
import ch.ige.jipps.text.Watermark;
import ch.ige.jipps.vo.AuszugVO;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.RegistrationState;
import ch.ige.util.enums.printer.ReportPaper;
import ch.ige.util.enums.report.HistoryStart;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;

/**
 * Loads and formats value objects for the reports, using the BAGIS database.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @author Esther Bandschapp
 * @version $Id: AuszugAssembler.java,v 1.166 2008/10/23 13:35:15 lewis_lee Exp $
 */
public class AuszugAssembler
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( AuszugAssembler.class );

  /**
   * For accessing report texts
   */
  private RegFormTextDAO regFormTextDAO;

  /**
   * For accessing IP type report types
   */
  private RegAuszTypSTypDAO regAuszTypSTypDAO;

  /**
   * Empty constructor, creates all DAOs.
   */
  public AuszugAssembler()
  {
    this.regFormTextDAO = new RegFormTextDAO();
    this.regAuszTypSTypDAO = new RegAuszTypSTypDAO();

    ServicesFactory.getStandaloneServices();
  }

  /**
   * Loads all the data for a report.
   * @param ipKey The IP's unique identifying key
   * @param reportType The type of report to create
   * @param language The language to use
   * @param stateDate The date for which the report is created, in the format <code>dd.MM.yyyy</code>
   * @param userNumber The unique number of the user creating the report
   * @param printIpiHeader If this is <code>true</code>, the letter header with the IPI's address is printed
   * @throws DBException if the report can't be loaded
   * @throws Exception if other errors occur
   * @return The fully loaded VO for the report
   */
  public AuszugVO loadReport( final IPKeyVO ipKey, final ReportType reportType, final BAGISLanguageDomain language, final Date stateDate, final int userNumber,
      final boolean printIpiHeader ) throws DBException, Exception
  {
    try
    {
      // This may be changed by the report type, but by default take the last event date of the given state date.
      EreigIDat effectiveStateDate = EreigIDat.max( stateDate );

      AuszugVO report = new AuszugVO();
      report.setPrintIpiHeader( printIpiHeader );

      GenericIP ip = ServicesFactory.getInstance().findIP( IPKeyVO.create( ipKey ) );

      if ( Types.TRADEMARK.isInstance( ip )
          && ( reportType.equals( ReportType.CORRECTION_CONFIRMATION ) || reportType.equals( ReportType.REGISTRATION_CONFIRMATION ) ) )
      {
        if ( ip.getRegistrationEvent() == null )
        {
          throw new IllegalArgumentException( "Auszug kann nicht erzeugt werden, da der Schutztitel noch nicht eingetragen ist." );
        }
        // The registration confirmation needs the state date of the registration event, but the correction
        // confirmation needs the state date of *the last possible event date (EreigIDat)* of the registration event's
        // date, because
        // corrections have been made later, which were back-dated to the registration date (but after the registration
        // event).
        //
        // Example: The trademark was registered on the 21.5.2007 (event number 55). The user specifies a state date of
        // 25.5.2007.
        // Normally, the event date is set to 20070525#99.
        // If the registration confirmation is required, the state date is set to 20070521#55
        // If the correction confirmation is required, the state date is set to 20070521#99
        if ( reportType.equals( ReportType.REGISTRATION_CONFIRMATION ) )
        {
          effectiveStateDate = ip.getRegistrationEvent().getEreig_idat();
        }
        else if ( reportType.equals( ReportType.CORRECTION_CONFIRMATION ) )
        {
          effectiveStateDate = EreigIDat.max( ip.getRegistrationEvent().getEreig_idat().getDate() );
        }
      }

      Date applicationDate = ip.getApplicationDate();
      if ( applicationDate != null && effectiveStateDate.getDate().before( applicationDate ) )
      {
        throw new IllegalArgumentException( "Auszug wird für den " + DateFormatter.shortFormat( effectiveStateDate.getDate() )
            + " gewünscht, der Schutztitel existiert aber erst seit dem " + DateFormatter.shortFormat( applicationDate ) );
      }

      if ( reportType.usesRegisterView() )
      {
        ip.applyRegisterTransformation( effectiveStateDate );

        // Deferred designs must not show their events between registration and publication, except on
        // the owner's special register report.
        if ( Types.DESIGN.isInstance( ip ) )
        {
          Design design = (Design) ip;
          if ( design.hasDeferment() && design.wasReducedWithinDeferment() && !reportType.equals( ReportType.REGISTER_REPORT_FOR_OWNER ) )
          {
            design.hideRemovedItemsAfterDeferment();
          }
        }
      }
      else
      {
        ip.removeAllEventsAfter( effectiveStateDate );
        ip.removeEventsWithNegativePublStat();
      }

      java.sql.Date reportDate = new java.sql.Date( effectiveStateDate.getDate().getTime() );
      Locale reportLanguage = null;
      // The language may be the one from the IP
      if ( language == null )
      {
        reportLanguage = ip.getLocale();
      }
      else
      {
        reportLanguage = language.toLocale();
      }

      RegAuszTypSTypVO ipTypeReportType = this.regAuszTypSTypDAO.findIpTypeReportType( ip.getBAGISType(), reportType );
      if ( ipTypeReportType == null )
      {
        throw new Exception( "Unbekannte Kombination von Schutztitel-Typ " + ip.getBAGISType() + " und Auszugstyp " + reportType );
      }

      report.setIpHeader( Header.getIpHeader( ip, reportDate, ipTypeReportType, reportLanguage ) );

      // The report language may have changed!
      reportLanguage = report.getIpHeader().getReportLanguage();
      report.setPrintHeader( Header.getPrintHeader( report.getIpHeader(), reportType, reportDate, reportLanguage ) );

      LOG.debug( "Header loaded" );

      String watermark = Watermark.getWatermark( reportType, ip, reportLanguage, effectiveStateDate.getDate() );
      report.setWatermark( watermark );
      LOG.debug( "Watermark is '" + watermark + "'" );

      // For certain reports, Swiss trademarks take the registration date as state date.
      // Note: This doesn't apply to the header!
      if ( Types.TRADEMARK.isInstance( ip ) && ( reportType.equals( ReportType.PRIORITY_REPORT ) )
          || ( reportType.equals( ReportType.REGISTRATION_CONFIRMATION ) ) )
      {
        if ( ip.getRegistrationState().equals( RegistrationState.REGISTERED ) && ip.getRegistrationEvent() != null )
        {
          effectiveStateDate = ip.getRegistrationEvent().getEreig_idat();
          LOG.debug( "Take the registration date for report: " + effectiveStateDate );
        }
      }

      // Load the image for trademarks. Note that this depends on the report type for setting the state date.
      if ( Types.TRADEMARK.isInstance( ip ) )
      {
        Trademark trademark = (Trademark) ip;
        TimePeriodCollection<ImageVO> images = trademark.getImages().getPeriodsContainingTime( effectiveStateDate );
        if ( !images.isEmpty() )
        {
          if ( images.size() == 1 )
          {
            report.setImage( images.get( 0 ) );
          }
          else
          {
            throw new IllegalStateException( "Found more than one current trademark image for IP " + ipKey );
          }
        }
      }

      report.setCurrentStateElements( CurrentState.getCurrentState( ip, effectiveStateDate, reportLanguage, reportType ) );
      LOG.debug( "Loaded current state" );

      // The registration snapshot exists only for certain trademark reports
      if ( Types.TRADEMARK.isInstance( ip ) && ( reportType.equals( ReportType.REGISTER_REPORT_AFTER_REGISTRATION ) )
          || ( reportType.equals( ReportType.DOSSIER_REPORT_AFTER_APPLICATION ) ) )
      {
        Event registrationEvent = ip.getRegistrationEvent();
        if ( registrationEvent != null )
        {
          SnapshotVO registrationSnapshot = Snapshot.getSnapshot( ip, registrationEvent );
          if ( registrationSnapshot != null )
          {
            Locale currentReportLanguage = report.getIpHeader().getReportLanguage();
            RegFormTextVO snapshotTitle = this.regFormTextDAO.findReportText( ReportTextConstants.REGISTRATION_STATE_INTEGER, currentReportLanguage,
                ReportArea.MISCELLANEOUS );
            registrationSnapshot.setTitle( snapshotTitle.getText() );

            report.setRegistrationSnapshot( registrationSnapshot );
            LOG.debug( "Loaded registration snapshot" );
          }
        }
      }

      // Check if history data should be loaded for this report
      if ( !ipTypeReportType.getHistoryStart().equals( HistoryStart.NO_HISTORY ) )
      {
        Locale currentReportLanguage = report.getIpHeader().getReportLanguage();
        RegFormTextVO historyTitle = this.regFormTextDAO.findReportText( ReportTextConstants.HISTORY_INTEGER, currentReportLanguage, ReportArea.HISTORY );
        report.setHistoryTitle( historyTitle.getText() );

        report.setHistoryItems( History.getHistory( ip, effectiveStateDate, reportLanguage, reportType ) );
        LOG.debug( "Loaded history data" );
      }

      // Load signature
      report.setSignatureLines( Signature.getSignature( ip, reportLanguage, reportType, userNumber ) );
      report.setSignatureStamp( Signature.getStamp( ip, reportType ) );
      LOG.debug( "Signature loaded" );

      return report;
    }
    catch ( Exception ex )
    {
      throw new Exception( "Fehler im AuszugAssembler", ex );
    }
  }

  /**
   * Gets the paper tray for the report type and IP type.
   * @param reportType The report type
   * @param ipType The IP type
   * @return The paper tray containing a certain paper type, see below
   * @throws SQLException if the paper tray can't be fetched
   * @see ch.ige.util.enums.printer.ReportPaper
   */
  ReportPaper getReportPaperWithDefault( ReportType reportType, IPType ipType ) throws SQLException
  {
    try
    {
      RegAuszTypSTypVO ipTypeReportType = this.regAuszTypSTypDAO.findIpTypeReportType( ipType, reportType );
      ReportPaper reportPaper = ipTypeReportType.getReportPaper();
      if ( reportPaper != null && ( reportPaper.equals( ReportPaper.PAPER_WITH_IPI_LOGO ) || reportPaper.equals( ReportPaper.PAPER_PLAIN ) ) )
      {
        return reportPaper;
      }
      else
      {
        return ReportPaper.PAPER_WITH_IPI_LOGO;
      }
    }
    catch ( SQLException ex )
    {
      SQLException exception = new SQLException( "AuszugAssembler: getSchacht" );
      exception.initCause( ex );
      throw exception;
    }
  }
}