package ch.ige.jipps;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.common.DetailTerminatedException;
import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.design.model.DesignEvent;
import ch.ige.ipobjects.design.vo.ModOrdnrVO;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.report.dao.RegAuszTypSTypDAO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.exception.DBException;
import ch.ige.jipps.text.EventTextLoader;
import ch.ige.jipps.vo.DesignImageVO;
import ch.ige.jipps.vo.DesignImagesVO;
import ch.ige.util.collection.MapConverter;
import ch.ige.util.date.LanguageDateFormatter;
import ch.ige.util.enums.report.ReportType;

/**
 * Loads and formats image objects for the design image list report.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Esther Bandschapp
 * @version $Id: DesignBilderAssembler.java,v 1.38 2008/05/15 12:32:44 bandschapp Exp $
 */
public class DesignBilderAssembler
{
  /**
   * This is shown if a design has no IP number
   */
  private static final String NO_ENTRY_NUMBER = "000000";

  /**
   * The empty constructor
   */
  public DesignBilderAssembler()
  {
    // empty constructor
  }

  /**
   * Loads the images for a design's image list report.
   * @param ipKey The unique key for the IP whose image list is fetched
   * @param stateDate The date for which the image list is fetched
   * @param printIpiHeader If this is <code>true</code>, the letter header with the IPI's address is printed
   * @return The object containing the list of images
   * @throws DBException if no images are found
   */
  public DesignImagesVO loadDesignImages( int ipKey, Date stateDate, boolean printIpiHeader ) throws DBException
  {
    try
    {
      Design design = loadDesign( ipKey, stateDate );

      RegAuszTypSTypVO ipTypeReportType = new RegAuszTypSTypDAO().findIpTypeReportType( design.getBAGISType(), ReportType.DESIGN_IMAGE_LIST );
      if ( ipTypeReportType == null )
      {
        throw new Exception( "Unbekannte Kombination von Schutztitel-Typ " + design.getBAGISType() + " und Auszugstyp " + ReportType.DESIGN_IMAGE_LIST );
      }

      DesignImagesVO designImages = new DesignImagesVO();
      designImages.reportTitle = ipTypeReportType.getReportTitle( design.getLocale(), design.getRegistrationState() );
      designImages.ipKey = ipKey;
      designImages.printIpiHeader = printIpiHeader;
      designImages.applicationNo = design.getApplicationNumber();

      if ( design.getNumber() != null )
      {
        designImages.ipNumber = design.getNumber();
      }
      else
      {
        designImages.ipNumber = NO_ENTRY_NUMBER;
      }
      // Note: According to the Design department, the design image list must always be
      // printed in the IP's language, regardless of what language was selected by the user
      designImages.language = design.getLanguage();
      LanguageDateFormatter dateFormatter = new LanguageDateFormatter( design.getLocale() );
      designImages.reportDate = dateFormatter.longFormat( stateDate );

      setImages( design, stateDate, designImages );

      // Now load the event texts for the history
      EventTextLoader.insertEventText( designImages.historyImages, design.getLocale(), false );

      return designImages;
    }
    catch ( Exception e )
    {
      throw new DBException( "Exception while loading Design for Bilderliste", e );
    }
  }

  /**
   * Loads the basic design.
   * @param ipKey The unique key identifying the design
   * @param stateDate The date for which the image list is fetched
   * @return The basic design or <code>null</code> if there's none
   * @throws DetailTerminatedException if the detail is terminated
   */
  private Design loadDesign( int ipKey, Date stateDate ) throws DetailTerminatedException
  {
    IPKeyVO ipKeyVO = IPKeyVO.create( ipKey );
    Design design = ServicesFactory.getStandaloneServices().findDesign( ipKeyVO );
    design.applyRegisterTransformation( EreigIDat.max( stateDate ) );
    return design;
  }

  /**
   * Sets the relevant images for the current state and the history into <code>designImages</code>.
   * @param design The original design to take the images from
   * @param stateDate The date for which the image list is fetched
   * @param designImages The desin images whose {@link DesignImagesVO#images} and {@link DesignImagesVO#historyImages}
   * are filled.
   */
  private void setImages( Design design, Date stateDate, DesignImagesVO designImages )
  {
    List<DesignImageVO> currentStateImages = new ArrayList<DesignImageVO>();
    List<DesignImageVO> historyImages = new ArrayList<DesignImageVO>();
    // Create a map of event dates, each pointing to a List of order numbers
    Map<EreigIDat, List<ModOrdnrVO>> orderNumberMap = MapConverter.toGroupMap( design.getDesignItems(), EreigIDat.class, "von_ereig_idat" );
    EreigIDat currentState = EreigIDat.max( stateDate );

    // Within the history, the event that contains the order numbers is the application,
    // but the event that must be shown on the image list is the registration.
    // There are two kinds of registrations, one for 'secret' designs, one for normal ones.
    DesignEvent eventForPublicVisibility = design.getEndOfDelayEvent();
    if ( eventForPublicVisibility == null )
    {
      eventForPublicVisibility = DesignEvent.wrap( design.getRegistrationEvent() );
    }

    for ( Event event: design.getEvents() )
    {
      DesignEvent designEvent = DesignEvent.wrap( event );
      // The order number belonging to the even must have the same event date.
      List<ModOrdnrVO> orderNumbers = orderNumberMap.get( designEvent.getEreig_idat() );
      if ( orderNumbers != null && !orderNumbers.isEmpty() )
      {
        for ( ModOrdnrVO orderNumber: orderNumbers )
        {
          List<DesignImageVO> targetPartition = null;
          DesignEvent eventToShow;
          // If the image is still valid, it's part of the current state,
          // otherwise it's part of the image history.
          if ( orderNumber.containsTime( currentState ) )
          {
            targetPartition = currentStateImages;
            eventToShow = null;
          }
          else
          {
            targetPartition = historyImages;
            // 
            if ( eventForPublicVisibility != null && eventForPublicVisibility.after( designEvent ) )
            {
              eventToShow = eventForPublicVisibility;

              // Theoretically, the order number could be finished *before* it must be shown.
              // Thus, in the register view, it has never existed. So skip the adding by setting the List to null.
              if ( orderNumber.isTerminated() && eventForPublicVisibility.after( orderNumber.getBis_ereig_idat() ) )
              {
                continue;
              }
            }
            else
            {
              eventToShow = DesignEvent.wrap( designEvent );
            }
          }
          // Create all images for the order number
          int i = 1;
          for ( Iterator<ImageVO> imageIterator = orderNumber.getImages().iterator(); imageIterator.hasNext(); i++ )
          {
            ImageVO image = imageIterator.next();
            targetPartition.add( new DesignImageVO( image.getImage_nr(), new Integer( i ), orderNumber, image, eventToShow ) );
          }
        }
      }
    }

    designImages.images = currentStateImages;
    designImages.historyImages = historyImages;
  }
}