package ch.ige.jipps;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import ch.ige.jipps.constants.XslConstants;
import ch.ige.jipps.pdf.PdfMerger;
import ch.ige.jipps.pdf.PdfRenderer;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.DesignImagesVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.xml.DesignBilderWriteXml;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Creates the design image list as PDF files.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Priska Steiner
 * @version $Id: DesignBilderControl.java,v 1.66 2008/05/22 13:54:10 bandschapp Exp $
 * @see DesignBilderAssembler
 * @see DesignBilderWriteXml
 */
public class DesignBilderControl extends AuszugControl
{
  /**
   * Used to create the XML
   */
  private DesignBilderWriteXml designWriteXml;

  /**
   * Assembles the data for creating design image lists
   */
  protected DesignBilderAssembler designBilderAssembler;

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( DesignBilderControl.class );

  /**
   * The constructor
   */
  public DesignBilderControl()
  {
    this.designWriteXml = new DesignBilderWriteXml();
  }

  /**
   * Creates and prints PDF files from the given print jobs.
   * @param printJobs a <code>List</code> of {@link ch.ige.jipps.vo.PrintJobVO} objects
   * @return A <code>List</code> of {@link ch.ige.jipps.vo.AnswerVO} objects representing the print stati
   */
  @Override
  public List<AnswerVO> makePrint( List<PrintJobVO> printJobs )
  {
    LOG.debug( "In Makeprint" );
    String status;
    this.designBilderAssembler = new DesignBilderAssembler();
    DesignImagesVO designImages = null;

    for ( PrintJobVO printJobVO: printJobs )
    {
      status = "ok";
      try
      {
        try
        {
          // Design image lists are always printed on plain white paper, so print the IPI header
          printJobVO.printIgekopf = true;
          designImages = this.designBilderAssembler.loadDesignImages( printJobVO.reportId, printJobVO.standDatum, printJobVO.printIgekopf );
        }
        catch ( Exception ex )
        {
          status = "Exception beim laden der VOs:" + ex.getMessage();
          LOG.error( status, ex );
        }

        String reportXml = null;
        try
        {
          reportXml = this.designWriteXml.toXml( designImages );
        }
        catch ( IOException ex )
        {
          status = "IOException beim Schreiben des XMLStreams: " + ex.getMessage();
          LOG.error( status, ex );
        }
        catch ( JAXBException ex )
        {
          status = "JAXBException beim Schreiben des XMLStreams: " + ex.getMessage();
          LOG.error( status, ex );
        }
        LOG.debug( "XMLFile erstellt" );

        try
        {
          printJobVO.printJobFile = createPdf( reportXml, XslConstants.BILDER_LISTE );
        }
        catch ( Exception ex )
        {
          status = "Fehler beim erstellen des PDFs: " + ex.getMessage();
          LOG.error( status, ex );
        }

        try
        {
          ReportType reportType = ReportType.parse( printJobVO.regauszTyp );
          IPType ipType = IPType.parse( printJobVO.schutitTyp );
          printJobVO.paperMode = new AuszugAssembler().getReportPaperWithDefault( reportType, ipType ).toDB();
        }
        catch ( SQLException ex )
        {
          status = "DesignBilderControl: " + ex.getMessage();
          LOG.error( status, ex );
        }

        int pdfId = getNextPrintJobId();
        LOG.debug( "printId in DesignBilderControl:" + pdfId );
        printJobVO.printJobId = pdfId;

        if ( designImages != null )
        {
          printJobVO.formattedApplicationNumber = designImages.applicationNo;
          printJobVO.formattedIpNumber = designImages.ipNumber;
        }

        storePdf( printJobVO );
        LOG.debug( "Daten gespeichert" );

        // If the job should be printed, send it to the JMS queue
        if ( printJobVO.druckStatus == 1 )
        {
          sendPrintJob( printJobVO.printJobId, null, printJobVO.druckerId );
        }
      }
      finally
      {
        // Get the status for the printing, including the IP key for errors
        if ( printJobVO != null )
        {
          addAnswer( printJobVO.printJobId, printJobVO.formTyp, printJobVO.reportId, status, printJobVO.formattedApplicationNumber, printJobVO.formattedIpNumber );
        }
      }
    }

    return this.answers;
  }

  /**
   * See {@link ch.ige.jipps.JIPPSFacadeServices#getXml(List)}.
   */
  @Override
  public String[] makeXml( List<PrintJobVO> printJobs ) throws Exception
  {
    LOG.debug( "makeXml" );

    Map<PrintJobVO, String> printJobMap = toXml( loadReports( printJobs ) );
    // During the whole processing, something may have gone wrong
    for ( PrintJobVO printJob: printJobs )
    {
      if ( printJob.status != null )
      {
        throw new Exception( printJob.status );
      }
    }

    return printJobMap.values().toArray( new String[0] );
  }

  /**
   * Converts the reports in XML format into XSL-FO documents. <strong>Note that this is only used for testing!</strong>
   * @param printJobs The print jobs
   * @return An array of <code>String</code>s, each of them an XSL-FO document corresponding to its print job
   * @throws Exception if the conversion fails
   */
  @Override
  public String[] makeXslFo( List<PrintJobVO> printJobs ) throws Exception
  {
    LOG.debug( "makeXslFo" );
    PdfRenderer xslFoPdfRenderer = new PdfRenderer();
    List<String> xslFoDocuments = new ArrayList<String>( printJobs.size() );

    Map<PrintJobVO, String> printJobMap = toXml( loadReports( printJobs ) );
    for ( PrintJobVO printJob: printJobMap.keySet() )
    {
      String xmlDocument = printJobMap.get( printJob );
      String xslFoDocument = xslFoPdfRenderer.renderXslFo( null, xmlDocument, XslConstants.BILDER_LISTE );
      xslFoDocuments.add( xslFoDocument );
    }

    return xslFoDocuments.toArray( new String[0] );
  }

  /**
   * Creates one merged PDF file from all the given print jobs.
   * @param printJobs The print jobs
   * @return The PDF data containing all merged PDF files for the print jobs
   * @throws Exception if the PDF file can't be created
   */
  @Override
  public byte[] makePdf( final List<PrintJobVO> printJobs ) throws Exception
  {
    DesignImagesVO designImages = null;
    this.designBilderAssembler = new DesignBilderAssembler();
    List<byte[]> pdfFiles = new ArrayList<byte[]>();

    for ( PrintJobVO printJobVO: printJobs )
    {
      try
      {
        designImages = this.designBilderAssembler.loadDesignImages( printJobVO.reportId, printJobVO.standDatum, printJobVO.printIgekopf );
      }
      catch ( Exception ex )
      {
        throw new Exception( "Exception beim laden der VOs", ex );
      }

      String reportXml = this.designWriteXml.toXml( designImages );
      LOG.debug( "Created report XML with " + reportXml.length() + " characters" );

      byte[] pdfFile = createPdf( reportXml, XslConstants.BILDER_LISTE );
      LOG.debug( "Adding PDF file with " + pdfFile.length + " bytes" );
      pdfFiles.add( pdfFile );
    }

    LOG.debug( "Merging all PDF files" );
    byte[] mergedPdfFiles = PdfMerger.mergePdfs( pdfFiles );
    LOG.debug( "Got merged PDF file with " + mergedPdfFiles.length + " bytes" );
    return mergedPdfFiles;
  }

  /**
   * Loads the reports from the database for the given print jobs.
   * @param printJobs The print jobs
   * @return a <code>Map</code> where the print job is the key and the corresponding
   * {@link ch.ige.jipps.vo.DesignImagesVO} is the value
   */
  private Map<PrintJobVO, DesignImagesVO> loadReports( final List<PrintJobVO> printJobs )
  {
    LOG.debug( "loadReports" );
    Map<PrintJobVO, DesignImagesVO> reports = new HashMap<PrintJobVO, DesignImagesVO>( printJobs.size() );

    this.designBilderAssembler = new DesignBilderAssembler();
    for ( PrintJobVO printJobVO: printJobs )
    {
      LOG.debug( "Loading report: Print job " + printJobVO );

      try
      {
        DesignImagesVO designImages = this.designBilderAssembler.loadDesignImages( printJobVO.reportId, printJobVO.standDatum, printJobVO.printIgekopf );
        LOG.debug( "Got report with IP key <" + designImages.ipKey + ">" );
        reports.put( printJobVO, designImages );
      }
      catch ( Exception ex )
      {
        LOG.error( "Schutit-Key: " + printJobVO.reportId + ", Exception while loading VOs", ex );
        printJobVO.status = "Fehler beim Laden des Schutztitel-VO: " + ex.getMessage();
      }
    }
    return reports;
  }

  /**
   * Converts the reports to their XML equivalent.
   * @param reports a <code>Map</code> of print jobs and design images
   * @return The same kind of <code>Map</code>, but the the value is a <code>String</code> object containing the
   * XML for each report
   */
  private Map<PrintJobVO, String> toXml( final Map<PrintJobVO, DesignImagesVO> reports )
  {
    LOG.debug( "toXml" );
    HashMap<PrintJobVO, String> printJobXml = new HashMap<PrintJobVO, String>( reports.size() );
    for ( PrintJobVO printJobVO: reports.keySet() )
    {
      LOG.debug( "Converting to XML: Print job with ID <" + printJobVO.printJobId + ">" );

      DesignImagesVO designImages = reports.get( printJobVO );
      reports.put( printJobVO, null );
      String xml = null;
      try
      {
        xml = this.designWriteXml.toXml( designImages );
      }
      catch ( IOException ex )
      {
        LOG.error( "IOException while writing XMLStream", ex );
        printJobVO.status = "IOException beim Schreiben des XMLStreams:" + ex.getMessage();
      }
      catch ( JAXBException ex )
      {
        LOG.error( "JAXBException while writing XMLStream", ex );
        printJobVO.status = "JAXBException beim Schreiben des XMLStreams:" + ex.getMessage();
      }
      LOG.debug( "Got XML: " + xml );
      printJobXml.put( printJobVO, xml );
    }
    return printJobXml;
  }
}
