package ch.ige.jipps;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ch.ige.ipobjects.patent.vo.PatentDocumentEventKeyVO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.jipps.ftp.web.util.EPAUploadClientConfigWrapper;
import ch.ige.jipps.vo.AmvErrorVO;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.DruckPeriVO;
import ch.ige.jipps.vo.DruckerVO;
import ch.ige.jipps.vo.LoeschungErrorVO;
import ch.ige.jipps.vo.LoeschungLaufVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.PrintjobSearchVO;
import ch.ige.jipps.vo.PropertyJippsVO;
import ch.ige.patdocutil.receipts.PatentDocumentVOWithPDF;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.RegistrationState;

/**
 * The main interface to JIPPS in a non-JEE environment.
 * <p>
 * Copyright (c) 2002-2004, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Priska Steiner
 * @author Johannes Lang
 * @author Marc Bouquet
 * @author Elmar Isler
 * @author Kurt Jegge
 * @author Eric Lewis
 * @version $Id: JIPPSFacadeServices.java,v 1.53 2009/02/04 14:45:46 fabrizioli Exp $
 */
public interface JIPPSFacadeServices
{
  /**
   * Holt die Daten für einen Preview, diese Methode wird vom Client, via Delegate aufgerufen
   * @param pdfId int, enthält die ID des zu holenden Pdf's
   * @return pdfArr byte[] mit dem Wert aus pdf_file
   * @throws RemoteException
   */
  public byte[] getPreviewData( int pdfId ) throws RemoteException;

  /**
   * Holt ein Sammel-Pdf das anhand der Liste erstellt wird
   * @param printJobs Liste von PrintJobVO's als Parameter
   * @return byte[] gemergede Pdf-Datei
   * @throws Exception Unerwarteter Fehler
   * @throws RemoteException
   */
  public byte[] getMergePdf( List<PrintJobVO> printJobs ) throws Exception, RemoteException;

  /**
   * Gets the XML documents for the print jobs.
   * @param printJobs A <code>List</code> of {@link ch.ige.jipps.vo.PrintJobVO} objects, which define what reports to
   * print
   * @return An array of XML documents, where every String represents the XML document for a print job
   * @throws Exception if creation of XML documents fails
   * @throws RemoteException if the server fails
   */
  public String[] getXml( List<PrintJobVO> printJobs ) throws Exception, RemoteException;

  /**
   * Prints a print job after a preview has been requested.
   * When showing a preview, the print job is in fact already created. To print it, only the print
   * status has to be changed, and the print jobs have to be sent to the print server.
   * @param printJobIds The IDs of the print jobs to print
   * @param printStatus The new print status, one of {@link PrintStatusConstants}
   * @param printerId The ID of the printer which prints the print jobs
   * @throws RemoteException if printing fails
   */
  public void printAfterPreview( List<Integer> printJobIds, int printStatus, int printerId ) throws RemoteException;

  /**
   * Prints a report print job after a preview has been requested.
   * When showing a preview, the print job is in fact already created. To print it, only the print
   * status has to be changed, and the print jobs have to be sent to the print server.
   * @param printJob The report print job to print
   * @param printStatus The new print status, one of {@link PrintStatusConstants}
   * @param printerId The ID of the printer which prints the print jobs
   * @throws RemoteException if printing fails
   */
  public void printReportAfterPreview( PrintJobVO printJob, int printStatus, int printerId ) throws RemoteException;

  /**
   * Erstellt aus dem Parameter formTyp via Reflection-API eine Instanz der entsprechenden Formklasse und gibt dieser
   * die Strings weiter
   * @param printJobs Liste mit den Parametern vom Client
   * @return mAnswer Rückgabewert der Hashmap
   * @throws RemoteException
   */
  public List<AnswerVO> createPrintjobs( List<PrintJobVO> printJobs ) throws RemoteException;

  /**
   * Gets the valid report types for the IP type and the IP's registration state.
   * @param ipType The type of IP
   * @param registrationState The registration state
   * @return The combination of IP type and report type
   * @throws RemoteException if the loading fails
   */
  public Set<RegAuszTypSTypVO> getReportTypes( IPType ipType, RegistrationState registrationState ) throws RemoteException;

  /**
   * Gets the printer that's defined for the user and the print job type.
   * @param employeeNumber The employee's unique number
   * @return The JIPPS printer to use for the employee
   * @throws RemoteException if the selection fails
   */
  public Integer getMaDrucker( int employeeNumber ) throws RemoteException;

  /**
   * Gets the printer that's defined for the user and the print job type.
   * @param employeeUserName The employee's user name used on the system
   * @return The JIPPS printer to use for the employee
   * @throws RemoteException if the selection fails
   */
  public Integer getMaDrucker( String employeeUserName ) throws RemoteException;

  /**
   * Gets the names of all printers that are installed on the print server(s).
   * @return The printers installed on the print servers or <code>null</code> if they can't be found
   * @throws RemoteException
   */
  public List<String> getInstalledPrinters() throws RemoteException;

  /**
   * Gets the IDs and names of all printers that are defined in Jipps.
   * @return The defined printers, where the key is the printer ID and the value is the printer name
   * @throws RemoteException
   */
  public Map<Integer, String> getPrintersAsMap() throws RemoteException;

  /**
   * Gets all printers for the given print job and IP types, which are defined both in BAGIS and JIPPS.
   * @param printJobType The print job type to print, one of the values referenced by
   * {@link ch.ige.jipps.vo.PrintJobVO#formTyp}
   * @param ipType The type of IP, which is used for color printers
   * @return The printers defined both in BAGIS and JIPPS
   * @throws RemoteException if the select fails
   */
  public Set<DruckerVO> getPrinters( int printJobType, IPType ipType ) throws RemoteException;

  /**
   * Mit dieser Methode wird der Druckstatus im Bagis updated.
   * @param bfaktId Bfaktur-Nummer
   * @param statusDruck Status des Drucks
   * @throws Exception Unerwarteter Fehler
   * @throws RemoteException
   */
  public void setBfaktDruckstatus( int bfaktId, String statusDruck ) throws Exception, RemoteException;

  /**
   * Holt die Loeschungsfehler auf der Tabelle LoeschungError
   * @return List mit den LoeschungErrorVO's
   * @throws RemoteException
   */
  public List<LoeschungErrorVO> getLoeschungErrorList() throws RemoteException;

  /**
   * Ruft LoeschungControl auf zum Rendern der Loeschungxml's
   * @param selList Liste mit Druckperioden
   * @param drucker Id des JIPPSDruckers
   * @param pakete gewünschte Paketgrösse
   * @param userName Name des Auslösers
   * @param loeschungLaufId Nummer des Löschungslaufes.
   * @return String mit einer Message zum Status der Loeschungen
   * @throws RemoteException
   */
  public String renderLoeschungen( List<DruckPeriVO> selList, int drucker, int pakete, String userName, int loeschungLaufId ) throws RemoteException;

  /**
   * Holt ein Loeschungsfehler auf der Tabelle LoeschungError
   * @param keyId Schlüssel des Fehlers
   * @return LoeschungErrorVO mit dem gewünschten LoeschungsError
   * @throws RemoteException
   */
  public LoeschungErrorVO getLoeschungError( int keyId ) throws RemoteException;

  /**
   * Löscht die Fehlermeldungen in LoeschungError
   * @param loeschList Liste der zu löschenden Meldungen
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   * @throws RemoteException
   */
  public int removeLoeschungError( List<LoeschungErrorVO> loeschList ) throws RemoteException;

  /**
   * Gets the employee's unique number.
   * @param employeeUserName The employee's user name used on the system
   * @return The employee's unique number or <code>0</code> if none was found
   * @throws RemoteException if the selection fails
   */
  public int getMaNr( String employeeUserName ) throws RemoteException;

  /**
   * Erstellt das Statusmail für die Löschungsanzeigen
   * @param meldung die zu erstellende Meldung
   * @param log die zu erstellende Log-Meldung
   * @throws RemoteException
   */
  public void createMail( String meldung, String log ) throws RemoteException;

  /**
   * Löst den Batchdruck der Eintragungsbescheinigungen aus
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   * @throws RemoteException
   */
  public int printEintrBesch() throws RemoteException;

  /**
   * Prints letters for changes that are done automatically to trademarks: They are either cancelled or prolongated.
   * @param hasCover Inserts a cover page if <code>true</code>, otherwise none if <code>false</code>
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   * @throws RemoteException if the printing fails
   */
  public int printAMVChanges( boolean hasCover ) throws RemoteException;

  /**
   * Holt die Drucker auf der Tabelle Drucker. Dies wird für den JippsAdmin benötigt. Die Methode getJIPPSDrucker macht
   * das selbe über das PrintJobStoreBean, da ein two-phase commit gemacht wird. Für den JippsAdmin ist dies nicht
   * nötig. Es würde eine unnötige Connection aufgebaut.
   * @return List mit den DruckerVO's
   * @throws RemoteException
   */
  public List<DruckerVO> getPrinters() throws RemoteException;

  /**
   * Holt ein Drucker mit aus der Tabelle Drucker
   * @param druckerId Drucker-Nr.
   * @return druckerVO Value-Object eines Druckers
   * @throws RemoteException
   */
  public DruckerVO getPrinter( int druckerId ) throws RemoteException;

  /**
   * Gets the next free printer ID.
   * @return The new printer ID
   * @throws RemoteException if the selection fails
   */
  public int getNextDruckerId() throws RemoteException;

  /**
   * Mit dieser Methode wird das übergebene Value-Object, dass Angabe zu einen neuen Drucker enthält, in die Tabelle
   * Drucker und Logidrucker geschrieben.
   * @param druckerVO Value-Object mit den neuen Daten.
   * @throws RemoteException
   */
  public void insertDrucker( DruckerVO druckerVO ) throws RemoteException;

  /**
   * Mit dieser Methode wird das übergebene Value-Object, dass Angabe zu korrigierten Drucker enthält, in die Tabelle
   * Drucker und Logidrucker geschrieben.
   * @param druckerVO Value-Object mit den korrigierten Daten.
   * @throws RemoteException
   */
  public void updateDrucker( DruckerVO druckerVO ) throws RemoteException;

  /**
   * Deletes a printer identified by <code>printerId</code>. This means that the printer's entry in the table
   * <code>LOGIDRUCKER</code> is also removed.
   * @param printerId The printer's unique identifier
   * @throws RemoteException if the printer can't be deleted; note that the Exception's cause must be checked for
   * further information
   */
  public void deleteDrucker( int printerId ) throws RemoteException;

  /**
   * Deletes a print job identified by <code>printJobId</code>.
   * @param printJobId The print job's unique identifier
   * @throws RemoteException if the print job can't be deleted
   */
  public void deletePrintjob( int printJobId ) throws RemoteException;

  /**
   * Deletes print jobs with the corresponding print job IDs. 
   * @param printjobIds The print jobs' unique identifiers
   * @throws RemoteException if the print job can't be deleted
   */
  public void deletePrintjobs( List<Integer> printjobIds ) throws RemoteException;

  /**
   * Returns the distinct names of the print control servers.
   * @return The distinct names of the print control servers
   * @throws RemoteException if the servers can't be fetched
   */
  public Set<String> selectControlServers() throws RemoteException;

  /**
   * Returns the distinct types of print jobs.
   * @return The distinct types of print jobs in lowercase
   * @throws RemoteException if the print job types can't be fetched
   */
  public Set<String> selectPrintJobTypes() throws RemoteException;

  /**
   * Veranlasst die Löschung der PrintJobs gemäss der Periode in den Properties
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   * @throws RemoteException
   */
  public int removePrintJobs() throws RemoteException;

  /**
   * Holt die Meta-Daten der Datenbanken und gibt sie als Array-Liste zurück.
   * @return list Liste mit den URL-Strings der Datenbanken
   * @throws RemoteException
   */
  public List<String> getDbUrls() throws RemoteException;

  /**
   * Gibt die Version der Servers (Jipps) bekannt.
   * @return String Versions-Nummer
   * @throws RemoteException
   */
  public String getJippsVersion() throws RemoteException;

  /**
   * Holt die DruckPerioden auf der Tabelle Loeschung.
   * @param druckArt Druckart 5 = Nichtbezahlen, 6 = Schutzablauf
   * @return List mit den DruckPeriVO's
   * @throws RemoteException
   */
  public List<DruckPeriVO> getDruckPeriListByDruckArt( int druckArt ) throws RemoteException;

  /**
   * Holt die DruckPerioden auf der Tabelle Loeschung.
   * @param druckArt Druckart 5 = Nichtbezahlen, 6 = Schutzablauf
   * @param id Die Löschungslauf-ID
   * @return List mit den DruckPeriVO's
   * @throws RemoteException
   */
  public List<DruckPeriVO> getDruckPeriListById( int druckArt, int id ) throws RemoteException;

  /**
   * Holt die letzten paar Ids der Löschung Läufe.
   * @param cancellationType Druckart 5 = Nichtbezahlen, 6 = Schutzablauf
   * @param maxNumber Maximale Anzahl von Datensätzen. Es werden immer die "jüngsten" zurückgegeben
   * @return Die IDs
   * @throws RemoteException
   */
  public List<LoeschungLaufVO> getLoeschungsLaeufe( Integer cancellationType, Integer maxNumber ) throws RemoteException;

  /**
   * Mit dieser Methode wird das übergebene Value-Object, dass Angabe zu korrigierten Properties enthält, in die Tabelle
   * PropertyJipps geschrieben.
   * @param propVO Value-Object mit den korrigierten Daten.
   * @throws RemoteException
   */
  public void updateProperty( PropertyJippsVO propVO ) throws RemoteException;

  /**
   * Mit dieser Methode wird das übergebene Value-Object, dass Angaben einer Property enthält, in die Tabelle
   * PropertyJipps geschrieben.
   * @param propVO Value-Object mit den neuen Daten.
   * @throws RemoteException
   */
  public void insertProperty( PropertyJippsVO propVO ) throws RemoteException;

  /**
   * Mit dieser Methode wird die übergebene Property aus der Tabelle PropertyJipps gelöscht.
   * @param key Schluesselwert.
   * @throws RemoteException
   */
  public void deleteProperty( String key ) throws RemoteException;

  /**
   * Gets all existing properties.
   * @return All properties
   * @throws RemoteException if the properties can't be found
   */
  public List<PropertyJippsVO> getPropertyList() throws RemoteException;

  /**
   * Gets a property identified by its key.
   * @param key The property key
   * @return The property
   * @throws RemoteException if the property can't be found
   */
  public PropertyJippsVO getPropertyByKey( String key ) throws RemoteException;

  /**
   * Holt die Printjobs aus der Tabelle Printjob
   * @param searchVO PrintJobSearchVO
   * @return List mit den PrintjobSearchVO's
   * @throws RemoteException
   */
  public List<PrintjobSearchVO> getPrintJobSearchList( PrintjobSearchVO searchVO ) throws RemoteException;

  /**
   * Methode zum Updaten der Printjobs die erneut gedruckt werden sollen ohne Aufbereitung
   * @param updateList Liste mit den zu bearbeitenden Jobs
   * @param isBatch Kennzeichen, ob bei der Suche nach Batchid gesucht wurde
   * @throws RemoteException
   */
  public void updatePrintjobs( List<PrintjobSearchVO> updateList, boolean isBatch ) throws RemoteException;

  /**
   * Löst den Batchdruck der Automatischen Markenverlängerung - Erinnerung 1 aus
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   * einfügen)
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   * @throws RemoteException
   */
  public int printAMVOne( boolean hasCover ) throws RemoteException;

  /**
   * Löst den Batchdruck der Automatischen Markenverlängerung - Erinnerung 1 aus.
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   * einfügen)
   * @param amvLaufId
   * @param sendMail
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   * @throws RemoteException
   */
  //TODO: Add description for args amvLaufId and sendMail
  public int printAMVOne( boolean hasCover, int amvLaufId, boolean sendMail ) throws RemoteException;

  /**
   * Setzt den Verarb_Stat auf 3 (erneuter Druck) in der Tabelle Loeschung.
   * @param periVO Value Object
   * @throws RemoteException
   */
  public void updateVerarbStat( DruckPeriVO periVO ) throws RemoteException;

  /**
   * Löst den Wiederholungsdruck der Automatischen Markenverlängerung - Erinnerung 1 aus und schliesst diesen auch bei
   * einem Fehlerfall ab.
   * @param hasCover boolean Flag für Zwischenblätter (true = Zwischenblatt einfügen, false = kein Zwischenblatt
   * einfügen)
   * @param amvLaufId int Lauf-Nummer für Wiederholungsdruck
   * @param sendMail boolean true = Bestätigungs-Mail wird gesendet, false = Bestätigungs-Mail wird nicht gesendet
   * @throws RemoteException
   */
  public void finishAMVOne( boolean hasCover, int amvLaufId, boolean sendMail ) throws RemoteException;

  /**
   * Gibt ein Value-Object eines Amv-Laufes zurück.
   * @param amvLaufId int Amvlauf-Nummer
   * @return AmvLaufVO Value-Object
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  public AmvLaufVO getAmvLauf( int amvLaufId ) throws RemoteException;

  /**
   * Gibt eine Liste mit AMV-Errors VO's zurück.
   * @param amvLaufId int Amvlauf-Nummer
   * @return AmvLaufVO Value-Object
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  public List<AmvErrorVO> findErrorList( int amvLaufId ) throws RemoteException;

  /**
   * Gibt ein Liste mit Value-Object von Amv-Läufe zurück.
   * @param typCd String Amvlauf-Nummer
   * @return List mit Value-Objects mit AmvLaufVO's
   * @throws RemoteException Unerwarteter Remote-Fehler
   */
  public List<AmvLaufVO> findAmvLauf( String typCd ) throws RemoteException;

  /**
   * Generate the PDF and return the xml, image and pdf files of the published patent documents given a list of patent document identifiers (table PATDOC).
   * The patent documents are only returned if published.
   * @param patdocIdsPrintJobs map of patent documents identifiers and PrintJobVOs for which the pdf shall be generated
   * @return a list <code>PatentDocuemntVOWithPDF</code> containing the pdf, xml and image files. 
   * @throws RemoteException 
   * @deprecated
   */
  public List<PatentDocumentVOWithPDF> getPublishedPatentDocumentsWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsPrintJobs )
      throws RemoteException;

  /**
   * Generate the PDF and return the xml, image and pdf files of the patent documents given a list of patent document identifiers (table PATDOC).
   * The patent documents are only returned if published.
   * @param patdocIdsPrintJobs map of patent documents identifiers and PrintJobVOs for which the pdf shall be generated
   * @param publishedOnly exclude patent documents which are not published yet
   * @return a list <code>PatentDocuemntVOWithPDF</code> containing the pdf, xml and image files. 
   * @throws RemoteException 
   */
  public List<PatentDocumentVOWithPDF> getPatentDocumentsWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsPrintJobs, boolean publishedOnly )
      throws RemoteException;

  /**
   * Generate the bibliographic data and the PDF and return the xml, image and pdf files of the bibliographic data given a list of patent document identifiers (table PATDOC).
   * The patent documents are only returned if published.
   * @param patdocIdsPrintJobs map of patent documents identifiers and PrintJobVOs for which the bibliographic data and pdf shall be generated
   * @param publishedOnly exclude patent documents which are not published yet
   * @return a list <code>PatentDocuemntVOWithPDF</code> containing the pdf, xml and image files. 
   * @throws RemoteException 
   */
  public List<PatentDocumentVOWithPDF> getBiblioDataWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsPrintJobs, boolean publishedOnly )
      throws RemoteException;

  /**
   * Get the information (identifier and event) from the patent documents available for a given publication date and patentnumber 
   * @param publicationDate Das Publikationsdatum der Schrift
   * @param patentNumber Die Patentnummer, wird ignoriert wenn negativ oder 0 oder null
   * @return Die Schlüssel der Schriften und deren Ereignisse
   * @throws RemoteException 
   */
  public List<PatentDocumentEventKeyVO> getAllPatdocIds( final Date publicationDate, final Integer patentNumber ) throws RemoteException;

  /**
   * Get the configuration for the client of the FTP Servlet for the EPA Upload
   * @return the configuration
   * @throws RemoteException
   */
  public EPAUploadClientConfigWrapper getConfigurationForEPAUploadClient() throws RemoteException;

  /**
   * returns a List of all the available PrintjobVOs for the given BfaktID
   * @param bfaktId
   * @return a List of all the available PrintjobVOs for the given BfaktID
   * @throws RemoteException
   */
  public List<Integer> getPrintJobsByBfaktId( final Integer bfaktId ) throws RemoteException;

}
