package ch.ige.jipps;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.adr.server.AdresseFacade;
import ch.ige.adr.server.AdresseFacadeHome;
import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAOPool;
import ch.ige.ipobjects.common.dao.ImageDAO;
import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.constants.FormAreaConstants;
import ch.ige.jipps.constants.LetterTypeConstants;
import ch.ige.jipps.dao.BFormTxtDruckDAO;
import ch.ige.jipps.dao.BfakTxtDAO;
import ch.ige.jipps.dao.BfaktPosDAO;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.dao.FormTextDAO;
import ch.ige.jipps.dao.JoinSchreibenDAO;
import ch.ige.jipps.dao.PotZahlDAO;
import ch.ige.jipps.exception.DBException;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.text.P2d2Tags;
import ch.ige.jipps.text.P2d2Utils;
import ch.ige.jipps.vo.BFormTxtDruckVO;
import ch.ige.jipps.vo.BfakTxtVO;
import ch.ige.jipps.vo.BfaktPosVO;
import ch.ige.jipps.vo.FormTextVO;
import ch.ige.jipps.vo.JippsFaktPosVO;
import ch.ige.jipps.vo.JippsFakturVO;
import ch.ige.jipps.vo.LetterVO;
import ch.ige.util.IGEDate;
import ch.ige.util.enums.BAGISLanguage;
import ch.ige.util.server.replace.P2d2;
import ch.ige.util.vo.VersadrVO;
import ch.ige.wdlutil.exception.WdlSyntaxException;
import ch.ige.wdlutil.format.WdlFormatter;

/**
 * Loads the data for letters and creates the corresponding value objects.
 * <p>
 * Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Marc Bouquet
 * @author Johannes Lang
 * @author Eric Lewis
 * @version $Id: LetterAssembler.java,v 1.158 2009/02/11 09:46:02 lewis_lee Exp $
 */
public class LetterAssembler
{
  /**
   * The key for a System Property which forces this to use a dummy address
   * instead of calling the address EJB; this is used for unit tests which
   * don't rely on a running application server with all EARs deployed. 
   */
  public static final String USE_DUMMY_ADDRESS = "dummy_address";

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( LetterAssembler.class );

  /**
   * The length of the {@link P2d2Tags#WDL} tag.
   */
  private static final int WDL_START_LENGTH = P2d2Tags.WDL.length();

  /**
   * The length of the {@link P2d2Tags#WDL_END} tag.
   */
  private static final int WDL_END_LENGTH = P2d2Tags.WDL_END.length();

  /**
   * Loads the data for an invoice / letter.
   * @param invoiceId The unique ID of the invoice
   * @param publicationDate The date when the events contained in the invoice were published; if it's <code>null</code>, it's taken from the invoice
   * @param letterDate The date when the letter is sent; if it's <code>null</code>, it's taken from the invoice
   * @param canUpdateInvoiceDate is this method allowed to update the InvoiceDate? this param is only forwarded, but not used directly in this method.
   * @return The letters containing the data to transform into PDFs
   * @throws Exception if the letter creation fails
   */
  public List<LetterVO> loadLetterVO( int invoiceId, Date publicationDate, Date letterDate, boolean canUpdateInvoiceDate ) throws Exception
  {
    List<LetterVO> letters = new ArrayList<LetterVO>();

    try
    {
      LetterVO letter = loadLetter( invoiceId, null, true, publicationDate, letterDate, false, canUpdateInvoiceDate );
      letters.add( letter );

      List<Integer> additionalAddressNumbers = loadAdditionalAddressNumbers( invoiceId );
      for ( Integer additionalAddressNumber: additionalAddressNumbers )
      {
        LOG.debug( "Creating additional letter for address number " + additionalAddressNumber );
        LetterVO additionalLetter = new LetterVO();
        // Copying this stuff around without deep cloning may have some side effects, like changing all objects at the same time!!
        additionalLetter.jippsFakturVO = letter.jippsFakturVO;
        additionalLetter.jippsFaktPositionen = letter.jippsFaktPositionen;
        additionalLetter.formTexte = letter.formTexte;
        additionalLetter.bformTexte = letter.bformTexte;
        additionalLetter.bfakTexte = letter.bfakTextePotZahl;
        additionalLetter.versadrVO = loadAddress( additionalAddressNumber );
        additionalLetter.bild = letter.bild;

        letters.add( additionalLetter );
      }
    }
    catch ( DBException ex )
    {
      LOG.error( "SQL-Fehler beim Laden des LetterVO", ex );
    }
    catch ( ParseException ex )
    {
      LOG.error( "ParseException beim Laden des LetterVO", ex );
    }
    catch ( Exception ex )
    {
      LOG.warn( "Fehler beim Laden des LetterVO", ex );
      throw ex;
    }
    return letters;
  }

  /**
   * Loads exactly one letter.
   * @param invoiceId The invoice's unique ID
   * @param addressId The address's ID
   * @param canUpdateInvoiceDate is this method allowed to update the InvoiceDate? this param is only forwarded, but not used directly in this method.
   * @return The List of letters, which contains only one letter
   * @throws Exception if loading fails
   */
  public List<LetterVO> loadLetterVO( int invoiceId, int addressId, boolean canUpdateInvoiceDate ) throws Exception
  {
    List<LetterVO> letters = new ArrayList<LetterVO>( 1 );
    // laj/lee 11.02.2009
    // Note: Normally, the optional texts depend on the hasPotZahl, but since it's only false for AMV,
    // and some letters always need the optional texts, we have to set the flag to true.
    letters.add( loadLetter( invoiceId, addressId, true, null, null, true, canUpdateInvoiceDate ) );
    return letters;
  }

  /**
   * Loads a letter.
   * @param invoiceId The invoice's unique ID
   * @param addressId The address's ID
   * @param loadOptionalTexts if <code>true</code>, this loads optional texts by calling {@link #loadOptionalTexts(LetterVO, boolean)}
   * @param publicationDate The date when the events contained in the invoice were published; if it's <code>null</code>, it's taken from the invoice
   * @param letterDate The date when the letter is sent; if it's <code>null</code>, it's taken from the invoice
   * @param isOnlyOneLetter If only one letter is created, this is <code>true</code>, otherwise it's <code>false</code>
   * @param canUpdateInvoiceDate is this method allowed to update the InvoiceDate? this param is only forwarded, but not used directly in this method.
   * @return The letter
   * @throws Exception if loading fails
   */
  private static LetterVO loadLetter( Integer invoiceId, Integer addressId, boolean loadOptionalTexts, Date publicationDate, Date letterDate,
      boolean isOnlyOneLetter, boolean canUpdateInvoiceDate ) throws Exception
  {
    LetterVO letter = new LetterVO();

    letter.jippsFakturVO = loadInvoice( invoiceId, canUpdateInvoiceDate );

    if ( publicationDate != null )
    {
      letter.jippsFakturVO.publicationDate = publicationDate;
    }
    if ( letterDate != null )
    {
      letter.jippsFakturVO.bfaktDat = letterDate;
    }

    letter.jippsFaktPositionen = loadInvoiceLineItems( invoiceId );
    if ( addressId != null )
    {
      letter.versadrVO = loadAddress( addressId );
    }
    else
    {
      letter.versadrVO = loadAddress( letter.jippsFakturVO.adrNr );
    }
    letter.formTexte = loadFormTexts( BAGISLanguage.parse( letter.jippsFakturVO.sprachCd ), FormAreaConstants.BAGIS );
    letter.bformTexte = loadBFormTexts( BAGISLanguage.parse( letter.jippsFakturVO.sprachCd ), Integer.valueOf( letter.jippsFakturVO.formartCd ) );
    if ( loadOptionalTexts )
    {
      letter.bfakTexte = loadOptionalTexts( letter, false );
      letter.bfakTextePotZahl = loadOptionalTexts( letter, true );
    }
    replaceP2d2Tags( letter, isOnlyOneLetter );

    letter.bild = loadImage( letter );

    return letter;
  }

  /**
   * Loads the invoice and the IP.
   * If {@link JippsFakturVO#bfaktDat} is <code>null</code>, it's set to the current date, both in the returned object and in the database.
   * @param invoiceId The invoice's unique ID
   * @param canUpdateInvoiceDate is this method allowed to update the InvoiceDate?
   * @return The combined invoice and IP
   * @throws DBException if the invoice can't be loaded
   */
  private static JippsFakturVO loadInvoice( int invoiceId, boolean canUpdateInvoiceDate ) throws DBException
  {
    try
    {
      JippsFakturVO jippsInvoice = DAOPool.get( JoinSchreibenDAO.class ).select( invoiceId );

      if ( jippsInvoice.bfaktDat == null )
      {
        LOG.debug( "BfaktDat for Bfaktur " + invoiceId + " is null. The invoice is set to the actual date." );
        java.sql.Date date = new java.sql.Date( System.currentTimeMillis() );
        jippsInvoice.bfaktDat = date;
        if ( canUpdateInvoiceDate )
        {
          LOG.debug( "The BfaktDat for Bfaktur " + invoiceId + " can be updated." );
          DAOPool.get( BfakturDAO.class ).updateBFakturBfaktDat( invoiceId, date );
          LOG.info( "Invoice " + invoiceId + " was updated to date " + IGEDate.dateToString( date ) );
        }
      }

      return jippsInvoice;
    }
    catch ( SQLException ex )
    {
      throw new DBException( "Couldn't load JippsFakturVO for invoice " + invoiceId, ex );
    }
  }

  /**
   * Gets all the line items for an invoice.
   * @param invoiceId The invoice's unique ID
   * @return The line items for the invoice
   * @throws DBException if the line items can't be loaded
   * @throws IllegalStateException If the first line item has no text
   */
  private static List<JippsFaktPosVO> loadInvoiceLineItems( int invoiceId ) throws DBException, IllegalStateException
  {
    try
    {
      List<JippsFaktPosVO> items = new ArrayList<JippsFaktPosVO>();

      List<BfaktPosVO> invoiceLineItems = DAOPool.get( BfaktPosDAO.class ).select( invoiceId );
      if ( invoiceLineItems.isEmpty() )
      {
        LOG.error( "Invoice (BFaktur) with ID " + invoiceId + " has no items. This might lead to an invalid letter XML document later!" );
      }
      else if ( invoiceLineItems.get( 0 ).textNumber <= 0 )
      {
        throw new IllegalStateException( "Invoice (BFaktur) " + invoiceId + " has a first line item (no. " + invoiceLineItems.get( 0 ).lineItemNumber
            + "), which contains no text." );
      }

      for ( BfaktPosVO invoiceLineItem: invoiceLineItems )
      {
        // Text numbers <= 0 must not be printed
        if ( invoiceLineItem.textNumber > 0 )
        {
          JippsFaktPosVO item = new JippsFaktPosVO();
          item.invoiceLineItem = invoiceLineItem;
          items.add( item );
        }
      }
      return items;
    }
    catch ( SQLException ex )
    {
      throw new DBException( "LetterAssembler: SQL-Fehler beim Laden des BfaktPosVO", ex );
    }
  }

  /**
   * Loads the letter's main address.
   * For testing, the System Property {@link #USE_DUMMY_ADDRESS} may be set; in that
   * case (as the name says), a dummy address is created, without having to resort to
   * the address Bean on the application server.
   * @param addressId The address's ID
   * @return The address
   * @throws Exception if the address can't be fetched.
   */
  private static VersadrVO loadAddress( int addressId ) throws Exception
  {
    // Enable this to run without depending on a deployed EJB, otherwise local unit tests don't work.
    String useDummyAddress = System.getProperty( USE_DUMMY_ADDRESS );
    if ( useDummyAddress != null || ( !AbstractDAO.staticIsRunningInJ2EEServer() ) )
    {
      VersadrVO dummyAddress = new VersadrVO();
      dummyAddress.setName( "Dummy-Adresse" );
      dummyAddress.setZeile1( "Dieser Adressblock wird" );
      dummyAddress.setZeile2( "nur angezeigt, wenn ein lokaler" );
      dummyAddress.setZeile3( "Unit-Test läuft" );
      return dummyAddress;
    }
    else
    {
      Context context = new InitialContext();
      AdresseFacadeHome home = (AdresseFacadeHome) context.lookup( AdresseFacadeHome.JNDI_NAME );
      AdresseFacade facade = home.create();
      return facade.getVersadr( addressId );
    }
  }

  /**
   * Loads additional (potential) address numbers for the letter to be sent to.
   * @param invoiceId The invoice's unique ID
   * @return The address numbers of the additional addresses, which can be empty
   * @throws DBException if the load fails
   */
  private static List<Integer> loadAdditionalAddressNumbers( int invoiceId ) throws DBException
  {
    try
    {
      return DAOPool.get( PotZahlDAO.class ).select( invoiceId );
    }
    catch ( SQLException ex )
    {
      throw new DBException( "LetterAssembler: SQL-Fehler beim Laden des PotZahlVO", ex );
    }
  }

  /**
   * Gets form texts for the given language and form area.
   * @param language The language
   * @param formArea The form area, one of {@link FormAreaConstants}
   * @return The found form texts, which may be empty
   * @throws DBException if the load fails
   */
  private static List<FormTextVO> loadFormTexts( BAGISLanguage language, String formArea ) throws DBException
  {
    try
    {
      List<FormTextVO> texts = DAOPool.get( FormTextDAO.class ).select( language, formArea );
      for ( FormTextVO text: texts )
      {
        if ( StringUtils.isEmpty( text.text ) )
        {
          LOG.info( "Got empty form text for " + text );
        }
      }
      return texts;
    }
    catch ( SQLException ex )
    {
      throw new DBException( "Couldn't load form texts", ex );
    }
  }

  /**
   * Gets B form text for the given language and the form type.
   * @param language The language
   * @param formType The type of form
   * @return The found form texts, which may be empty
   * @throws DBException if the load fails
   */
  private static List<BFormTxtDruckVO> loadBFormTexts( BAGISLanguage language, Integer formType ) throws DBException
  {
    try
    {
      List<BFormTxtDruckVO> texts = DAOPool.get( BFormTxtDruckDAO.class ).select( language, formType );
      for ( BFormTxtDruckVO text: texts )
      {
        if ( StringUtils.isEmpty( text.text ) )
        {
          LOG.info( "Got empty BForm text for " + text );
        }
      }
      return texts;
    }
    catch ( SQLException ex )
    {
      throw new DBException( "Couldn't load B form texts", ex );
    }
  }

  /**
   * Loads all optional texts for the letter (<code>BFaktText</code> in German).
   * @param letter The letter being created
   * @param isAdditionalAddress if this is <code>true</code>, it's an additional address, otherwise it's the main address
   * @return The optional texts
   * @throws DBException if the load fails
   */
  private static List<BfakTxtVO> loadOptionalTexts( LetterVO letter, boolean isAdditionalAddress ) throws DBException
  {
    List<Integer> textNumbers = new ArrayList<Integer>();
    // Add an address addition only if it's an additional address
    if ( !isAdditionalAddress )
    {
      addExistingTextNumber( textNumbers, letter.jippsFakturVO.txtAdrZusatz );
    }

    // There's an attachment
    addExistingTextNumber( textNumbers, letter.jippsFakturVO.txtNrBeilage );

    // There's an ending text
    addExistingTextNumber( textNumbers, letter.jippsFakturVO.txtSchluss );

    // Add all optional line item texts
    for ( JippsFaktPosVO invoiceLineItem: letter.jippsFaktPositionen )
    {
      addExistingTextNumber( textNumbers, invoiceLineItem.invoiceLineItem.textNumber );
    }

    if ( textNumbers.isEmpty() )
    {
      return Collections.emptyList();
    }
    else
    {
      try
      {
        List<BfakTxtVO> texts = DAOPool.get( BfakTxtDAO.class ).select( textNumbers );
        // Convert WDLs if necessary
        convertWDL( texts );
        for ( BfakTxtVO text: texts )
        {
          if ( StringUtils.isEmpty( text.text ) )
          {
            LOG.info( "Got empty BFakText for " + text );
          }
          text.text = ConvertText.replaceHtmlTags( text.text );
        }
        return texts;
      }
      catch ( SQLException ex )
      {
        throw new DBException( "LetterAssembler: SQL-Fehler beim Laden des BfakTxtVO", ex );
      }
    }
  }

  /**
   * Adds a text number to the existing text numbers if it's valid (that is, greater than 0).
   * @param textNumbers The existing text numbers
   * @param textNumber The text number to potentially add
   */
  private static void addExistingTextNumber( List<Integer> textNumbers, Integer textNumber )
  {
    if ( textNumber != null && textNumber > 0 )
    {
      textNumbers.add( textNumber );
    }
  }

  /**
   * If the letter is an application confirmation, this gets the trademark's image.
   * @param letter The letter to check
   * @return The image for the trademark or <code>null</code> if there's none or <code>letter</code> is the wrong type
   * @throws DBException if the image can't be loaded
   */
  public static byte[] loadImage( LetterVO letter ) throws DBException
  {
    byte[] imageData = null;

    if ( LetterTypeConstants.isTrademarkApplicationConfirmation( letter.jippsFakturVO.formartCd ) )
    {
      // Double brackets mean that the name contains a <code>((fig.))</code>, which means it's an image trademark
      if ( letter.jippsFakturVO.beschreibung.contains( "((" ) )
      {
        try
        {
          LOG.debug( "Loading image for IP " + letter.jippsFakturVO.schutitKey + " and date " + IGEDate.dateToString( letter.jippsFakturVO.bfaktDat ) );
          ImageVO image = DAOPool.get( ImageDAO.class ).getSingleImage( IPKeyVO.create( letter.jippsFakturVO.schutitKey ),
              EreigIDat.max( letter.jippsFakturVO.bfaktDat ) );
          if ( image != null )
          {
            imageData = image.getImage().getBytes( 1, (int) image.getImage().length() );
            LOG.debug( "Loaded image for IP with key " + letter.jippsFakturVO.schutitKey + ", size: " + imageData.length );
          }
        }
        catch ( Exception ex )
        {
          throw new DBException( "LetterAssembler: Fehler beim Laden des Bildes", ex );
        }
      }
    }

    return imageData;
  }

  /**
   * Replaces all {@link P2d2} tags in a letter.
   * @param letter The letter
   * @param isOnlyOneLetter If only one letter is created, this is <code>true</code>, otherwise it's <code>false</code>
   */
  private static void replaceP2d2Tags( LetterVO letter, boolean isOnlyOneLetter )
  {
    Map<String, String> tags = new HashMap<String, String>();

    // Replace tags in BAGIS form texts.
    for ( BFormTxtDruckVO bFormText: letter.bformTexte )
    {
      if ( StringUtils.isNotEmpty( bFormText.text ) )
      {
        tags.clear();
        tags.putAll( P2d2.findP2d2( bFormText.text ) );
        if ( !tags.isEmpty() )
        {
          LOG.debug( "Got " + tags.size() + " P2D2 tags in BForm text " + bFormText.text );
        }
        if ( letter.jippsFaktPositionen.size() > 0 )
        {
          fillTagValues( tags, letter.jippsFakturVO, letter.jippsFaktPositionen.get( letter.jippsFaktPositionen.size() - 1 ), letter.bfakTexte );
        }
        else
        {
          fillTagValues( tags, letter.jippsFakturVO, null, letter.bfakTexte );
          LOG.info( "Text number in the table BfaktPos is <= 0" );
        }

        try
        {
          bFormText.text = P2d2.replaceP2d2( bFormText.text, tags );
        }
        catch ( Exception ex )
        {
          LOG.error( "Replacement of P2D2 tags fails for B form texts", ex );
        }
      }
    }

    // Replace tags in invoice line items
    replaceTagsInInvoiceText( letter.jippsFakturVO, letter.jippsFaktPositionen, letter.bfakTexte, tags );

    // Replace tags in additional addresses
    if ( !isOnlyOneLetter )
    {
      replaceTagsInInvoiceText( letter.jippsFakturVO, letter.jippsFaktPositionen, letter.bfakTextePotZahl, tags );
    }
  }

  /**
   * Replaces the P2D2 tags in invoice texts.
   * @param invoice The invoice
   * @param invoiceLineItems The line items of the invoice
   * @param texts The existing texts
   * @param tags The P2D2 tags
   */
  private static void replaceTagsInInvoiceText( JippsFakturVO invoice, List<JippsFaktPosVO> invoiceLineItems, List<BfakTxtVO> texts, Map<String, String> tags )
  {
    for ( JippsFaktPosVO invoiceLineItem: invoiceLineItems )
    {
      // Replace tags in primary address
      BfakTxtVO text = getText( texts, invoiceLineItem.invoiceLineItem.textNumber );
      if ( text != null )
      {
        if ( StringUtils.isNotEmpty( text.text ) )
        {
          tags.clear();
          tags.putAll( P2d2.findP2d2( text.text ) );
          if ( !tags.isEmpty() )
          {
            LOG.debug( "Got " + tags.size() + " P2D2 tags in text " + text.text );
          }
          fillTagValues( tags, invoice, invoiceLineItem, texts );
          try
          {
            text.text = P2d2.replaceP2d2( text.text, tags );
          }
          catch ( Exception ex )
          {
            LOG.error( "Can't replace tags in text " + text.text, ex );
          }
        }
      }
      else
      {
        LOG.error( "Found no text for invoice line item (BFaktPos) with text number " + invoiceLineItem.invoiceLineItem.textNumber );
      }
    }
  }

  /**
   * Fills in the values for all tags that are found in the letter.
   * @param tags The <code>Map</code> containing all tags, but no values
   * @param invoice The invoice
   * @param invoiceLineItem The invoice line item
   * @param invoiceTexts The texts that contain the ending text
   */
  private static void fillTagValues( Map<String, String> tags, JippsFakturVO invoice, JippsFaktPosVO invoiceLineItem, List<BfakTxtVO> invoiceTexts )
  {
    if ( !tags.isEmpty() )
    {
      if ( invoiceLineItem != null )
      {
        P2d2Utils.setIntegerReplacement( tags, P2d2Tags.ANZAHL, invoiceLineItem.invoiceLineItem.numberOfArticles );
        P2d2Utils.setStringReplacement( tags, P2d2Tags.SCHUTIT, invoiceLineItem.invoiceLineItem.ipNumber );
        P2d2Utils.setBigDecimalReplacement( tags, P2d2Tags.PREIS, invoiceLineItem.preis );
        P2d2Utils.setBigDecimalReplacement( tags, P2d2Tags.WERT, invoiceLineItem.wert );

        Date eventDate = null;
        if ( invoiceLineItem.invoiceLineItem.eventDate != null )
        {
          eventDate = invoiceLineItem.invoiceLineItem.eventDate.getDate();
        }
        P2d2Utils.setDateReplacement( tags, P2d2Tags.EREIG_DAT, eventDate );
        P2d2Utils.setDateReplacement( tags, P2d2Tags.FRIST, invoiceLineItem.invoiceLineItem.deadline );
        P2d2Utils.setStringReplacement( tags, P2d2Tags.PUBL_TEXT, invoiceLineItem.invoiceLineItem.publicationText );
      }
      else
      {
        P2d2Utils.setTagReplacement( tags, P2d2Tags.ANZAHL );
        P2d2Utils.setTagReplacement( tags, P2d2Tags.SCHUTIT );
        P2d2Utils.setTagReplacement( tags, P2d2Tags.PREIS );
        P2d2Utils.setTagReplacement( tags, P2d2Tags.WERT );
        P2d2Utils.setTagReplacement( tags, P2d2Tags.EREIG_DAT );
        P2d2Utils.setTagReplacement( tags, P2d2Tags.FRIST );
        P2d2Utils.setTagReplacement( tags, P2d2Tags.PUBL_TEXT );
      }

      P2d2Utils.setStringReplacement( tags, P2d2Tags.TEXT, invoice.varText );
      P2d2Utils.setStringReplacement( tags, P2d2Tags.INHABER, invoice.inhaber );
      P2d2Utils.setStringReplacement( tags, P2d2Tags.UNTERSCH, invoice.untName );
      P2d2Utils.setBigDecimalReplacement( tags, P2d2Tags.TOTAL, invoice.faktBetrag );
      P2d2Utils.setStringReplacement( tags, P2d2Tags.TITEL, invoice.beschreibung );
      P2d2Utils.setStringReplacement( tags, P2d2Tags.GESUCH, invoice.gesuchNr );

      BfakTxtVO endingText = getText( invoiceTexts, invoice.txtSchluss );
      P2d2Utils.setStringReplacement( tags, P2d2Tags.DIVERS, endingText == null ? null : endingText.text );

      P2d2Utils.setDateReplacement( tags, P2d2Tags.SCHUTZB, invoice.schutzbeginn );
      P2d2Utils.setDateReplacement( tags, P2d2Tags.SCHUTZENDE, invoice.schutzende );
      P2d2Utils.setDateReplacement( tags, P2d2Tags.VERFALL, invoice.verfallDat );
      P2d2Utils.setDateReplacement( tags, P2d2Tags.PUBL_DAT, invoice.publicationDate );

      P2d2Utils.checkAndDefaultValuelessTags( tags );
    }
  }

  /**
   * Converts texts if they contain a WDL.
   * @param invoiceTexts contains all texts for an invoice
   */
  public static void convertWDL( List<BfakTxtVO> invoiceTexts )
  {
    // Note: If there is a WDL in the texts, then it must contain a start and an end tag.
    // The WDL may have been split over several lines, but that is only the case for really old invoices.
    // In that case, the WDL formatting may not be correct...

    boolean wdlBeingProcessed = false;

    try
    {
      for ( BfakTxtVO invoiceText: invoiceTexts )
      {
        LOG.debug( "Text: " + invoiceText.text );
        int wdlStart = invoiceText.text.indexOf( P2d2Tags.WDL );

        // Found WDL in the text
        if ( wdlStart != -1 )
        {
          LOG.debug( "Found WDL start tag in text, converting WDL" );
          wdlBeingProcessed = true;

          int wdlEnd = invoiceText.text.indexOf( P2d2Tags.WDL_END );
          if ( wdlEnd == -1 )
          {
            wdlEnd = invoiceText.text.length();
            LOG.debug( "Found no WDL end tag, converting from text position " + wdlStart + " to end (" + wdlEnd + ")" );
          }
          else
          {
            wdlBeingProcessed = false;
            LOG.debug( "Found WDL end tag, converting from text position " + wdlStart + " to " + wdlEnd );
          }

          String newText = invoiceText.text.substring( 0, wdlStart );
          newText += WdlFormatter.toHtml( invoiceText.text.substring( wdlStart + WDL_START_LENGTH, wdlEnd ), false, false );
          if ( wdlEnd < invoiceText.text.length() )
          {
            newText += invoiceText.text.substring( wdlEnd + WDL_END_LENGTH, invoiceText.text.length() );
          }
          invoiceText.text = newText;
        }
        // There is a WDL, but we're in the middle of it
        else if ( wdlBeingProcessed )
        {
          LOG.debug( "Found no WDL start tag, but we're in the middle of the WDL text" );
          int wdlEnd = invoiceText.text.indexOf( P2d2Tags.WDL_END );

          if ( wdlEnd == -1 )
          {
            LOG.debug( "Found no WDL end tag, converting the whole text" );
            invoiceText.text = WdlFormatter.toHtml( invoiceText.text, false, false );
          }
          else
          {
            LOG.debug( "Found WDL end tag, converting from start to " + wdlEnd );
            String newText = WdlFormatter.toHtml( invoiceText.text.substring( 0, wdlEnd ), false, false );
            newText += invoiceText.text.substring( wdlEnd + WDL_END_LENGTH );
            invoiceText.text = newText;
            wdlBeingProcessed = false;
          }
        }
      }
    }
    catch ( WdlSyntaxException e )
    {
      LOG.error( "Couldn't format WDL", e );
    }
  }

  /**
   * Gets a text from the given invoice texts.
   * @param texts The texts to choose from
   * @param textNumber The text number to look for
   * @return The found text or <code>null</code> if none is found
   */
  private static BfakTxtVO getText( List<BfakTxtVO> texts, int textNumber )
  {
    for ( BfakTxtVO textVO: texts )
    {
      if ( textVO.textNr == textNumber )
      {
        return textVO;
      }
    }
    return null;
  }
}
