package ch.ige.jipps;

import java.sql.Date;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.jipps.constants.StatusConstants;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.BfakturVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.InvoiceConstants;
import ch.ige.util.constants.PrintStatusConstants;

/**
 * Controls the printing of letters in batch runs.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: LetterBatchControl.java,v 1.43 2009/01/12 09:32:27 lewis_lee Exp $
 */
public class LetterBatchControl extends LetterControl
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( LetterBatchControl.class );

  /**
   * The constructor
   */
  public LetterBatchControl()
  {
    // empty constructor
  }

  /**
   * Prints letters for changes that are done automatically to trademarks: They are either cancelled or prolongated.
   * @param hasCover Inserts a cover page if <code>true</code>, otherwise none if <code>false</code>
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  public int printAMVChanges( boolean hasCover )
  {
    final String publicationReportName = "AMV-Verlängerungen und -löschungen";
    final String publicationReportPrintjobName = "AMVBest";

    // TODO Create an enum
    final String propertyPrefix = "amv_regbest";

    try
    {
      LOG.info( "Started processing of '" + publicationReportName + "'." );

      BfakturDAO bfakturDAO = DAOPool.get( BfakturDAO.class );

      List<BfakturVO> cancellations = bfakturDAO.selectBFakturByForm( InvoiceConstants.FORM_TRADEMARK_CANCELLATION,
          PrintStatusConstants.READY_FOR_PRINTING_STRING );
      logInvoices( cancellations, "cancellation letters" );

      List<BfakturVO> prolongations = bfakturDAO.selectBFakturByForm( InvoiceConstants.FORM_TRADEMARK_PROLONGATION,
          PrintStatusConstants.READY_FOR_PRINTING_STRING );
      logInvoices( prolongations, "prolongation letters" );

      List<BfakturVO> invoices = new ArrayList<BfakturVO>();
      invoices.addAll( cancellations );
      invoices.addAll( prolongations );

      // Note that there's no publication date to print in the letter,
      // but the invoice must contain one.
      // Also, no copies are printed.
      Date now = new Date( System.currentTimeMillis() );
      for ( BfakturVO invoice: invoices )
      {
        invoice.publicationDate = now;
        invoice.anzKopie = 0;
      }
      return printLetters( invoices, publicationReportName, publicationReportPrintjobName, propertyPrefix, hasCover );
    }
    catch ( SQLException e )
    {
      LOG.error( "Error occured for '" + publicationReportName + "', couldn't load invoice IDs", e );
      return StatusConstants.ERROR;
    }
  }

  /**
   * Prints the given letters.
   * @param letters The letters to print, actually invoices
   * @param reportName The name of the report
   * @param propertyPrefix The property prefix used to get the printer and the path for the type of processing
   * @param hasCover Inserts a cover page if <code>true</code>, otherwise none if <code>false</code>
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  private int printLetters( List<BfakturVO> letters, String reportName, String printjobName, String propertyPrefix, boolean hasCover )
  {
    int status = StatusConstants.OK;

    int printerId = 0;

    try
    {
      try
      {
        JIPPSPropertiesServices jippsProperties = JIPPSPropertiesFactory.getInstance();

        printerId = jippsProperties.getDrucker( propertyPrefix );
        LOG.debug( "Loaded printer for " + reportName + " has ID: " + printerId );

        this.path = jippsProperties.getDirectoryPath( propertyPrefix );
        LOG.debug( "Loaded path for " + reportName + " is: " + this.path );
      }
      catch ( Exception ex )
      {
        LOG.error( "Exception beim Lookup auf jippsProperties", ex );
      }

      if ( letters == null )
      {
        LOG.warn( "Got <null> letter batch list for " + reportName );
      }
      else
      {
        if ( !letters.isEmpty() )
        {
          List<PrintJobVO> printJobs = createPrintJobs( letters, null, hasCover, printerId );
          setJMSProperties( printjobName );
          this.answers = super.makePrint( printJobs );
          resetJMSProperties();

          if ( !printJobs.isEmpty() )
          {
            status = sendAnswerMail( true, this.answers, printJobs.get( 0 ).batchId, letters.size(), printJobs.get( 0 ).druckerId, reportName, propertyPrefix,
                this.path );
            LOG.debug( "*** Status nach prepareAnswer: " + status );
          }
          else
          {
            status = StatusConstants.ERROR;
          }
        }
        else
        {
          LOG.warn( "There are no " + reportName + "." );
          // Send a mail to the batch user saying that there weren't any letters to print
          sendAnswerMail( false, this.answers, 0, 0, printerId, reportName, propertyPrefix, null );
          status = StatusConstants.OK;
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Fehler in LetterBatchControl", ex );
      status = StatusConstants.ERROR;
    }
    return status;
  }

  /**
   * Sets the additional JMS properties.
   * @param reportName The name of the report
   */
  private void setJMSProperties( String reportName )
  {
    super.saveInFileSystem = true;
    super.reportName = reportName;
  }

  /**
   * Resets the additional JMS properties.
   */
  private void resetJMSProperties()
  {
    super.saveInFileSystem = false;
    super.reportName = null;
  }

  /**
   * Sends a mail with information about the print jobs that were processed.
   * @param hasData Shows whether where printed print jobs
   * @param answersToCheck The answers of the print job processing
   * @param batchId The unique ID of the batch run that was executed
   * @param numberOfInvoices The number of processed invoices
   * @param printerId The ID of the printer where the print jobs are printed
   * @param reportName The name of the report
   * @param propertyPrefix The property prefix used to get the printer and the path for the type of processing
   * @param path The path where the PDF files are save
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  private int sendAnswerMail( boolean hasData, List<AnswerVO> answersToCheck, int batchId, int numberOfInvoices, int printerId, String reportName,
      String propertyPrefix, String path )
  {
    int status = StatusConstants.OK;
    String errorMsg = "";

    if ( answersToCheck != null )
    {
      for ( AnswerVO ansVO: answersToCheck )
      {
        if ( !ansVO.printStatus.equals( AnswerVO.STATUS_OK ) )
        {
          if ( ansVO.listPrintId.isEmpty() || ansVO.listPrintId.get( 0 ) == 0 )
          {
            status = StatusConstants.ERROR;
            errorMsg = ansVO.printStatus;
            LOG.error( "Die " + reportName + " konnten nicht erstellt werden:" + ansVO.printStatus );
          }
          else
          {
            errorMsg = errorMsg + "Schutztitel Key: " + ansVO.schutitKey + " / GesuchNr: " + ansVO.gesuchNr + "\n";
          }
        }
      }
    }

    try
    {
      JIPPSPropertiesServices jippsProperties = JIPPSPropertiesFactory.getInstance();
      if ( hasData )
      {
        jippsProperties.doMail( batchId, reportName, null, errorMsg, propertyPrefix, numberOfInvoices, this.numberOfErrors, printerId, path );
        LOG.debug( " *** Für die " + reportName + " wurde die Bestätigung gesendet." );
      }
      else
      {
        jippsProperties.doMailNoData( reportName, null, errorMsg, propertyPrefix, this.numberOfErrors );
        LOG.debug( " *** Für die " + reportName + " sind keine Daten vorhanden die Bestätigung wird gesendet." );
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception in prepareAnswer", ex );
      status = StatusConstants.ERROR;
    }
    return status;
  }
}
