package ch.ige.jipps.configuration;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.naming.InitialContext;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.jipps.JIPPSProperties;
import ch.ige.jipps.JIPPSPropertiesHome;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.constants.St36Constants;
import ch.ige.jipps.dao.PropertyJippsDAO;

/**
 * A utility for getting configuration paths, either from the DB or from a system property.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author lewis_lee
 * @version $Id: PathConfiguration.java,v 1.13 2007/12/19 17:55:34 lewis_lee Exp $
 */
public class PathConfiguration
{
  /**
   * The prefix for creating an URL from a file
   */
  public static final String URL_FILE_PREFIX = "file:///";

  /**
   * For local tests, XSL files are searched in this directory.
   */
  private static final String TEST_PATH_XSL = "C:\\jp\\projects\\jipps\\xsl";

  /**
   * For local tests, images are saved in this directory.
   */
  private static final String TEST_PATH_IMAGE = "C:\\temp\\";

  /**
   * The subpath for the st36 xml
   */
  private static final String TEST_ST36_FOP_BASEDIR_VALUE = TEST_PATH_IMAGE;

  /**
   * The base directory for the the ST36 files
   */
  private static final String TEST_ST36_BASE_DIRECTORY = TEST_PATH_XSL + "\\st36topdf\\ch-patent-doc-v_0.7.6.1\\";

  /**
   * The path to the fonts used in st36 xml's
   */
  private static final String TEST_ST36_FOP_FONT_BASEDIR_VALUE = TEST_PATH_XSL + "\\st36topdf\\ch-patent-doc-v_0.7.6.1\\font\\";

  /**
   * The path to the fonts used in st36 xml's
   */
  private static final String TEST_ST36_FOP_HYPH_BASEDIR_VALUE = TEST_PATH_XSL + "\\st36topdf\\ch-patent-doc-v_0.7.6.1\\hyph\\";

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PathConfiguration.class );

  /**
   * The jipps properties when running in J2EE server
   */
  private static JIPPSProperties jippsProperties;

  /**
   * Lookup for jipps properties only has to be done once
   */
  static
  {
    if ( AbstractDAO.staticIsRunningInJ2EEServer() )
    {
      try
      {
        jippsProperties = ( (JIPPSPropertiesHome) new InitialContext().lookup( JIPPSPropertiesHome.JNDI_NAME ) ).create();
      }
      catch ( Exception e )
      {
        throw new IllegalStateException( "Lookup for jipps properties failed", e );
      }
    }
  }

  /**
   * Gets the path (the directory) to files, looking in the System properties first, then in the JIPPS property DB
   * table.
   * @param propertiesKey The Properties key where the path is found
   * @return The configured path to the files
   * @throws IllegalArgumentException if <code>propertiesKey</code> is empty
   * @throws IllegalStateException if the configuration can't be found
   */
  public static String getPath( String propertiesKey ) throws IllegalArgumentException, IllegalStateException
  {
    if ( StringUtils.isEmpty( propertiesKey ) )
    {
      throw new IllegalArgumentException( "Properties key can't be empty" );
    }
    String path = getProperty( propertiesKey );
    // Just to be sure it ends with a file separator.
    // Note that the File.separator might be used, but then getting a UNIX
    // path wouldn't work on Windows
    if ( path != null && !path.endsWith( "\\" ) && !path.endsWith( "/" ) )
    {
      path += File.separator;
    }
    return path;
  }

  /**
   * Gets the value for a given property key.
   * @param propertiesKey The property key
   * @return The property value
   */
  public static String getProperty( String propertiesKey )
  {
    String property = StringUtils.EMPTY;
    try
    {
      if ( jippsProperties == null )
      {
        property = new PropertyJippsDAO().select( propertiesKey );
      }
      else
      {
        property = jippsProperties.getProperty( propertiesKey );
      }
    }
    catch ( SQLException e )
    {
      throw new IllegalStateException( "Can't find path property '" + propertiesKey + "'", e );
    }
    catch ( RemoteException e )
    {
      throw new IllegalStateException( "Can't find path property '" + propertiesKey + "'", e );
    }
    return property;
  }

  /**
   * Gets the path (the directory) to files, looking in the System properties first, then in the JIPPS property DB table
   * and makes sure it exists and is at least readable (and maybe also writeable, depending on
   * <code>checkForWriting</code>).
   * @param propertiesKey The Properties key where the path is found
   * @param checkForWriting If this is <code>true</code>, the directory must be writable, otherwise just readable
   * @return The configured and verified path to the files
   * @throws IllegalArgumentException if <code>propertiesKey</code> is empty
   * @throws IllegalStateException if the path can't be found
   */
  public static String getVerifiedPath( String propertiesKey, boolean checkForWriting ) throws IllegalArgumentException, IllegalStateException
  {
    String path = getPath( propertiesKey );
    checkPath( path, checkForWriting );
    return path;
  }

  /**
   * Gets the path (the directory) to files as URL, looking in the System properties first, then in the JIPPS property
   * DB table and makes sure it exists and is at least readable (and maybe also writeable, depending on
   * <code>checkForWriting</code>).
   * @param propertiesKey The Properties key where the path is found
   * @param checkForWriting If this is <code>true</code>, the directory must be writable, otherwise just readable
   * @return The configured and verified path to the files URL
   * @throws IllegalArgumentException if <code>propertiesKey</code> is empty
   * @throws IllegalStateException if the path can't be found
   */
  public static String getVerifiedPathAsUrl( String propertiesKey, boolean checkForWriting ) throws IllegalArgumentException, IllegalStateException
  {
    String path = getVerifiedPath( propertiesKey, checkForWriting );
    // Make it a real URI
    path = "file:///" + path;
    path = path.replace( '\\', '/' );
    return path;
  }

  /**
   * Checks that the path exists and is a non-empty readable (and maybe writable) directory.
   * @param path The path to check
   * @param checkForWriting If this is <code>true</code>, the directory must be writable, otherwise just readable
   * @throws IllegalArgumentException if the path is in any way wrong
   */
  private static void checkPath( String path, boolean checkForWriting ) throws IllegalArgumentException
  {
    if ( path == null )
    {
      throw new IllegalArgumentException( "Path is undefined" );
    }
    File pathFile = new File( path );
    if ( !pathFile.exists() )
    {
      throw new IllegalArgumentException( "Path '" + path + "' doesn't exist" );
    }
    if ( !pathFile.isDirectory() )
    {
      throw new IllegalArgumentException( "Path '" + path + "' is not a directory" );
    }
    if ( !pathFile.canRead() )
    {
      throw new IllegalArgumentException( "Path '" + path + "' is not readable" );
    }
    if ( checkForWriting && !pathFile.canWrite() )
    {
      throw new IllegalArgumentException( "Path '" + path + "' is not writable" );
    }
  }

  /**
   * <strong>This must only be used for local unit testing!</strong> This sets the local XSL and image directories, in
   * order to let JIPPS read XSLs from this project and write images into the local temporary directory.
   */
  public static void setPathsForLocalTest()
  {
    // TODO Write a function in util for checking if something is running under Windows or Linux (and locally for tests)
    // The local configuration is only done if this runs (locally) on
    // Windows. Otherwise, the configuration is expected to come from the DB.
    if ( System.getProperty( PropertyConstants.OS_NAME ).startsWith( "Windows" ) )
    {
      LOG.debug( "Setting XSL path to " + TEST_PATH_XSL );
      System.setProperty( PropertyConstants.JIPPS_PATH_XSL, TEST_PATH_XSL );
      LOG.debug( "Setting image path to " + TEST_PATH_IMAGE );
      System.setProperty( PropertyConstants.JIPPS_PATH_IMAGE, TEST_PATH_IMAGE );

      // fop base directory for Patdoc Images
      String testFOPBasedir = TEST_ST36_FOP_BASEDIR_VALUE;
      LOG.debug( "Setting fop base directory (images) to " + testFOPBasedir );
      System.setProperty( "st36_basedir", testFOPBasedir );

      // st36 base directory
      String testSt36Basedir = TEST_ST36_BASE_DIRECTORY;
      LOG.debug( "Setting st36 base directory (xsl) to " + testSt36Basedir );
      System.setProperty( St36Constants.getSt36SchemaDirectory( 4 ), testSt36Basedir );

      String testFontdir = PathConfiguration.URL_FILE_PREFIX + TEST_ST36_FOP_FONT_BASEDIR_VALUE.replace( '\\', '/' );
      LOG.debug( "Setting st36 font path to " + testFontdir );
      System.setProperty( PathConfiguration.URL_FILE_PREFIX + St36Constants.getSt36FontDirectory( 4 ), testFontdir );

      String testHyphdir = PathConfiguration.URL_FILE_PREFIX + TEST_ST36_FOP_HYPH_BASEDIR_VALUE.replace( '\\', '/' );
      LOG.debug( "Setting st36 hyph path to " + testHyphdir );
      System.setProperty( PathConfiguration.URL_FILE_PREFIX + St36Constants.getHyphUrl( 4 ), testHyphdir );

    }
  }
}
