package ch.ige.jipps.constants;

import java.io.File;
import java.sql.SQLException;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.ipobjects.patent.dao.PatDocSchemaDAO;
import ch.ige.ipobjects.patent.vo.PatentDocumentSchemaVO;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.util.text.Replacer;

/**
 * Contains the constants used for st36 / fop
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Esther Bandschapp
 * @version  $Id: St36Constants.java,v 1.10 2007/12/05 14:17:50 bandschapp Exp $
 */
public class St36Constants
{
  
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( St36Constants.class );
  
  /**
   * The name of the attribute baseDir which is used as a key in st36 / fop,
   * used to store temporary images
   */
  private static final String ST36_FOP_BASEDIR = "st36_basedir";

  /**
   * The name of the attribute for the main xsl file
   * Part of the main xsl file
   */
  private static final String ST36_XSL_NAME = "st36_xsl_name";

  /**
   * The name of the attribute for the directory name of the fonts
   */
  private static final String ST36_XSL_FONT_DIR = "st36_font_subdir";

  /**
   * The name of the attribute which contains the subpath of the fop xmls
   * (after the official xsl path - and before where the data lies) 
   */
  private static final String ST36_XSL_SUBDIR = "st36_subdir";

  /**
   * The name of the attribute for the configfile used for st36
   */
  private static final String ST36_CONFIG_FILE_NAME = "st36_user_config";

  /**
   * The name of the attribute for the subdirectory for hyphenation
   */
  private static final String ST36_HYPH_SUBDIR = "st36_hyph_subdir";

  /**
   * The existing patdoc schemas
   */
  private static final List<PatentDocumentSchemaVO> PATDOC_SCHEMAS;

  static
  {
    try
    {
      PATDOC_SCHEMAS = new PatDocSchemaDAO().selectAll();
    }
    catch ( SQLException e )
    {
      throw new IllegalStateException( "The patdoc schemas cannot be fetched", e );
    }
  }

  /**
   * The default constructor
   */
  public St36Constants()
  {
    super();
  }

  /**
   * Returns the full path for the fop st36 working directory, used to store images.
   * @return The working directory
   */
  public static String getSt36BaseDirectory()
  {
    String directory = PathConfiguration.getPath( ST36_FOP_BASEDIR );
    return replaceSlashes( new StringBuilder( directory ) );
  }

  /**
   * Returns the full path of the directory where the different schema xmls can be found.
   * @return The st36 directory
   */
  public static String getSt36Directory()
  {
    StringBuilder fopXslBasePath = new StringBuilder();
    fopXslBasePath.append( PathConfiguration.getPath( PropertyConstants.JIPPS_PATH_XSL ) );
    fopXslBasePath.append( PathConfiguration.getPath( ST36_XSL_SUBDIR ) );
    return replaceSlashes( fopXslBasePath );
  }

  /**
   * Returns the full path for the hyphenation subdirectory used in fop/st36.
   * @param schemaId The schema ID
   * @return The path to the hyph directory
   */
  public static String getHyphUrl( int schemaId )
  {
    StringBuilder fopHyphUrl = new StringBuilder();
    fopHyphUrl.append( getSt36SchemaDirectory(schemaId) );
    fopHyphUrl.append( PathConfiguration.getPath( ST36_HYPH_SUBDIR ) );
    String returnValue = replaceSlashes( fopHyphUrl );
    LOG.debug("Returning Hyph URL: "+ returnValue );
    return returnValue;
  }

  /**
   * Gets the full path of the user config file.
   * @param schemaId The schema ID
   * @return The full path of the user config file
   */
  public static String getConfigFileUrl( int schemaId )
  {
    StringBuilder fopConfigPath = new StringBuilder();
    fopConfigPath.append( getSt36SchemaDirectory(schemaId) );
    fopConfigPath.append( getSt36ConfigFileName() );
    String returnValue = replaceSlashes( fopConfigPath );
    LOG.debug("Returning config file URL: "+ returnValue );
    return returnValue;
  }

  /**
   * Gets the full path of the font directory of a particular schema.
   * @param schemaId The schema ID
   * @return The path of the font directory
   */
  public static String getSt36FontDirectory( int schemaId )
  {
    StringBuilder directory = new StringBuilder();
    directory.append( getSt36Directory() );
    directory.append( getSt36SchemaDirectoryName( schemaId ) );
    directory.append( File.separator );
    directory.append( getSt36FontDirectoryName() );
    directory.append( File.separator );
    String returnValue = replaceSlashes( directory );
    LOG.debug("Returning font directory: "+ returnValue );
    return returnValue;
  }

  /**
   * Gets the subdirectory name for the fop st36 font directory.
   * @return The subdirectory name for the fop st36 font directory
   */
  private static String getSt36FontDirectoryName()
  {
    String returnValue = PathConfiguration.getProperty( St36Constants.ST36_XSL_FONT_DIR );
    LOG.debug("Returning font directory name: "+ returnValue );
    return returnValue;
  }

  /**
   * Gets the full path of the directory for a particular schema.
   * @param schemaId The schema ID
   * @return The directory name of that particular schema
   * TODO: Implement getSt36SchemaSubDirectory()
   */
  public static String getSt36SchemaDirectory( int schemaId )
  {
    StringBuilder directory = new StringBuilder();
    directory.append( getSt36Directory() );
    directory.append( getSt36SchemaDirectoryName( schemaId ) );
    directory.append( File.separator );
    String returnValue = replaceSlashes( directory );
    LOG.debug("Returning schema directory: "+ returnValue );
    return returnValue;
  }

  /**
   * Checks whether the ID is valid and returns the correct name of the schema directory.
   * @param schemaId The schema ID
   * @return The correct name of the schema directory
   */
  private static String getSt36SchemaDirectoryName( int schemaId )
  {
    for ( PatentDocumentSchemaVO patentDocumentSchemaVO: PATDOC_SCHEMAS )
    {
      if ( patentDocumentSchemaVO.getPatdocschema_id().equals( schemaId ) )
      {
        String returnValue = Replacer.replaceAll( patentDocumentSchemaVO.getId_string(), FileConstants.EXTENSION_XSD, StringUtils.EMPTY );
        LOG.debug("Returning schema directory name: "+ returnValue );
        return returnValue;
      }
    }
    //Should not get here:
    throw new IllegalStateException( "The provided schema ID does not exist: " + schemaId );
  }

  /**
   * Gets the name of the main xml called in st36, either with suffix or not
   * @param withSuffix <code>true</code> if the file suffix is required, otherwise <code>false</code>.
   * @return The file name
   */
  public static String getSt36XslName( boolean withSuffix )
  {
    StringBuilder fopMainXslPath = new StringBuilder();
    fopMainXslPath.append( PathConfiguration.getPath( St36Constants.ST36_XSL_NAME ) );
    fopMainXslPath.deleteCharAt( fopMainXslPath.length() - 1 ); //remove last slash
    if ( withSuffix )
    {
      fopMainXslPath.append( FileConstants.EXTENSION_XSL );
    }
    String returnValue = fopMainXslPath.toString();
    LOG.debug("Returning xsl name: "+ returnValue );
    return returnValue;
  }

  /**
   * Gets the configuration file name, the file is used in st36.
   * @return The st36 configuration file name
   */
  private static String getSt36ConfigFileName()
  {
    StringBuilder returnValue = new StringBuilder( PathConfiguration.getPath( ST36_CONFIG_FILE_NAME ));
    returnValue.deleteCharAt( returnValue.length() - 1 ); //remove last slash
    LOG.debug("Returning config file name: "+ returnValue );
    return returnValue.toString();
  }

  /**
   * Replaces the slashes.
   * @param path The path where the slashes have to be replaces
   * @return The path with replaced slashes
   */
  private static String replaceSlashes( StringBuilder path )
  {
    return path.toString().replace( '\\', '/' );
  }

}
