package ch.ige.jipps.currentstate;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.vo.FeeVO;
import ch.ige.ipobjects.common.vo.PrioritaetVO;
import ch.ige.ipobjects.country.dao.CountryDAO;
import ch.ige.ipobjects.country.vo.CountryVO;
import ch.ige.ipobjects.event.vo.ArTextVO;
import ch.ige.ipobjects.exceptions.IPObjectsException;
import ch.ige.ipobjects.services.Services;
import ch.ige.ipobjects.time.PointInTimeCollection;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.visitors.PeriodPointInTimeFilter;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.util.constants.ReportTextNumberHandler;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.event.EventTypeTextType;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.format.FeeFormatter;
import ch.ige.util.format.PriorityFormatter;

/**
 * The base class of every <code>CurrentStateBuilder</code> implementation. This implements every method by returning
 * an empty <code>List</code>.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: BaseBuilder.java,v 1.30 2007/12/19 17:55:06 lewis_lee Exp $
 */
abstract class BaseBuilder implements CurrentStateBuilder
{
  /**
   * Adds a single text to the current state elements.
   * @param currentStateElements The current state elements to modify
   * @param newStateElement A new element to add
   * @param text The text to put into <code>newStateElement</code>
   * @return The modified <code>currentStateElements</code>
   */
  protected List<CurrentStateElement> addSingleText( List<CurrentStateElement> currentStateElements, CurrentStateElement newStateElement, String text )
  {
    if ( StringUtils.isEmpty( text ) )
    {
      return currentStateElements;
    }
    else
    {
      ArrayList<String> texts = new ArrayList<String>();
      texts.add( text );
      addMultipleTexts( currentStateElements, newStateElement, texts );
      return currentStateElements;
    }
  }

  /**
   * Adds multiple text lines to the current state elements.
   * @param currentStateElements A <code>List</code> of {@link ch.ige.jipps.currentstate.CurrentStateElement} objects
   * @param newStateElement A new element to add
   * @param texts The texts to put into <code>newStateElement</code>
   * @return The modified <code>currentStateElements</code>
   */
  protected List<CurrentStateElement> addMultipleTexts( List<CurrentStateElement> currentStateElements, CurrentStateElement newStateElement, List<String> texts )
  {
    if ( texts.isEmpty() )
    {
      return currentStateElements;
    }
    else
    {
      newStateElement.setTextLines( texts );
      currentStateElements.add( newStateElement );
      return currentStateElements;
    }
  }

  /**
   * Gets the latest payed fee for the IP.
   * @param ip The IP containing all data
   * @param currentDate The data is filtered for this date
   * @return The latest payed fee or <code>null</code> if there's none
   */
  protected FeeVO getLatestPayedFee( GenericIP ip, EreigIDat currentDate )
  {
    PointInTimeCollection<FeeVO> allFees = PeriodPointInTimeFilter.<FeeVO>olderThan( currentDate ).filter( ip.getPayedFees() );
    if ( allFees.isEmpty() )
    {
      return null;
    }
    else
    {
      return allFees.get( allFees.size() - 1 );
    }
  }

  /**
   * Gets the text for the latest payed fee.
   * @param fee The latest payed fee
   * @param language The language to use
   * @return The text for the latest payed fee or <code>null</code> if there's none
   * @throws SQLException if the fee's text can't be fetched
   */
  protected String getLatestPayedFeeText( FeeVO fee, Locale language ) throws SQLException
  {
    try
    {
      if ( fee == null )
      {
        return null;
      }
      else
      {
        final Services EventServices = ServicesFactory.getInstance();
        ArTextVO eventText = EventServices.getEventText( fee.getFeeEvent(), EventTypeTextType.REGISTER_REPORT, language );

        if ( eventText == null )
        {
          return null;
        }
        else
        {
          return FeeFormatter.formatFee( eventText.getText(), fee.getPayedUntilDate(), language );
        }
      }
    }
    catch ( IPObjectsException e )
    {
      SQLException exception = new SQLException( "Couldn't get text for last payed fee" );
      exception.initCause( e );
      throw exception;
    }
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getDescription( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getRegistration( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public final List<CurrentStateElement> getCancellation( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    List<CurrentStateElement> cancellations = new ArrayList<CurrentStateElement>();

    if ( !Types.PATENT_PCT.isInstance( ip ) && ip.getCancellationDate() != null )
    {
      CurrentStateElement cancellation = new StandDatVO();
      cancellation.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForCancellation( ip.getBAGISType() ) );
      addSingleText( cancellations, cancellation, DateFormatter.shortFormat( ip.getCancellationDate(), language ) );
    }

    return cancellations;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getPublication( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  public final List<CurrentStateElement> getAddresses( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return AddressBuilder.getAddresses( ip, reportType, currentState, language );
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getLatestFee( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getPriorities( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    ArrayList<CurrentStateElement> priorities = new ArrayList<CurrentStateElement>();

    TimePeriodCollection<PrioritaetVO> priorityVOs = ip.getPrios().getPeriodsContainingTime( currentState );
    if ( !priorityVOs.isEmpty() )
    {
      List<String> formattedPriorities = new ArrayList<String>();
      for ( PrioritaetVO priorityVO: priorityVOs )
      {
        String countryName = null;
        if ( StringUtils.isNotEmpty( priorityVO.getBland_cd() ) )
        {
          CountryVO country = new CountryDAO().findCountry( priorityVO.getBland_cd() );
          // Like with addresses, the country name is taken from the IP's language,
          // not the report language
          countryName = country.getLand_bez( ip.getLocale() );
        }

        String remark = null;
        if ( Types.PATENT_GENERIC.isInstance( ip ) )
        {
          formattedPriorities.add( PriorityFormatter.formatPatentPriority( priorityVO.getBland_cd(), priorityVO.getPrio_nr(), priorityVO.getPrio_dat() ) );
        }
        else if ( Types.SPC.isInstance( ip ) )
        {
          formattedPriorities.add( PriorityFormatter.formatSpcRegistration( priorityVO.getPrio_bem(), priorityVO.getPrio_nr(), priorityVO.getPrio_dat() ) );
        }
        else if ( Types.TRADEMARK.isInstance( ip ) )
        {
          // TODO Check if the priority number is never required
          formattedPriorities.add( PriorityFormatter.formatTrademarkPriority( priorityVO.getBland_cd(), countryName, null, priorityVO.getPrio_dat() ) );
          Trademark trademark = (Trademark) ip;
          remark = trademark.getPriorityRemark().getItemContaining( currentState ).getValue();
        }
        else if ( Types.DESIGN.isInstance( ip ) )
        {
          formattedPriorities.add( PriorityFormatter.formatDesignPriority( priorityVO.getBland_cd(), priorityVO.getPrio_nr(), priorityVO.getPrio_dat() ) );
        }

        if ( StringUtils.isNotEmpty( remark ) )
        {
          String[] remarks = StringUtils.split( remark, '\n' );
          for ( int i = 0; i < remarks.length; i++ )
          {
            formattedPriorities.add( remarks[i] );
          }
        }
      }

      CurrentStateElement priority = new StandDatVO();
      priority.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForPriority( ip.getBAGISType() ) );
      addMultipleTexts( priorities, priority, formattedPriorities );
    }

    return priorities;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getClassification( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getPct( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getProtectionStart( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getProtectionEnd( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getRenewal( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getTrademarkKind( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getItems( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getDeposit( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getType( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getExhibitionImmunity( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getUnsealing( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getRemark( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getColorClaim( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    return null;
  }
}
