package ch.ige.jipps.currentstate;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.dao.XxDomainDAO;
import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.common.vo.FeeVO;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.design.DesignPublicationDeferment;
import ch.ige.ipobjects.design.model.DesignItemsHistory.DesignItemsItem;
import ch.ige.ipobjects.design.vo.ModKlasseVO;
import ch.ige.ipobjects.design.vo.ModOrdnrVO;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.util.constants.PublicationDelayConstants;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.constants.ReportTextNumberHandler;
import ch.ige.util.constants.XxDomainConstants;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.format.LocarnoClassFormatter;

/**
 * Builds the objects for the current state in the design report.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignBuilder.java,v 1.28 2007/12/17 17:21:13 lewis_lee Exp $
 */
class DesignBuilder extends BaseBuilder
{
  // See interface
  @Override
  public List<CurrentStateElement> getLatestFee( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    Design design = (Design) ip;
    List<CurrentStateElement> fees = new ArrayList<CurrentStateElement>();

    // The fee isn't shown for a cancelled IP
    if ( !design.isCancelled() )
    {
      FeeVO fee = getLatestPayedFee( ip, currentState );
      if ( fee != null )
      {
        CurrentStateElement latestFee = new StandDatVO();
        latestFee.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForFee( design.getBAGISType() ) );
        String feeText = getLatestPayedFeeText( fee, language );
        addSingleText( fees, latestFee, feeText );
      }
    }

    return fees;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getDescription( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Design design = (Design) ip;
    List<CurrentStateElement> descriptions = new ArrayList<CurrentStateElement>();

    // The title
    CurrentStateElement title = new StandDatVO();
    title.setReportTextNumber( ReportTextConstants.DESIGN_DESCRIPTION_INTEGER );
    HistoryItem<String> titleHistory = design.getTitle().getItemContaining( currentState );
    if ( titleHistory != null )
    {
      String titleText = titleHistory.getValue();
      addSingleText( descriptions, title, titleText );
    }

    // The description
    HistoryItem<String> descriptionHistory = design.getDesignDescription().getItemContaining( currentState );
    if ( descriptionHistory != null )
    {
      String descriptionText = descriptionHistory.getValue();
      if ( StringUtils.isNotEmpty( descriptionText ) )
      {
        CurrentStateElement description = new StandDatVO();
        description.setReportTextNumber( ReportTextConstants.DESIGN_LONG_DESCRIPTION_INTEGER );
        addMultipleTexts( descriptions, description, ConvertText.splitLines( descriptionText ) );
      }
    }

    return descriptions;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getPublication( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> publications = new ArrayList<CurrentStateElement>();

    HistoryItem<DesignPublicationDeferment> delayHistory = design.getDefermentState().getItemContaining( currentState );
    if ( delayHistory != null )
    {
      // TODO use DesignPublicationDeferment here
      Integer delayState = delayHistory.getValue().toDB();
      if ( delayState != null && !delayState.equals( PublicationDelayConstants.PUBLICATION_WITHOUT_DELAY_INTEGER ) )
      {
        CurrentStateElement delayedPublication = new StandDatVO();
        delayedPublication.setReportTextNumber( ReportTextConstants.DESIGN_PUBLICATION_TYPE_INTEGER );
        String delayStateText = new XxDomainDAO().findNormalDomainValue( XxDomainConstants.DESIGN_PUBLICATION_TYPE, delayState, language );
        addSingleText( publications, delayedPublication, delayStateText );
      }
    }

    return publications;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getClassification( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> classes = new ArrayList<CurrentStateElement>();

    CurrentStateElement classList = new StandDatVO();
    classList.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForClassification( design.getBAGISType() ) );
    TimePeriodCollection<ModKlasseVO> theClasses = design.getClassifications().getPeriodsContainingTime( currentState );
    List<String> stringClasses = new ArrayList<String>();
    for ( ModKlasseVO theClass: theClasses )
    {
      stringClasses.add( theClass.getModkl_nr() );
    }
    addSingleText( classes, classList, LocarnoClassFormatter.format( stringClasses, true ) );

    return classes;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getItems( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> items = new ArrayList<CurrentStateElement>();

    CurrentStateElement numberOfItems = new StandDatVO();
    numberOfItems.setReportTextNumber( ReportTextConstants.DESIGN_NUMBER_OF_ITEMS_INTEGER );
    HistoryItem<Integer> itemNumberHistory = design.getNumberOfItems().getItemContaining( currentState );
    if ( itemNumberHistory != null )
    {
      Integer itemNumber = itemNumberHistory.getValue();
      addSingleText( items, numberOfItems, itemNumber.toString() );
    }

    CurrentStateElement item = new StandDatVO();
    item.setReportTextNumber( ReportTextConstants.DESIGN_ITEM_TYPE_INTEGER );
    String itemType = new XxDomainDAO().findNormalDomainValue( XxDomainConstants.DESIGN_ITEM_TYPE, design.getDesignItemRepresentation().toDB(), language );
    addSingleText( items, item, itemType );

    // Multiple order numbers without text are written as a list, separated by ';'.
    // But if they contain text, every order number and every text line is written on its own line.
    CurrentStateElement orderNumber = new StandDatVO();
    orderNumber.setReportTextNumber( ReportTextConstants.IMAGE_ORDER_NUMBER_INTEGER );
    DesignItemsItem designItemsItem = (DesignItemsItem) design.getGroupedDesignItems().getItemContaining( currentState );
    if ( designItemsItem != null )
    {
      TimePeriodCollection<ModOrdnrVO> orderNumbers = designItemsItem.getItems();
      List<String> orderNumberLines = new ArrayList<String>();
      List<String> orderNumberCodes = new ArrayList<String>();
      boolean containsText = false;
      for ( ModOrdnrVO orderNumberVO: orderNumbers )
      {
        orderNumberLines.add( orderNumberVO.getOrdn_nr() );
        orderNumberCodes.add( orderNumberVO.getOrdn_nr() );
        if ( StringUtils.isNotEmpty( orderNumberVO.getBeschreib_100() ) )
        {
          containsText = true;
          orderNumberLines.addAll( ConvertText.splitLines( orderNumberVO.getBeschreib_100() ) );
        }
      }
      if ( containsText )
      {
        addMultipleTexts( items, orderNumber, orderNumberLines );
      }
      else
      {
        addSingleText( items, orderNumber, StringUtils.join( orderNumberCodes.iterator(), "; " ) );
      }
    }

    return items;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getDeposit( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> deposits = new ArrayList<CurrentStateElement>();

    CurrentStateElement deposit = new StandDatVO();
    deposit.setReportTextNumber( ReportTextConstants.DESIGN_APPLICATION_TYPE_INTEGER );
    String depositType = new XxDomainDAO().findNormalDomainValue( XxDomainConstants.DESIGN_DEPOSIT_TYPE, design.getDepositionMode().toDB(), language );
    addSingleText( deposits, deposit, depositType );

    return deposits;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getType( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> types = new ArrayList<CurrentStateElement>();

    CurrentStateElement type = new StandDatVO();
    type.setReportTextNumber( ReportTextConstants.DESIGN_TYPE_INTEGER );
    String depositType = new XxDomainDAO().findNormalDomainValue( XxDomainConstants.DESIGN_TYPE, design.getDesignForm().toDB(), language );
    addSingleText( types, type, depositType );

    return types;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getExhibitionImmunity( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> exhibitionImmunities = new ArrayList<CurrentStateElement>();

    CurrentStateElement type = new StandDatVO();
    type.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForExhibitionImmunity( design.getBAGISType() ) );
    addSingleText( exhibitionImmunities, type, design.getExhibitionInformation() );

    return exhibitionImmunities;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getUnsealing( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Design design = (Design) ip;
    ArrayList<CurrentStateElement> unsealings = new ArrayList<CurrentStateElement>();

    CurrentStateElement unsealing = new StandDatVO();
    unsealing.setReportTextNumber( ReportTextConstants.DESIGN_UNSEALING_DATE_INTEGER );
    addSingleText( unsealings, unsealing, DateFormatter.shortFormat( design.getUnsealingDate(), language ) );

    return unsealings;
  }
}
