package ch.ige.jipps.currentstate;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.dao.XxDomainDAO;
import ch.ige.ipobjects.common.vo.XxDomainVO;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.ipobjects.trademark.vo.MarMarartVO;
import ch.ige.ipobjects.trademark.vo.MarWarklVO;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.util.collection.MapConverter;
import ch.ige.util.constants.EventProcessConstants;
import ch.ige.util.constants.NumericLiterals;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.constants.ReportTextNumberHandler;
import ch.ige.util.constants.XxDomainConstants;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.format.ReportIpNumberFormatter;
import ch.ige.util.format.WdlClassFormatter;
import ch.ige.wdlutil.exception.WdlSyntaxException;
import ch.ige.wdlutil.format.WdlFormatter;

/**
 * Builds the objects for the current state in the trademark report.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: TrademarkBuilder.java,v 1.27 2008/02/21 17:07:48 lewis_lee Exp $
 */
class TrademarkBuilder extends BaseBuilder
{
  // See interface
  @Override
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getRegistration( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Trademark trademark = (Trademark) ip;
    ArrayList<CurrentStateElement> registrations = new ArrayList<CurrentStateElement>();

    for ( Event originalEvent: trademark.getEvents() )
    {
      if ( originalEvent.getErfass_proc() != null && originalEvent.getErfass_proc().equals( EventProcessConstants.REGISTRATION_INTEGER ) )
      {
        CurrentStateElement registration = new StandDatVO();
        registration.setReportTextNumber( ReportTextConstants.REGISTRATION_TRADEMARK_INTEGER );
        String registrationDate = DateFormatter.shortFormat( originalEvent.getEreig_idat().getDate(), language );
        addSingleText( registrations, registration, registrationDate );
        break;
      }
    }

    return registrations;
  }

  // See interface
  @Override
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getClassification( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Trademark trademark = (Trademark) ip;
    ArrayList<CurrentStateElement> goodsAndServices = new ArrayList<CurrentStateElement>();

    // The free-form text
    CurrentStateElement freeFormText = new StandDatVO();
    freeFormText.setReportTextNumber( ReportTextConstants.WDL_INTEGER );
    String originalWdlText = trademark.getGoodsAndServicesText().getItemContaining( currentState ).getValue();
    if ( !StringUtils.isEmpty( originalWdlText ) )
    {
      try
      {
        List<String> convertedLines = new ArrayList<String>();
        convertedLines.addAll( WdlFormatter.toHtmlListWithH1( originalWdlText, false, false ) );
        addMultipleTexts( goodsAndServices, freeFormText, convertedLines );
      }
      catch ( WdlSyntaxException e )
      {
        throw new RuntimeException( "Can't format WDL '" + originalWdlText + "'", e );
      }
    }

    // The WDL/Nice class numbers
    CurrentStateElement wdlClasses = new StandDatVO();
    wdlClasses.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForClassification( ip.getBAGISType() ) );
    List<MarWarklVO> wdlClassObjects = trademark.getGoodsAndServicesClasses().getPeriodsContainingTime( currentState );
    if ( !wdlClassObjects.isEmpty() )
    {
      // The WdlClassFormatter needs a List of Integers
      List<Integer> wdlClassNumbers = new ArrayList<Integer>();
      for ( MarWarklVO niceClass: wdlClassObjects )
      {
        wdlClassNumbers.add( niceClass.getWarkl_nr() );
      }
      addSingleText( goodsAndServices, wdlClasses, WdlClassFormatter.format( wdlClassNumbers, false ) );
    }

    return goodsAndServices;
  }

  // See interface
  @Override
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getTrademarkKind( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    Trademark trademark = (Trademark) ip;
    ArrayList<CurrentStateElement> trademarkKinds = new ArrayList<CurrentStateElement>();

    CurrentStateElement trademarkKind = new StandDatVO();
    trademarkKind.setReportTextNumber( ReportTextConstants.TRADEMARK_KIND_INTEGER );

    Map<Integer, XxDomainVO> trademarkKindMap = MapConverter.toMap( new XxDomainDAO().findDomainValue( XxDomainConstants.TRADEMKARK_KIND, language ),
        Integer.class, "domainLnr" );
    TimePeriodCollection<MarMarartVO> numericTrademarkKinds = trademark.getTrademarkKinds().getPeriodsContainingTime( currentState );

    List<String> trademarkKindLines = new ArrayList<String>();
    for ( MarMarartVO numericTrademarkKind: numericTrademarkKinds )
    {
      XxDomainVO trademarkKindName = trademarkKindMap.get( numericTrademarkKind.getTrademarkKind().toDB() );
      if ( trademarkKindName == null )
      {
        throw new IllegalArgumentException( "Found no trademark kind (domain value) for " + numericTrademarkKind );
      }
      trademarkKindLines.add( trademarkKindName.getBez() );
    }

    return addMultipleTexts( trademarkKinds, trademarkKind, trademarkKindLines );
  }

  // See interface
  @Override
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getRenewal( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Trademark trademark = (Trademark) ip;
    ArrayList<CurrentStateElement> renewals = new ArrayList<CurrentStateElement>();

    CurrentStateElement renewal = new StandDatVO();
    renewal.setReportTextNumber( ReportTextConstants.TRADEMARK_RENEWAL_INTEGER );
    StringBuilder renewalText = new StringBuilder();
    if ( StringUtils.isNotEmpty( trademark.getRenewalOldTrademark() ) )
    {
      renewalText.append( ReportIpNumberFormatter.formatTrademark( new Long( trademark.getRenewalOldTrademark() ), ip.getBAGISType(), NumericLiterals.ZERO ) );
      renewalText.append( "; " );
    }
    if ( StringUtils.isNotEmpty( trademark.getRenewalText() ) )
    {
      renewalText.append( trademark.getRenewalText() );
    }
    return addSingleText( renewals, renewal, renewalText.toString() );
  }

  // See interface
  @Override
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getRemark( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Trademark trademark = (Trademark) ip;
    String remarkText = trademark.getPublishedRemark().getItemContaining( currentState ).getValue();

    if ( StringUtils.isNotEmpty( remarkText ) )
    {
      ArrayList<CurrentStateElement> remarks = new ArrayList<CurrentStateElement>();
      CurrentStateElement remark = new StandDatVO();
      remark.setReportTextNumber( ReportTextConstants.TRADEMARK_REMARKS_INTEGER );
      // Some remark texts contain hyphenation, but without carriage returns.
      remarkText = remarkText.replaceAll( "\\- ", "" );
      String[] textLines = StringUtils.split( remarkText, '\n' );
      return addMultipleTexts( remarks, remark, Arrays.asList( textLines ) );
    }
    else
    {
      return null;
    }
  }

  // See interface
  @Override
  @SuppressWarnings( "unused" )
  public List<CurrentStateElement> getColorClaim( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    Trademark trademark = (Trademark) ip;
    String colorClaimText = trademark.getColorClaim().getItemContaining( currentState ).getValue();

    if ( StringUtils.isNotEmpty( colorClaimText ) )
    {
      ArrayList<CurrentStateElement> colorClaims = new ArrayList<CurrentStateElement>();
      CurrentStateElement colorClaim = new StandDatVO();
      colorClaim.setReportTextNumber( ReportTextConstants.TRADEMARK_COLOR_CLAIM_INTEGER );
      return addSingleText( colorClaims, colorClaim, colorClaimText );
    }
    else
    {
      return null;
    }
  }
}
