package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLFragment;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.jipps.vo.AmvVO;
import ch.ige.util.validation.Check;

/**
 * Accesses the table <code>JIPPS_AMV</code> that contains trademark renewals within batch runs.
 * Note that <code>AMV</code> means <code>Automatische Markenverlängerung</code>, it's
 * a trademark renewal.
 * 
 * <p>Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: AmvDAO.java,v 1.25 2008/05/06 07:37:43 lewis_lee Exp $
 */
public class AmvDAO extends AbstractJippsDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<AmvVO> VO_TYPE = AmvVO.class;

  /**
   * Selects the columns
   */
  private final static String SELECT_COLUMNS = "SELECT amv_lauf_id, adr_nr, bfakt_id, schutit_key, TRIM(schutit_nr) AS schutit_nr, verarb_stat, printjob_file AS print_job_file, created_dat, changed_dat ";

  /**
   * Selects the table and the ID
   */
  private final static SQLFragment FROM_TABLE_BY_ID = SQL.fragment( " FROM jipps_amv WHERE amv_lauf_id = ? " );

  /**
   * Selects the processing that's not ok.
   */
  private final static SQLFragment STATUS_OK = SQL.fragment( " AND verarb_stat = " + AmvLaufVO.PROCESSING_SUCCESSFUL );

  /**
   * Selects the processing that's not ok.
   */
  private final static SQLFragment STATUS_NOT_OK = SQL.fragment( " AND verarb_stat IN ( " + AmvLaufVO.PROCESSING_STARTED + ", "
      + AmvLaufVO.READY_FOR_PROCESSING + ", " + AmvLaufVO.FAULTY_DATA + " )" );

  /**
   * Counts the distinct IDs of trademark renewals that were not finished
   */
  private final static SQLStatement<AmvVO> SELECT_NUMBER_OF_UNFINISHED_AMV = SQL.<AmvVO>select( "SELECT DISTINCT COUNT( verarb_stat ) AS verarb_stat " )
      .append( FROM_TABLE_BY_ID ).append( STATUS_NOT_OK );

  /**
   * Selects all trademark renewals 
   */
  private final static SQLStatement<AmvVO> SELECT_AMV = SQL.<AmvVO>select( SELECT_COLUMNS ).append( FROM_TABLE_BY_ID );

  /**
   * Selects all trademark renewals invoice IDs
   */
  private final static SQLStatement<AmvVO> SELECT_INVOICE_IDS = SQL.<AmvVO>select( "SELECT DISTINCT bfakt_id, adr_nr, schutit_key " ).append( FROM_TABLE_BY_ID );

  /**
   * Selects the finished trademark renewals 
   */
  private final static SQLStatement<AmvVO> SELECT_FINISHED_AMV = SQL.<AmvVO>select( SELECT_COLUMNS ).append( FROM_TABLE_BY_ID ).append( STATUS_OK );

  /**
   * Selects the trademark renewals that were not finished 
   */
  private final static SQLStatement<AmvVO> SELECT_UNFINISHED_AMV = SQL.<AmvVO>select( SELECT_COLUMNS ).append( FROM_TABLE_BY_ID ).append( STATUS_NOT_OK );

  /**
   * Inserts one trademark renewal
   */
  private final static SQLStatement<Integer> INSERT = SQL.insert( "INSERT INTO jipps.jipps_amv "
      + "( amv_lauf_id, bfakt_id, adr_nr, schutit_key, schutit_nr, verarb_stat, printjob_file ) " + "VALUES ( ?, ?, ?, ?, ?, ?, EMPTY_BLOB() )" );

  /**
   * Updates the print job file
   */
  private final static SQLStatement<Integer> UPDATE_PRINT_JOB = SQL.update( "UPDATE jipps_amv " + "SET printjob_file = ? "
      + "WHERE amv_lauf_id = ? AND bfakt_id = ? AND adr_nr = ? AND schutit_key = ?" );

  /**
   * Updates the status to 'ready for processing'.
   */
  private final static SQLStatement<Integer> UPDATE_STATUS_TO_READY = SQL.update( "UPDATE jipps_amv SET verarb_stat = " + AmvLaufVO.READY_FOR_PROCESSING
      + ", changed_dat = sysdate " + "WHERE amv_lauf_id = ?" );

  /**
   * Updates the status.
   */
  private final static SQLStatement<Integer> UPDATE_STATUS = SQL.update( "UPDATE jipps.jipps_amv " + "SET verarb_stat = ?, changed_dat = sysdate "
      + "WHERE amv_lauf_id = ? AND adr_nr = ? AND schutit_key = ?" );

  /**
   * Checks whether the data of an AMV batch run has successfully been processed.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return <code>true</code> if the processing was successful, otherwise <code>false</code>
   * @throws SQLException if the query fails
   */
  public boolean selectCheckVerarbStat( int amvBatchRunId ) throws SQLException
  {
    AmvVO amv = selectVO( VO_TYPE, SELECT_NUMBER_OF_UNFINISHED_AMV, new Integer( amvBatchRunId ) );
    return amv.verarbStat == 0;
  }

  /**
   * Gets the trademark renewals that have been successfully processed in the given batch run.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return The successful trademark renewals of the given AMV batch run
   * @throws SQLException if the query fails
   */
  public List<AmvVO> selectVerarbStatOk( int amvBatchRunId ) throws SQLException
  {
    return selectVOList( 50, VO_TYPE, SELECT_FINISHED_AMV, new Integer( amvBatchRunId ) );
  }

  /**
   * Gets the trademark renewals that have not been successfully processed in the given
   * batch run. Normally, this is used to identify them, and then the run is restarted. 
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return The trademark renewals of the given AMV batch run that were not successful
   * @throws SQLException if the query fails
   */
  public List<AmvVO> selectVerarbStatNok( int amvBatchRunId ) throws SQLException
  {
    return selectVOList( 50, VO_TYPE, SELECT_UNFINISHED_AMV, new Integer( amvBatchRunId ) );
  }

  /**
   * Gets all trademark renewals for a batch run.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return The trademark renewals of the given AMV batch run
   * @throws SQLException if the query fails
   */
  public List<AmvVO> selectAMVToId( int amvBatchRunId ) throws SQLException
  {
    return selectVOList( 50, VO_TYPE, SELECT_AMV, new Integer( amvBatchRunId ) );
  }

  /**
   * Selects all distinct invoice IDs which need to be updated for the trademark renewal batch run.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return The trademark renewals containing the distinct invoice IDs
   * @throws SQLException if the query fails
   */
  public List<AmvVO> selectBfakturList( int amvBatchRunId ) throws SQLException
  {
    return selectVOList( 50, VO_TYPE, SELECT_INVOICE_IDS, new Integer( amvBatchRunId ) );
  }

  /**
   * Inserts a trademark renewal (AMV).
   * @param amv The trademark renewal to insert
   */
  @SuppressWarnings( "unchecked" )
  public void insert( final AmvVO amv )
  {
    Check.notNull( amv, "AMV" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_insert";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( INSERT, amv.amvLaufId, amv.bfaktId, amv.adrNr, amv.schutitKey, amv.schutitNr, amv.verarbStat );
        verifyUpdateCount( count, 1, "Update count must be 1" );
      }
    } );
  }

  /**
   * Updates the trademark renewal (AMV) with the PDF file.
   * @param amv The trademark renewal whose PDF file is updated
   */
  @SuppressWarnings( "unchecked" )
  public void updateReportPdf( final AmvVO amv )
  {
    Check.notNull( amv, "AMV" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_insert_pdf";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( UPDATE_PRINT_JOB, amv.printJobFile, amv.amvLaufId, amv.bfaktId, amv.adrNr, amv.schutitKey );
        verifyUpdateCount( count, 1, "Update count must be 1" );
      }
    } );
  }

  /**
   * Updates the status of the trademark renewal (AMV).
   * @param amv The trademark renewal whose status is updated
   */
  @SuppressWarnings( "unchecked" )
  public void update( final AmvVO amv )
  {
    Check.notNull( amv, "trademark renewal (AMV)" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_update_status";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( UPDATE_STATUS, amv.verarbStat, amv.amvLaufId, amv.adrNr, amv.schutitKey );
        verifyUpdateCount( count, 1, "Update count must be 1" );
      }
    } );
  }

  /**
   * Sets the status of all trademark renewals within the given batch to
   * {@link ch.ige.jipps.vo.AmvLaufVO#READY_FOR_PROCESSING}.
   * @param amvBatchRunId The unique ID of the AMV batch run
   */
  @SuppressWarnings( "unchecked" )
  public void update( final int amvBatchRunId )
  {
    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_update_stati";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( UPDATE_STATUS_TO_READY, amvBatchRunId );
        verifyMinimalUpdateCount( count, 1, "Update count must be at least 1" );
      }
    } );
  }
}
