package ch.ige.jipps.dao;

import java.sql.SQLException;

import org.apache.commons.lang.StringUtils;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.JippsFakturVO;
import ch.ige.util.enums.IPType;
import ch.ige.util.format.ReportApplicationNumberFormatter;

/**
 * Acesses both <code>BFAKTUR</code> and <code>SCHUTIT</code> for quicker access by joining them.
 * 
 * <p>Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Johannes Lang
 * @author Eric Lewis
 * @version $Id: JoinSchreibenDAO.java,v 1.28 2008/05/19 11:52:59 lewis_lee Exp $
 */
public class JoinSchreibenDAO extends AbstractBagisDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<JippsFakturWithBaseIpVO> VO_TYPE = JippsFakturWithBaseIpVO.class;

  /**
   * Selects the letters
   */
  private static final SQLStatement<JippsFakturWithBaseIpVO> SELECT = SQL
      .select( "SELECT b.bfakt_id, b.bfakt_dat, TRIM(b.formart_cd) AS formart_cd, TRIM(b.sprach_cd) AS sprach_cd, b.adr_nr, "
          + "b.schutit_key, TRIM(b.schutit_nr) AS schutit_nr, b.schutit_typ, "
          + "b.inhaber, TRIM(b.unt_name) AS unt_name, TRIM(b.ihr_zeichen) AS ihr_zeichen, "
          + "TRIM(b.unsere_ref) AS unsere_ref, TRIM(b.tel_nr) AS tel_nr, "
          + "TRIM(b.kz_einschr) AS kz_einschr, TRIM(b.kz_korrektur) AS kz_korrektur, "
          + "DECODE(b.txt_beilage, null, 0, b.txt_beilage) AS txt_nr_beilage, DECODE(b.var_text, null, '', b.var_text) AS var_text, b.verfall_dat, b.fakt_betrag, "
          + "DECODE(b.txt_schluss, null, 0, b.txt_schluss) AS txt_schluss, " + "DECODE(b.txt_adr_zusatz, null, 0, b.txt_adr_zusatz) AS txt_adr_zusatz, "
          + "s.schutz_beginn AS schutzbeginn, s.schutz_ende AS schutzende, DECODE(s.beschreibung, null, '', s.beschreibung) AS beschreibung, "
          + "DECODE(s.gesuch_nr, null, '', s.gesuch_nr) AS gesuch_nr, "
          + "s2.schutit_typ AS base_ip_type "
          + "FROM bfaktur b, schutit s "
          + "LEFT OUTER JOIN schutit_schutit ss ON s.schutit_key = ss.schutit_key_verkn "
          + "LEFT OUTER JOIN schutit s2 ON ss.schutit_key = s2.schutit_key "
          + "WHERE b.bfakt_id = ? "
          + "AND b.schutit_key = s.schutit_key " );

  /**
   * Selects the data for an invoice, including some IP data.
   * @param invoiceId The invoice's identifier
   * @return The invoice including some IP data
   * @throws SQLException if the query fails
   */
  @SuppressWarnings("static-access")
  public JippsFakturVO select( int invoiceId ) throws SQLException
  {
    JippsFakturWithBaseIpVO invoice = selectVO( VO_TYPE, SELECT, Integer.valueOf( invoiceId ) );
    if ( invoice != null && StringUtils.isNotEmpty( invoice.gesuchNr ) )
    {
      IPType ipType = IPType.parse( invoice.schutitTyp );
      // Some application numbers come preformatted with a slash
      String applicationNumber = invoice.gesuchNr.replaceAll( "/", "" );
      switch( ipType )
      {
        case SPC:
          invoice.gesuchNr = ReportApplicationNumberFormatter.formatSpc( applicationNumber, ipType, invoice.baseIpType );
          break;
        case PATENT_PCT:
          // The first two characters contain the ISO country code, the rest is a number
          String countryCode = applicationNumber.substring( 0, 2 );
          invoice.gesuchNr = ReportApplicationNumberFormatter.formatPct( applicationNumber.substring( 2 ), ipType, countryCode );
          break;
        default:
          invoice.gesuchNr = ReportApplicationNumberFormatter.format( applicationNumber, ipType );
          break;
      }
    }
    return invoice;
  }
  
  /**
   * The VO is only exentede to get the base IP type in case of an SPC.
   */
  public static class JippsFakturWithBaseIpVO extends JippsFakturVO
  {
    /**
     * The SPC's base IP type
     */
    public IPType baseIpType;
  }
}
