package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.LoeschungErrorVO;
import ch.ige.util.validation.Check;

/**
 * Accesses the table <code>LOESCHUNG_ERROR</code>.
 * <p>Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: LoeschungErrorDAO.java,v 1.25 2008/05/22 09:23:16 lewis_lee Exp $
 */
public class LoeschungErrorDAO extends AbstractJippsDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<LoeschungErrorVO> VO_TYPE = LoeschungErrorVO.class;

  /**
   * The name of the cancellation error ID sequence
   */
  private static final String CANCELLATION_ERROR_ID_SEQUENCE = "seq_loeschung_error";

  /**
   * Selects the cancellation errors
   */
  private static final SQLStatement<LoeschungErrorVO> SELECT_ALL = SQL
      .select( "SELECT loeschung_lauf_id, bfakt_id, errormessage AS error_message, key_id FROM loeschung_error" );

  /**
   * Selects one cancellation error
   */
  private static final SQLStatement<LoeschungErrorVO> SELECT_ONE = SELECT_ALL.append( " WHERE key_id = ?" );

  /**
   * Inserts a cancellation error
   */
  private static final SQLStatement<Integer> INSERT = SQL.insert( "INSERT INTO loeschung_error " + "(key_id, loeschung_lauf_id, bfakt_id, errormessage) "
      + "VALUES (" + CANCELLATION_ERROR_ID_SEQUENCE + ".nextval,?,?,?)" );

  /**
   * Deletes a number of cancellation errors
   */
  private static final String DELETE = "DELETE FROM loeschung_error WHERE key_id IN ( {0} )";

  /**
   * Gets all cancellation errors.
   * @return All cancellation errors
   * @throws SQLException if the query fails
   */
  public List<LoeschungErrorVO> selectLoeschungError() throws SQLException
  {
    return selectVOList( 900, VO_TYPE, SELECT_ALL );
  }

  /**
   * Loads one cancellation error.
   * @param loeschungErrorId The unique ID of the error, also known as <code>keyId</code>
   * @return The cancellation error or <code>null</code> if none was found
   * @throws SQLException if the query fails
   */
  public LoeschungErrorVO selectLoeschungError( int loeschungErrorId ) throws SQLException
  {
    return selectVO( VO_TYPE, SELECT_ONE, Integer.valueOf( loeschungErrorId ) );
  }

  /**
   * Inserts a cancellation error.
   * @param cancellationRunId The ID of the cancellation run
   * @param invoiceId The ID of the invoice to which the cancellation belongs
   * @param errorMessage The error message to show
   * @return The new cancellation error ID
   */
  public Integer insert( final int cancellationRunId, final int invoiceId, final String errorMessage )
  {
    return this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_error_insert";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( INSERT, Integer.valueOf( cancellationRunId ), Integer.valueOf( invoiceId ), errorMessage );
        verifyUpdateCount( count, 1, "update count must be 1" );
        this.setResult( selectCurrentSequenceValue( runner, CANCELLATION_ERROR_ID_SEQUENCE ) );
      }
    } );
  }

  /**
   * Deletes the cancellation errors with the given IDs.
   * @param loeschungErrorIds The IDs of the cancellation errors to delete
   * @return The number of deleted cancellation errors
   */
  @SuppressWarnings( "unchecked" )
  public Integer deleteLoeschungError( final List<Integer> loeschungErrorIds )
  {
    Check.notNull( loeschungErrorIds, "cancellation error IDs" );
    if ( loeschungErrorIds.isEmpty() )
    {
      return 0;
    }

    return this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_error_delete";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        String idList = StringUtils.join( loeschungErrorIds.iterator(), ", " );
        String sqlStatement = MessageFormat.format( DELETE, idList );
        int numberOfUpdates = runner.update( sqlStatement );
        this.setResult( Integer.valueOf( numberOfUpdates ) );
      }
    } );
  }
}
