package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.List;

import oracle.sql.BLOB;
import oracle.sql.CLOB;
import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.enums.BAGISLanguageDomain;

/**
 * Accesses the data from ReferenceXML through a VO.
 * <p/>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * @author Esther Bandschapp
 * @version $Id: ReferenceXMLDAO.java,v 1.25 2007/12/05 12:59:25 lewis_lee Exp $
 */
public class ReferenceXMLDAO extends AbstractJippsDAO
{
  /**
   * The VO class to use
   */
  private static final Class<ReferenceXMLVO> VO_TYPE = ReferenceXMLVO.class;

  /**
   * The base selection
   */
  private static final SQLStatement<ReferenceXMLVO> SELECT = SQL.select( "SELECT ref_xml_id, schutit_key AS ip_key, schutit_typ AS ip_type, ip_language,  "
      + "ausz_typ AS report_type, stand_dat AS report_date, xml_data, xsl_fo_data FROM referenz_xml WHERE " );

  // TODO The old data was saved using IDs beginning with 500. At some point in time they must be deleted, and then this
  // query can be changed back.
  /**
   * The old data was saved using IDs beginning with 500. This makes sure only the new data is selected.
   */
  private static final SQLStatement<ReferenceXMLVO> SELECT_ALL_ITEMS = SELECT.append( "ref_xml_id < 500 ORDER BY ref_xml_id" );

  /**
   * Inserts a reference XML
   */
  private static final SQLStatement<Integer> INSERT_VO = SQL.select( "INSERT INTO referenz_xml ("
      + "ref_xml_id, schutit_key, schutit_typ, ip_language, ausz_typ, stand_dat, xml_data, xsl_fo_data, pdf_file) VALUES (?,?,?,?,?,?,?,?,?)" );

  /**
   * The only constructor
   */
  public ReferenceXMLDAO()
  {
    super();
  }

  /**
   * Reads all reference XMLs, but doesn't read the PDF files.
   * 
   * @return The reference XML objects
   * @throws SQLException if the selection fails
   */
  public List<ReferenceXMLVO> readReferenceXMLs() throws SQLException
  {
    return selectVOList( 1000, VO_TYPE, SELECT_ALL_ITEMS );
  }

  /**
   * Inserts a reference XML document.
   * 
   * @param referenceXml The reference XML to insert
   */
  @SuppressWarnings( "unchecked" )
  public void insert( final ReferenceXMLVO referenceXml )
  {
    this.execute( new SingleDAOUnitOfWork()
    {
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        BLOB pdfFile = null;
        CLOB xmlData = null;
        CLOB xslFoData = null;
        try
        {
          pdfFile = runner.createTemporaryOracleBLOB( referenceXml.getPdfFile() );
          xmlData = runner.createTemporaryOracleCLOB( referenceXml.getXmlData() );
          xslFoData = runner.createTemporaryOracleCLOB( referenceXml.getXslFoData() );
          int updateCount = runner.update( INSERT_VO, referenceXml.getRefXmlId(), referenceXml.getIpKey(), referenceXml.getIpType(), BAGISLanguageDomain.parse(
              referenceXml.getLocale() ).toDB(), referenceXml.getReportType(), referenceXml.getReportDate(), xmlData, xslFoData, pdfFile );
          verifyUpdateCount( updateCount, 1, "insert failed, update count = " + updateCount );
        }
        finally
        {
          if ( pdfFile != null )
          {
            pdfFile.freeTemporary();
          }
          if ( xmlData != null )
          {
            xmlData.freeTemporary();
          }
          if ( xslFoData != null )
          {
            xslFoData.freeTemporary();
          }
        }
      }
    } );
  }
}
