package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.List;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.AmvErrorDAO;
import ch.ige.jipps.vo.AmvErrorVO;
import ch.ige.jipps.vo.AmvVO;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.dao.AmvErrorDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: AmvErrorDAOTest.java,v 1.3 2007/06/21 07:18:20 lewis_lee Exp $
 */
public class AmvErrorDAOTest extends BaseDAOTestTemplate<AmvErrorDAO, AmvErrorVO>
{
  /**
   * The ID of an AMV run that has error
   */
  private static final int AMV_FAULTY_RUN = 33;

  /**
   * An IP that was processed during {@link #AMV_FAULTY_RUN}
   */
  private static final int FAULTY_IP_KEY = 1200093441;

  /**
   * The ID of the AMV batch run to delete during {@link #tearDown()}
   */
  private Integer amvBatchRunIdToDelete = null;

  /**
   * The renewal that points to this error message
   */
  private AmvVO trademarkRenewal = null;

  /**
   * The only constructor
   * @param name The test's name
   */
  public AmvErrorDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<AmvErrorDAO> getDAOClass()
  {
    return AmvErrorDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvErrorDAO#selectAMVErrorToId(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectAMVErrorToId() throws Exception
  {
    List<AmvErrorVO> errors = getDAO().selectAMVErrorToId( 99999 );
    assertEquals( "Checking for unknown batch run ID", 0, errors.size() );

    //TODO Reactivate as soon as there is data in the table JIPPS_AMV_ERROR
//    errors = getDAO().selectAMVErrorToId( AMV_FAULTY_RUN );
//    assertEquals( "Checking for finished batch run ID", 31, errors.size() );
//    for ( AmvErrorVO error: errors )
//    {
//      if ( error.schutitKey == FAULTY_IP_KEY )
//      {
//        assertEquals( "Checking batch run ID", AMV_FAULTY_RUN, error.amvLaufId );
//        assertEquals( "Checking address number", 6015350, error.adrNr );
//        assertEquals( "Checking IP key", FAULTY_IP_KEY, error.schutitKey );
//        assertTrue( "Checking error message", error.errorMsg.startsWith( "Fehler im Auszug: " ) );
//        assertEquals( "Checking creation date", "2005-04-11 12:14:17.0", error.createdDat.toString() );
//      }
//    }
  }

  /**
   * Prepares the database for this test,
   * inserting both a whole batch run and one trademark renewal within that batch run
   * and then creates an error based on those objects.
   * @return The error containing data which exists in the database
   * @throws SQLException if the inserts fail
   */
  private AmvErrorVO createPreparedError() throws SQLException
  {
    this.amvBatchRunIdToDelete = AmvLaufDAOTest.createAndSaveBatchRun();
    this.trademarkRenewal = AmvDAOTest.createAndSaveTrademarkRenewal( this.amvBatchRunIdToDelete );

    AmvErrorVO error = getSerializableVO();
    error.amvLaufId = this.amvBatchRunIdToDelete;
    error.adrNr = this.trademarkRenewal.adrNr;
    error.schutitKey = this.trademarkRenewal.schutitKey;
    return error;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvErrorDAO#insert(ch.ige.jipps.vo.AmvErrorVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    // Lose the milliseconds and go back a minute, the times may differ between this server and the DB server
    Timestamp testStart = new Timestamp( ( System.currentTimeMillis() / 1000 * 1000 ) - 60 * 1000 );

    try
    {
      getDAO().insert( null );
      fail( "Null error must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    AmvErrorVO error = createPreparedError();
    getDAO().insert( error );

    List<AmvErrorVO> loadedErrors = getDAO().selectAMVErrorToId( this.amvBatchRunIdToDelete );
    assertEquals( "Checking for inserted error", 1, loadedErrors.size() );

    // Note that the creation date is set automatically to the current date/time
    AmvErrorVO loadedError = loadedErrors.get( 0 );
    assertEquals( "Checking batch run ID", this.amvBatchRunIdToDelete.intValue(), loadedError.amvLaufId );
    assertEquals( "Checking address number", error.adrNr, loadedError.adrNr );
    assertEquals( "Checking IP key", error.schutitKey, loadedError.schutitKey );
    assertEquals( "Checking error message", error.errorMsg, loadedError.errorMsg );
    assertTrue( "Checking creation date", loadedError.createdDat.after( testStart ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvErrorDAO#delete(int, int, int)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    try
    {
      getDAO().delete( 9999999 );
      fail( "Deleting unknown ID must fail" );
    }
    catch ( UnitOfWorkException e )
    {
      // This is expected
    }
    
    AmvErrorVO error = createPreparedError();
    getDAO().insert( error );

    List<AmvErrorVO> errors = getDAO().selectAMVErrorToId( this.amvBatchRunIdToDelete );
    assertEquals( "Checking for inserted batch run ID", 1, errors.size() );

    getDAO().delete( this.amvBatchRunIdToDelete, error.adrNr, error.schutitKey );
    errors = getDAO().selectAMVErrorToId( this.amvBatchRunIdToDelete );
    assertEquals( "Checking for deleted errors", 0, errors.size() );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvErrorDAO#delete(int)}.
   * @throws Exception if the test fails
   */
  public void testDeleteWholeBatchRun() throws Exception
  {
    AmvErrorVO error = createPreparedError();
    getDAO().insert( error );

    List<AmvErrorVO> errors = getDAO().selectAMVErrorToId( this.amvBatchRunIdToDelete );
    assertEquals( "Checking for inserted batch run ID", 1, errors.size() );

    getDAO().delete( this.amvBatchRunIdToDelete );
    errors = getDAO().selectAMVErrorToId( this.amvBatchRunIdToDelete );
    assertEquals( "Checking for deleted errors", 0, errors.size() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected AmvErrorVO getSerializableVO() throws SQLException
  {
    try
    {
      AmvErrorVO error = new AmvErrorVO();
      error.adrNr = 33;
      error.amvLaufId = 751;
      error.createdDat = DateConverter.stringToTimestamp( "15.12.2004 11:33:41.991" );
      error.changedDat = DateConverter.stringToTimestamp( "15.12.2004 12:44:41.991" );
      error.errorMsg = "Oh no, the world is coming to an end!";
      error.schutitKey = 571235;
      return error;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( AmvErrorVO originalError, AmvErrorVO serializedError )
  {
    return originalError.adrNr == serializedError.adrNr && originalError.amvLaufId == serializedError.amvLaufId
        && originalError.createdDat.equals( serializedError.createdDat ) && originalError.changedDat.equals( serializedError.changedDat )
        && originalError.errorMsg.equals( serializedError.errorMsg ) && originalError.schutitKey == serializedError.schutitKey;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    if ( this.amvBatchRunIdToDelete != null )
    {
      // Note that this is a cascading delete that also deletes the AmvVOs and AMVErrorVOs
      AmvLaufDAOTest.deleteBatchRun( this.amvBatchRunIdToDelete );
    }
    super.tearDown();
  }
}
