package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.List;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.AmvLaufDAO;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.dao.AmvLaufDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: AmvLaufDAOTest.java,v 1.8 2008/07/22 13:27:13 bandschapp Exp $
 */
public class AmvLaufDAOTest extends BaseDAOTestTemplate<AmvLaufDAO, AmvLaufVO>
{
  /**
   * The ID of an AMV run that has finished
   */
  private static final int AMV_FINISHED_RUN = 4653;

  /**
   * The sequence number corresponding to {@link #AMV_FINISHED_RUN}
   */
  private static final String FINISHED_SEQUENCE_NUMBER = "2008071401";

  /**
   * The dummy batch run type which makes a batch run identifiable for this test.
   */
  public static final String BATCH_RUN_TYPE = "unitTest";

  /**
   * The ID of the AMV batch run to delete during {@link #tearDown()}
   */
  private Integer amvBatchRunIdToDelete = null;

  /**
   * The only constructor
   * @param name The test's name
   */
  public AmvLaufDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<AmvLaufDAO> getDAOClass()
  {
    return AmvLaufDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#select(java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    try
    {
      getDAO().select( null );
      fail( "Null batch run type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().select( "" );
      fail( "Empty batch run type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<AmvLaufVO> batchRuns = getDAO().select( "whatever" );
    assertEquals( "Checking unknown batch run type", 0, batchRuns.size() );

    batchRuns = getDAO().select( AmvLaufVO.REMINDER_1 );
    assertTrue( "Checking existing batch run type", batchRuns.size() > 0 );

    checkKnownBatchRun( batchRuns );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#selectAMVLaufToId(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectAMVLaufToId() throws Exception
  {
    AmvLaufVO batchRun = getDAO().selectAMVLaufToId( 999999 );
    assertNull( "Checking unknown batch run type", batchRun );

    batchRun = getDAO().selectAMVLaufToId( AMV_FINISHED_RUN );
    if ( batchRun != null ) // can be null if AMV_FINISHED_RUN - ID does not exist, which may happen.
    {
      checkKnownBatchRun( batchRun );
    }
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#getAmvLaufId(java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testGetAmvLaufId() throws Exception
  {
    try
    {
      getDAO().getAmvLaufId( null );
      fail( "Null batch run type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().getAmvLaufId( "" );
      fail( "Empty batch run type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    int highestBatchRunId = getDAO().getAmvLaufId( "whatever" );
    assertEquals( "Checking unknown batch run type", 0, highestBatchRunId );

    highestBatchRunId = getDAO().getAmvLaufId( AmvLaufVO.REMINDER_1 );
    assertTrue( "Checking existing batch run type", highestBatchRunId > 0 );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#selectAmvLaufId(java.lang.String, java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testSelectAmvLaufId() throws Exception
  {
    try
    {
      getDAO().selectAmvLaufId( null, AmvLaufVO.REMINDER_1 );
      fail( "Null sequence number must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectAmvLaufId( "", AmvLaufVO.REMINDER_1 );
      fail( "Empty sequence number must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    int batchRunId = getDAO().selectAmvLaufId( "bla", AmvLaufVO.REMINDER_1 );
    assertEquals( "Checking unknown batch run type", 0, batchRunId );

    try
    {
      getDAO().selectAmvLaufId( FINISHED_SEQUENCE_NUMBER, null );
      fail( "Null batch run type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectAmvLaufId( FINISHED_SEQUENCE_NUMBER, "" );
      fail( "Empty batch run type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    batchRunId = getDAO().selectAmvLaufId( FINISHED_SEQUENCE_NUMBER, "whatever" );
    assertEquals( "Checking unknown batch run type", 0, batchRunId );

    batchRunId = getDAO().selectAmvLaufId( FINISHED_SEQUENCE_NUMBER, AmvLaufVO.REMINDER_1 );
    assertEquals( "Checking correct batch run", AMV_FINISHED_RUN, batchRunId );
  }

  /**
   * Checks a well-known batch run from batch runs.
   * @param batchRuns The loaded batch runs
   */
  private void checkKnownBatchRun( List<AmvLaufVO> batchRuns )
  {
    boolean foundKnownBatchRun = false;
    for ( AmvLaufVO batchRun: batchRuns )
    {
      if ( batchRun.amvLaufId == AMV_FINISHED_RUN )
      {
        foundKnownBatchRun = true;
        checkKnownBatchRun( batchRun );
        break;
      }
    }
    assertTrue( "Checking for known batch run", foundKnownBatchRun );
  }

  /**
   * Checks a well-known batch run.
   * @param batchRun The batch run to check
   */
  private void checkKnownBatchRun( AmvLaufVO batchRun )
  {
    assertEquals( "Checking batch run ID", AMV_FINISHED_RUN, batchRun.amvLaufId );
    assertEquals( "Checking sequence", FINISHED_SEQUENCE_NUMBER, batchRun.laufNr );
    assertEquals( "Checking batch run type", AmvLaufVO.REMINDER_1, batchRun.typCd );
    assertEquals( "Checking processing status", AmvLaufVO.NO_DATA, batchRun.verarbStat );
    assertEquals( "Checking number of renewals", 0, batchRun.anzAmv );
    assertEquals( "Checking batch ID", 0, batchRun.batchId );
    assertEquals( "Checking creation date", "2008-07-14 17:08:21.0", batchRun.createdDat.toString() );
    assertEquals( "Checking end of processing date", "2008-07-14 17:08:28.0", batchRun.verarbEndeDat.toString() );
  }

  /**
   * Gets the current time without milliseconds minus one minute.
   * This is used to check DB-generated timestamps, since they must be
   * younger than this. The minute is subtracted because the clock between
   * this computer and the DB server is not synchronized.
   * @return The test start
   */
  private Timestamp getTestStart()
  {
    return new Timestamp( ( System.currentTimeMillis() / 1000 * 1000 ) - 60 * 1000 );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#insert(ch.ige.jipps.vo.AmvLaufVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    Timestamp testStart = getTestStart();

    try
    {
      getDAO().insert( null );
      fail( "Null batch run must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    AmvLaufVO batchRun = getSerializableVO();
    this.amvBatchRunIdToDelete = getDAO().insert( batchRun );

    AmvLaufVO loadedBatchRun = getDAO().selectAMVLaufToId( this.amvBatchRunIdToDelete );
    assertNotNull( "Checking that the batch run was saved", loadedBatchRun );
    assertEquals( "Checking sequence", batchRun.laufNr, loadedBatchRun.laufNr );
    assertNull( "Checking end of processing date", loadedBatchRun.verarbEndeDat );
    assertEquals( "Checking processing status", AmvLaufVO.READY_FOR_PROCESSING, loadedBatchRun.verarbStat );
    assertEquals( "Checking number of renewals", 0, loadedBatchRun.anzAmv );
    assertEquals( "Checking batch run type", BATCH_RUN_TYPE, loadedBatchRun.typCd );
    assertEquals( "Checking batch ID", 0, loadedBatchRun.batchId );
    assertTrue( "Checking creation date", loadedBatchRun.createdDat.after( testStart ) );
    assertTrue( "Checking change date", loadedBatchRun.changedDat.after( testStart ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#update(ch.ige.jipps.vo.AmvLaufVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    Timestamp testStart = getTestStart();

    try
    {
      getDAO().update( null );
      fail( "Null batch run must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    AmvLaufVO batchRun = getSerializableVO();
    this.amvBatchRunIdToDelete = getDAO().insert( batchRun );

    AmvLaufVO loadedBatchRun = getDAO().selectAMVLaufToId( this.amvBatchRunIdToDelete );
    assertNotNull( "Checking that the batch run was saved", loadedBatchRun );

    // Change only the variables that are saved during update
    loadedBatchRun.verarbStat = AmvLaufVO.FAULTY_DATA;
    loadedBatchRun.anzAmv = 95821;
    loadedBatchRun.batchId = 76;
    getDAO().update( loadedBatchRun );

    AmvLaufVO reloadedBatchRun = getDAO().selectAMVLaufToId( this.amvBatchRunIdToDelete );
    assertNotNull( "Checking that the batch run was saved", reloadedBatchRun );
    assertEquals( "Checking sequence", batchRun.laufNr, reloadedBatchRun.laufNr );
    assertTrue( "Checking end of processing date", reloadedBatchRun.verarbEndeDat.after( testStart ) );
    assertEquals( "Checking processing status", loadedBatchRun.verarbStat, reloadedBatchRun.verarbStat );
    assertEquals( "Checking number of renewals", loadedBatchRun.anzAmv, reloadedBatchRun.anzAmv );
    assertEquals( "Checking batch run type", BATCH_RUN_TYPE, reloadedBatchRun.typCd );
    assertEquals( "Checking batch ID", loadedBatchRun.batchId, reloadedBatchRun.batchId );
    assertEquals( "Checking creation date", loadedBatchRun.createdDat, reloadedBatchRun.createdDat );
    assertTrue( "Checking change date", reloadedBatchRun.changedDat.after( loadedBatchRun.changedDat ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvLaufDAO#delete(int)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    try
    {
      getDAO().delete( 9999999 );
      fail( "Deleting unknown ID must fail" );
    }
    catch ( UnitOfWorkException e )
    {
      // This is expected
    }

    AmvLaufVO batchRun = getSerializableVO();
    this.amvBatchRunIdToDelete = getDAO().insert( batchRun );

    getDAO().delete( this.amvBatchRunIdToDelete );
    AmvLaufVO loadedBatchRun = getDAO().selectAMVLaufToId( this.amvBatchRunIdToDelete );
    assertNull( "Checking that the batch run was deleted", loadedBatchRun );
    this.amvBatchRunIdToDelete = null; // Was already deleted
  }

  /**
   * Creates and inserts a batch run.
   * This is normally used by other tests that require a batch run to exist
   * before they can save their objects.
   * @return The batch run's ID
   * @throws SQLException if the batch run can't be created
   */
  public static int createAndSaveBatchRun() throws SQLException
  {
    AmvLaufDAOTest thisTest = new AmvLaufDAOTest( "whatever" );
    AmvLaufVO batchRun = thisTest.getSerializableVO();
    thisTest.getDAO().insert( batchRun );
    List<AmvLaufVO> batchRuns = thisTest.getDAO().select( BATCH_RUN_TYPE );
    assertEquals( "Checking for test batch runs", 1, batchRuns.size() );
    return batchRuns.get( 0 ).amvLaufId;
  }

  /**
   * Deletes a batch run.
   * Normally, this was created for other tests by {@link #createAndSaveBatchRun()}.
   * @param amvBatchRunId The unique ID of the AMV batch run
   */
  public static void deleteBatchRun( int amvBatchRunId )
  {
    AmvLaufDAOTest thisTest = new AmvLaufDAOTest( "whatever" );
    thisTest.getDAO().delete( amvBatchRunId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected AmvLaufVO getSerializableVO() throws SQLException
  {
    try
    {
      Timestamp theDate = DateConverter.stringToTimestamp( "01.01.2007 00:00:00.000" );
      AmvLaufVO batchRun = new AmvLaufVO();
      batchRun.amvLaufId = 999999;
      batchRun.laufNr = "test";
      batchRun.verarbEndeDat = theDate;
      batchRun.verarbStat = 1;
      batchRun.anzAmv = 1;
      batchRun.typCd = BATCH_RUN_TYPE;
      batchRun.batchId = 1;
      batchRun.createdDat = theDate;
      batchRun.changedDat = theDate;
      return batchRun;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( AmvLaufVO originalBatchRun, AmvLaufVO serializedBatchRun )
  {
    return originalBatchRun.amvLaufId == serializedBatchRun.amvLaufId && originalBatchRun.laufNr.equals( serializedBatchRun.laufNr )
        && originalBatchRun.verarbEndeDat.equals( serializedBatchRun.verarbEndeDat ) && originalBatchRun.verarbStat == serializedBatchRun.verarbStat
        && originalBatchRun.anzAmv == serializedBatchRun.anzAmv && originalBatchRun.typCd.equals( serializedBatchRun.typCd )
        && originalBatchRun.batchId == serializedBatchRun.batchId && originalBatchRun.createdDat.equals( serializedBatchRun.createdDat )
        && originalBatchRun.changedDat.equals( serializedBatchRun.changedDat );

  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    if ( this.amvBatchRunIdToDelete != null )
    {
      // Note that this is a cascading delete that also deletes the AmvVOs and AMVErrorVOs
      AmvLaufDAOTest.deleteBatchRun( this.amvBatchRunIdToDelete );
    }
    super.tearDown();
  }
}
