package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.List;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.LoeschungDAO;
import ch.ige.jipps.vo.DruckPeriVO;
import ch.ige.jipps.vo.LoeschungLaufVO;
import ch.ige.jipps.vo.LoeschungVO;
import ch.ige.util.constants.CancellationConstants;
import ch.ige.util.constants.CancellationProcessingConstants;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.IPType;

/**
 * Tests {@link ch.ige.jipps.dao.LoeschungDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: LoeschungDAOTest.java,v 1.7 2008/05/22 11:43:55 lewis_lee Exp $
 */
public class LoeschungDAOTest extends BaseDAOTestTemplate<LoeschungDAO, LoeschungVO>
{
  /**
   * The recipient group to use in the tests.
   */
  private static final String PSEUDO_RECIPIENT_GROUP = "t";

  /**
   * Used for setting up a cancellation run.
   */
  private LoeschungLaufDAOTest loeschungLaufDAOTest;

  /**
   * The cancellation run set up for this test.
   */
  private LoeschungLaufVO cancellationRun = null;

  /**
   * The cancellation inserted during this test, which must be deleted at {@link #tearDown()}.
   */
  private LoeschungVO insertedCancellation;

  /**
   * Shows when a test method starts, for inserting into the DB.
   */
  private Timestamp testStart;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public LoeschungDAOTest( String name )
  {
    super( name );
  }

  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.loeschungLaufDAOTest = new LoeschungLaufDAOTest( getName() );
    this.cancellationRun = this.loeschungLaufDAOTest.createAndInsert();

    // Lose the milliseconds and go back a minute, the times may differ between this server and the DB server
    this.testStart = new Timestamp( ( System.currentTimeMillis() / 1000 * 1000 ) - 60 * 1000 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<LoeschungDAO> getDAOClass()
  {
    return LoeschungDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#selectSorted(java.lang.String, int, int, int[])}.
   * @throws Exception if the test fails
   */
  public void testSelectSortedWithPrintStati() throws Exception
  {
    int numberOfCancellations = 1;

    try
    {
      getDAO().selectSorted( "9", numberOfCancellations, this.cancellationRun.getIdentifier().intValue(), new int[] { 1, 2, 3, 4, 5, 6 } );
      fail( "Too many print stati must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    LoeschungVO cancellation = createAndInsertCancellation( this.testStart );
    List<LoeschungVO> loadedCancellations = getDAO().selectSorted( PSEUDO_RECIPIENT_GROUP, numberOfCancellations,
        this.cancellationRun.getIdentifier().intValue(), new int[] { CancellationProcessingConstants.NEW } );
    assertEquals( "Checking number of cancellations", numberOfCancellations, loadedCancellations.size() );

    checkCancellations( cancellation, loadedCancellations.get( 0 ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#selectSorted(java.lang.String, int, int)}.
   * @throws Exception if the test fails
   */
  public void testSelectSorted() throws Exception
  {
    int numberOfCancellations = 1;

    LoeschungVO cancellation = createAndInsertCancellation( this.testStart );
    List<LoeschungVO> cancellations = getDAO().selectSorted( PSEUDO_RECIPIENT_GROUP, numberOfCancellations, this.cancellationRun.getIdentifier().intValue() );
    assertEquals( "Checking number of cancellations", numberOfCancellations, cancellations.size() );

    assertEquals( "Checking invoice ID", cancellation.bfaktId, cancellations.get( 0 ).bfaktId );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#selectDruckPeriByDruckArt(int, int)}.
   * @throws Exception if the test fails
   */
  public void testSelectDruckPeriByDruckArt() throws Exception
  {
    List<DruckPeriVO> printRecipients = getDAO().selectDruckPeriByDruckArt( -1, -2 );
    assertEquals( "Checking invalid printing period", 0, printRecipients.size() );

    createAndInsertCancellation( this.testStart );
    printRecipients = getDAO().selectDruckPeriByDruckArt( CancellationConstants.IP_NOT_PAID, this.cancellationRun.getIdentifier().intValue() );
    assertEquals( "Checking valid printing period", 1, printRecipients.size() );
    DruckPeriVO printRecipient = printRecipients.get( 0 );
    assertEquals( "Checking print recipient ID", PSEUDO_RECIPIENT_GROUP, printRecipient.druckPeri );
    assertEquals( "Checking number of cancellations", 1, printRecipient.anzahl );
    assertEquals( "Checking cancellation run ID", this.cancellationRun.getIdentifier().intValue(), printRecipient.loeschungLaufId );
    assertEquals( "Checking whether this was printed", false, printRecipient.isPrinted );
  }

  /**
   * Checks that the cancellations contain the required data.
   * @param actualCancellation The cancellation loaded from the DB
   * @param expectedCancellation The cancellation expected to be found
   */
  private void checkCancellations( LoeschungVO expectedCancellation, LoeschungVO actualCancellation )
  {
    assertEquals( "Checking cancellation run ID", this.cancellationRun.getIdentifier().intValue(), actualCancellation.loeschungLaufId );
    assertEquals( "Checking invoice ID", expectedCancellation.bfaktId, actualCancellation.bfaktId );
    assertEquals( "Checking address number", expectedCancellation.adrNr, actualCancellation.adrNr );
    assertEquals( "Checking print recipient ID", PSEUDO_RECIPIENT_GROUP, actualCancellation.druckPeri );
    assertEquals( "Checking tray ID", expectedCancellation.fachNr, actualCancellation.fachNr );
    assertEquals( "Checking IP type", IPType.ALL.toDB().intValue(), actualCancellation.schutitTyp );
    assertEquals( "Checking other address number", 0, actualCancellation.adrNrVtr );
    assertEquals( "Checking sort text", expectedCancellation.sortTxt, actualCancellation.sortTxt.trim() );
    assertEquals( "Checking status", expectedCancellation.verarbStat, actualCancellation.verarbStat );
    assertEquals( "Checking XML document", expectedCancellation.loeschungXml, actualCancellation.loeschungXml );
    assertNotNull( "Checking creation date", actualCancellation.createDate );
    assertTrue( "Checking creation date", actualCancellation.createDate.after( this.testStart ) );
    assertNotNull( "Checking change date", actualCancellation.changedDate );
    assertTrue( "Checking change date", actualCancellation.changedDate.after( this.testStart ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#getLoeschungLaufId(int)}.
   * @throws Exception if the test fails
   */
  public void testGetLoeschungLaufId() throws Exception
  {
    int cancellationRunId = getDAO().getLoeschungLaufId( 9999999 );
    assertEquals( "Checking for unknown printing status", 0, cancellationRunId );

    cancellationRunId = getDAO().getLoeschungLaufId( this.cancellationRun.druckArt );
    assertEquals( "Checking for status", this.cancellationRun.getIdentifier().intValue(), cancellationRunId );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#insert(ch.ige.jipps.vo.LoeschungVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    try
    {
      getDAO().insert( null );
      fail( "Null cancellation must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    createAndInsertCancellation( this.testStart );
  }

  /**
   * Creates and inserts a cancellation, reloads it and tests it.
   * @param start The start time of the test method
   * @return The created cancellation
   * @throws SQLException if the insert fails
   */
  private LoeschungVO createAndInsertCancellation( Timestamp start ) throws SQLException
  {
    LoeschungVO cancellation = getCancellationToInsert();
    getDAO().insert( cancellation );
    this.insertedCancellation = cancellation;

    List<LoeschungVO> loadedCancellations = getDAO().selectSorted( cancellation.druckPeri, 10, cancellation.loeschungLaufId );
    LoeschungVO loadedCancellation = loadedCancellations.get( 0 );
    checkCancellation( cancellation, loadedCancellation, start );
    return cancellation;
  }

  /**
   * Gets the cancellation to insert.
   * @return The cancellation to insert
   */
  private LoeschungVO getCancellationToInsert()
  {
    // Since there's no way to load this directly, make sure that it
    // turns up as the first cancellation by setting its values to
    // the minimal values for sorting.
    LoeschungVO cancellation = new LoeschungVO();
    assertNotNull( "Checking that a cancellation run exists before inserting an cancellation error", this.cancellationRun );
    cancellation.loeschungLaufId = this.cancellationRun.loeschungLaufId;
    cancellation.bfaktId = 99;
    cancellation.adrNr = 0;
    cancellation.adrNrVtr = 0;
    // The create/change dates are inserted automatically
    cancellation.druckPeri = PSEUDO_RECIPIENT_GROUP;
    cancellation.fachNr = 0;
    cancellation.loeschungXml = "<dummy>This is a test!</dummy>";
    cancellation.schutitTyp = IPType.ALL.toDB().intValue();
    cancellation.sortTxt = "bla";
    cancellation.verarbStat = CancellationProcessingConstants.NEW_INTEGER;
    return cancellation;
  }

  /**
   * Checks a cancellation.
   * @param originalCancellation The cancellation as created
   * @param loadedCancellation The cancellation as loaded from the DB
   * @param start The test start, which is compared to the dates generated during the insert
   */
  private void checkCancellation( LoeschungVO originalCancellation, LoeschungVO loadedCancellation, Timestamp start )
  {
    assertEquals( "Checking cancellation run ID", originalCancellation.loeschungLaufId, loadedCancellation.loeschungLaufId );
    assertEquals( "Checking invoice ID", originalCancellation.bfaktId, loadedCancellation.bfaktId );
    assertEquals( "Checking address number", originalCancellation.adrNr, loadedCancellation.adrNr );
    assertEquals( "Checking other address number", originalCancellation.adrNrVtr, loadedCancellation.adrNrVtr );
    assertEquals( "Checking print recipient ID", originalCancellation.druckPeri, loadedCancellation.druckPeri );
    assertEquals( "Checking tray ID", originalCancellation.fachNr, loadedCancellation.fachNr );
    assertEquals( "Checking IP type", originalCancellation.schutitTyp, loadedCancellation.schutitTyp );
    assertEquals( "Checking sort text", originalCancellation.sortTxt, loadedCancellation.sortTxt );
    assertEquals( "Checking status", originalCancellation.verarbStat, loadedCancellation.verarbStat );
    assertEquals( "Checking for XML document", originalCancellation.loeschungXml, loadedCancellation.loeschungXml );

    assertTrue( "Checking creation date", loadedCancellation.createDate.after( start ) );
    assertTrue( "Checking change date", loadedCancellation.changedDate.after( start ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#update(int, int, int, Integer)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    Integer updatedCancellations = getDAO().update( 9999999, 9999999, 9999999, CancellationProcessingConstants.PRINT_AGAIN );
    assertEquals( "Checking unknown cancellation run", 0, updatedCancellations.intValue() );

    LoeschungVO cancellation = createAndInsertCancellation( this.testStart );

    // Change the processing status to something that will be selected in selectSorted()
    cancellation.verarbStat = CancellationProcessingConstants.PRINT_AGAIN_INTEGER;
    updatedCancellations = getDAO().update( cancellation.loeschungLaufId, cancellation.bfaktId, cancellation.adrNr, cancellation.verarbStat );
    assertEquals( "Checking that cancellation was updated", 1, updatedCancellations.intValue() );

    recheckAfterUpdate( cancellation, this.testStart );
  }

  /**
   * Re-checks the cancellation after an update.
   * @param cancellation The cancellation to check
   * @param start The start time of the test method
   * @throws SQLException if the selection fails
   */
  private void recheckAfterUpdate( LoeschungVO cancellation, Timestamp start ) throws SQLException
  {
    List<LoeschungVO> loadedCancellations = getDAO().selectSorted( cancellation.druckPeri, 10, cancellation.loeschungLaufId );
    LoeschungVO loadedCancellation = loadedCancellations.get( 0 );
    checkCancellation( cancellation, loadedCancellation, start );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#updateDruckPeri(int, String, Integer)}.
   * @throws Exception if the test fails
   */
  public void testUpdateDruckPeri() throws Exception
  {
    Integer updatedCancellations = getDAO().updateDruckPeri( 9999999, "t", CancellationProcessingConstants.PRINT_AGAIN_INTEGER );
    assertEquals( "Checking unknown cancellation run", 0, updatedCancellations.intValue() );

    LoeschungVO cancellation = createAndInsertCancellation( this.testStart );

    // Change the processing status to something that will be selected in selectSorted()
    cancellation.verarbStat = CancellationProcessingConstants.PRINT_AGAIN_INTEGER;
    updatedCancellations = getDAO().updateDruckPeri( cancellation.loeschungLaufId, cancellation.druckPeri, cancellation.verarbStat );
    assertEquals( "Checking that cancellation was updated", 1, updatedCancellations.intValue() );

    recheckAfterUpdate( cancellation, this.testStart );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungDAO#delete(int, int, int)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    try
    {
      getDAO().delete( 9999999, 999999, 99999 );
      fail( "Unknown cancellation run must fail" );
    }
    catch ( UnitOfWorkException e )
    {
      // This is expected
    }

    LoeschungVO cancellation = createAndInsertCancellation( this.testStart );

    getDAO().delete( cancellation.loeschungLaufId, cancellation.bfaktId, cancellation.adrNr );

    // Re-check the whole thing
    List<LoeschungVO> loadedCancellations = getDAO().selectSorted( cancellation.druckPeri, 10, cancellation.loeschungLaufId );
    assertEquals( "Checking for missing cancellation", 0, loadedCancellations.size() );

    this.insertedCancellation = null; // Was already deleted
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected LoeschungVO getSerializableVO() throws SQLException
  {
    try
    {
      LoeschungVO cancellation = new LoeschungVO();
      cancellation.loeschungLaufId = 8;
      cancellation.bfaktId = 81;
      cancellation.adrNr = 33;
      cancellation.adrNrVtr = 55;
      cancellation.changedDate = DateConverter.stringToTimestamp( "03.01.2002 23:33:00.000" );
      cancellation.createDate = DateConverter.stringToTimestamp( "01.01.2001 18:44:44.000" );
      cancellation.druckPeri = "abc";
      cancellation.fachNr = 55;
      cancellation.loeschungXml = "<dummy>Hello world</dummy>";
      cancellation.schutitTyp = 5;
      cancellation.sortTxt = "fghIIJ";
      cancellation.verarbStat = 99;
      return cancellation;
    }
    catch ( ParseException e )
    {
      throw new SQLException( "Couldn't parse date: " + e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( LoeschungVO originalCancellation, LoeschungVO serializedCancellation )
  {
    return originalCancellation.adrNr == serializedCancellation.adrNr && originalCancellation.adrNrVtr == serializedCancellation.adrNrVtr
        && originalCancellation.bfaktId == serializedCancellation.bfaktId && originalCancellation.changedDate.equals( serializedCancellation.changedDate )
        && originalCancellation.createDate.equals( serializedCancellation.createDate )
        && originalCancellation.druckPeri.equals( serializedCancellation.druckPeri ) && originalCancellation.fachNr == serializedCancellation.fachNr
        && originalCancellation.loeschungLaufId == serializedCancellation.loeschungLaufId
        && originalCancellation.loeschungXml.equals( serializedCancellation.loeschungXml )
        && originalCancellation.schutitTyp == serializedCancellation.schutitTyp && originalCancellation.sortTxt.equals( serializedCancellation.sortTxt )
        && originalCancellation.verarbStat.equals( serializedCancellation.verarbStat );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    if ( this.insertedCancellation != null )
    {
      this.getDAO().delete( this.insertedCancellation.loeschungLaufId, this.insertedCancellation.bfaktId, this.insertedCancellation.adrNr );
    }
    if ( this.cancellationRun != null )
    {
      this.loeschungLaufDAOTest.delete( this.cancellationRun.getIdentifier() );
    }
    super.tearDown();
  }
}