package ch.ige.jipps.dao.test;

import java.io.File;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.FileUtils;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.dao.PrintjobDAO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.date.DateConverter;
import ch.ige.util.validation.Check;

/**
 * Tests {@link ch.ige.jipps.dao.PrintjobDAO}.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: PrintjobDAOTest.java,v 1.24 2008/05/22 12:00:39 lewis_lee Exp $
 */
public class PrintjobDAOTest extends BaseDAOWithCRUDTestTemplate<PrintjobDAO, PrintJobVO, Integer>
{
  /**
   * The ID of a printjob which was created to print a report (from the GUI).
   */
  private static final Integer PRINT_JOB_ID = 50037;

  /**
   * A print job ID that doesn't exist in the database
   */
  private static final Integer UNKNOWN_PRINT_JOB_ID = 2;

  /**
   * A wrong print job ID
   */
  private static final Integer WRONG_PRINT_JOB_ID = -1;

  /**
   * The batch run ID to use.
   */
  private static final Integer BATCH_ID = 5613;

  /**
   * A batch run ID that doesn't exist in the database
   */
  private static final Integer UNKNOWN_BATCH_ID = 2;

  /**
   * A wrong batch run ID
   */
  private static final Integer WRONG_BATCH_ID = -1;

  /**
   * If a print job is created in this test, then it must have this ID.
   */
  private static final Integer CREATED_PRINT_JOB_ID = 999999;

  /**
   * A known report ID matching {@link #PRINT_JOB_ID}.
   */
  private static final int REPORT_ID = 6031380;

  /**
   * Shows when a test method starts, for inserting into the DB.
   */
  private Timestamp testStart;

  /**
   * Default constructor
   * 
   * @param name The test's name
   */
  public PrintjobDAOTest( String name )
  {
    super( name );
  }

  @Override
  public void setUp() throws Exception
  {
    super.setUp();

    // Lose the milliseconds and go back a minute, the times may differ between this server and the DB server
    this.testStart = new Timestamp( ( System.currentTimeMillis() / 1000 * 1000 ) - 60 * 1000 );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#getNextPrintJobId()}.
   * @throws SQLException if the test fails
   */
  public void testGetNextPrintJobId() throws SQLException
  {
    int printJobId = getDAO().getNextPrintJobId();
    assertTrue( "Checking print job ID", printJobId > 10 );
    int furtherPrintJobId = getDAO().getNextPrintJobId();
    assertTrue( "Checking one more print job ID", furtherPrintJobId > printJobId );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#getNextBatchId()}.
   * @throws SQLException if the test fails
   */
  public void testGetNextBatchId() throws SQLException
  {
    int batchId = getDAO().getNextBatchId();
    assertTrue( "Checking batch ID", batchId > 10 );
    int furtherBatchId = getDAO().getNextBatchId();
    assertTrue( "Checking one more batch ID", furtherBatchId > batchId );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#readPrintjob(int)}.
   * 
   * @throws Exception if the test fails
   */
  public void testReadPrintjob() throws Exception
  {
    PrintJobVO printJob = createAndInsert();
    setSavedId( printJob.getIdentifier() );
    defaultTestRead( printJob, WRONG_PRINT_JOB_ID, UNKNOWN_PRINT_JOB_ID, printJob.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#readPrintJobs(java.lang.Integer)}.
   * 
   * @throws Exception if the test fails
   */
  public void testReadPrintJobs() throws Exception
  {
    defaultTestReadCollectionWithInvalidParameters( new Object[] { null } );

    defaultTestReadCollectionWithInvalidParameters( WRONG_BATCH_ID );

    defaultTestReadCollectionForUnknownResult( UNKNOWN_BATCH_ID );

    createAndInsert();
    defaultTestReadCollection( getExpectedVO(), 1, BATCH_ID );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( PrintJobVO printJob )
  {
    printJob.printJobId = CREATED_PRINT_JOB_ID;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#writePrintjob(ch.ige.jipps.vo.PrintJobVO)}.
   * @throws SQLException if the test fails
   */
  public void testWritePrintjob() throws SQLException
  {
    defaultTestCreate();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOAfterInsert(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOAfterInsert( PrintJobVO printJob, PrintJobVO loadedPrintJob )
  {
    printJob.erstDatum = loadedPrintJob.erstDatum;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( PrintJobVO printJob )
  {
    printJob.druckStatus = 7;
    printJob.druckerId = 16;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#updatePrintjob(List, int, int)}.
   * @throws SQLException if the test fails
   */
  public void testUpdatePrintjob() throws SQLException
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#deletePrintjob(int)}.
   * @throws SQLException if the test fails
   */
  public void testDeletePrintjob() throws SQLException
  {
    defaultTestDelete( WRONG_PRINT_JOB_ID, UNKNOWN_PRINT_JOB_ID );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#deletePrintjobs(int)}.
   * @throws SQLException if the test fails
   */
  public void testDeletePrintjobsWithBatchId() throws SQLException
  {
    try
    {
      getDAO().deletePrintjobs( WRONG_BATCH_ID );
      fail( "Invalid ID must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    Integer deletedPrintJobs = getDAO().deletePrintjobs( UNKNOWN_BATCH_ID );
    assertEquals( "Checking for unknown batch ID", 0, deletedPrintJobs.intValue() );

    PrintJobVO printJob = createAndInsert();
    deletedPrintJobs = getDAO().deletePrintjobs( printJob.batchId );
    assertEquals( "Checking for known batch ID", 1, deletedPrintJobs.intValue() );
    assertNull( "Checking that print job was deleted", read( printJob.printJobId ) );
    clearSavedId();
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#deletePrintjob(int, int)}.
   * @throws SQLException if the test fails
   */
  public void testDeletePrintjobsWithBatchIdAndReportId() throws SQLException
  {
    try
    {
      getDAO().deletePrintjob( -1, BATCH_ID );
      fail( "Invalid report ID must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().deletePrintjob( REPORT_ID, WRONG_BATCH_ID );
      fail( "Invalid batch ID must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    Integer deletedPrintJobs = getDAO().deletePrintjob( REPORT_ID, UNKNOWN_BATCH_ID );
    assertEquals( "Checking for unknown batch ID", 0, deletedPrintJobs.intValue() );

    PrintJobVO printJob = createAndInsert();
    deletedPrintJobs = getDAO().deletePrintjob( printJob.reportId, printJob.batchId );
    assertEquals( "Checking for known report and batch ID", 1, deletedPrintJobs.intValue() );
    assertNull( "Checking that print job was deleted", read( printJob.printJobId ) );
    clearSavedId();
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PrintjobDAO#deletePrintjobs(java.util.Date)}.
   * @throws Exception if the test fails
   */
  public void testDeletePrintjobsWithDate() throws Exception
  {
    try
    {
      getDAO().deletePrintjobs( (Date) null );
      fail( "Null date must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    // Since the print job is inserted with the current date/time
    // and deletePrintjobs() deletes everything younger than the given
    // date, it's impossible to simply test it without deleting all existing
    // print jobs.
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<PrintjobDAO> getDAOClass()
  {
    return PrintjobDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( PrintJobVO printJob ) throws SQLException
  {
    getDAO().writePrintjob( printJob );
    return printJob.getIdentifier();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected PrintJobVO read( Integer printJobId ) throws SQLException
  {
    return getDAO().readPrintjob( printJobId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  protected Collection<PrintJobVO> read( Object... searchParameters ) throws SQLException
  {
    return getDAO().readPrintJobs( (Integer) searchParameters[0] );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( PrintJobVO printJob )
  {
    // Since a list is expected for the real update, this satisfies the test's expectations.
    Check.notNull( printJob, "print job" );

    List<Integer> printJobIds = getIdentifiers( printJob );
    getDAO().updatePrintjob( printJobIds, printJob.druckStatus, printJob.druckerId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer printJobId )
  {
    getDAO().deletePrintjob( printJobId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( PrintJobVO printJob, PrintJobVO loadedPrintJob )
  {
    assertEquals( "Checking print job ID", printJob.printJobId, loadedPrintJob.printJobId );
    assertEquals( "Checking batch ID", printJob.batchId, loadedPrintJob.batchId );
    assertEquals( "Checking printer ID", printJob.druckerId, loadedPrintJob.druckerId );
    assertEquals( "Checking number of copies", printJob.anzKopien, loadedPrintJob.anzKopien );
    assertEquals( "Checking print status", printJob.druckStatus, loadedPrintJob.druckStatus );
    assertEquals( "Checking sending application", printJob.senderApp, loadedPrintJob.senderApp );
    assertEquals( "Checking sending user", printJob.senderUser, loadedPrintJob.senderUser );
    assertTrue( "Checking creation date", loadedPrintJob.erstDatum.after( this.testStart ) );
    assertEquals( "Checking form type", printJob.formTyp.toLowerCase(), loadedPrintJob.formTyp.toLowerCase() );
    assertEquals( "Checking report ID", printJob.reportId, loadedPrintJob.reportId );
    assertEquals( "Checking PDF length", printJob.printJobFile.length, loadedPrintJob.printJobFile.length );
    assertEquals( "Checking cover", printJob.coverCd, loadedPrintJob.coverCd );
    assertEquals( "Checking paper mode", printJob.paperMode, loadedPrintJob.paperMode );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected PrintJobVO getExpectedVO()
  {
    try
    {
      return getSerializableVO();
    }
    catch ( SQLException e )
    {
      throw new RuntimeException( "Can't create print job", e );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected PrintJobVO getSerializableVO() throws SQLException
  {
    try
    {
      PrintJobVO printJob = new PrintJobVO();
      printJob.printJobId = CREATED_PRINT_JOB_ID;
      printJob.batchId = BATCH_ID;
      printJob.adrNr = 123;
      printJob.druckerId = 15;
      printJob.paperMode = 3;
      printJob.anzKopien = 5;
      printJob.druckStatus = 9;
      printJob.senderApp = "bla";
      printJob.senderUser = "blo";
      printJob.erstDatum = DateConverter.stringToTimestamp( "01.02.1902 23:11:22.000" );
      printJob.formTyp = "Hello";
      printJob.reportId = 552231;
      printJob.printJobFile = new byte[] { 23, 62, 42, 88, 99, 0, -1, 127, -128 };
      printJob.printIgekopf = true;
      printJob.status = "It's ok";
      printJob.listData = "This is the list data";
      printJob.standDatum = DateConverter.stringToSqlDate( "05.03.1903" );
      printJob.registerKz = 23;
      printJob.schutitTyp = 7;
      printJob.regauszTyp = 1;
      printJob.sprache = 3;
      printJob.archiveCopy = false;
      printJob.maNr = 749;
      printJob.coverCd = false;
      printJob.hasPotZahl = true;
      printJob.watermark = "I'm a test, tralala";
      return printJob;

    }
    catch ( ParseException e )
    {
      throw new SQLException( "Couldn't parse date: " + e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( PrintJobVO originalPrintJob, PrintJobVO serializedPrintJob )
  {
    return originalPrintJob.printJobId == serializedPrintJob.printJobId && originalPrintJob.batchId == serializedPrintJob.batchId
        && originalPrintJob.druckerId == serializedPrintJob.druckerId && originalPrintJob.paperMode == serializedPrintJob.paperMode
        && originalPrintJob.anzKopien == serializedPrintJob.anzKopien && originalPrintJob.druckStatus == serializedPrintJob.druckStatus
        && originalPrintJob.senderApp.equals( serializedPrintJob.senderApp ) && originalPrintJob.senderUser.equals( serializedPrintJob.senderUser )
        && originalPrintJob.formTyp.equals( serializedPrintJob.formTyp ) && originalPrintJob.reportId == serializedPrintJob.reportId
        && originalPrintJob.coverCd == serializedPrintJob.coverCd && Arrays.equals( originalPrintJob.printJobFile, serializedPrintJob.printJobFile )
        && originalPrintJob.adrNr == serializedPrintJob.adrNr && originalPrintJob.printIgekopf == serializedPrintJob.printIgekopf
        && originalPrintJob.status.equals( serializedPrintJob.status ) && originalPrintJob.listData.equals( serializedPrintJob.listData )
        && originalPrintJob.standDatum.equals( serializedPrintJob.standDatum ) && originalPrintJob.registerKz == serializedPrintJob.registerKz
        && originalPrintJob.schutitTyp == serializedPrintJob.schutitTyp && originalPrintJob.regauszTyp == serializedPrintJob.regauszTyp
        && originalPrintJob.sprache == serializedPrintJob.sprache && originalPrintJob.archiveCopy == serializedPrintJob.archiveCopy
        && originalPrintJob.maNr == serializedPrintJob.maNr && originalPrintJob.hasPotZahl == serializedPrintJob.hasPotZahl
        && originalPrintJob.watermark.equals( serializedPrintJob.watermark ) && originalPrintJob.erstDatum.equals( serializedPrintJob.erstDatum );
  }

  /**
   * Gets the PDFs of a print job and saves them to the local temp path.
   * @param args Just one argument: The print job ID
   * @throws Exception if the PDF can't be read
   */
  public static void main( String[] args ) throws Exception
  {
    if ( args == null || args.length != 1 )
    {
      throw new IllegalArgumentException( "Expecting one argument: Print job ID" );
    }
    Integer printJobId = null;
    try
    {
      printJobId = Integer.valueOf( args[0] );
    }
    catch ( Exception e )
    {
      throw new IllegalArgumentException( "Expecting one argument: Print job ID" );
    }

    PrintJobVO printJob = new PrintjobDAO().readPrintjob( printJobId );
    File pdfFile = File.createTempFile( Integer.toString( printJob.printJobId ), FileConstants.EXTENSION_PDF );
    FileUtils.writeByteArrayToFile( pdfFile, printJob.printJobFile );
    System.out.print( "PDF written to " + pdfFile );
  }
}
