package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.DATASOURCE_FOLDER_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.DESTINATION_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FAILURE_CODE;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FOLDER_MARK_FOR_SESSION;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.SHARE_DESTINATION;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.io.FilenameUtils;

import ch.ige.ftp.IGEFTPClientAdapter;
import ch.ige.ftp.jakarta.IGEFTPClientAdapterJakarta;
import ch.ige.jipps.ftp.web.util.FTPAccessConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

/**
 * Download the data from a folder on an FTP server to a local folder.
 * <p>
 * Required request parameters:
 * <ol>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#DATASOURCE_FOLDER_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#EMPLOYEE_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#DESTINATION_REQUEST_PARAMETER}</li>
 * </ol>
 * <p>
 * Optional parameters:
 * <ol>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#SERVERPPATH_REQUEST_PARAMETER}</li>
 * </ol> 
 * @author fabrizioli
 *
 */
public class DownloadService extends AbstractService
{

  public DownloadService( ServletContext context )
  {
    super( context );
  }

  public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
  {

    String clientfolderPath = null;
    String serverfolderPath = null;
    String provenience = null;
    String ma = null;
    File clientfolder = null;
    try
    {
      ma = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.EMPLOYEE_REQUEST_PARAMETER, true );
      clientfolderPath = FTPServletConfiguration.extractRequestParameter( request, DATASOURCE_FOLDER_REQUEST_PARAMETER, true );
      provenience = FTPServletConfiguration.extractRequestParameter( request, DESTINATION_REQUEST_PARAMETER, true );
      serverfolderPath = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.SERVERPPATH_REQUEST_PARAMETER, false );
      HttpSession session = request.getSession();
      if (session != null)
      {
         String sessionMarker = (String)session.getAttribute( FOLDER_MARK_FOR_SESSION );
         if ( sessionMarker == null)
         {
           LOG.error( "No session entry for marking folder for session" );
         }
         else
         {
           serverfolderPath += sessionMarker;
           LOG.info( "Folder for writing patent documents " + serverfolderPath );
         }
      }
    }
    catch ( Exception e )
    {
      String msg = "FTP request parameter for destination or folderpath or employee do not exist: Cannot execute ftp request";
      LOG.error( msg );
      // configuration failure, data are not stored 
      throw new FTPServletRuntimeException( msg );
    }
    Map<String, FTPAccessConfigVO> ftpLoginParameters = (Map<String, FTPAccessConfigVO>) this.context
        .getAttribute( FTPServletConfiguration.CONTEXT_FTPLOGIN_PARAMTERER_MAP );

    FTPAccessConfigVO ftpParameters = ftpLoginParameters.get( provenience );
    if ( ftpParameters == null )
    {
      String msg = "FTP Paramters for destination [" + provenience + "] do not exist: Cannot execute ftp request";
      LOG.error( msg );
      // Configuration failure, data are not stored 
      throw new FTPServletRuntimeException( msg );
    }

    clientfolderPath = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( clientfolderPath, ma );
    clientfolder = new File( clientfolderPath );
    if ( !clientfolder.exists() )
    {
      boolean created = clientfolder.mkdirs();
      if ( !created )
      {
        String msg = "Folder [" + clientfolderPath + "] could not be created: Cannot execute ftp request";
        LOG.error( msg );

        throw new FTPServiceException( FAILURE_CODE + " Keine Daten von " + provenience + " Server gelesen: Verzeichnis [" + clientfolderPath
            + "] konnte nicht generiert werden." );
      }
      else
      {
        LOG.debug( "Foldre created for download: " + clientfolderPath );
      }
    }

    List<String> downloadedFilesPaths = null;

    IGEFTPClientAdapter client = new IGEFTPClientAdapterJakarta( ftpParameters.getFTPHost(), ftpParameters.getFTPUser(), ftpParameters.getFTPPasswort(),
        clientfolderPath, ftpParameters );

    try
    {
      client.clearLastRequestInfo();
      client.login();
      if ( serverfolderPath != null )
      {
        // separators are changed to client system, but adapted to server system, when calling setServerPath
        serverfolderPath = FilenameUtils.concat( ftpParameters.getFTPServerPath(), serverfolderPath );
      }
      else
      {
        serverfolderPath = ftpParameters.getFTPServerPath();
      }
      client.setRelativeServerPath( serverfolderPath, false );
      downloadedFilesPaths = client.downloadData( null );
      LOG.debug( "The number of files uploaded is " + downloadedFilesPaths.size() );
      client.logout();
      String serverUrl = storeDataOnShareForDownload( clientfolderPath, ma, ftpLoginParameters.get( FTPServletConfiguration.SHARE_DESTINATION ), false );
      bean.setServerUrl( serverUrl );
      if ( downloadedFilesPaths == null || downloadedFilesPaths.size() == 0 )
      {
        throw new FTPServiceException( FAILURE_CODE + " Keine Daten auf " + provenience + " Server gesandt." );
      }
      else
      {
        bean.setUploadedFilesPaths( downloadedFilesPaths );
      }
      return "downloadResult";
    }
    catch ( Throwable e )
    {
      LOG.error( "Error in ftp download", e );
      FTPAccessConfigVO ftpParametersShare = ftpLoginParameters.get( SHARE_DESTINATION );

      downloadedFilesPaths = client.getLastRequestInfo();
      LOG.debug( "The files downloaded were " + ( downloadedFilesPaths == null ? 0 : downloadedFilesPaths.size() ) );
      if ( downloadedFilesPaths != null && !downloadedFilesPaths.isEmpty() )
      {
        String serverUrl = storeDataOnShareForDownload( clientfolderPath, ma, ftpParametersShare, true );
        LOG.info( "Data stored after Download failure to " + serverUrl );
        bean.setServerUrl( serverUrl );
        bean.setUploadedFilesPaths( downloadedFilesPaths );
      }
      throw new FTPServiceException( FTPServletConfiguration.generateErrorMessage( e, " Keine Daten /Nicht alle Daten von " + provenience
          + " Server gelesen, Fehler beim Download  [" + e.getMessage() + "]" ) );

    }
    finally
    {
      if ( clientfolder != null )
      {
        try
        {
          deleteTmpDirectory( clientfolder );
        }
        catch ( IOException e )
        {
          LOG.error( "Error in deleting the folder after upload " + clientfolder.getAbsolutePath(), e );
        }
        if ( clientfolder.exists() )
        {
          LOG.error( " The temporary Folder was not deleted successfully: " + clientfolder );
        }
      }

    }
  }

}
