package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.DATASOURCE_FOLDER_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.DESTINATION_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FAILURE_CODE;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FOLDER_MARK_FOR_SESSION;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.SHARE_DESTINATION;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.io.FilenameUtils;

import ch.ige.ftp.IGEFTPClientAdapter;
import ch.ige.ftp.jakarta.IGEFTPClientAdapterJakarta;
import ch.ige.jipps.ftp.web.util.FTPAccessConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

/**
 * Upload the data read recursively from a local folder to an FTP server, generating the same folders on the server
 * <p>
 * Required request parameters:
 * <ol>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#DATASOURCE_FOLDER_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#EMPLOYEE_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#DESTINATION_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#BIBLIOGRAPHIC_DATA_PARAMETER}</li>
 * </ol> 
 * @author fabrizioli
 *
 */
public class UploadService extends AbstractService
{

  public UploadService( ServletContext context )
  {
    super( context );
  }

  public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
  {

    String folderPath = null;
    String destination = null;
    String ma = null;
    File folder = null;
    boolean isBiblioRequest = false;
    try
    {
      ma = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.EMPLOYEE_REQUEST_PARAMETER, true );
      folderPath = FTPServletConfiguration.extractRequestParameter( request, DATASOURCE_FOLDER_REQUEST_PARAMETER, true );
      String biblio = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.BIBLIOGRAPHIC_DATA_REQUEST_PARAMETER, false );
      isBiblioRequest = ( biblio != null && biblio.length() > 0 ) ? true : false;
      HttpSession session = request.getSession();
      if ( session != null )
      {
        String sessionMarker = (String) session.getAttribute( FOLDER_MARK_FOR_SESSION );
        if ( sessionMarker == null )
        {
          LOG.error( "No session entroy for marking folder for session" );
        }
        else
        {
          folderPath += File.separator + sessionMarker;
          LOG.info( "Folder for reading patent documents " + folderPath );
        }
      }
      destination = FTPServletConfiguration.extractRequestParameter( request, DESTINATION_REQUEST_PARAMETER, true );
    }
    catch ( Exception e )
    {
      String msg = "FTP request parameter for destination or folderpath or employee do not exist: Cannot execute ftp request";
      LOG.error( msg );
      // configuration failure, data are not stored 
      //response.sendError( 400, msg );
      throw new FTPServletRuntimeException( msg );
    }
    Map<String, FTPAccessConfigVO> ftpLoginParameters = (Map<String, FTPAccessConfigVO>) this.context
        .getAttribute( FTPServletConfiguration.CONTEXT_FTPLOGIN_PARAMTERER_MAP );

    FTPAccessConfigVO ftpParameters = ftpLoginParameters.get( destination );
    if ( ftpParameters == null )
    {
      String msg = "FTP Paramters for destination [" + destination + "] do not exist: Cannot execute ftp request";
      LOG.error( msg );
      // Configuration failure, data are not stored 
      //response.sendError( 400, msg );
      throw new FTPServletRuntimeException( msg );
    }

    folderPath = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( folderPath, ma );
    if ( isBiblioRequest )
    {
      folderPath = FilenameUtils.concat( folderPath, FTPServletConfiguration.MARKER_FOR_BIBLIO_FOLDER_NAME );
    }
    folder = new File( folderPath );
    if ( !folder.exists() )
    {
      String msg = "Folder [" + folderPath + "] does not exist: Cannot execute ftp request";
      LOG.error( msg );
      throw new FTPServiceException( FAILURE_CODE + " Keine Daten auf " + destination + " Server gesandt: Verzeichnis [" + folderPath + "] existiert nicht." );
    }

    List<String> uploadedFilesPaths = null;

    IGEFTPClientAdapter client = new IGEFTPClientAdapterJakarta( ftpParameters.getFTPHost(), ftpParameters.getFTPUser(), ftpParameters.getFTPPasswort(),
        folderPath, ftpParameters );

    try
    {
      client.clearLastRequestInfo();
      client.login();
      client.setRelativeServerPath( ftpParameters.getFTPServerPath(), false );
      uploadedFilesPaths = client.uploadData( null, true, true );
      LOG.debug( "The number of files uploaded from " + folderPath + " is " + uploadedFilesPaths.size() );
      client.logout();
      folderPath = getFolderForCopy( folderPath, isBiblioRequest );
      String serverUrl = storeDataOnShareForUpload( folderPath, ma, ftpLoginParameters.get( FTPServletConfiguration.SHARE_DESTINATION ), false );
      bean.setServerUrl( serverUrl );
      if ( uploadedFilesPaths == null || uploadedFilesPaths.size() == 0 )
      {
        throw new FTPServiceException( FAILURE_CODE + " Keine Daten auf " + destination + " Server gesandt." );
      }
      else
      {
        bean.setUploadedFilesPaths( uploadedFilesPaths );
      }
      return "uploadResult";
    }
    catch ( Throwable e )
    {
      LOG.error( "Error in ftp upload", e );
      folderPath = getFolderForCopy( folderPath, isBiblioRequest );
      FTPAccessConfigVO ftpParametersShare = ftpLoginParameters.get( SHARE_DESTINATION );
      String serverUrl = storeDataOnShareForUpload( folderPath, ma, ftpParametersShare, true );
      LOG.info( "Data stored after Upload failure to " + serverUrl );
      uploadedFilesPaths = client.getLastRequestInfo();
      LOG.debug( "The files uploaded were " + ( uploadedFilesPaths == null ? 0 : uploadedFilesPaths.size() ) );
      bean.setServerUrl( serverUrl );
      bean.setUploadedFilesPaths( uploadedFilesPaths );
      throw new FTPServiceException( FTPServletConfiguration.generateErrorMessage( e, " Keine Daten /Nicht alle Daten auf " + destination
          + " Server gesandt, Fehler beim Upload  [" + e.getMessage() + "]" ) );

    }
    finally
    {
      if ( folder != null )
      {
        try
        {
          deleteTmpDirectory( folder );
        }
        catch ( IOException e )
        {
          LOG.error( "Error in deleting the folder after upload " + folder.getAbsolutePath(), e );
        }
        if ( folder.exists() )
        {
          LOG.error( " The temporary Folder was not deleted successfully: " + folder );
        }
      }
      else
      {
        LOG.info( "No folder deleted." );
      }

    }
  }

  private String getFolderForCopy( String folderPath, boolean isBiblioRequest )
  {
//    if ( isBiblioRequest )
//    {
//      // also load pdf to copy
//      int biblioIndex = folderPath.lastIndexOf( FTPServletConfiguration.MARKER_FOR_BIBLIO_FOLDER_NAME );
//      if ( biblioIndex > 0 )
//      {
//        folderPath = folderPath.substring( 0, biblioIndex );
//      }
//      else
//      {
//        LOG.warn( "The folderPath is not the expected one, biblio not contained " + folderPath );
//      }
//    }
    return folderPath;
  }
}
