package ch.ige.jipps.header;

import java.sql.Date;
import java.util.Locale;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.util.constants.NumericLiterals;
import ch.ige.util.date.LanguageDateFormatter;
import ch.ige.util.enums.PublicationTarget;
import ch.ige.util.format.ApplicationNumberFormatter;
import ch.ige.util.format.ReportApplicationNumberFormatter;
import ch.ige.util.format.ReportIpNumberFormatter;

/**
 * Builds the special cases for the IP header in the report for all default cases.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: BaseIpHeaderBuilder.java,v 1.24 2008/09/23 07:54:48 lewis_lee Exp $
 */
abstract class BaseIpHeaderBuilder implements IpHeaderBuilder
{
  /**
   * This is usually the default to return.
   */
  protected static final String EMPTY = "";

  // See interface
  @SuppressWarnings( "unused" )
  public String getDescription( GenericIP ip, Date reportDate )
  {
    return ip.getDescription();
  }

  // See interface
  public Locale getReportLanguage( GenericIP ip, Locale language )
  {
    Locale reportLanguage = null;
    // Take the language that the user requested, if possible, but
    // default to the IP's language.
    if ( language != null )
    {
      reportLanguage = language;
    }
    else
    {
      reportLanguage = ip.getLocale();
    }
    // In general, there are no English reports, use German instead
    if ( reportLanguage.equals( Locale.ENGLISH ) )
    {
      reportLanguage = Locale.GERMAN;
    }
    return reportLanguage;
  }

  // See interface
  public String getFormattedIpNumber( GenericIP ip )
  {
    return ReportIpNumberFormatter.formatWithDefault( ip.getIPNumber(), ip.getBAGISType() );
  }

  // See interface
  @SuppressWarnings( "unused" )
  public String getFormattedBaseIpNumber( GenericIP ip )
  {
    return EMPTY;
  }

  // See interface
  public String getApplicationDate( GenericIP ip, LanguageDateFormatter dateFormatter )
  {
    if ( ip.getApplicationDate() == null )
    {
      return EMPTY;
    }
    else
    {
      return Header.formatDate( ip.getApplicationDate(), dateFormatter );
    }
  }

  // See interface
  @SuppressWarnings( "unused" )
  public String getApplicationPublicationDate( GenericIP ip, LanguageDateFormatter dateFormatter )
  {
    return EMPTY;
  }

  // See interface
  public String getFormattedApplicationNumber( GenericIP ip )
  {
    if ( ip.getApplicationNumber() != null )
    {
      return ReportApplicationNumberFormatter.format( ip.getApplicationNumber(), ip.getBAGISType() );
    }
    else
    {
      return getZeroFormattedApplicationNumber( ip );
    }
  }

  /**
   * Gets a formatted dummy application number, which is zero.
   * This is only done because it's always been done like this ;-)
   * @param ip The IP containing all data
   * @return The formatted dummy application number
   */
  protected String getZeroFormattedApplicationNumber( GenericIP ip )
  {
    if ( Types.PATENT_PCT.isInstance( ip ) )
    {
      return ApplicationNumberFormatter.formatPct( NumericLiterals.ZERO.toString(), ip.getBAGISType(), "  " );
    }
    else
    {
      return ReportApplicationNumberFormatter.format( NumericLiterals.ZERO.toString(), ip.getBAGISType() );
    }
  }

  // See interface
  @SuppressWarnings( "unused" )
  public String getBaseProtectionStart( GenericIP ip, LanguageDateFormatter dateFormatter )
  {
    return EMPTY;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public String getOppositionDeadline( GenericIP ip, Date reportDate, LanguageDateFormatter dateFormatter )
  {
    return EMPTY;
  }

  // See interface
  @SuppressWarnings( "unused" )
  public PublicationVO getPublication( GenericIP ip, LanguageDateFormatter dateFormatter, Date reportDate, boolean isFirstPublication )
  {
    return getDefaultPublicationResults( ip, reportDate, isFirstPublication );
  }

  /**
   * Gets the event for publication.
   * @param ip The IP containing all data
   * @param reportDate The date when the report is printed
   * @param isFirstPublication If this is <code>true</code>, the first publication
   *        is fetched, otherwise the last publication
   * @return The event for first or last publication
   */
  protected Event getPublicationEvent( GenericIP ip, Date reportDate, boolean isFirstPublication )
  {
    Event publication = null;
    if ( isFirstPublication )
    {
      publication = ip.getFirstPublication( EreigIDat.max( reportDate ) );
    }
    else
    {
      publication = ip.getLastPublication( EreigIDat.max( reportDate ) );
    }
    return publication;
  }

  /**
   * Gets the default results for publication.
   * @param ip The IP to check for electronic publication
   * @param reportDate The date when the report is printed
   * @param isFirstPublication If this is <code>true</code>, the first publication
   *        is fetched, otherwise the last publication
   * @return The default values for the publication
   * @see #getPublication(GenericIP, LanguageDateFormatter, Date, boolean)
   */
  protected PublicationVO getDefaultPublicationResults( GenericIP ip, Date reportDate, boolean isFirstPublication )
  {
    // Set default values
    PublicationVO publication = new PublicationVO();
    publication.formattedPublicationDate = EMPTY;
    publication.publicationYear = NumericLiterals.ZERO;
    publication.publicationNumber = null;
    publication.publishPublicationEvent = false;

    Event publicationEvent = getPublicationEvent( ip, reportDate, isFirstPublication );
    publication.isPublishedElectronically = false;
    if ( publicationEvent != null && publicationEvent.getDate() != null )
    {
      publication.isPublishedElectronically = publicationEvent.getPublicationTarget().equals( PublicationTarget.SWISSREG2 );
    }

    return publication;
  }
}
