package ch.ige.jipps.history;

import java.sql.Date;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.vo.PrioritaetVO;
import ch.ige.ipobjects.country.dao.CountryDAO;
import ch.ige.ipobjects.country.vo.CountryVO;
import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriod;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.jipps.text.ConvertText;
import ch.ige.util.constants.ReportTextNumberHandler;

/**
 * The base class for the detail lines of an IP's priorities.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: BasePrioritiesDetail.java,v 1.9 2007/12/19 08:56:33 lewis_lee Exp $
 * @param <T> The time period type
 */
public abstract class BasePrioritiesDetail<T extends TimePeriod> extends BaseHistoryItemDetail<T>
{
  /**
   * The log
   */
  static final Logger LOG = Logger.getLogger( BasePrioritiesDetail.class );

  /**
   * The IP whose language is used for the country name
   */
  protected GenericIP ip;

  /**
   * Gets the IP.
   * @return The IP
   */
  public GenericIP getIp()
  {
    return this.ip;
  }

  /**
   * The constructor
   * @param priorities The IP's priorities
   * @param ipType The IP's type
   * @param ip The generic IP
   */
  public BasePrioritiesDetail( TimePeriodCollection<T> priorities, IPType ipType, GenericIP ip )
  {
    super( priorities, ipType, null );
    this.ip = ip;
  }

  /**
   * The class that actually formats the priorities.
   */
  public abstract class BasePrioritiesDetailBuilder extends AbstractHistoryItemDetailBuilder<T, Detail>
  {
    /**
     * Creates the builder.
     */
    public BasePrioritiesDetailBuilder()
    {
      super();
    }

    /**
     * Builds the detail lines for the priorities.
     * @param detailData The priorities time period
     * @param priorities The priorities
     * @param globalRemark A remark that's valid for the whole detail, which is ignored if <code>null</code>
     * @return The formatted details
     */
    protected HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<T> detailData, List<PrioritaetVO> priorities, String globalRemark )
    {
      if ( !detailData.isEmpty() )
      {
        List<String> formattedPriorities = new ArrayList<String>();
        for ( PrioritaetVO priority: priorities )
        {
          // The country name must be in the IP language, not the report language
          formattedPriorities.addAll( getFormattedPriorities( priority, BasePrioritiesDetail.this.getIp().getLocale() ) );
        }

        // Add global remarks
        formattedPriorities.addAll( ConvertText.splitLines( globalRemark ) );

        if ( !formattedPriorities.isEmpty() )
        {
          Detail detail = new Detail( detailData.get( 0 ), BasePrioritiesDetail.this );
          detail.reportTextNumber = ReportTextNumberHandler.getReportTextNumberForPriority( BasePrioritiesDetail.this.getIpType().getBAGISType() );
          detail.textLines = formattedPriorities;
          return createListManyDataItemsSingleRepresentation( detailData, detail );
        }
        else
        {
          return EmptyHistoryItemDetailList.getInstance();
        }
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }

    /**
     * Formats and adds the given priority data to <code>allFormattedPriorities</code>.
     * @param priority The priority
     * @param priorityLanguage The language to use for formatting
     * @return The formatted priorities
     */
    private List<String> getFormattedPriorities( PrioritaetVO priority, Locale priorityLanguage )
    {
      List<String> formattedPriorities = new ArrayList<String>();
      String countryName = null;
      if ( StringUtils.isNotEmpty( priority.getBland_cd() ) )
      {
        try
        {
          CountryVO country = new CountryDAO().findCountry( priority.getBland_cd() );
          // Like with addresses, the country name is taken from the IP's language,
          // not the report language
          countryName = country.getLand_bez( priorityLanguage );
        }
        catch ( SQLException ex )
        {
          BasePrioritiesDetail.LOG.error( "Can't load country with code " + priority.getBland_cd(), ex );
        }
      }

      String formattedPriority = formatPriority( priority.getBland_cd(), countryName, priority.getPrio_dat(), priority.getPrio_nr(), priority.getPrio_bem() );
      if ( StringUtils.isNotEmpty( formattedPriority ) )
      {
        formattedPriorities.add( formattedPriority );
      }

      formattedPriorities.addAll( ConvertText.splitLines( priority.getPrio_bem() ) );

      return formattedPriorities;
    }
  }

  /**
   * Formats the given priority and returns the formatted version.
   * @param countryCode The two-letter country code (actually an ST.3 code)
   * @param countryName The country's name
   * @param priorityDate The date of the priority
   * @param priorityNumber The number of the priority
   * @param priorityRemark The remark about the priority
   * @return The formatted priority
   */
  protected abstract String formatPriority( String countryCode, String countryName, Date priorityDate, String priorityNumber, String priorityRemark );
}
