package ch.ige.jipps.history;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.report.dao.RegAuszGrpTypDAO;
import ch.ige.ipobjects.report.dao.RegAuszTypSTypDAO;
import ch.ige.ipobjects.report.dao.RegFormTextDAO;
import ch.ige.ipobjects.report.history.HistoryBuilder.EmptyHistoryException;
import ch.ige.ipobjects.report.history.impl.DefaultHistoryBuilder;
import ch.ige.ipobjects.report.vo.RegAuszGrpTypVO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.ipobjects.report.vo.RegFormTextVO;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.jipps.history.description.BaseHistoryDescription;
import ch.ige.jipps.history.description.DesignHistoryDescription;
import ch.ige.jipps.history.description.PatentPCTHistoryDescription;
import ch.ige.jipps.history.description.TrademarkHistoryDescription;
import ch.ige.jipps.text.EventTextLoader;
import ch.ige.jipps.text.ReportTextLoader;
import ch.ige.jipps.vo.HistoryItemVO;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.enums.report.HistoryStart;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;

/**
 * Builds the objects for the history in the report.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: History.java,v 1.47 2008/04/18 14:15:54 lewis_lee Exp $
 */
public class History
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( History.class );

  /**
   * Gets the history for the report.
   * @param ip The IP with all required data
   * @param currentState The history is built for this date
   * @param language The language of the report
   * @param reportType The report type
   * @return The history items in the correct (chronological) sequence for printing
   * @throws SQLException if the report texts can't be fetched
   */
  public static List<HistoryItemVO> getHistory( GenericIP ip, PointInTime currentState, Locale language, ReportType reportType ) throws SQLException
  {
    RegAuszTypSTypVO ipTypeReportType = new RegAuszTypSTypDAO().findIpTypeReportType( ip.getBAGISType(), reportType );
    Boolean startFromApplication = null;
    if ( ipTypeReportType.getHistoryStart().equals( HistoryStart.STARTING_FROM_APPLICATION ) )
    {
      startFromApplication = Boolean.TRUE;
    }
    else if ( ipTypeReportType.getHistoryStart().equals( HistoryStart.STARTING_FROM_REGISTRATION ) )
    {
      startFromApplication = Boolean.FALSE;
    }
    else
    {
      throw new IllegalArgumentException( "The report type <" + reportType + "> has an invalid flag for report history" );
    }

    /** TODO This should be configurable and saved in the database (regausztyp_styp) */
    boolean includeWaitListEvents = false;
    if ( reportType.equals( ReportType.DOSSIER_REPORT_AFTER_APPLICATION ) )
    {
      includeWaitListEvents = true;
    }

    List<HistoryItemVO> historyItems = new ArrayList<HistoryItemVO>();
    Map<Integer, String> allowedReportGroups = getAllowedReportGroups( reportType );
    BaseHistoryDescription historyDescription = createHistoryDescription( ip, currentState, language, includeWaitListEvents, startFromApplication
        .booleanValue(), allowedReportGroups, reportType );
    try
    {
      // Visit the history events and build the VOs for the history items
      historyItems = new DefaultHistoryBuilder<HistoryItemVO>( ip, historyDescription ).createHistory();
    }
    catch ( EmptyHistoryException ex )
    {
      LOG.warn( "Found no history for IP with key <" + ip.getIPKey() + ">", ex );
    }

    // Load the texts
    EventTextLoader.insertOptionalEventText( historyItems, language, false );
    ReportTextLoader.insertOptionalReportText( historyItems, ip.getType(), language, reportType, ReportArea.HISTORY, false );

    // Set the end date label
    RegFormTextVO endDateLabel = new RegFormTextDAO().findReportText( ReportTextConstants.VALID_TO_INTEGER, language, ReportArea.MISCELLANEOUS );
    for ( HistoryItemVO historyItem: historyItems )
    {
      if ( historyItem.getEndDate() != null )
      {
        historyItem.setEndDateLabel( endDateLabel.getText() + " " );
      }
    }

    return historyItems;
  }

  /**
   * Creates the history description which is specific for an IP type.
   * @param ip The IP with all required data
   * @param currentState The history is built for this date
   * @param language The language of the report
   * @param includeWaitListEvents If this is <code>true</code>, events that are on the wait list are also shown
   * @param startFromApplication If this is <code>true</code>, events starting from the application are shown,
   * otherwise from the registration
   * @param allowedReportGroups The report groups, a <code>Map</code> where the keys show whether an event must be
   * included on a report. The values can be ignored.
   * @param reportType The report type
   * @return The IP-specific history description
   */
  private static BaseHistoryDescription createHistoryDescription( GenericIP ip, PointInTime currentState, Locale language, boolean includeWaitListEvents,
      boolean startFromApplication, Map<Integer, String> allowedReportGroups, ReportType reportType )
  {
    if ( Types.PATENT_PCT.isInstance( ip ) )
    {
      return new PatentPCTHistoryDescription( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
    }
    else if ( Types.TRADEMARK.isInstance( ip ) )
    {
      return new TrademarkHistoryDescription( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
    }
    else if ( Types.DESIGN.isInstance( ip ) )
    {
      return new DesignHistoryDescription( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
    }
    else
    {
      return new BaseHistoryDescription( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
    }
  }

  /**
   * Gets the report groups which are allowed for this report type.
   * @param reportType The report type
   * @return The report groups, a <code>Map</code> where the keys are <code>Integer</code> objects that show whether
   * an event must be included on a report. The values can be ignored.
   * @throws SQLException if the report groups can't be fetched
   */
  private static Map<Integer, String> getAllowedReportGroups( ReportType reportType ) throws SQLException
  {
    Map<Integer, String> reportGroups = new HashMap<Integer, String>();

    RegAuszGrpTypDAO reportGroupDAO = new RegAuszGrpTypDAO();
    List<RegAuszGrpTypVO> reportGroupObjects = reportGroupDAO.findReportGroups( reportType );
    for ( RegAuszGrpTypVO reportGroup: reportGroupObjects )
    {
      reportGroups.put( reportGroup.regAuszGrp, reportGroup.bem );
    }
    return reportGroups;
  }
}
