package ch.ige.jipps.test;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.jipps.LetterAssembler;
import ch.ige.jipps.constants.LetterTypeConstants;
import ch.ige.jipps.vo.BfakTxtVO;
import ch.ige.jipps.vo.JippsFakturVO;
import ch.ige.jipps.vo.LetterVO;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.LetterAssembler}
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: LetterAssemblerTest.java,v 1.7 2008/07/23 13:49:18 lewis_lee Exp $
 */
public class LetterAssemblerTest extends TestCase
{
  static
  {
    ClientLogInitalizer.initializeLoggingForTests( true, true );
    ClientInitializer.initStandaloneClient();
  }

  /**
   * This is an image trademark.
   */
  private static final int TRADEMARK_KEY = 1201844266;

  /**
   * The ID of the letter sent for the application confirmation.
   */
  private static final int INVOICE_ID = 6218309;

  /**
   * The date of the letter.
   */
  private static final String INVOICE_DATE = "08.01.2007";

  /**
   * The only constructor
   * @param name The test's name
   */
  public LetterAssemblerTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link LetterAssembler#loadImage(ch.ige.jipps.vo.LetterVO)}.
   * @throws Exception if the test fails
   */
  public void testLoadBild() throws Exception
  {
    LetterVO letter = createLetter();
    letter.jippsFakturVO.formartCd = "9991";
    assertNull( "Checking for invalid letter type", LetterAssembler.loadImage( letter ) );

    letter = createLetter();
    letter.jippsFakturVO.beschreibung = "Signore Pierpaoli, molto gentile";
    assertNull( "Checking for non-image trademark", LetterAssembler.loadImage( letter ) );

    letter = createLetter();
    letter.jippsFakturVO.schutitKey = 1200084524;
    assertNull( "Checking for non-trademark", LetterAssembler.loadImage( letter ) );

    letter = createLetter();
    letter.jippsFakturVO.bfaktDat = DateConverter.stringToDate( "01.01.1955" );
    assertNull( "Checking for unknown letter date", LetterAssembler.loadImage( letter ) );

    letter = createLetter();
    byte[] image = LetterAssembler.loadImage( letter );
    assertNotNull( "Checking for valid letter", image );
    assertEquals( "Checking image size", 21596, image.length );
  }

  /**
   * Tests {@link LetterAssembler#convertWDL(java.util.List)}.
   */
  public void testConvertWDL()
  {
    // Non-split WDL text, with WDL end tag
    List<BfakTxtVO> invoiceTexts = createInvoiceTexts( "Neu ist folgendes Waren- und/oder Dienstleistungsverzeichnis erfasst:<br><br>%%$$WDL 5 Produits pharmaceutiques; compléments nutritionnels à usage<br>   médical, tous les produits précités selon la formule tibétaine.<br>29 Compléments nutritionels à base de fruits à usage non médical.<br>%%*$WDL<br>Cl. int.: 5, 29." );
    String expectedResult = "Neu ist folgendes Waren- und/oder Dienstleistungsverzeichnis erfasst:<br><br><b>5</b><br>Produits pharmaceutiques; compléments nutritionnels à usage médical, tous les produits précités selon la formule tibétaine.<p/><b>29</b><br>Compléments nutritionels à base de fruits à usage non médical.<br><br>Cl. int.: 5, 29.";
    LetterAssembler.convertWDL( invoiceTexts );
    assertEquals( "Checking WDL within text", expectedResult, invoiceTexts.get( 0 ).text );

    // Several texts, non-split WDL text, with WDL end tag
    invoiceTexts = createInvoiceTexts(
        "bla",
        "Neu ist folgendes Waren- und/oder Dienstleistungsverzeichnis erfasst:<br><br>%%$$WDL 5 Produits pharmaceutiques; compléments nutritionnels à usage<br>   médical, tous les produits précités selon la formule tibétaine.<br>29 Compléments nutritionels à base de fruits à usage non médical.<br>%%*$WDL<br>Cl. int.: 5, 29.",
        "xyz" );
    expectedResult = "Neu ist folgendes Waren- und/oder Dienstleistungsverzeichnis erfasst:<br><br><b>5</b><br>Produits pharmaceutiques; compléments nutritionnels à usage médical, tous les produits précités selon la formule tibétaine.<p/><b>29</b><br>Compléments nutritionels à base de fruits à usage non médical.<br><br>Cl. int.: 5, 29.";
    LetterAssembler.convertWDL( invoiceTexts );
    assertEquals( "Checking non-WDL text", "bla", invoiceTexts.get( 0 ).text );
    assertEquals( "Checking WDL within text", expectedResult, invoiceTexts.get( 1 ).text );
    assertEquals( "Checking non-WDL text", "xyz", invoiceTexts.get( 2 ).text );

    // Several texts, split WDL text, with WDL end tag
    invoiceTexts = createInvoiceTexts(
        "bla",
        "Neu ist folgendes Waren- und/oder Dienstleistungsverzeichnis erfasst:<br><br>%%$$WDL 5 Produits pharmaceutiques; compléments nutritionnels à usage<br>   médical, tous les produits précités selon la formule tibétaine.<br>",
        "29 Compléments nutritionels à base de fruits à usage non médical.<br>%%*$WDL<br>Cl. int.: 5, 29.", "xyz" );
    expectedResult = "Neu ist folgendes Waren- und/oder Dienstleistungsverzeichnis erfasst:<br><br><b>5</b><br>Produits pharmaceutiques; compléments nutritionnels à usage médical, tous les produits précités selon la formule tibétaine.<br>";
    String expectedResult2 = "<b>29</b><br>Compléments nutritionels à base de fruits à usage non médical.<br><br>Cl. int.: 5, 29.";
    LetterAssembler.convertWDL( invoiceTexts );
    assertEquals( "Checking non-WDL text", "bla", invoiceTexts.get( 0 ).text );
    assertEquals( "Checking WDL within text", expectedResult, invoiceTexts.get( 1 ).text );
    assertEquals( "Checking WDL within text", expectedResult2, invoiceTexts.get( 2 ).text );
    assertEquals( "Checking non-WDL text", "xyz", invoiceTexts.get( 3 ).text );
  }

  /**
   * Creates invoice texts containing WDLs.
   * @param wdlTexts The WDL texts.
   * @return The invoice texts containing <code>wdlTexts</code>
   */
  private List<BfakTxtVO> createInvoiceTexts( String... wdlTexts )
  {
    List<BfakTxtVO> invoiceTexts = new ArrayList<BfakTxtVO>();
    for ( String wdlText: wdlTexts )
    {
      BfakTxtVO invoiceText = new BfakTxtVO();
      invoiceText.text = wdlText;
      invoiceTexts.add( invoiceText );
    }
    return invoiceTexts;
  }

  /**
   * Creates the letter to get the image with.
   * @return The letter for testing
   * @throws ParseException if the letter date can't be parsed
   */
  private LetterVO createLetter() throws ParseException
  {
    LetterVO letter = new LetterVO();
    letter.jippsFakturVO = new JippsFakturVO();
    letter.jippsFakturVO.formartCd = LetterTypeConstants.APPLICATION_CONFIRMATION_422;
    letter.jippsFakturVO.beschreibung = "Dr.Pierpaoli ((fig.))"; // This means it's an image trademark
    letter.jippsFakturVO.schutitKey = TRADEMARK_KEY;
    letter.jippsFakturVO.bfaktId = INVOICE_ID;
    letter.jippsFakturVO.bfaktDat = DateConverter.stringToDate( INVOICE_DATE );
    return letter;
  }
}
