package ch.ige.jipps.test;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;

import junit.framework.TestCase;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.jipps.EintrBeschControl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.StatusConstants;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.dao.PrintjobDAO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.printer.ReportPaper;

/**
 * Tests the registration confirmation (<code>Eintragungsbescheinigung</code>) using
 * {@link ch.ige.jipps.JIPPSFacadeServices#printEintrBesch()} and
 * {@link ch.ige.jipps.EintrBeschControl}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: RegistrationConfirmationTest.java,v 1.11 2008/09/22 08:31:39 lewis_lee Exp $
 */
public class RegistrationConfirmationTest extends TestCase
{
  static
  {
    ServicesFactory.initStandaloneClient();
  }

  /**
   * Used for general utility stuff, preparing the test etc.
   */
  private PrintjobDAO printJobDAO;

  /**
   * Used for preparing the test.
   */
  private BfakturDAO bfakturDAO;

  /**
   * The batch ID used for testing.
   */
  private Integer batchId;

  /**
   * The IDs of the invoices which are set up for printing during this test.
   */
  private static final int[] INVOICE_IDS = { 6758891, 6759044, 6759069 };

  /**
   * The only constructor
   * @param name The test's name
   */
  public RegistrationConfirmationTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    PathConfiguration.setPathsForLocalTest();

    this.printJobDAO = new PrintjobDAO();
    this.bfakturDAO = new BfakturDAO();
  }

  /**
   * Tests when the E-Publication is active and there are registation confirmations.
   * @throws Exception if the test fails
   */
  public void testRegistrationConfirmations() throws Exception
  {
    // Set up the DB to have something to process.
    for ( int invoiceId: INVOICE_IDS )
    {
      this.bfakturDAO.updateBFakturDruckstatus( invoiceId, PrintStatusConstants.READY_FOR_PRINTING_STRING );
    }

    // Lose the milliseconds and go back a minute, the times may differ between this server and the DB server
    Timestamp testStart = new Timestamp( ( System.currentTimeMillis() / 1000 * 1000 ) - 60 * 1000 );

    int status = new EintrBeschControl().printEintrBesch( false );
    assertEquals( "Checking that printing is ok", StatusConstants.OK, status );

    // It's not like this is especially intelligent...  ;-)
    // But it's all that we have... Just pray that there wasn't a batch run in-between!
    // Note that the batch run actually creates two batch jobs, one with reports and one with letters.
    this.batchId = this.printJobDAO.getNextBatchId() - 2;
    checkBatchRun( false, this.batchId, INVOICE_IDS.length, testStart );
    checkBatchRun( true, this.batchId + 1, INVOICE_IDS.length, testStart );
  }

  /**
   * Checks a batch run.
   * @param isLetter If this is <code>true</code>, it's a letter batch run; if it's <code>false</code>, it's a report batch run
   * @param currentBatchId The ID of the batch run
   * @param expectedNumber The expected number of print jobs
   * @param testStart The time when this test was started
   * @throws SQLException if the print jobs can't be read
   */
  private void checkBatchRun( boolean isLetter, int currentBatchId, int expectedNumber, Timestamp testStart ) throws SQLException
  {
    List<PrintJobVO> printJobs = this.printJobDAO.readPrintJobs( currentBatchId );
    assertTrue( "Checking that print jobs exist", !printJobs.isEmpty() );
    // There may be other print jobs ready to print, so don't check for strict equality.
    assertTrue( "Checking number of print jobs", printJobs.size() >= expectedNumber );
    for ( PrintJobVO printJob: printJobs )
    {
      assertEquals( "Checking batch ID", currentBatchId, printJob.batchId );
      assertTrue( "Checking printer ID", printJob.druckerId > 0 );
      assertEquals( "Checking print status", PrintStatusConstants.READY_FOR_PRINTING, printJob.druckStatus );
      assertEquals( "Checking sender application", PrintJobVO.SYSTEM, printJob.senderApp );
      assertEquals( "Checking sender user", PrintJobVO.SYSTEM, printJob.senderUser );
      assertEquals( "Checking cover", false, printJob.coverCd );
      assertTrue( "Checking PDF file", printJob.printJobFile.length > 5000 );
      assertTrue( "Checking creation date", printJob.erstDatum.after( testStart ) );
      if ( isLetter )
      {
        assertEquals( "Checking print job type", PrintJobVO.LETTER, printJob.formTyp );
        assertEquals( "Checking number of copies", 0, printJob.anzKopien );
        assertEquals( "Checking paper type for letters:", ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB().intValue(), printJob.paperMode );
      }
      else
      {
        assertEquals( "Checking print job type", PrintJobVO.AUSZUG, printJob.formTyp );
        assertEquals( "Checking number of copies", 1, printJob.anzKopien );
        assertEquals( "Checking paper type for reports:", ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB().intValue(), printJob.paperMode );
      }
    }
  }

  @Override
  protected void tearDown() throws Exception
  {
    // Clean up the mess...
    if ( this.batchId != null )
    {
      this.printJobDAO.deletePrintjobs( this.batchId );
      this.printJobDAO.deletePrintjobs( this.batchId + 1 );
    }

    super.tearDown();
  }
}
