package ch.ige.jipps.test;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Locale;

import org.custommonkey.xmlunit.SimpleXpathEngine;
import org.custommonkey.xmlunit.XMLTestCase;
import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAO;
import ch.ige.jipps.ReportControl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.header.IpHeaderVO;
import ch.ige.jipps.vo.AuszugVO;
import ch.ige.jipps.xml.AuszugWriteXml;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Checks whether the character encoding is 100% correct when marshalling objects to XML.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version FacadeXmlIpHeaderTest.java,v 1.3 2006/09/06 13:14:36 lewis_lee Exp
 */
public class XmlCharacterEncodingTest extends XMLTestCase
{
  /**
   * The first line of special characters
   */
  private static String LINE_1 = "! \" # $ % & ' ( ) * + , - . / ê ë ì í î ï ð ñ ò ó ô õ ö ÷ ø ù ú û ü ý þ ÿ € Š š Ž ž Œ œ Ÿ";

  /**
   * The second line of special characters
   */
  private static String LINE_2 = "? @ [ \\ ] ^ _ ` { | } ~ ¡ ¢ £ ¤ ¥ ¦ § ¨ © ª « ¬ ­";

  /**
   * The third line of special characters
   */
  private static String LINE_3 = "® ¯ ° ± ² ³ ´ µ ¶ • ¸ ¹ º » ¼";

  /**
   * The fourth line of special characters
   */
  private static String LINE_4 = "½ ¾ ¿ À Á Â Ã Ä Å Æ Ç È É Ê Ë";

  /**
   * The fifth line of special characters
   */
  private static String LINE_5 = "Ì Í Î Ï Ð Ñ Ò Ó Ô Õ Ö × Ø Ù Ú";

  /**
   * The sixth line of special characters
   */
  private static String LINE_6 = "Û Ü Ý Þ ß à á â ã ä å æ ç è é";

  static
  {
    ClientLogInitalizer.initializeLoggingForTests( true, true );
    AbstractDAO.setRunningInJ2EEServer( false );
    System.setProperty( DAO.STANDALONE_CONFIG_PROPERTY_NAME, "true" );
  }

  /**
   * The default constructor
   * @param args Any parameters
   */
  public XmlCharacterEncodingTest( String args )
  {
    super( args );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    XMLUnit.setIgnoreWhitespace( true );

    PathConfiguration.setPathsForLocalTest();
  }

  /**
   * Tests all characters for correct encoding in XML using JAXB.
   * @throws Exception
   */
  public void testAllCharacters() throws Exception
  {
    AuszugWriteXml writeXml = new AuszugWriteXml();
    String xml = writeXml.toXml( getReport() );

    Document xmlDocument = XMLUnit.buildTestDocument( xml );
    checkResult( xmlDocument, "/auszug/kopf/schutitel", LINE_1 );
    checkResult( xmlDocument, "/auszug/kopf/erstPublDat", LINE_2 );
    checkResult( xmlDocument, "/auszug/kopf/eintrDat", LINE_3 );
    checkResult( xmlDocument, "/auszug/kopf/anmeldDat", LINE_4 );
    checkResult( xmlDocument, "/auszug/kopf/gesuchNr", LINE_5 );
    checkResult( xmlDocument, "/auszug/kopf/stammEintrNr", LINE_6 );
  }

  /**
   * Tests all characters for correct encoding in a PDF.
   * @throws Exception
   */
  public void testAllCharactersInPdf() throws Exception
  {
    AuszugWriteXml writeXml = new AuszugWriteXml();
    AuszugVO report = getReport();
    String xml = writeXml.toXml( report );

    String xslName = ReportType.DOSSIER_REPORT_AFTER_APPLICATION.toDB() + "_" + report.getIpHeader().getIpType().toDB();
    TestControl control = new TestControl();
    byte[] pdfContent = control.createPdf( xml, xslName );

    File pdfFile = File.createTempFile( xslName, FileConstants.EXTENSION_PDF );
    FileOutputStream saveIt = new FileOutputStream( pdfFile.getPath() );
    saveIt.write( pdfContent );
    saveIt.close();
    System.out.print( "Test PDF written to " + pdfFile );
  }

  /**
   * Checks whether the content of the defined node is as expected.
   * @param document The whole XML document
   * @param xPath The XPath expression to check, defining the node
   * @param expectedContent The expected content, one of the weird lines
   * @throws Exception if the checking fails
   */
  private static void checkResult( Document document, String xPath, String expectedContent ) throws Exception
  {
    SimpleXpathEngine xPathEngine = new SimpleXpathEngine();
    NodeList nodes = xPathEngine.getMatchingNodes( xPath, document );
    assertTrue( "Checking that the node " + xPath + " was found ", nodes.getLength() == 1 );
    Node node = nodes.item( 0 );
    Node firstChild = node.getFirstChild();
    String receivedValue = firstChild.getNodeValue();
    assertEquals( "Checking node value for " + xPath, expectedContent, receivedValue );
  }

  /**
   * Creates a report containing all the weird characters one can imagine!
   * @return The report for testing
   */
  public static AuszugVO getReport()
  {
    AuszugVO report = new AuszugVO();

    // Fill the header with required stuff...
    IpHeaderVO ipHeader = new IpHeaderVO();
    ipHeader.setReportType( ReportType.REGISTER_REPORT_AFTER_REGISTRATION );
    ipHeader.setReportTitle( "Registerauszug - ESZ" );
    ipHeader.setIpKey( Integer.valueOf( 123456 ) );
    ipHeader.setIpType( IPType.SPC );
    ipHeader.setFirstPublicationYear( Integer.valueOf( 2006 ) );
    ipHeader.setLastPublicationYear( Integer.valueOf( 2006 ) );
    ipHeader.setEventOfFirstPublicationPublished( Boolean.FALSE );
    ipHeader.setEventOfLastPublicationPublished( Boolean.FALSE );
    ipHeader.setReportLanguage( Locale.GERMAN );
    report.setIpHeader( ipHeader );

    ipHeader.setDescription( LINE_1 );
    ipHeader.setFirstPublicationDate( LINE_2 );
    ipHeader.setRegistrationDate( LINE_3 );
    ipHeader.setApplicationDate( LINE_4 );
    ipHeader.setFormattedApplicationNumber( LINE_5 );
    ipHeader.setFormattedBaseIpNumber( LINE_6 );

    return report;
  }

  /**
   * This is used to create the PDF file.
   */
  private static class TestControl extends ReportControl
  {
    /**
     * Creates this.
     */
    TestControl()
    {
      // Nothing to do
    }

    @Override
    protected byte[] createPdf( String xml, String xslName ) throws Exception
    {
      return super.createPdf( xml, xslName );
    }
  }
}
