package ch.ige.jipps.text.test;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import junit.framework.TestCase;
import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.ipobjects.design.model.DesignType;
import ch.ige.ipobjects.model.IPType;
import ch.ige.jipps.text.ReportTextContainer;
import ch.ige.jipps.text.ReportTextLoader;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;

/**
 * Tests {@link ch.ige.jipps.text.ReportTextLoader}.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: ReportTextLoaderTest.java,v 1.12 2007/05/04 13:55:53 lewis_lee Exp $
 */
public class ReportTextLoaderTest extends TestCase
{
  static
  {
    ClientInitializer.initStandaloneClient();
  }

  /**
   * The report texts numbers to check
   */
  private static final Integer[] REPORT_TEXT_NUMBERS = new Integer[]
      {
      ReportTextConstants.DESIGN_CANCELLATION,
      ReportTextConstants.DESIGN_DESCRIPTION,
      ReportTextConstants.DESIGN_EXHIBITION_IMMUNITY,
      ReportTextConstants.DESIGN_APPLICATION_TYPE
  };

  /**
   * The German report texts to check
   */
  private static final String[] REPORT_TEXTS_GERMAN = new String[]
      {
      "Design gelöscht per:",
      "Bezeichnung des Gegenstandes",
      "Ausstellungsimmunität",
      "Art der Hinterlegung"
  };

  /**
   * The French report texts to check
   */
  private static final String[] REPORT_TEXTS_FRENCH = new String[]
      {
      "Design radié:",
      "Désignation de l'objet",
      "Immunité dérivée d'une exposition",
      "Forme de dépôt"
  };

  /**
   * The only constructor
   * @param name The test's name
   */
  public ReportTextLoaderTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.jipps.text.ReportTextLoader#insertReportText(List, IPType, Locale, ReportType, ReportArea, boolean)} with German texts.
   * @throws Exception if the test fails
   */
  public void testInsertReportTextForGerman() throws Exception
  {
    List<ReportTextContainer> reportTextContainers = createReportTextContainers();
    ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
    checkReportTextContainers( reportTextContainers, Locale.GERMAN );
  }

  /**
   * Tests {@link ch.ige.jipps.text.ReportTextLoader#insertReportText(List, IPType, Locale, ReportType, ReportArea, boolean)} with French texts.
   * @throws Exception if the test fails
   */
  public void testInsertReportTextForFrench() throws Exception
  {
    List<ReportTextContainer> reportTextContainers = createReportTextContainers();
    ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.FRENCH, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
    checkReportTextContainers( reportTextContainers, Locale.FRENCH );
  }

  /**
   * Tests {@link ch.ige.jipps.text.ReportTextLoader#insertReportText(List, IPType, Locale, ReportType, ReportArea, boolean)}, checking that the texts are unique.
   * @throws Exception if the test fails
   */
  public void testInsertReportTextForUniqueness() throws Exception
  {
    // Yes, I know that index and numbering aren't the same...  ;-)

    List<ReportTextContainer> reportTextContainers = createReportTextContainers();
    ReportTextContainer container2 = reportTextContainers.get( 2 );
    ReportTextContainer container3 = reportTextContainers.get( 3 );
    container3.setReportTextNumber( container2.getReportTextNumber() );
    ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.FRENCH, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, false );
    assertEquals( "Ignoring that the texts must be unique", REPORT_TEXT_NUMBERS[ 2 ], container2.getReportTextNumber() );
    assertEquals( "Ignoring that the texts must be unique", REPORT_TEXTS_FRENCH[ 2 ], container2.getReportText() );
    assertEquals( "Ignoring that the texts must be unique", REPORT_TEXT_NUMBERS[ 2 ], container3.getReportTextNumber() );
    assertEquals( "Ignoring that the texts must be unique", REPORT_TEXTS_FRENCH[ 2 ], container3.getReportText() );

    reportTextContainers = createReportTextContainers();
    container2 = reportTextContainers.get( 2 );
    container3 = reportTextContainers.get( 3 );
    container3.setReportTextNumber( container2.getReportTextNumber() );
    ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.FRENCH, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
    assertEquals( "Assuring that the texts must be unique", REPORT_TEXT_NUMBERS[ 2 ], container2.getReportTextNumber() );
    assertEquals( "Assuring that the texts must be unique", REPORT_TEXTS_FRENCH[ 2 ], container2.getReportText() );
    assertEquals( "Assuring that the texts must be unique", REPORT_TEXT_NUMBERS[ 2 ], container3.getReportTextNumber() );
    assertEquals( "Assuring that the texts must be unique", null, container3.getReportText() );
  }

  /**
   * Tests {@link ch.ige.jipps.text.ReportTextLoader#insertOptionalReportText(List, IPType, Locale, ReportType, ReportArea, boolean)}, checking that the texts are unique.
   * @throws Exception if the test fails
   */
  public void testInsertOptionalReportTextForUniqueness() throws Exception
  {
    // Yes, I know that index and numbering aren't the same...  ;-)

    List<ReportTextContainer> reportTextContainers = createReportTextContainers();
    ReportTextContainer container2 = reportTextContainers.get( 2 );
    ReportTextContainer container3 = reportTextContainers.get( 3 );
    container3.setReportTextNumber( null );
    ReportTextLoader.insertOptionalReportText( reportTextContainers, DesignType.getInstance(), Locale.FRENCH, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, false );
    assertEquals( "Checking existing text number", REPORT_TEXT_NUMBERS[ 2 ], container2.getReportTextNumber() );
    assertEquals( "Checking existing text", REPORT_TEXTS_FRENCH[ 2 ], container2.getReportText() );
    assertEquals( "Checking null text number", null, container3.getReportTextNumber() );
    assertEquals( "Checking null text", null, container3.getReportText() );
  }

  /**
   * Tests {@link ch.ige.jipps.text.ReportTextLoader#insertReportText(List, IPType, Locale, ReportType, ReportArea, boolean)} with faults.
   * @throws Exception if the test fails
   */
  public void testInsertReportTextWithFaults() throws Exception
  {
    List<ReportTextContainer> reportTextContainers = new ArrayList<ReportTextContainer>();
    try
    {
      ReportTextLoader.insertReportText( null, DesignType.getInstance(), Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
      fail( "Null text containers must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    // Not a fault per se, just check that an empty list comes out empty
    ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
    assertTrue( "Checking empty list", reportTextContainers.isEmpty() );

    try
    {
      ReportTextLoader.insertReportText( reportTextContainers, null, Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
      fail( "Null IP type must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    try
    {
      ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), null, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
      fail( "Null language must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    try
    {
      ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.GERMAN, null, ReportArea.CURRENT_STATE, true );
      fail( "Null report type must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    try
    {
      ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, null, true );
      fail( "Null report area must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    reportTextContainers = createReportTextContainers();

    try
    {
      ReportTextContainer container2 = reportTextContainers.get( 1 );
      container2.setReportTextNumber( null );
      ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
      fail( "Null report text number must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    try
    {
      ReportTextContainer container2 = reportTextContainers.get( 1 );
      container2.setReportTextNumber( -33 );
      ReportTextLoader.insertReportText( reportTextContainers, DesignType.getInstance(), Locale.GERMAN, ReportType.DOSSIER_REPORT_AFTER_APPLICATION, ReportArea.CURRENT_STATE, true );
      fail( "Unknown report text number must fail" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }
  }

  /**
   * Creates report text containers with report text numbers.
   * @return A <code>List</code> of report text containers
   */
  private List<ReportTextContainer> createReportTextContainers()
  {
    List<ReportTextContainer> reportTextContainers = new ArrayList<ReportTextContainer>();
    for ( int i = 0; i < REPORT_TEXT_NUMBERS.length; i++ )
    {
      ReportTextContainer reportTextContainer = new StandDatVO();
      reportTextContainer.setReportTextNumber( REPORT_TEXT_NUMBERS[ i ] );
      reportTextContainers.add( reportTextContainer );
    }
    return reportTextContainers;
  }

  /**
   * Checks the report text containers after filling the texts.
   * @param reportTextContainers The report text containers to check
   * @param language The language to check for
   */
  private void checkReportTextContainers( List<ReportTextContainer> reportTextContainers, Locale language )
  {
    assertEquals( "Checking number of report text containers", REPORT_TEXT_NUMBERS.length, reportTextContainers.size() );
    for ( int i = 0; i < reportTextContainers.size(); i++ )
    {
      ReportTextContainer container = reportTextContainers.get( i );
      assertEquals( "Checking number", REPORT_TEXT_NUMBERS[ i ], container.getReportTextNumber() );
      String expectedReportText = null;
      if ( language.equals( Locale.GERMAN ) )
      {
        expectedReportText = REPORT_TEXTS_GERMAN[ i ];
      }
      else
      {
        expectedReportText = REPORT_TEXTS_FRENCH[ i ];
      }
      assertEquals( "Checking text", expectedReportText, container.getReportText() );
    }
  }
}
