package ch.ige.jipps.transformation;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Logger;
import org.apache.xalan.processor.TransformerFactoryImpl;

/**
 * Caching implementation of JAXP transformer factory. This implementation caches templates that were loaded from local
 * files so that consequent calls to local stylesheets require stylesheet reparsing only if stylesheet was changed.
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * @author Johannes Lang
 * @version $Id: CachingTransformerFactory.java,v 1.8 2007/12/18 13:09:23 lewis_lee Exp $
 */
public class CachingTransformerFactory extends TransformerFactoryImpl
{
  private static Map<String, TemplatesCacheEntry> templatesCache = new HashMap<String, TemplatesCacheEntry>();

  private static Logger LOG = Logger.getLogger( CachingTransformerFactory.class );

  private static int activeReaders = 0;

  private static int activeWriters = 0;

  /**
   * Process the source into a Transformer object. If source is a StreamSource with <code>systemID</code> pointing to
   * a file, transformer is produced from a cached templates object. Cache is done in soft references; cached objects
   * are reloaded, when file's date of last modification changes.
   * @param source An object that holds a URI, input stream, etc.
   * @return A Transformer object that may be used to perform a transformation in a single thread, never null.
   * @throws TransformerConfigurationException - May throw this during parsing when it is constructing the Templates
   * object and fails.
   */
  @Override
  public Transformer newTransformer( final Source source ) throws TransformerConfigurationException
  {
    if ( source instanceof StreamSource )
    {
      try
      {
        final URI uri = new URI( source.getSystemId() );
        // If URI points to a file, load transformer from the file (or from the cache)
        if ( "file".equalsIgnoreCase( uri.getScheme() ) )
        {
          return newTransformer( new File( uri ) );
        }
      }
      catch ( URISyntaxException urise )
      {
        throw new TransformerConfigurationException( urise );
      }
    }
    return super.newTransformer( source );
  }

  /**
   * Creates a transformer from a file (and caches templates) or from cached templates object.
   * @param file file to load transformer from.
   * @return Transformer, built from given file.
   * @throws TransformerConfigurationException if there was a problem loading transformer from the file.
   */
  protected Transformer newTransformer( final File file ) throws TransformerConfigurationException
  {
    TemplatesCacheEntry templatesCacheEntry = getFromCache( file.getAbsolutePath() );
    if ( templatesCacheEntry != null )
    {
      if ( templatesCacheEntry.lastModified < templatesCacheEntry.templatesFile.lastModified() )
      {
        templatesCacheEntry = null; // to enforce new loading of resource
      }
    }
    // If no templatesEntry is found or this entry was obsolete
    if ( templatesCacheEntry == null )
    {
      LOG.debug( "Loading transformation [" + file.getAbsolutePath() + "]." );
      if ( !file.exists() )
      {
        throw new TransformerConfigurationException( "Requested transformation [" + file.getAbsolutePath() + "] does not exist." );
      }
      templatesCacheEntry = new TemplatesCacheEntry( newTemplates( new StreamSource( file ) ), file );
      writeIntoCache( file.getAbsolutePath(), templatesCacheEntry );
    }
    else
    {
      LOG.debug( "Using cached transformation [" + file.getAbsolutePath() + "]." );
    }
    return templatesCacheEntry.templates.newTransformer();
  }

  /**
   * Returns a templates cache entry for the specified absolute path.
   * @param absolutePath absolute path of the entry.
   * @return Templates cache entry for the specified path.
   */
  protected TemplatesCacheEntry getFromCache( String absolutePath )
  {
    beforeRead();
    final TemplatesCacheEntry templatesCacheEntry = templatesCache.get( absolutePath );
    afterRead();
    return templatesCacheEntry;
  }

  /**
   * Saves templates cache entry for the specified absolute path.
   * @param absolutePath absolute path of the entry.
   * @param templatesCacheEntry templates cache entry to save.
   */
  protected void writeIntoCache( String absolutePath, TemplatesCacheEntry templatesCacheEntry )
  {
    beforeWrite();
    templatesCache.put( absolutePath, templatesCacheEntry );
    afterWrite();
  }

  /**
   * Invoked just before reading, waits until reading is allowed.
   */
  protected synchronized void beforeRead()
  {
    while ( activeWriters > 0 )
    {
      try
      {
        wait();
      }
      catch ( InterruptedException iex )
      {
        LOG.error( "Process was interupted", iex );
      }
    }
    ++activeReaders;
  }

  /**
   * Invoked just after reading.
   */
  protected synchronized void afterRead()
  {
    --activeReaders;
    notifyAll();
  }

  /**
   * Invoked just before writing, waits until writing is allowed.
   */
  protected synchronized void beforeWrite()
  {
    while ( activeReaders > 0 || activeWriters > 0 )
    {
      try
      {
        wait();
      }
      catch ( InterruptedException iex )
      {
        LOG.error( "Process was interupted", iex );
      }
    }
    ++activeWriters;
  }

  /**
   * Invoked just after writing.
   */
  protected synchronized void afterWrite()
  {
    --activeWriters;
    notifyAll();
  }

  /**
   * Private class to hold a templates cache entry.
   */
  private class TemplatesCacheEntry
  {
    /** When was the cached entry last modified. */
    public long lastModified;

    /** Cached templates object. */
    public Templates templates;

    /** Templates file object. */
    public File templatesFile;

    /**
     * Constructs a new cache entry.
     * @param templates templates to cache.
     * @param templatesFile file, from which this transformer was loaded.
     */
    public TemplatesCacheEntry( final Templates templates, final File templatesFile )
    {
      this.templates = templates;
      this.templatesFile = templatesFile;
      this.lastModified = templatesFile.lastModified();
    }
  }

}
