package ch.ige.jipps.vo.comparator.test;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;
import ch.ige.jipps.text.ReportTextContainer;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.jipps.vo.comparator.ReportTextNumberComparator;

/**
 * Tests {@link ch.ige.jipps.vo.comparator.ReportTextNumberComparator}.
 *
 * <p>Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author    Eric Lewis
 * @version   $Id: ReportTextNumberComparatorTest.java,v 1.9 2007/05/04 08:56:36 lewis_lee Exp $
 */
public class ReportTextNumberComparatorTest extends TestCase
{
  /**
   * The report text containers which are sorted
   */
  private List<StandDatVO> reportTextContainers;

  /**
   * The unit under test
   */
  private ReportTextNumberComparator comparator;

  /**
   * The events are sorted by these report text numbers.
   * Note that they don't have to be in ascending order!
   */
  private static final int[] REPORT_TEXT_NUMBERS = { 100, 300, 1000, 200, 400, 900, 700, 500, 600, 800 };

  /**
   * The default constructor
   * @param args Any arguments
   */
  public ReportTextNumberComparatorTest( String args )
  {
    super( args );
    this.reportTextContainers = create( REPORT_TEXT_NUMBERS );
  }

  /**
   * Tests how the Comparator works with report text containers.
   * @throws Exception if the test fails
   */
  public void testCompareReportTextContainers() throws Exception
  {
    this.comparator = new ReportTextNumberComparator( createReportTextNumbers( REPORT_TEXT_NUMBERS ) );
    Collections.sort( this.reportTextContainers, this.comparator );
    assertEquals( "Checking number of elements", 5, this.reportTextContainers.size() );

    ReportTextContainer container = this.reportTextContainers.get( 0 );
    assertEquals( "Checking event 1", 300, container.getReportTextNumber().intValue() );

    container = this.reportTextContainers.get( 1 );
    assertEquals( "Checking event 2", 200, container.getReportTextNumber().intValue() );

    container = this.reportTextContainers.get( 2 );
    assertEquals( "Checking event 3", 900, container.getReportTextNumber().intValue() );

    container = this.reportTextContainers.get( 3 );
    assertEquals( "Checking event 4", 500, container.getReportTextNumber().intValue() );

    container = this.reportTextContainers.get( 4 );
    assertEquals( "Checking event 5", 800, container.getReportTextNumber().intValue() );
  }

  /**
   * Tests how the Comparator handles faults.
   * @throws Exception if the test fails
   */
  public void testCompareWithFaults() throws Exception
  {
    try
    {
      this.comparator = new ReportTextNumberComparator( null );
      fail( "Must fail with null report texts" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    try
    {
      this.comparator = new ReportTextNumberComparator( new ArrayList<Integer>() );
      fail( "Must fail with empty report texts" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }

    this.comparator = new ReportTextNumberComparator( createReportTextNumbers( REPORT_TEXT_NUMBERS ) );
    this.reportTextContainers = new ArrayList<StandDatVO>();
    StandDatVO currentState1 = new StandDatVO();
    currentState1.setReportTextNumber( -1 );
    StandDatVO currentState2 = new StandDatVO();
    currentState2.setReportTextNumber( REPORT_TEXT_NUMBERS[0] );
    this.reportTextContainers.add( currentState1 );
    this.reportTextContainers.add( currentState2 );
    try
    {
      Collections.sort( this.reportTextContainers, this.comparator );
      fail( "Must fail with report text number that can't be found" );
    }
    catch ( IllegalArgumentException ex )
    {
      // This is expected
    }
  }

  /**
   * Creates events for testing, containing every second report text number
   * from <code>reportTextNumbers</code> in reverse order.
   * @param reportTextNumbers The predefined report text numbers to sort with
   * @return A <code>List</code> of objects containing report text numbers
   */
  private List<StandDatVO> create( int[] reportTextNumbers )
  {
    List<StandDatVO> textObjects = new ArrayList<StandDatVO>();
    for ( int i = reportTextNumbers.length - 1; i >= 0; i -= 2 )
    {
      StandDatVO container = new StandDatVO();
      container.setReportTextNumber( reportTextNumbers[i] );
      textObjects.add( container );
    }
    return textObjects;
  }

  /**
   * Creates the predefined report text numbers for sorting.
   * @param reportTextNumbers The predefined report text numbers for sorting
   * @return The same as <code>reportTextNumbers</code>, but as <code>Integer</code> objects
   */
  private List<Integer> createReportTextNumbers( int[] reportTextNumbers )
  {
    List<Integer> numbers = new ArrayList<Integer>();
    for ( int reportTextNumber: reportTextNumbers )
    {
      numbers.add( reportTextNumber );
    }
    return numbers;
  }
}
