package ch.ige.jipps.xml;

import java.io.IOException;
import java.util.Locale;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.BFormTxtDruckVO;
import ch.ige.jipps.vo.BfakTxtVO;
import ch.ige.jipps.vo.FormTextVO;
import ch.ige.jipps.vo.JippsFaktPosVO;
import ch.ige.jipps.vo.LetterVO;
import ch.ige.jipps.xml.hinterlegung.Adresse;
import ch.ige.jipps.xml.hinterlegung.Hinterlegung;
import ch.ige.jipps.xml.hinterlegung.HinterlegungType;
import ch.ige.jipps.xml.hinterlegung.Igekopf;
import ch.ige.jipps.xml.hinterlegung.ObjectFactory;
import ch.ige.util.address.IpiAddress;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.BAGISLanguage;

/**
 * Collects the data to create the xml file
 * <p>
 * Copyright (c) 2003-2006 Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author Priska Steiner
 * @author Esther Bandschapp
 * @version $Id: HinterlegungWriteXml.java,v 1.7 2008/07/23 15:19:14 lewis_lee Exp $
 */
public class HinterlegungWriteXml
{
  private ObjectFactory objectFactory;

  private static JAXBContext jaxbContext;

  private static Marshaller marshaller;

  private Hinterlegung hinterlegung;

  /**
   * Creates an XML document from the letter.
   * 
   * @param letterVO A value object containing the data for the letter
   * @return A <code>String</code> containing the XML document
   * @throws JAXBException
   * @throws IOException
   */
  public String toXml( LetterVO letterVO ) throws JAXBException, IOException
  {
    if ( this.objectFactory == null )
    {
      jaxbContext = JAXBContext.newInstance( "ch.ige.jipps.xml.hinterlegung" );
      this.objectFactory = new ObjectFactory();
      marshaller = jaxbContext.createMarshaller();
    }

    this.hinterlegung = this.objectFactory.createHinterlegung();

    // Igekopf füllen
    Igekopf igekopf = writeIgekopf( letterVO );
    this.hinterlegung.setIgekopf( igekopf );

    // Adresse füllen
    Adresse adresse = writeAdresse( letterVO );
    this.hinterlegung.setAdresse( adresse );

    // Einzelfelder fürs XML holen
    Locale language = BAGISLanguage.parse( letterVO.jippsFakturVO.sprachCd.trim() ).toLocale();
    String formattedInvoiceDate = DateFormatter.longFormat( letterVO.jippsFakturVO.bfaktDat, language );
    this.hinterlegung.setBfaktDat( formattedInvoiceDate );
    this.hinterlegung.setFormartCd( letterVO.jippsFakturVO.formartCd );

    for ( JippsFaktPosVO posVO: letterVO.jippsFaktPositionen )
    {
      if ( posVO != null && posVO.invoiceLineItem.textNumber > 0 )
      {
        for ( BfakTxtVO bfaktxt: letterVO.bfakTexte )
        {
          if ( bfaktxt.textNr == posVO.invoiceLineItem.textNumber )
          {
            if ( bfaktxt.text.indexOf( "\n" ) != -1 )
            {
              bfaktxt.text = replaceNewlines( ConvertText.replaceHtmlTags( bfaktxt.text ) );
            }
            this.hinterlegung.setTxtEreignisVar( ConvertText.replaceHtmlTags( bfaktxt.text ) );
          }
        }
      }
    }
    if ( letterVO.bild != null )
    {
      writeBildFile( letterVO.bild, letterVO.jippsFakturVO.schutitKey );
    }
    else
    {
      this.hinterlegung.setSchutitel( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.beschreibung ) );
    }

    if ( letterVO.jippsFakturVO.txtNrBeilage > 0 )
    {
      for ( BfakTxtVO bfaktxt: letterVO.bfakTexte )
      {
        if ( bfaktxt.textNr == letterVO.jippsFakturVO.txtNrBeilage )
        {
          this.hinterlegung.setTxtBeilage( ConvertText.replaceHtmlTags( bfaktxt.text ) );
        }
      }
    }

    // Formtexte holen und abfüllen; Werte aus Bfaktur auch dazu holen
    writeFormTexte( letterVO );

    // BformTexte holen und abfüllen
    writeBformTexte( letterVO );

    return XmlMarshaller.toString( jaxbContext, marshaller, "hinterlegung.xsd", this.hinterlegung );
  }

  /**
   * Creates the IGE header
   * 
   * @param letterVO A value object containing the data for the letter
   * @return igekopf der geschriebener Igekopf
   * @throws JAXBException
   */
  private Igekopf writeIgekopf( LetterVO letterVO ) throws JAXBException
  {
    Igekopf igekopf = this.objectFactory.createIgekopf();

    if ( !letterVO.printIgekopf )
    {
      igekopf.setIgekopfDe( "" );
      igekopf.setIgekopfFr( "" );
      igekopf.setIgekopfIt( "" );
      igekopf.setIgekopfEn( "" );
      igekopf.setIgekopfAdr( "" );
      igekopf.setIgekopfKto( "" );
    }
    else
    {
      igekopf.setIgekopfDe( IpiAddress.getGermanName() );
      igekopf.setIgekopfFr( IpiAddress.getFrenchName() );
      igekopf.setIgekopfIt( IpiAddress.getItalianName() );
      igekopf.setIgekopfEn( IpiAddress.getEnglishName() );
      igekopf.setIgekopfAdr( IpiAddress.getFullGermanAddress() );
      igekopf.setIgekopfKto( "" );
    }
    return igekopf;
  }

  /**
   * Creates an <code>Adresse object</code> containing the address data
   * 
   * @param letterVO A value object containing the data for the letter
   * @return <code>Adresse object</code> containing the address data
   * @throws JAXBException
   */
  private Adresse writeAdresse( LetterVO letterVO ) throws JAXBException
  {
    Adresse adresse = this.objectFactory.createAdresse();
    adresse.setName( ConvertText.replaceHtmlTags( letterVO.versadrVO.getNameOrEmptyStr() ) );
    adresse.setZeile1( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile1OrEmptyStr() ) );
    adresse.setZeile2( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile2OrEmptyStr() ) );
    adresse.setZeile3( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile3OrEmptyStr() ) );
    adresse.setZeile4( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile4OrEmptyStr() ) );
    adresse.setPlzOrt( getPlzOrt( letterVO.versadrVO.getLandCdOrEmptyStr(), letterVO.versadrVO.getPlzOrEmptyStr(), letterVO.versadrVO.getOrtOrEmptyStr() ) );

    if ( letterVO.jippsFakturVO.txtAdrZusatz > 0 )
    {
      for ( BfakTxtVO bfaktxt: letterVO.bfakTexte )
      {
        if ( bfaktxt.textNr == letterVO.jippsFakturVO.txtAdrZusatz )
        {
          adresse.setTxtAdrZusatz( ConvertText.replaceHtmlTags( bfaktxt.text ) );
        }
      }
    }
    return adresse;
  }

  /**
   * Matches the post code (PLZ) and the place (Ort) with the country code <strong>NOTE: Changings of this method
   * require also changings in the class LetterWriteXml</strong>
   * 
   * @param landCd The country code (CH = Switzerland, AUS = Australia)
   * @param plz The post code
   * @param ort The place
   * @return A <code>String</code> with the combination of country, postcode and place
   */
  private String getPlzOrt( String landCd, String plz, String ort )
  {
    if ( landCd.trim().equalsIgnoreCase( "CH" ) )
    {
      return plz + " " + ort;
    }
    else
    {
      return landCd + " - " + plz + " " + ort;
    }
  }

  /**
   * Writes the form texts into the XML
   * 
   * @param letterVO A value object containing the data for the letter
   */
  private void writeFormTexte( LetterVO letterVO )
  {
    for ( FormTextVO formtext: letterVO.formTexte )
    {
      int zeile = 0;
      if ( formtext.lineNumber != null )
      {
        zeile = formtext.lineNumber.intValue();
      }
      switch( zeile )
      {
        //TODO Use enums
        case 35:
        {
          this.hinterlegung.setLblUnsereRef( ConvertText.replaceHtmlTags( formtext.text ) );
          this.hinterlegung.setTxtUnsereRef( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.unsereRef ) );
          break;
        }
        case 40:
        {
          if ( letterVO.jippsFakturVO.kzEinschr.equals( "1" ) )
          {
            this.hinterlegung.setTxtEinschreiben( ConvertText.replaceHtmlTags( formtext.text ) );
          }
          break;
        }
        case 55:
        {
          this.hinterlegung.setLblTelNr( ConvertText.replaceHtmlTags( formtext.text + " " ) );
          this.hinterlegung.setTxtTelNr( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.telNr ) );
          break;
        }
        case 80:
        {
          this.hinterlegung.setLblIhrZeichen( ConvertText.replaceHtmlTags( formtext.text ) );
          this.hinterlegung.setTxtIhrZeichen( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.ihrZeichen ) );
          break;
        }
        case 160:
        {
          this.hinterlegung.setTxtSeite( ConvertText.replaceHtmlTags( formtext.text ) );
          break;
        }
        default:
          break;
      }
    }
  }

  /**
   * Writes the BFormTexte into the XML
   * 
   * @param letterVO A value object containing the data for the letter
   */
  private void writeBformTexte( LetterVO letterVO )
  {
    for ( BFormTxtDruckVO bformtxt: letterVO.bformTexte )
    {
      switch( bformtxt.textType.intValue() )
      {
        //TODO Use enums
        case 0:
        {
          this.hinterlegung.setTxtAbteilung( ConvertText.replaceHtmlTags( bformtxt.text ) );
          break;
        }
        case 1:
        {
          this.hinterlegung.setTxtBetreff( ConvertText.replaceHtmlTags( bformtxt.text ) );
          break;
        }
        case 2:
        {
          this.hinterlegung.setTxtKopf( ConvertText.replaceHtmlTags( bformtxt.text ) );
          break;
        }
        case 3:
        {
          this.hinterlegung.setTxtSchluss( ConvertText.replaceHtmlTags( bformtxt.text ) );
          break;
        }
        default:
          break;
      }
    }
  }

  /**
   * Writes the image into a member object
   * 
   * @param bild A byte[] containing the image
   * @param schutitKey The IP's unique key
   * @throws IOException
   */
  private void writeBildFile( byte[] bild, int schutitKey ) throws IOException, JAXBException
  {
    String pathWithoutFileType = PathConfiguration.getVerifiedPath( PropertyConstants.JIPPS_PATH_IMAGE, true ) + "bild_" + String.valueOf( schutitKey );
    Map<String, Object> imageInformation = ImageProcessor.getInstance().writeImage( pathWithoutFileType, bild, false );

    HinterlegungType.BildType bildType = this.objectFactory.createHinterlegungTypeBildType();
    bildType.setSrc( pathWithoutFileType + "." + imageInformation.get( ImageProcessor.IMAGE_TYPE ) );

    /** TODO As soon as we only use jai, we can delete this */
    double w = ( (Double) imageInformation.get( "width" ) ).doubleValue();
    double h = ( (Double) imageInformation.get( "height" ) ).doubleValue();
    if ( ( w > 8 ) || ( h > 8 ) )
    {
      if ( w > h )
      {
        bildType.setWidth( "8cm" );
      }
      else
      {
        bildType.setHeight( "8cm" );
      }
    }
    else
    {
      bildType.setWidth( imageInformation.get( "width" ) + "cm" );
      bildType.setHeight( imageInformation.get( "height" ) + "cm" );
    }
    this.hinterlegung.setBild( bildType );
  }

  /**
   * Formats a <code>String</code>: It replaces the Newlines with a "<br/>" Tag
   * 
   * @param text A <code>String</code> containing Newlines
   * @return A <code>String</code> where the Newlines are replaced by a "<br/>" Tag
   */
  private String replaceNewlines( String text )
  {
    //TODO Can this be replaced by ConvertText?
    int start = 0;
    String repText = text;
    StringBuffer sb = new StringBuffer( repText );

    start = repText.indexOf( "\n" );
    while ( start != -1 )
    {
      repText = sb.replace( start, start + 1, "<br/>" ).toString();
      start = repText.indexOf( "\n" );
    }

    return repText;
  }
}
