package ch.ige.jipps.xml;

import java.util.Map;

import ch.ige.jipps.imageconv.ImageProcessor;

/**
 * Crops the size of an image to its maximum defined by {@link #MAXIMUM_WIDTH}
 * and {@link #MAXIMUM_HEIGHT}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ImageCropper.java,v 1.3 2007/05/15 13:09:28 lewis_lee Exp $
 */
public class ImageCropper
{
  /**
   * The maximum width of an image in centimeters
   */
  public static final double MAXIMUM_WIDTH = 8;

  /**
   * The maximum height of an image in centimeters
   */
  public static final double MAXIMUM_HEIGHT = 8;

  /**
   * The unit of the image's dimensions
   */
  public static final String IMAGE_SIZE_UNIT = "cm";
  
  /**
   * Automatic scaling by XSL-FO, takes the size of the image itself
   */
  public static final String AUTO = "auto";

  /**
   * Creates the formatted sizes for an image in a report, cropping it
   * to the maximum size (see below).
   * @param imageProperties The <code>Map</code> created by {@link ch.ige.jipps.imageconv.ImageProcessor}
   *        from where the height and the width are taken
   * @return Two <code>String</code>, the first being the formatted width, the second the formatted height.
   *         Note that if one dimension has been cropped, the other one is <code>null</code>, in order to let
   *         FOP resize it in the PDF.
   * @see #MAXIMUM_WIDTH
   * @see #MAXIMUM_HEIGHT
   */
  public static String[] getCroppedAndFormattedSizes( Map<String, Object> imageProperties )
  {
    if ( imageProperties == null )
    {
      throw new IllegalArgumentException( "No image properties" );
    }
    Object assumedWidth = imageProperties.get( ImageProcessor.IMAGE_WIDTH );
    if ( assumedWidth == null || !( assumedWidth instanceof Double ) )
    {
      throw new IllegalArgumentException( "No valid width found" );
    }
    double width = ( (Double) assumedWidth ).doubleValue();
    if ( width <= 0 )
    {
      throw new IllegalArgumentException( "Width too small" );
    }

    Object assumedHeight = imageProperties.get( ImageProcessor.IMAGE_HEIGHT );
    if ( assumedHeight == null || !( assumedHeight instanceof Double ) )
    {
      throw new IllegalArgumentException( "No valid height found" );
    }
    double height = ( (Double) assumedHeight ).doubleValue();
    if ( height <= 0 )
    {
      throw new IllegalArgumentException( "Height too small" );
    }

    String[] sizes = new String[2];
    sizes[0] = width + IMAGE_SIZE_UNIT;
    sizes[1] = height + IMAGE_SIZE_UNIT;
    // Limit the size of the image and let FOP decide how to scale
    // the image by omitting the other dimension (height or width).
    if ( width > MAXIMUM_WIDTH || height > MAXIMUM_HEIGHT )
    {
      if ( width > height )
      {
        sizes[0] = MAXIMUM_WIDTH + IMAGE_SIZE_UNIT;
        sizes[1] = AUTO;
      }
      else
      {
        sizes[0] = AUTO;
        sizes[1] = MAXIMUM_WIDTH + IMAGE_SIZE_UNIT;
      }
    }
    return sizes;
  }
}
