package ch.ige.ses.client.batch;

import java.io.*;
import org.apache.log4j.*;
import ch.ige.util.client.*;
import ch.ige.ses.BuildInfo;
import ch.ige.ses.util.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.server.batchctrl.*;
import ch.ige.ses.server.monitor.MonitorDelegate;

/**
 * SES Batch Programm zur Starten der JGB-Verarbeitung.
 * Wichtig:
 * Dieser Job darf erst NACH der JGB Fristablauf Verarbeitung gestartet werden !
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Stefan Nydegger, thomas Iten
 * @version $Id: SesJgbStart.java,v 1.24 2007/01/10 19:08:23 iten Exp $
 */
public class SesJgbStart extends BatchBasis
{
  //
  // const
  //
  private static final Logger LOG      = Logger.getLogger("ch.ige.ses.client.batch");

  private static final String JOB_NAME            = "SesJgbStart";    // job name
  private static final int    JOB_CTRL_INTERVAL   = 300;              //  5 min =   300 sec
  private static final int    JOB_TIMEOUT         = 14400;            //  4 h   = 14400 sec

  private static final String OK_NUM_SUCCESS      = "0";
  private static final String OK_MSG_SUCCESS      = "Batch Job erfolgreich beendet";

  private static final String ERR_NUM_EXCEPTION   = "-1";
  private static final String ERR_MSG_EXCEPTION   = "Eine unerwartete Exception ist aufgetreten: ";

  private static final String ERR_NUM_WRONG_ARG   = "-2";
  private static final String ERR_MSG_WRONG_ARG   = "Aufruf Program mit ungültigen Parametern";

  private static final String ERR_NUM_MISSING_ID  = "-3";
  private static final String ERR_MSG_MISSING_ID  = "Fehler beim ermitteln der neuen JGB Lauf ID";

  private static final String ERR_NUM_STATE       = "-4";
  private static final String ERR_MSG_STATE       = "Fehler beim ermitteln des Status";

  private static final String ERR_NUM_JOB_STOPPED = "-5";
  private static final String ERR_MSG_JOB_STOPPED = "Der JGB Job wurde unterbrochen";

  private static final String ERR_NUM_TIMEOUT     = "-6";
  private static final String ERR_MSG_TIMEOUT     = "Der JGB Job hat die Timeout Zeit überschritten";

  private static final String ERR_NUM_DISABLED    = "-7";
  private static final String ERR_MSG_DISABLED    = "Der JGB Job ist nicht Enabled. " +
                                                    "Bitte auf Tabelle SES.Property den Eintrag <batch.job." + JOB_NAME + ".enabled> überprüfen.";

  //
  // fields
  //
  public String  modus          = null;     // Mögliche Werte
                                            // - SESConstants.MODUS_PROD (program param)
                                            // - SESConstants.MODUS_TEST (program Param)
                                            // - "USAGE"                 (program intern)

  public int     lastJgbLaufId  = 0;        // get last id during start call
  public int     newJgbLaufId   = 0;        // get new id during control call
  public int     timeRunning    = 0;        // time the job already is running in seconds

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SesJgbStart()
  {
    super(JOB_NAME);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Ausgabe Programmhilfe
   * Zeigt mit welchen Parametern das Program gestartet werden kann.
   */
  private void printUsage()
  {
    System.out.println("Usage: " + BuildInfo.CVS_TAG);
    System.out.println("SesJgbStart modus");
    System.out.println("            [-h] [-?] [help] [?]");
    System.out.println();
    System.out.println("  modus             Program Modus [" +
                                            SESConstants.MODUS_PROD    + " | " +
                                            SESConstants.MODUS_TEST    + " | " +
                                            SESConstants.MODUS_STOP    + " | " +
                                            SESConstants.MODUS_RESTART + " | " +
                                            "SHOW]" );
    System.out.println("  -h, -?, help, ?   Anzeige Hilfe (usage)");
  }


  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args             programm arguments
   * @throws BatchException   in case of wrong arguments
   */
  private void loadParams(String[] args) throws BatchException
  {
    if (args.length ==1)
    {
      if ( args[0].equals(SESConstants.MODUS_PROD) )
      {
        modus = SESConstants.MODUS_PROD;
      }
      else if ( args[0].equals(SESConstants.MODUS_TEST) )
      {
        modus = SESConstants.MODUS_TEST;
      }
      else if ( args[0].equals(SESConstants.MODUS_STOP) )
      {
        modus = SESConstants.MODUS_STOP;
      }
      else if ( args[0].equals(SESConstants.MODUS_RESTART) )
      {
        modus = SESConstants.MODUS_RESTART;
      }
      else if ( args[0].equals("SHOW") )
      {
        modus = "SHOW";
      }
      else if ( args[0].equals("-h")   || args[0].equals("-?") ||
                args[0].equals("help") || args[0].equals("?") )
      {
        modus = "USAGE";
      }
      else
      {
        setNotifyConsole(true);
        throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
      }
    }
    else
    {
      setNotifyConsole(true);
      throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
    }
  }


  /**
   * Start Produktiv Modus.
   * Dabei wird die JgbLaufId des letzten Jobs ermittelt. Diese wird beim ersten
   * Control Aufruf verwendet um die aktuelle Job Id zu ermitteln.
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startProd() throws Exception
  {
    LOG.debug("startProd - modus=" + modus + ", bfaktIdVon=0, bfaktIdBis=0");

    lastJgbLaufId = BatchDelegate.getInstance().jgbStart(modus,0,0);

    LOG.debug("startProd - received lastJgbLaufId=" + lastJgbLaufId);
  }


  /**
   * Start Test Modus
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startTest() throws Exception
  {
    int     bfaktIdVon = 0;
    int     bfaktIdBis = 0;


    System.out.println("");
    System.out.println(JOB_NAME + " - Test Modus" );

    System.out.println("");
    System.out.println("Bitte folgende Werte eingeben:");
    bfaktIdVon = readInt("  bfaktIdVon = ");
    bfaktIdBis = readInt("  bfaktIdBis = ");

    BatchDelegate.getInstance().jgbStart(modus,bfaktIdVon,bfaktIdBis);

    System.out.println("");
    System.out.println("Die JGB Verarbeitung wurde erfolgreich gestartet...");
  }

  /**
    * Start Stop Modus
    *
    * @throws Exception  unerwarteter Fehler
    */
   public void startStop() throws Exception
   {
     System.out.println("");
     System.out.println(JOB_NAME + " - Stop Modus" );

     BatchDelegate.getInstance().jgbStart(modus, 0, 0);

     System.out.println("");
     System.out.println("Die JGB Verarbeitung wird gestoppt...");
  }

  /**
   * Start Restart Modus
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startRestart() throws Exception
  {
    int     setRange   = 0;
    int     bfaktIdVon = 0;
    int     bfaktIdBis = 0;

    System.out.println("");
    System.out.println(JOB_NAME + " - Restart Modus" );

    System.out.println("");
    System.out.println("Bitte Restart Modus wählen:");
    setRange = readInt("  bfaktId range manuell setzten (1=ja, 0=nein) = ");

    if ( setRange == 1 )
    {
      System.out.println("Bitte folgende Werte eingeben:");
      bfaktIdVon = readInt("  bfaktIdVon = ");
      bfaktIdBis = readInt("  bfaktIdBis = ");
    }

    BatchDelegate.getInstance().jgbStart(modus,bfaktIdVon,bfaktIdBis);

    System.out.println("");
    System.out.println("Die JGB Verarbeitung wurde erfolgreich gestartet...");
  }


  /**
   * Start Show Modus
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startShow()
  {
    int   cmd     = 0;
    int   laufId  = 0;

    while ( cmd != 99)
    {
      try
      {
        System.out.println("");
        System.out.println(JOB_NAME + " - Show Modus" );
        System.out.println("   1 : Anzeige Statistik");
        System.out.println("   2 : Anzeige Fehlerprotokoll ");
        System.out.println("  99 : Exit Program");
        System.out.println("");

        cmd = readInt("  Eingabe = ");

        switch (cmd)
        {
          case 1:
            laufId = readInt("  Lauf Id = ");
            System.out.println(getStatistik(laufId));
            break;

          case 2:
            laufId = readInt("  Lauf Id = ");
            System.out.println(getErrorProtokol(laufId));
            break;
        }

      }
      catch (Exception ex)
      {
        System.out.println("Exception: \n" + ex.getMessage());
      }
    }
  }


  /**
   * get new jgb lauf id. the id is needed to control the state of the job
   * from time to time.
   *
   * note:  after the start of the jgb job, wait a while before calling this
   *        function, since the job is starting up asynchronous
   *
   * @throws BatchException in case of a missing id
   */
  public void getNewLaufId() throws BatchException
  {
    try
    {
      LOG.debug("getNewLaufId - lastJgbLaufId=" + lastJgbLaufId);

      newJgbLaufId = BatchDelegate.getInstance().jgbGetNewLaufId(lastJgbLaufId);

      LOG.debug("getNewLaufId - received newJgbLaufId=" + newJgbLaufId);
    }
    catch (Exception ex)
    {
      throw new BatchException ( ERR_NUM_MISSING_ID,
                                 ERR_MSG_MISSING_ID + "\n" + ex.getMessage() );
    }
  }

  /**
   * control job state
   *
   * @return true if job is finished sucessfully, false if job is still running
   * @throws BatchException in case job ist stopped or an error is occured
   */
  public boolean ctrlState() throws BatchException
  {
    int     sapState  = 0;
    boolean finished  = false;


    // get state
    try
    {
      LOG.debug("ctrlState - getting actual state...");

      sapState = BatchDelegate.getInstance().jgbCtrlJobState(newJgbLaufId);

      LOG.debug("ctrlState - sapState=" + sapState);
    }
    catch (Exception ex)
    {
      throw new BatchException ( ERR_NUM_STATE,
                                 ERR_MSG_STATE + "\n" + ex.getMessage() );
    }


    // control state
    if ( sapState == JGBLaufVO.SAP_STAT_FINISHED )
    {
      LOG.debug("ctrlState - SAP_STAT_FINISHED received");
      finished = true;
    }
    else if (sapState == JGBLaufVO.SAP_STAT_STOPPED)
    {
      LOG.debug("ctrlState - SAP_STAT_STOPPED received");
      throw new BatchException( ERR_NUM_JOB_STOPPED,
                                ERR_MSG_JOB_STOPPED + "\n" + getErrorProtokol(newJgbLaufId) );
    }
    else if ( timeRunning > JOB_TIMEOUT )
    {
      LOG.debug("ctrlState - JOB_TIMEOUT occured");
      throw new BatchException( ERR_NUM_TIMEOUT,
                                ERR_MSG_TIMEOUT + " (Timeout=" + JOB_TIMEOUT + "sec)" );
    }

    // all done
    return finished;
  }


  /**
   * sleep for the given JOB_CTRL_INTERVAL and increment timeRunning variable
   */
  public void waitInterval()
  {
    try
    {
      LOG.debug("waitInterval - sleep for " + JOB_CTRL_INTERVAL + " seconds...");
      Thread.sleep(JOB_CTRL_INTERVAL * 1000);
      timeRunning = timeRunning + JOB_CTRL_INTERVAL;
    }
    catch (java.lang.InterruptedException ie)
    {
      // do nothing
    }
  } // end method


  /**
   * Ausgabe Textmeldung und einlesen Eingabe als int
   *
   * @param prompt        Textmeldung
   * @return              int Wert
   * @throws Exception    Unerwarteter Fehler
   */
  public int readInt(String prompt) throws Exception
  {
    BufferedReader  stdin = new BufferedReader(new InputStreamReader(System.in));

    System.out.print (prompt);
    return( Integer.parseInt( stdin.readLine() ) );
  }


  /**
   * Gibt die Statistik des aktuellen Jahresgebühren Laufes zurück.
   * Wenn nicht nötig keine Neuberechnung durchführen
   * Im Fehlerfall wird ein Leerstring zurückgegeben.
   */
  public String getStatistik(int laufId)
  {
    String buf = "";

    try
    {
      buf = MonitorDelegate.getInstance().getStatistik(laufId, false);
    }
    catch (Exception ex)
    {
      LOG.warn("getStatistik - Exception occured: " + ex.getMessage());
    }

    return buf;
  }


  /**
   * Gibt das Fehlerprotokoll des aktuellen Jahresgebühren Laufes zurück.
   * Im Fehlerfall wird ein Leerstring zurückgegeben.
   */
  public String getErrorProtokol(int laufId)
  {
    String buf = "";

    try
    {
      buf = MonitorDelegate.getInstance().getProtokoll(laufId);
    }
    catch (Exception ex)
    {
      LOG.warn("getErrorProtokol - Exception occured: " + ex.getMessage());
    }

    return buf;
  }


  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    SesJgbStart job = new SesJgbStart();

    try
    {
      //
      // init
      //
      ClientLogInitalizer.initalizeLogging(ClientLogInitalizer.BATCH_CONFIG);
      job.loadParams(args);


      //
      // run job produktion
      //
      if ( job.modus.equals(SESConstants.MODUS_PROD) )
      {

        if ( ! job.enabled() )                // check if job is enabled
        {
          throw new BatchException(ERR_NUM_DISABLED,ERR_MSG_DISABLED);
        }

        job.startProd();                      // start job

        job.waitInterval();                   // get new lauf id after wait
        job.getNewLaufId();

        while ( ! job.ctrlState() )           // check state
        {
          job.waitInterval();
        }

                                              // notify that job successfully finished
        job.setMessage ( MSG_TYPE_OK, OK_NUM_SUCCESS,
                         OK_MSG_SUCCESS + "\n" + job.getStatistik(job.newJgbLaufId) );
        job.notifyMessage();
      }

      //
      // run job test
      //
      else if ( job.modus.equals(SESConstants.MODUS_TEST) )
      {
        job.setNotifyAll(false,false,true);   // set notif options to console only

        job.startTest();                      // start test
      }

      //
      // run job stop
      //
      else if ( job.modus.equals(SESConstants.MODUS_STOP) )
      {
        job.setNotifyAll(false,false,true);   // set notif options to console only

        job.startStop();                   // start stop
      }

      //
      // run job restart
      //
      else if ( job.modus.equals(SESConstants.MODUS_RESTART) )
      {
        job.setNotifyAll(false,false,true);   // set notif options to console only

        job.startRestart();                   // start restart
      }

      //
      // run job show
      //
      else if ( job.modus.equals("SHOW") )
      {
        job.setNotifyAll(false,false,true);   // set notif options to console only

        job.startShow();                      // start show
      }

      //
      // Usage
      //
      else if ( job.modus.equals("USAGE") )
      {
        job.printUsage();
      }

      //
      // End
      //
      System.exit(0);

    }

    catch (BatchException bex)
    {
      LOG.debug("main - BatchException occured: " + bex.getErrorMsg());
      job.notifyMessage ( MSG_TYPE_NOK, bex.getErrorNr(), bex.getErrorMsg() );
      System.exit( Integer.parseInt(bex.getErrorNr()) );
    }
    catch (Exception ex)
    {
      LOG.error("main - exception occured: " + ex.getMessage());
      job.notifyMessage ( MSG_TYPE_NOK, ERR_NUM_EXCEPTION, ERR_MSG_EXCEPTION + ex.toString() );
      System.exit( Integer.parseInt(ERR_NUM_EXCEPTION) );
    }

  } // end main


} // end class

