package ch.ige.ses.client.sapinbound;

import java.util.*;
import org.apache.log4j.*;

import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.server.sapctrl.SapDelegateInbound;
import ch.ige.ses.client.saputil.*;

/**
 * SAP Inbound Client - Jahresgebühren
 * Diese Klasse implementiert den monatlichen Jahresgebührenlauf von
 * SES ins SAP System.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: JGB.java,v 1.19 2007/12/28 12:30:04 iten Exp $
 */
public class JGB
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(JGB.class);


  //
  // fields
  //
  private SapDelegateInbound  fDelegate = null; // execute inbound requests

  private AutoRFC     fAutoRfc          = null; // rfc handle auto mode
  private ManualRFC   fManualRfc        = null; // rfc handle manual mode

  private int         fBlockSizeSesUpd  = 0;    // block size for ses update
  private int         fBfaktIdOffset    = 0;    // simulation.jgb.bfaktid.offset (for tests ONLY)

  private boolean     fErrOccured   = false;    // state and message from calls
  private String      fErrMsg       = "";       // to process() & finish()

  private int         fRecCntTotal  = 0;        // statistic info about
  private int         fRecCntSapOk  = 0;        // the processed records
  private int         fRecCntSapErr = 0;
  private int         fRecCntSesOk  = 0;
  private int         fRecCntSesErr = 0;

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * construct object for processing jgb's
   *
   * @param rfc                 sap remote function call handle
   * @throws Exception          unexpected error
   */
  public JGB(RFC rfc) throws Exception
  {
    SapProperties sp = new SapProperties();

    // init delegate
    fDelegate = SapDelegateInbound.getInstance();

    // set rfc depending the program action
    if ( SapInbound.getProgramAction().equals("auto") )
    {
      fAutoRfc = (AutoRFC)rfc;
    }
    else
    {
      fManualRfc = (ManualRFC)rfc;
    }

    // get sap jgb properties
    fBlockSizeSesUpd = sp.getInt(SapInbound.getProperties(),"sap.jgb.block_size.ses_update",10);
    
    // get bfaktid offset for simulation mode (for tests ONLY)
    if ( sp.getInt(SapInbound.getProperties(),"simulation.jgb.modify",0) == 1 )
    {
      fBfaktIdOffset = sp.getInt(SapInbound.getProperties(),"simulation.jgb.bfaktid.offset",0);
      if ( fBfaktIdOffset != 0 )
        LOG.debug("simulation.jgb.modify enabled with bfaktid offset " + fBfaktIdOffset );
      
    }
  }


  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * process jgb job
   *  - get list from ses db with SAPRechnungVO to process
   *  - divide list in block's ses update size
   *  - process each block
   *  - check end conditions and finish run if true
   *
   * @param sapJGBJobVO jgb job
   * @throws Exception  unexpectetd error
   */
  public void process(SAPJGBJobVO sapJGBJobVO)
  {
    ArrayList     list  = null;     // list with all SAPRechnungVO's
    String        info  = null;     // sapJGBJobVO info


    //
    // start
    //
    info = "(jgbLaufId="  + sapJGBJobVO.jgbLaufId   + ", " +
           "bfaktIdFrom=" + sapJGBJobVO.bfaktIdFrom + ", " +
           "bfaktIdTo="   + sapJGBJobVO.bfaktIdTo   + ", " +
           "lastJob="     + sapJGBJobVO.lastJob     + ")";

    if ( LOG.isDebugEnabled() )
      LOG.debug ("process - *** start " + info + " ***");


    //
    // get list from ses db and set sap stat on ses jgb lauf table
    //
    try
    {
      LOG.debug("process - requestJgbRechnungJob");

      list = fDelegate.requestJgbRechnungJob(sapJGBJobVO);

      if ( LOG.isDebugEnabled() )
        LOG.debug("process - requestJgbRechnungJob " + list.size() + " records selected");

    }
    catch (Exception ex)
    {
      fErrOccured = true;
      fErrMsg     = "requestJgbRechnungJob " + info + " failed " +
                    "with exception: " + ex.getMessage();

      LOG.error("process - " + fErrMsg);
      return;
    }


    //
    // process list (in block's of ses update size or full list)
    //
    if (list.size() > 0)
    {
      // process list
      if (fBlockSizeSesUpd == 0)
      {
        processBlock(list);
      }
      else
      {
        Iterator  it    = list.iterator();
        ArrayList block = null;
        int       cnt = 0;

        while ( it.hasNext() )
        {
          block = new ArrayList();
          cnt   = 0;

          while ( it.hasNext() && (cnt < fBlockSizeSesUpd) )
          {
            block.add( it.next() );
            cnt++;
          }

          processBlock(block);
        }
      }

      // check for errors
      if ( fErrOccured )
      {
        fErrMsg = "processBlock ended with errors, " + getStatistic();

        LOG.error("process - " + fErrMsg);

        if ( checkStop(sapJGBJobVO.jgbLaufId) )
          processStop(sapJGBJobVO.jgbLaufId);

        return;
      }
    }


    //
    // check end conditions and finisch job if reached
    //
    LOG.debug("process - check end conditions...");
    if ( checkFinish(sapJGBJobVO.jgbLaufId) )
    {
      processFinish(sapJGBJobVO.jgbLaufId);
    }
    else
    {
      LOG.debug("process - end conditions NOT reached, check stop conditions...");
      if ( checkStop(sapJGBJobVO.jgbLaufId) )
      {
        processStop(sapJGBJobVO.jgbLaufId);
      }
    }

    if ( fErrOccured ) return;                // check for errors


    //
    // all done fine
    //
    if ( LOG.isDebugEnabled() )
      LOG.debug("process - *** end successfully, " + getStatistic() + " ***");


  } // end method


  /**
   * indicate error state of process() and finish() method calls
   *
   * @return state
   */
  public boolean isErrorOccured()
  {
    return fErrOccured;
  }


  /**
   * get error message as string
   *
   * @return message
   */
  public String getErrorMessage()
  {
    return fErrMsg;
  }

  /**
   * gather statistic info about the processed records by calling the
   * method process().
   *
   * @return statistic info
   */
  public String getStatistic()
  {
     return ( "processed records: " +
              "total="  + fRecCntTotal  + " : " +
              "sap ok=" + fRecCntSapOk  + ", sap err=" + fRecCntSapErr + " : " +
              "ses ok=" + fRecCntSesOk  + ", ses err=" + fRecCntSesErr  );
  }


  /**
   * override value from property file
   * note:
   * this method is only used in manual mode.
   *
   * @param size block size for ses updates
   */
  public void setBlockSizeSesUpdate(int size)
  {
    fBlockSizeSesUpd = size;
  }


  /**
   * finishes jgb run without checking the end conditions.
   * important:
   * this method is only intended for spezial error handling conditions and
   * should be used very carefully !
   *
   * @param jgbLaudId jgb lauf id from run to finish
   */
  public void forceFinish (int jgbLaudId)
  {
    processFinish(jgbLaudId);
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * process block of SAPRechnungVO's
   *  - insert vo's into sap system
   *  - update state of vo
   *  - update ses db
   *
   * @param block list of SAPRechnungVO
   */
  private void processBlock(ArrayList block)
  {
    int           retCode   = RFC.RET_CODE_SUCCESS;   // sap call return code
    StringBuffer  retMsg    = new StringBuffer();     // sap call return message

    Iterator      it        = block.iterator();
    SAPRechnungVO vo        = null;
    String        range     = "";


    // update record count total
    fRecCntTotal = fRecCntTotal + block.size();


    // insert vo's into sap system and update state of vo
    while ( it.hasNext() )
    {
      // init
      vo    = (SAPRechnungVO) it.next();

      range = range + vo.bfakt_id + " ";

      if (retMsg.length() > 0)
        retMsg.delete(0,retMsg.length());

      
      // create billing request's on sap (single commit)
      vo.bfakt_id += fBfaktIdOffset;    // simulation for tests ONLY
      
      if ( fAutoRfc != null)
      {
        retCode = fAutoRfc.createBillingReq(vo,retMsg);
      }
      else
      {
        retCode = fManualRfc.testJgbCreateBillingReq(vo,retMsg);
      }

      vo.bfakt_id -= fBfaktIdOffset;  // simulation for tests ONLY

      // update status of SAPRechnungVO
      if (retCode == RFC.RET_CODE_SUCCESS)
      {
        fRecCntSapOk++;
        vo.status_verarbeitung = SAPRechnungVO.STATUS_VERARBEITUNG_SAP;
      }
      else
      {
        fErrOccured = true;
        fRecCntSapErr++;

        vo.status_verarbeitung = SAPRechnungVO.STATUS_VERARBEITUNG_ERROR;
        vo.error_source        = "SapInbound" + SapInbound.getProgramId();
        vo.error_description   = retMsg.toString();

        // limit description to max. db size, since the sap retMsg may be very long.
        // (note: limitation is done here for keep network trafic small)
        if (vo.error_description.length() > 1000)
          vo.error_description = vo.error_description.substring(0,997) + "...";
      }
    }

    // update ses db
    try
    {
      fDelegate.requestJgbRechnungUpdate(block);

      fRecCntSesOk = fRecCntSesOk + block.size();

      if ( LOG.isDebugEnabled() )
        LOG.debug("processBlock - bfakt_id: " + range + " processed.");

    }
    catch (Exception ex)
    {
      fErrOccured   = true;
      fRecCntSesErr = fRecCntSesErr + block.size();
      LOG.fatal("processBlock - requestJgbRechnungUpdate for bfakt_id: " + range + " failed !");
      LOG.fatal("processBlock - requestJgbRechnungUpdate exception: " + ex.getMessage() );
      //TODO maintenance (optional) - weitere behandlung: inkonsistenz zwischen ses db und sap system
    }

  } // end method


  /**
   * check finish conditions
   *
   * @param   jgbLaufId   jgb job id
   * @return  true if stop condition is reached
   */
  private boolean checkFinish (int jgbLaufId)
  {
    boolean ret = false;

    try
    {
      if ( fDelegate.requestJgbEndCondition(jgbLaufId) )
      {
        LOG.debug("checkFinish - conditions reached");
        ret = true;
      }
      else
      {
        LOG.debug("checkFinish - conditions NOT reached");
      }
    }
    catch (Exception ex)
    {
      if ( ! fErrOccured )
      {
        fErrOccured   = true;
        fErrMsg = "checkFinish - requestJgbEndCondition for jgbLaufId=" + jgbLaufId +
                  " failed with exception:" + ex.toString();
      }

      LOG.fatal(fErrMsg);
    }

    return ret;

  } // end method


  /**
   * finish jgb run
   *  - get lauf number from ses db
   *  - commit annual fee on sap system
   *  - update ses db
   *
   * @param jgbLaufId   jgb job id
   * @throws Exception  unexpectetd error
   */
  private void processFinish (int jgbLaufId)
  {
    int           retCode   = RFC.RET_CODE_SUCCESS;   // sap call return code
    StringBuffer  retMsg    = new StringBuffer();     // sap call return
    JGBLaufVO     jgbLaufVO = null;


    // get lauf number from ses db
    try
    {
      jgbLaufVO = fDelegate.requestJgbLaufSelect(jgbLaufId);

      if ( jgbLaufVO == null )
      {
        fErrOccured = true;
        fErrMsg     = "requestJgbLaufSelect no records found for jgbLaufId=" + jgbLaufId;

        LOG.error("processFinish - " + fErrMsg);
        return;
      }

    }
    catch (Exception ex)
    {
      fErrOccured = true;
      fErrMsg     = "requestJgbLaufSelect with jgbLaufId=" + jgbLaufId +
                    " failed with exception: " + ex.getMessage();

      LOG.error("processFinish - " + fErrMsg);
      return;
    }


    // commit annual fee on sap
    if ( fAutoRfc != null)
    {
      retCode = fAutoRfc.commitAnnualFee(jgbLaufVO.laufNr,retMsg);
    }
    else
    {
      retCode = fManualRfc.testCommitAnnualFee(jgbLaufVO.laufNr,retMsg);
    }

    if ( retCode != RFC.RET_CODE_SUCCESS )
    {
      fErrOccured = true;
      fErrMsg     = "commitAnnualFee failed: \n" + retMsg.toString();

      LOG.error("processFinish - " + fErrMsg);
      return;
    }


    // update jgb_lauf on ses db
    try
    {
      fDelegate.requestJgbLaufUpdate ( jgbLaufId,
                                       JGBLaufVO.SAP_STAT_FINISHED );
    }
    catch (Exception ex)
    {
      fErrOccured   = true;
      fErrMsg = "requestJgbLaufUpdate for jgbLaufId=" + jgbLaufId +
                " failed with exception:" + ex.toString();

      LOG.fatal("processFinish - " + fErrMsg);
      //TODO maintenance (optional) - weitere behandlung: inkonsistenz zwischen ses db und sap system
      return;
    }

    // end without error's
    LOG.debug("processFinish - successfully done");

  } // end method


  /**
   * check stop conditions
   *
   * @param   jgbLaufId   jgb job id
   * @return  true if stop condition is reached
   */
  private boolean checkStop (int jgbLaufId)
  {
    boolean ret = false;

    try
    {
      if ( fDelegate.requestJgbStopCondition(jgbLaufId) )
      {
        LOG.debug("checkStop - stop conditions reached");
        ret = true;
      }
      else
      {
        LOG.debug("checkStop - stop conditions NOT reached");
      }
    }
    catch (Exception ex)
    {
      if ( ! fErrOccured )
      {
        fErrOccured   = true;
        fErrMsg = "checkStop - requestJgbStopCondition for jgbLaufId=" + jgbLaufId +
                  " failed with exception:" + ex.toString();
      }

      LOG.error(fErrMsg);
    }

    return ret;

  } // end method


  /**
   * stop jgb run
   *
   * @param jgbLaufId   jgb job id
   * @throws Exception  unexpectetd error
   */
  private void processStop (int jgbLaufId)
  {
    try
    {
      fDelegate.requestJgbLaufUpdate ( jgbLaufId,
                                       JGBLaufVO.SAP_STAT_STOPPED );

      LOG.debug("processStop - successfully done");

    }
    catch (Exception ex)
    {
      if ( ! fErrOccured )
      {
        fErrOccured   = true;
        fErrMsg = "processStop - requestJgbLaufUpdate for jgbLaufId=" + jgbLaufId +
                  " failed with exception:" + ex.toString();
      }

      LOG.error(fErrMsg);
    }

  } // end method

} // end class


