package ch.ige.ses.client.sapoutbound;

import com.sap.mw.jco.*;
import org.apache.log4j.*;
import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.client.sapinbound.SapInbound;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.server.sapctrl.SapDelegateOutbound;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.TraceVO;

/**
 * Erhält SAP Calls und leitet diese an die J2EE Umgebung weiter.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapServer.java,v 1.36 2007/12/28 12:30:04 iten Exp $
 */
public class SapServer extends JCO.Server
{
  //
  // const
  //
  private static final Logger   LOG                 = Logger.getLogger(SapServer.class);
  private static final int      DEF_SAP_TRACE       = 0;
  private static final int      DEF_BLOCK_SIZE_UPD  = 100;

  //
  // fields
  //
  protected String    fSapServerNr;       // programm id and thread number (example: "01-0")
  protected String    fSapServerName;     // server name for log messages

  private   boolean   fRequestProcess;    // process request, i.e send to bagis
  private   boolean   fLogTableDump;      // sapoutbound log sap table dump
  private   int       fSapTrace;          // sapoutbound trace options
  private   int       fBlockSizeUpdate;   // block size for updateing bagis & ses db
  private int         fBfaktIdOffset=0;   // simulation.jgb.bfaktid.offset (for tests ONLY)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SapServer ( String gwhost, String gwserv, String progid,
                     IRepository repository, int threadNr ) throws Exception
  {
    super(gwhost,gwserv,progid,repository);

    fSapServerNr   = SapOutbound.getProgramId() + "-" + Integer.toString(threadNr);
    fSapServerName = "SapOutbound " + fSapServerNr + " : ";

    // get sap outbound trace options & block size for updates
    SapProperties sp = new SapProperties();
    fSapTrace         = sp.getInt(SapOutbound.getProperties(),"sapoutbound.trace",DEF_SAP_TRACE);
    fBlockSizeUpdate  = sp.getInt(SapOutbound.getProperties(),"bagis.send.state.block_size.update",DEF_BLOCK_SIZE_UPD);

    // get sapserver.request.process option
    if (SapOutbound.getProperties().getProperty("sapserver.request.process").equals("1"))
      fRequestProcess = true;
    else
      fRequestProcess = false;

    // get sapoutbound.log.detail option
    if (SapOutbound.getProperties().getProperty("sapoutbound.log.tabledump").equals("1"))
      fLogTableDump = true;
    else
      fLogTableDump = false;
    
    // get bfaktid offset for simulation mode (for tests ONLY)
    if ( sp.getInt(SapOutbound.getProperties(),"simulation.jgb.modify",0) == 1 )
    {
      fBfaktIdOffset = sp.getInt(SapOutbound.getProperties(),"simulation.jgb.bfaktid.offset",0);
      if ( fBfaktIdOffset != 0 )
        LOG.debug( fSapServerName + "simulation.jgb.modify enabled with bfaktid offset " + fBfaktIdOffset );
    }
  }

  //---------------------------------------------------------------------------
  // public / protected methods
  //---------------------------------------------------------------------------

  protected void handleRequest(JCO.Function function)
  {
    try
    {
      String name = function.getName();

      if ( LOG.isInfoEnabled() )
        LOG.info(fSapServerName + "handleRequest - request " + name + " received");

      if ( name.equals("Z_BAGIS_SEND_STATE") )
      {
        this.handleZ_BAGIS_SEND_STATE ( function.getTableParameterList() );
      }
      else if ( name.equals("STFC_CONNECTION") )
      {
        this.handleSTFC_CONNECTION ( function.getImportParameterList(),
                                     function.getExportParameterList() );
      }
      else if ( name.equals("STFC_STRUCTURE") )
      {
        this.handleSTFC_STRUCTURE ( function.getImportParameterList(),
                                    function.getExportParameterList() );
      }
      else
      {
        LOG.warn(fSapServerName + "handleRequest - received & SKIPPED unknown request " + name + " !");
      }

      if ( LOG.isInfoEnabled() )
        LOG.info(fSapServerName + "handleRequest - request " + name + " done.");

    }
    catch (Exception ex)
    {
      LOG.error(fSapServerName + "handleRequest - exception occured: " + ex.getMessage());
    }

  } // end method

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void handleZ_BAGIS_SEND_STATE ( JCO.ParameterList tblParamList )
  {
    JCO.Table                 tbl     = tblParamList.getTable("STATE");
    SAPRechnungStatusListVO   list    = new SAPRechnungStatusListVO();
    SAPRechnungStatusListVO   jgblist = new SAPRechnungStatusListVO();
    SAPRechnungStatusVO       vo      = null;
    int                       state   = 0;
    int                       step    = 0;            // process step


    try
    {
      //
      // check if table is empty
      //
      step = 10;
      if (tbl.getNumRows() == 0)
      {
        throw new Exception("received & SKIPPED empty table " + tbl.getName() + " !");
      }

      //
      // loop table and get values
      //
      step = 20;
      do {
        // init
        if (fLogTableDump)
        {
          LOG.info(fSapServerName + "handleZ_BAGIS_SEND_STATE - table STATE dump:");
          LOG.info(tbl.toString());
        }
        vo = new SAPRechnungStatusVO();

        // fill values
        vo.bfakt_id  = Convert.toBagisBfaktId( tbl.getString("DOC_NUMBER") ) + fBfaktIdOffset;
        vo.jgbLaufNr = tbl.getString("ANN_FEE_SEQ_NO");
        vo.amvLaufNr = tbl.getString("TM_EXT_SEQ_NO");

        state = tbl.getInt("STATE");

        if (state == 1)                                   // status druck
        {
          vo.status_druck  = Convert.toBagisStatusDruck(Integer.toString(state));
          vo.bfakt_dat     = tbl.getDate("DATE");
        }
        else                                              // status zahlung
        {
          vo.status_zahlg   = Convert.toBagisStatusZahlg(Integer.toString(state));
          vo.status_datum   = tbl.getDate("DATE");
          vo.buch_datum     = tbl.getDate("DATE1");
          vo.fakt_betrag    = tbl.getBigDecimal("NETWR");
          // vo.waehr_cd    = tbl.getString("WAERS");     // not used, constant set to CHF by vo
        }

        // add to list
        list.statusList.add(vo);
        if ( LOG.isInfoEnabled() )
        {
          LOG.info(fSapServerName + "handleZ_BAGIS_SEND_STATE - add to list record with bfakt_id = " + vo.bfakt_id);
        }

        // add to jgblist (ONLY status druck)
        if (! vo.jgbLaufNr.equals("") && (state == 1) )
        {
          jgblist.statusList.add(vo);
          if ( LOG.isInfoEnabled() )
            LOG.info(fSapServerName + "handleZ_BAGIS_SEND_STATE - add to jgblist record with bfakt_id = " + vo.bfakt_id);
        }

      } while(tbl.nextRow());

      //
      // process message
      //
      step = 30;
      if ( this.fRequestProcess )
      {
        // update bagis db
        step = 31;
        LOG.info(fSapServerName + "handleZ_BAGIS_SEND_STATE - process step 1 - update bagis db");
        updateStatusBagis(list);

        // update ses jgb db
        step = 32;
        if (jgblist.statusList.size() > 0)
        {
          LOG.info(fSapServerName + "handleZ_BAGIS_SEND_STATE - process step 2 - update ses jgb db");
          updateStatusSES(jgblist);
        }

        // successfully done
        LOG.info(fSapServerName + "handleZ_BAGIS_SEND_STATE - process all steps successfully done");
      }
      else
      {
        LOG.warn ( fSapServerName +
                   "handleZ_BAGIS_SEND_STATE - received (unprocessed) list: \n" +
                   list.toStringValidDataOnly() );
      }

    }
    catch (Exception ex)
    {
      // log error
      LOG.error( fSapServerName + "handleZ_BAGIS_SEND_STATE - " +
                 "step #" + step + " failed : " + ex.getMessage() );


      // write error info into vo
      ErrorVO err = new ErrorVO();

      err.source          = "SapOutbound" + fSapServerNr;
      err.employeeId      = null;
      err.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
      err.errorStat       = ErrorVO.ERROR_STAT_PENDENT;

      switch (step)
      {
        case 10:
          err.description = "handleZ_BAGIS_SEND_STATE - step 10: " + ex.getMessage();
          err.dataTypeCd  = ErrorVO.DATA_TYPE_CD_NO_DATA;
          break;

        case 20:
          err.description = "handleZ_BAGIS_SEND_STATE - step 20: get values from sap table failed, exception: " + ex.getMessage();
          err.dataTypeCd  = ErrorVO.DATA_TYPE_CD_STRING;
          err.data        = tbl.toString();
          break;

        case 31:
          err.description = "handleZ_BAGIS_SEND_STATE - step 31: update bagis db failed, exception: " + ex.getMessage();
          err.dataTypeCd  = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_STATUS_LIST_VO;
          err.data        = list;
          break;

        case 32:
          err.description = "handleZ_BAGIS_SEND_STATE - step 32: update ses jgb db failed, exception: " + ex.getMessage();
          err.dataTypeCd  = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_STATUS_LIST_VO;
          err.data        = jgblist;
          break;

        default:
          err.description = "handleZ_BAGIS_SEND_STATE - step #" + step + ": exception: " + ex.getMessage();
          err.dataTypeCd  = ErrorVO.DATA_TYPE_CD_NO_DATA;
          break;
      }

      // save vo to ses error db
      this.saveError(err);

      // add trace message
      trace(false,"Z_BAGIS_SEND_STATE",err.toString());
    }

  } // end method


  private void handleSTFC_CONNECTION ( JCO.ParameterList in,
                                       JCO.ParameterList out )
  {
    String request  = in.getString("REQUTEXT");
    String response = "SapOutbound #" + SapOutbound.getProgramId() + " - " +
                      fSapServerName + "test response from STFC_CONNECTION";

    if ( LOG.isInfoEnabled() )
      LOG.info(fSapServerName + "handleSTFC_CONNECTION - REQUTEXT = " + request);

    out.setValue(request, "ECHOTEXT");
    out.setValue(response,"RESPTEXT");
  }


  private void handleSTFC_STRUCTURE ( JCO.ParameterList in,
                                      JCO.ParameterList out )
  {
    JCO.Structure imp  = in.getStructure("IMPORTSTRUCT");
    JCO.Structure echo = (JCO.Structure)imp.clone();
    String        resp = "SapOutbound #" + SapOutbound.getProgramId() + " - " +
                         fSapServerName + "test response from STFC_STRUCTURE";

    if ( LOG.isInfoEnabled() )
      LOG.info(fSapServerName + "handleSTFC_STRUCTURE - IMPORTSTRUCT");

    out.setValue(echo,"ECHOSTRUCT");
    out.setValue(resp,"RESPTEXT");
  }


  private void updateStatusBagis (SAPRechnungStatusListVO vo) throws Exception
  {
    Iterator                  it      = vo.statusList.iterator();
    SAPRechnungStatusListVO   block   = null;
    int                       size    = 0;              // block size
    int                       number  = 0;              // block number

    LOG.info(fSapServerName + "updateStatusBagis - list size=" + vo.statusList.size());
    while ( it.hasNext() )
    {
      block = new SAPRechnungStatusListVO();
      size  = 0;
      number++;

      while ( it.hasNext() && (size < fBlockSizeUpdate) )
      {
        block.statusList.add( it.next() );
        size++;
      }

      LOG.info ( fSapServerName + "updateStatusBagis - update block #" + number +
                 " (blocksize=" + size + ")" );

      SapDelegateOutbound.getInstance().receiveStatus(block);

      trace ( true,"Z_BAGIS_SEND_STATE updateStatusBagis (Block #" + number + ")",
              block.toStringValidDataOnly());

    } // end while
  }


  private void updateStatusSES (SAPRechnungStatusListVO vo) throws Exception
  {
    Iterator                  it      = vo.statusList.iterator();
    SAPRechnungStatusListVO   block   = null;
    int                       size    = 0;              // block size
    int                       number  = 0;              // block number

    LOG.info(fSapServerName + "updateStatusSES - list size=" + vo.statusList.size());
    while ( it.hasNext() )
    {
      block = new SAPRechnungStatusListVO();
      size  = 0;
      number++;

      while ( it.hasNext() && (size < fBlockSizeUpdate) )
      {
        block.statusList.add( it.next() );
        size++;
      }

      LOG.info ( fSapServerName + "updateStatusSES - update block #" + number +
                 " (blocksize=" + size + ")" );

      SapDelegateOutbound.getInstance().receiveStatusJGB(block);

      trace ( true,"Z_BAGIS_SEND_STATE updateStatusSES (Block #" + number + ")",
              block.toStringValidDataOnly());

    }
  }


  private boolean saveError (ErrorVO err)
  {
    try
    {
      LOG.info("saveError - save vo to ses error db");
      SapDelegateOutbound.getInstance().receiveError(err);
      return true;
    }
    catch (Exception ex)
    {
      LOG.error("saveError - save vo to ses error db failed: " + ex.getMessage() );

      // save error vo to local file
      new ErrorFile ( SapOutbound.getProperties().getProperty("sapoutbound.data.path"),
                      SapOutbound.getProgramIdentifier() ).save(err);

      return false;
    }
  }


  private void trace (boolean actionOk, String actionDesc, String data)
  {
    TraceVO  vo;

    try
    {
      if (fSapTrace == 0) return;                     // trace is switched off

      vo = new TraceVO();                             // assemble message
      vo.process    = SapOutbound.getProgramName();
      vo.processNr  = fSapServerNr;

      if (actionOk)
        vo.action = vo.ACTION_OK;
      else
        vo.action = vo.ACTION_NOK;
      vo.action = vo.action + vo.ACTION_SEP + actionDesc;

      vo.data       = data;

      if (fSapTrace == 1)                             // trace to stdout
      {
        System.out.println("### TRACE - START");
        System.out.println("TRACE-ACTION:");
        System.out.println(vo.action);
        System.out.println("TRACE-DATA:");
        System.out.println(vo.data);
        System.out.println("### TRACE - END");
      }
      else if (fSapTrace == 2)                        // trace to ses db
      {
        LOG.info("trace - write message to ses db");
        SapDelegateOutbound.getInstance().receiveTrace(vo);
      }

    }
    catch (Exception ex)
    {
      LOG.error("trace - exception: " + ex.getMessage() );
    }
  }

} // end class

