package ch.ige.ses.server.controlling;

import java.sql.SQLException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.ses.StakoKontrolleDAO;
import ch.ige.ses.server.error.SESErrorProcessLocal;
import ch.ige.ses.server.error.SESErrorProcessLocalHome;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.StakoKontrolleVO;

/**
 * SES: Bean zum Einlesen und updaten der Stako relevanten Artikel<br>
 * Liest die für die Jahresgebühren benötigten Bfakturen von Bagis ein
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Stefan Nydegger
 * @version  $Id: StakoControlBean.java,v 1.19 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="StakoControl"  
 *  display-name="StakoControl"
 *  view-type="local"
 *  jndi-name="StakoControl"
 *  local-jndi-name="StakoControlLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESErrorProcess"
 *   view-type="local"
 */
public class StakoControlBean implements SessionBean
{
  private static final Logger LOG     = Logger.getLogger(StakoControlBean.class);
  SESErrorProcessLocalHome sesErrorProcessLocalHome = null;
  SESErrorProcessLocal     sesErrorProcessLocal     = null;
  SessionContext sessionContext;

  //---------------------------------------------------------------------------
  //  bean methods
  //---------------------------------------------------------------------------

  /**
   * Holt mittels Lookup das HomeInterface und instanziert das RemoteInterface
   * des Bean SESErrorProcess für das Error-Handling
   */
  public void ejbCreate() 
  {
    try
    {
      //Naming-Kontext ermitteln
      Context ctx = new InitialContext();
      Object ref = ctx.lookup(sesErrorProcessLocalHome.COMP_NAME);
      sesErrorProcessLocalHome = (SESErrorProcessLocalHome)ref;
      sesErrorProcessLocal = sesErrorProcessLocalHome.create();
    }
    catch (CreateException ex)
    {
      LOG.error("StakoControlBean Methode ejbCreate: CreateException: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("StakoControlBean Methode ejbCreate: NamingException: " + ex.getMessage());
    }
  }

  public void ejbRemove()
  {
    // lazy bone
  }
  
  public void ejbActivate()
  {
    // lazy bone
  }
  
  public void ejbPassivate()
  {
    // lazy bone
  }
  
  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  //   methods
  //---------------------------------------------------------------------------
  
  /**
   * Kreiert neuen Stako_Kontrolle Eintrag
   * 
   * @param stakoKontrolleVO ValueObject mit Daten einer Stako-Verarbeitung
   * 
   * @ejb.interface-method
   */
  public void createStakoKontrolle(StakoKontrolleVO stakoKontrolleVO)
  {
    try
    {
      new StakoKontrolleDAO().insert(stakoKontrolleVO);
    }
    catch (SQLException sqlex)
    {
      try
      {
        // Eintrag in Tabelle ERROR
        writeToError("StakoControlBean Methode createStakoKontrolle",
                     "StakoControlBean Methode createStakoKontrolle: Insert des Datensatzes fehlgeschlagen");
      }
      catch (Exception ex)
      {
        LOG.error("StakoControlBean Methode createStakoKontrolle: Eintrag in ERROR nicht möglich");
        throw new EJBException(ex);
      }
    }
  }

  
  /**
  * Aktualisiert neuen Stako_Kontrolle Eintrag
  * 
  * @param stakoKontrolleVO ValueObject mit Daten einer Stako-Verarbeitung
  * 
  * @ejb.interface-method
  */
  public void setStakoKontrolle(StakoKontrolleVO stakoKontrolleVO)
  {
    try
    {
      new StakoKontrolleDAO().update(stakoKontrolleVO);
    }
    catch (SQLException sqlex)
    {
      try
      {
        //Eintrag in Tabelle ERROR
        writeToError("StakoControlBean Methode updateStakoKontrolle",
                     "StakoControlBean Methode updateStakoKontrolle: Update des Datensatzes fehlgeschlagen");
      }
      catch (Exception ex)
      {
        LOG.error("StakoControlBean Methode updateStakoKontrolle: Eintrag in ERROR nicht möglich");
        throw new EJBException(ex);
      }
    }
  }

  
  /**
   * Selektiert Stako_Kontrolle eintrag mit übergebener KontrollId
   * 
   * @param kontrollId  Primary Key der Stako Verarbeitung
   * @return ValueObject mit Daten einer Stako-Verarbeitung
   * 
   * @ejb.interface-method
   */
  public StakoKontrolleVO getStakoKontrolle(int kontrollId)
  {
    try
    {
      return new StakoKontrolleDAO().select(kontrollId);
    }
    catch (SQLException sqlex)
    {
      try
      {
        //Eintrag in Tabelle ERROR
        writeToError("StakoControlBean Methode getStakoKontrolle",
                     "StakoControlBean Methode getStakoKontrolle: Lesen von DB fehlgeschlagen");
        return null;
      }
      catch (Exception ex)
      {
        LOG.error("StakoControlBean Methode getStakoKontrolle: Eintrag in Tabelle ERROR nicht möglich");
        throw new EJBException(ex);
      }
    }
  }

  
  /**
   * Selektiert letzten Stako_Kontrolle Eintrag
   * 
   * @return ValueObject mit Daten einer Stako-Verarbeitung
   * 
   * @ejb.interface-method
   */
  public StakoKontrolleVO getLastStakoKontrolle()
  {
    try
    {
      return new StakoKontrolleDAO().selectMaxKontrollId();
    }
    catch (SQLException sqlex)
    {
      try
      {
        // Eintrag in Tabelle ERROR
        writeToError("StakoControlBean Methode getLastStakoKontrolle",
                     "StakoControlBean Methode getLastStakoKontrolle: Lesen von DB fehlgeschlagen");
        return null;
      }
      catch (Exception ex)
      {
        LOG.error("StakoControlBean Methode getLastStakoKontrolle: Eintrag in ERROR nicht möglich");
        throw new EJBException(ex);
      }
    }
  }

  
  /**
   * Methode writeToError schreibt bei Fehler bei der Stako Verarbeitung
   * ein ErrorVO in die Tabelle ERROR und loggt den Fehler
   * 
   * @param src         Klasse und Methode in der der Fehler auftritt
   * @param errMsg      Fehlermeldung
   * @throws Exception  Unerwarteter Fehler
   */
  private void writeToError(String src, String errMsg) throws Exception
  {
    // Fehler loggen
    LOG.error(errMsg);
    // ErrorVO füllen
    ErrorVO  errorVO  = new ErrorVO();
    errorVO.source = src;
    errorVO.description = errMsg;
    errorVO.employeeNotify = ErrorVO.EMPL_NOTIFY_PENDENT;
    errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_STAKO;
    errorVO.dataTypeCd = ErrorVO.DATA_TYPE_CD_NO_DATA;
    errorVO.errorStat = ErrorVO.ERROR_STAT_PENDENT;
    //Aufruf des Beans SESErrorProcess
    sesErrorProcessLocal.processError(errorVO);

  }
  
} // end of class
