package ch.ige.ses.server.dao.bagis;

import java.sql.*;
import java.util.*;

import ch.ige.util.server.*;
import ch.ige.ses.vo.bagis.*;

/**
 * DataAccessObject für Stako relevante Artikel
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: JoinStakoArtikelDAO.java,v 1.10 2006/12/11 12:43:32 iten Exp $
 */

public class JoinStakoArtikelDAO extends BagisDAO
{
  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Selektiert die Stako relevanten Artikel aus den Tabellen bfaktur,
   * bfaktpos, artikel und ma
   *
   * @param   von           Beginn der zu selektierenden Datum-Range
   * @param   bis           Ende der zu selektierenden Datum-Range  (optional)
   * @param   exclMa        Auszuschliessende ma_nr                 (optional)
   * @throws  SQLException  Unerwarteter SQL Fehler
   * @return  ArrayList     Liste mit StakoVO's
   */
  public ArrayList selectBfakturBfaktPos(java.util.Date von, java.util.Date bis, String exclMa) throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;
    ArrayList         al    = new ArrayList();

    try
    {
      con = getConnection();

      setQuery(getBfakturBfaktPosStatement(bis == null ? true : false, exclMa));
      ps = con.prepareStatement(getQuery());

      ps.setDate(1,toSqlDate(von));
      if(bis != null)
      {
        ps.setDate(2, toSqlDatePlusOneDay(bis));
      }

      rs = ps.executeQuery();

      while(rs.next())
      {
        StakoVO stakoVO    = new StakoVO();

        stakoVO.bfaktId    = rs.getInt("bfakt_id");
        stakoVO.bfaktPosNr = rs.getInt("bfakt_posnr");
        stakoVO.employeeId = rs.getString("employee_id");
        stakoVO.schutitNr  = rs.getString("schutit_nr");
        stakoVO.artNr      = rs.getString("art_nr");
        stakoVO.datum      = rs.getDate("bfakt_dat");
        stakoVO.anzahl     = rs.getBigDecimal("anzahl");
        stakoVO.herkunft   = rs.getString("herkunft");

        al.add(stakoVO);
      }

      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert die Stako relevanten Artikel artikel, ma, ereignis
   *
   * @param   von           Beginn der zu selektierenden Datum-Range
   * @param   bis           Ende der zu selektierenden Datum-Range
   * @param   exclMa        Auszuschliessende ma_nr
   * @throws  SQLException  Unerwarteter SQL Fehler
   * @return  ArrayList     Liste mit StakoVO's
   */
  public ArrayList selectEreignisMA(java.util.Date von, java.util.Date bis, String exclMa) throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;
    ArrayList         al    = new ArrayList();

    try
    {
      con = getConnection();

      setQuery(getEreignisMAStatement(bis == null ? true : false, exclMa));
      ps = con.prepareStatement(getQuery());

      ps.setDate(1,toSqlDate(von));
      if(bis != null)
      {
        ps.setDate(2,toSqlDatePlusOneDay(bis));
      }

      rs = ps.executeQuery();

      while(rs.next())
      {
        StakoVO stakoVO    = new StakoVO();

        stakoVO.employeeId = rs.getString("employee_id");
        stakoVO.schutitNr  = rs.getString("schutit_nr");
        stakoVO.artNr      = rs.getString("art_nr");
        stakoVO.datum      = rs.getDate("erfass_dat");
        stakoVO.anzahl     = rs.getBigDecimal("anzahl");
        stakoVO.herkunft   = rs.getString("herkunft");

        al.add(stakoVO);
      }
      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Setzt den Standpreis aller Artikel auf 0.
   *
   * @throws  SQLException  Unerwarteter SQL Fehler
   */
  public void resetStakoArtikel() throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps   = null;

    try
    {
      con = getConnection();

      ps = con.prepareStatement("UPDATE artikel SET standpreis = 0");
      ps.executeQuery();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert den standpreis der Stako relevanten Artikel
   *
   * @param     al            ArrayList mit art_nr
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public void updateStakoArtikel(ArrayList al) throws SQLException
  {
    Connection        con       = null;
    Statement         stmt      = null;
    StringBuffer      bufArtNr  = new StringBuffer();

    try
    {
      // Artikel in String umwandeln
      for (int i = 0; i < al.size(); i++)
      {
        bufArtNr.append(al.get(i) + " ,");
      }
      //letztes Komma entfernen
      bufArtNr.deleteCharAt(bufArtNr.length()-1);

      con = getConnection();
      stmt  = con.createStatement();
      setQuery("UPDATE artikel SET standpreis=1 WHERE art_nr IN ( " + bufArtNr.toString() + ")");

      stmt.executeQuery(getQuery());
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Liefert SQL-Statement für die Methode selectBfakturBfaktPos
   * Bemerkung:
   *   BFAKTUR.KTO_NR = Regulierer einer Rechnung
   *
   * @param   bisIsNull true=wenn kein End-Datum angegeben, false=wenn eine Range angegeben ist
   * @param   exclMa    Auszuschliessende ma_nr
   * @return  String    SQL-Statement
   */
  private String getBfakturBfaktPosStatement(boolean bisIsNull, String exclMa)
  {
    String sql    = null;

    String select = "SELECT  bp.bfakt_id, bp.bfakt_posnr, ma.employee_id, trim(bp.schutit_nr) as schutit_nr, " +
                            "trim(bp.art_nr) as art_nr, b.bfakt_dat, bp.anzahl, '2' herkunft ";
    String from   = "FROM bfaktur b, bfaktpos bp, ma ma, artikel a ";
    String where  = "WHERE b.bfakt_id > " + BFAKTID_BASE + " AND ";


    String whereVon     = " b.bfakt_dat > ? AND ";

    String whereRange   = "(b.bfakt_dat >=? AND b.bfakt_dat <?) AND ";


    String excludingMa  = "ma.ma_nr NOT IN ( " + exclMa + ") AND ";

    if(bisIsNull)
    {
      where += whereVon;
    }
    else
    {
      where += whereRange;
    }

    where = where + "b.status_zahlg NOT IN ('9','2') AND " +
                    "b.status_druck > '0' AND " +
                    "( ( b.bfakt_typ IN ('0','1') AND bp.schutit_key IS NOT NULL) OR " +
                    "( b.bfakt_typ > '1' ) ) AND " +
                    "b.ma_nr = ma.ma_nr AND ";

    String whereEnd ="b.bfakt_id = bp.bfakt_id AND " +
                    "bp.bfakt_posnr > 0 AND " +
                    "bp.art_nr = a.art_nr AND " +
                    "a.standpreis > 0 " +
                    "ORDER BY b.bfakt_dat ";

    if(exclMa != null && exclMa.length() > 0)
    {
      sql = select + from + where + excludingMa + whereEnd;
    }
    else
    {
      sql = select + from + where + whereEnd;
    }

    return sql;
  }


  /**
   * Liefert SQL-Statement für die Methode selectEreignisMA
   * Bemerkungen:
   *   formart_cd is null ist nötig, weil alle Datensätze mit vorhandenem
   *   formart_cd bereits bei der Herkunft bfaktur/bfaktpos selektiert werden
   *
   * @param   bisIsNull true=wenn kein End-Datum angegeben, false=wenn eine Range angegeben ist
   * @param   exclMa    Auszuschliessende ma_nr
   * @return  String    SQL-Statement
   */
  private String getEreignisMAStatement(boolean bisIsNull, String exclMa)
  {
    String sql    = null;

    String select = "SELECT ma.employee_id, schutit.schutit_nr, trim(e.art_nr) as art_nr, e.erfass_dat, 1 anzahl, '1' herkunft ";
    String from   = "FROM ereignis e, ma, schutit ";
    String where  = "WHERE e.art_nr IN (SELECT a.art_nr FROM artikel a, bartikel b WHERE a.standpreis >0 AND a.art_nr=b.art_nr AND b.formart_cd is null) " +
                    "AND   e.schutit_key > 0 " +
                    "AND   e.ereig_idat > 0 " +
                    "AND   e.ereig_stat_nr NOT IN (30, 21, 22, 89, 90, 91) " +
                    "AND   e.erfass_dat is not null " +
                    "AND ";

    String whereVon     = "e.erfass_dat >= ? AND ";

    String whereRange   = "(e.erfass_dat >=? AND e.erfass_dat <?) AND ";


    String whereEnd1    = "e.ma_nr = ma.ma_nr AND ";
    String excludingMa  = "e.ma_nr NOT IN ( " + exclMa + ") AND ";
    String whereEnd2    = "e.schutit_key = schutit.schutit_key ORDER BY e.erfass_dat ";

    if(bisIsNull)
    {
      where += whereVon;
    }
    else
    {
      where += whereRange;
    }

    if(exclMa != null && exclMa.length() > 0)
    {
      sql = select + from + where + whereEnd1 + excludingMa + whereEnd2;
    }
    else
    {
      sql = select + from + where + whereEnd1 + whereEnd2;
    }

    return sql;
  }

} // end class
