package ch.ige.ses.server.dao.ses;

import java.util.ArrayList;
import java.sql.*;
import java.sql.Date;

import ch.ige.ses.vo.ses.AmvLaufVO;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle AMV_LAUF
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author T. Iten
 * @version $Id: AmvLaufDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */
public class AmvLaufDAO
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------
  public AmvLaufDAO()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Fügt einen Datensatz in die Tabelle AMV_LAUF ein.
   * Das Feld AMV_LAUF_ID wird dabei mit Hilfe der Sequenz SES.SEQ_AMV_LAUF_ID
   * automatisch generiert.
   *
   * @param       vo              AmvLauf Value Object
   * @exception   SQLException    Unerwarteter SQL Fehler
   */
  public void insert(AmvLaufVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement (
              "insert into ses.amv_lauf " +
              "(amv_lauf_id, lauf_nr, lauf_stat, created_dat, changed_dat) " +
              "values (ses.seq_amv_lauf_id.nextval, ?, ?, sysdate, sysdate)"
            );
      ps.setString (1,vo.laufNr);
      ps.setInt    (2,vo.laufStat);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert alle Datensätze der Tabelle AMV_LAUF
   *
   * @return  ArrayList     ArrayList von AmvLaufVO's
   * @throws  SQLException  Unerwarteter SQL-Fehler
   */
  public ArrayList select() throws SQLException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    ArrayList           al  = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "ORDER BY amv_lauf_id DESC" );

      rs = ps.executeQuery();

      while(rs.next())
      {
        al.add(fillAmvLaufVO(rs));
      }

      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert einen Datensatz der Tabelle AMV_LAUF
   *
   * @param   amvLaufId     Lauf ID zum selektieren des Datensatzes.
   * @return  AmvLaufVO     Value Object
   * @throws  SQLException  Unerwarteter SQL-Fehler
   */
  public AmvLaufVO select(int amvLaufId) throws SQLException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    AmvLaufVO           vo  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement (getSelectStatement() + "where amv_lauf_id=?");
      ps.setInt(1,amvLaufId);

      rs = ps.executeQuery();
      if( rs.next() )
      {
        vo = fillAmvLaufVO(rs);
      }

      return vo;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert einen Datensatz der Tabelle AMV_LAUF
   *
   * @param   amvLaufNr     Lauf Nummer zum selektieren des Datensatzes.
   * @return  AmvLaufVO     Value Object
   * @throws  SQLException  Unerwarteter SQL-Fehler
   */
  public AmvLaufVO select(String amvLaufNr) throws SQLException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    AmvLaufVO           vo  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement (getSelectStatement() + "where lauf_nr=?");
      ps.setString(1,amvLaufNr);

      rs = ps.executeQuery();
      if( rs.next() )
      {
        vo = fillAmvLaufVO(rs);
      }

      return vo;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert alle LAUF_NR Einträge die älter sind als das angegebene
   * Erstellungsdatum.
   *
   * @param  created        Erstellungsdatum
   * @return ArrayList      ArrayList mit AmvLaufVO's
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList select(java.util.Date created) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    ArrayList         al  = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "WHERE CREATED_DAT < ? " +
              "ORDER BY LAUF_NR" );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );

      rs = ps.executeQuery();

      while(rs.next())
      {
        al.add(fillAmvLaufVO(rs));
      }

      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Gibt die Anzahl Einträge in der Tabelle AMV_LAUF zurück.
   *
   * @return  int           Anzahl Einträge
   * @throws  SQLException  Unerwarteter SQL-Fehler
   */
  public int selectCount() throws SQLException
   {
     Connection          con = null;
     PreparedStatement   ps  = null;
     ResultSet           rs  = null;
     int                 cnt = 0;

     try
     {
       con  = DBHelper.getConnection(DBHelper.SES);

       ps = con.prepareStatement
           ("select count(amv_lauf_id) as cnt from ses.amv_lauf");
       rs = ps.executeQuery();

       if( rs.next() )
       {
         cnt = rs.getInt("cnt");
       }

       return cnt;

     }
     finally
     {
       DBHelper.close(ps, con);
     }
   }


  /**
   * Gibt die maximale AMV_LAUF_ID zurück.
   *
   * @return  int           Maximale Id
   * @throws  SQLException  Unerwarteter SQL-Fehler
   */
  public int selectMaxLaufID() throws SQLException
   {
     Connection          con = null;
     PreparedStatement   ps  = null;
     ResultSet           rs  = null;
     int                 id  = 0;

     try
     {
       con  = DBHelper.getConnection(DBHelper.SES);

       ps = con.prepareStatement
           ("select max(amv_lauf_id) as max_id from ses.amv_lauf");
       rs = ps.executeQuery();

       if( rs.next() )
       {
         id = rs.getInt("max_id");
       }

       return id;

     }
     finally
     {
       DBHelper.close(ps, con);
     }
   }




  /**
   * Aktualisiert das Feld LAUF_STAT und automatisch auch das Feld CHANGED_DAT.
   *
   * @param   amvLaufId     Lauf ID des Datensatzes, welche aktualisiert wird
   * @param   laufStat      Neuer Statuswert
   * @return                Anzahl aktualisierter Datensätze
   * @throws  SQLException  Unerwarteter SQL Fehler
   */
  public int updateLaufStat(int amvLaufId, int laufStat) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps  = con.prepareStatement (
              "update ses.amv_lauf " +
              "set    lauf_stat=?, changed_dat=sysdate " +
              "where  amv_lauf_id=? " );

      ps.setInt(1,laufStat);
      ps.setInt(2,amvLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert das Feld created_dat. Die Funktion wird im Zusammenhang
   * mit der Datenbank Reorganisatin zu Testzwecken verwendet.
   *
   * @param   amvLaufId     eindeutige Lauf Id
   * @param   created       neues created_dat
   * @throws SQLException   unerwartete Fehlermeldung
   * @return int            Anzahl aktualisierte Datensätze
   */
  public int updateCreatedDat (int amvLaufId, java.util.Date created) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement
              ( "update ses.amv_lauf " +
                "set    created_dat = ?, " +
                "       changed_dat = ? " +
                "WHERE  amv_lauf_id = ? "
              );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );
      ps.setTimestamp( 2, new java.sql.Timestamp(created.getTime()) );
      ps.setInt(3, amvLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle AMV_LAUF
   *
   * @param     amvLaufId     Lauf ID des Datensatzes, welcher gelöscht wird
   * @return                  Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(int amvLaufId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
            ("delete from ses.amv_lauf where amv_lauf_id = ?");
      ps.setInt(1,amvLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * delivers sql statement for the select methods WITHOUT the where part
   *
   * note: the field must correspondent with the fields used in the
   *       method fillAmvLaufVO !
   *
   * @return  select statement
   */
  private String getSelectStatement()
  {
    return
      ( "select amv_lauf_id, lauf_nr, lauf_stat, created_dat, changed_dat " +
        "from   ses.amv_lauf " );
  }


  /**
    * fill AmvLaufVO with data from (a valid) result set.
    *
    * @param      rs            valid result set
    * @return     AmvLaufVO     value object
    * @exception  SQLException  unexpected sql error
    */
  private AmvLaufVO fillAmvLaufVO (ResultSet rs) throws SQLException
  {
    AmvLaufVO vo = new AmvLaufVO();

    vo.amvLaufId  = rs.getInt("amv_lauf_id");
    vo.laufNr     = rs.getString("lauf_nr");
    vo.laufStat   = rs.getInt("lauf_stat");
    vo.createdDat = rs.getTimestamp("created_dat") == null ?
                      null : new Date(rs.getTimestamp("created_dat").getTime());
    vo.changedDat = rs.getTimestamp("changed_dat") == null ?
                      null : new Date(rs.getTimestamp("changed_dat").getTime());

    return vo;
  } // end method


} // end class

