package ch.ige.ses.server.sapctrl;

import ch.ige.ses.exception.SapCallException;
import ch.ige.util.vo.VersadrVO;
import ch.ige.ses.vo.ses.SAPAdrStatusVO;
import ch.ige.util.ServiceLocator;

/**
 * Delegate für die Adress Schnittstelle.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapDelegateAdresse.java,v 1.11 2006/12/13 14:48:40 iten Exp $
 */
public class SapDelegateAdresse
{
  //
  // fields
  //
  private static SapDelegateAdresse   fDelegate       = null;
  private        SapInboundCtrl       fSapInboundCtrl = null;


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  private SapDelegateAdresse() throws Exception
  {
    SapInboundCtrlHome  home;

    home = (SapInboundCtrlHome) ServiceLocator.getInstance().getHome
                                  (SapInboundCtrlHome.JNDI_NAME, SapInboundCtrlHome.class);

    fSapInboundCtrl = home.create();
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Gibt eine Instanz von SapDelegateAdresse zurück, die die Aufrufe des SAP
   * Systems kappselt. Die Aufrufe an das SAP System verlaufen Asynchron.
   *
   * @return Instanz von SapDelegateAdresse
   * @throws Exception Unerwarteter Fehler bie der Instanzierung
   */
  public static SapDelegateAdresse getInstance() throws Exception
  {
    if (fDelegate == null)
    {
      fDelegate = new SapDelegateAdresse();
    }

    return fDelegate;
  }


  /**
   * Erstellt eine neue Adresse im SAP System
   *
   * @param   vo                Adressdaten
   * @throws  SapCallException  Fehler beim Verbindungsaufbau zum SAP System
   */
  public void create(VersadrVO vo) throws SapCallException
  {
    vo.setIsNew(true);
    sendAdressCreate(vo);
  }


  /**
   * Aktualisiert eine bestehende Adresse im SAP System
   *
   * @param   vo                Adressdaten
   * @throws  SapCallException  Fehler beim Verbindungsaufbau zum SAP System
   */
  public void update(VersadrVO vo) throws SapCallException
  {
    vo.setIsNew(false);
    sendAdressCreate(vo);
  }


  /**
   * Löschvormerkung für Adresse setzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void delete(int adrNr) throws SapCallException
  {
      sendAdressStatus( new SAPAdrStatusVO(adrNr,true,"X") );
  }


  /**
   * Löschvormerkung für Adresse zurücksetzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void undelete(int adrNr) throws SapCallException
  {
      sendAdressStatus( new SAPAdrStatusVO(adrNr,true," ") );
  }


  /**
   * Blockierung für Adresse setzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void block(int adrNr) throws SapCallException
  {
      sendAdressStatus( new SAPAdrStatusVO(adrNr,false,"X") );
  }


  /**
   * Blockierung für Adresse zurücksetzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void unblock(int adrNr) throws SapCallException
  {
      sendAdressStatus( new SAPAdrStatusVO(adrNr,false," ") );
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  public void sendAdressCreate(VersadrVO vo) throws SapCallException
  {
    try
    {
      fSapInboundCtrl.sendAdressCreate(vo);
    }
    catch (Exception ex)
    {
      throw new SapCallException ( "Fehler beim Verbindungsaufbau zum SAP System",
                                    ex.getMessage() );

    }
  }


  public void sendAdressStatus(SAPAdrStatusVO vo) throws SapCallException
  {
    try
    {
      fSapInboundCtrl.sendAdressStatus(vo);
    }
    catch (Exception ex)
    {
      throw new SapCallException ( "Fehler beim Verbindungsaufbau zum SAP System",
                                    ex.getMessage() );
    }
  }


} // end class


