package ch.ige.wdltool.analyzer;

import java.io.Serializable;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

/**
 * The result of the analysis of a single WDL.
 * Serveral counters are used for book-keeping information about the analysis. 
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   heinzer
 * @version  $Id: AnalysisResultVO.java,v 1.2 2008/04/18 09:48:12 heinzer Exp $
 */
public class AnalysisResultVO implements Serializable
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( AnalysisResultVO.class );

  private String analyzerLog = StringUtils.EMPTY;

  private int counterNewTerms = 0;

  private int counterExistingTerms = 0;

  private int counterExistingCandidateTerms = 0;

  private int counterInsertionError = 0;

  private boolean syntaxError;

  private boolean generalError;

  private int ipKey;

  /**
   * Public constructor.
   * @param ipKey The IP key of the trademark of this result.  
   */
  public AnalysisResultVO( int ipKey )
  {
    this.ipKey = ipKey;
  }

  /**
   * Gets the IP key of the trademakr of this result.
   * @return The IP key of the trademakr of this result
   */
  public int getIPKey()
  {
    return this.ipKey;
  }

  /**
   * Shows whether the analysis of the WDL produced an unexpected error.
   * @return <code>true</code> when the WDL has a syntax error, <code>false</code> otherwise.
   */
  public boolean hasGeneralError()
  {
    return this.generalError;
  }

  /**
   * Sets the general error flag.
   * This also logs the error on debug level and sets the analyzer log accordingly 
   * @param throwable The cause of this error
   */
  public void setGeneralError( Throwable throwable )
  {
    this.generalError = true;
    LOG.debug( "Generic exception for key: " + this.ipKey + ", " + throwable.getMessage(), throwable );
    setAnalyzerLog( "Genereller Fehler für Marke mit Schutit-Key " + this.ipKey + "; Technischer Grund: " + throwable.getMessage() + throwable );
  }

  /**
   * Shows whether the analysed WDL has a syntax error.
   * @return <code>true</code> when the WDL has a syntax error, <code>false</code> otherwise.
   */
  public boolean hasSyntaxError()
  {
    return this.syntaxError;
  }

  /**
   * Sets the syntax error flag.
   */
  public void setSyntaxError()
  {
    this.syntaxError = true;
  }

  /**
   * Gets the analyzer log.
   * @return The analyzer log
   */
  public String getAnalyzerLog()
  {
    return this.analyzerLog;
  }

  /**
   * Sets the analyzer log.
   * @param analyzerLog The log to set
   */
  public void setAnalyzerLog( String analyzerLog )
  {
    this.analyzerLog = analyzerLog;
  }

  /**
   * Gets the number of new terms.
   * @return The analyzer log
   */
  public int getCounterNewTerms()
  {
    return this.counterNewTerms;
  }

  /**
   * Increases the number of new terms.
   */
  public void incCounterNewTerms()
  {
    this.counterNewTerms++;
  }

  /**
   * Gets the number of existing terms.
   * @return The analyzer log
   */
  public int getCounterExistingTerms()
  {
    return this.counterExistingTerms;
  }

  /**
   * Increases the number of existing terms.
   */
  public void incCounterExistingTerms()
  {
    this.counterExistingTerms++;
  }

  /**
   * Gets the number of existing term candidates.
   * @return The number of existing term candidates.
   */
  public int getCounterExistingCandidateTerms()
  {
    return this.counterExistingCandidateTerms;
  }

  /**
   * Increases the number of existing term candidates.
   */
  public void incCounterExistingCandidateTerms()
  {
    this.counterExistingCandidateTerms++;
  }

  /**
   * Gets the number of term candidates that could not be added.
   * @return The number of term candidates where the insert operation failed
   */
  public int getCounterInsertionError()
  {
    return this.counterInsertionError;
  }

  /**
   * Increases the number of term candidates that could not be added.
   */
  public void incCounterInsertionError()
  {
    this.counterInsertionError++;
  }

}
