package ch.ige.wdltool.checker;

import java.sql.SQLException;
import java.util.Date;
import java.util.List;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.common.dao.ApplicationFinderDAO;
import ch.ige.ipobjects.common.dao.RegistrationFinderDAO;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.IPType;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.util.BatchUtils;
import ch.ige.wdltool.vo.CheckedClassTextGroupVO;
import ch.ige.wdltool.vo.CheckedClassTextPart;
import ch.ige.wdltool.vo.ClassTextPartVO;
import ch.ige.wdltool.vo.WdlCheckResultVO;
import ch.ige.wdlutil.exception.WdlSyntaxException;
import ch.ige.wdlutil.vo.ClassBlockVO;
import ch.ige.wdlutil.vo.ClassTextGroupVO;
import ch.ige.wdlutil.vo.WdlVO;

/**
 * Checks the WDLs of all trademarks that have an application or a registration
 * in the given time period.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: BatchChecker.java,v 1.9 2008/09/16 09:57:01 finger Exp $
 */
public class BatchChecker
{
  /**
   * Checks the WDL for all trademarks whose application date is in the specified time period
   * and writes the corresponding log and result files.
   * @param from The start of the time period
   * @param to The end of the time period
   * @return The log file content and the summary file content 
   * @throws SQLException if the query for trademarks fails
   */
  public static String[] checkWdlForApplication( Date from, Date to ) throws SQLException
  {
    Check.notNull( from, "'from' date" );
    Check.notNull( to, "'to' date" );
    List<IPKeyVO> trademarkKeys = DAOPool.get( ApplicationFinderDAO.class ).findApplicationsBetween( IPType.TRADEMARK_CH, EreigIDat.min( from ),
        EreigIDat.max( to ) );
    return checkWdl( trademarkKeys, from, to, true );
  }

  /**
   * Checks the WDL for all trademarks whose registration date is in the specified time period
   * and writes the corresponding log and result files.
   * @param from The start of the time period
   * @param to The end of the time period
   * @return The log file content and the summary file content 
   * @throws SQLException if the query for trademarks fails
   */
  public static String[] checkWdlForRegistration( Date from, Date to ) throws SQLException
  {
    Check.notNull( from, "'from' date" );
    Check.notNull( to, "'to' date" );
    List<IPKeyVO> trademarkKeys = DAOPool.get( RegistrationFinderDAO.class ).findRegistrationsBetween( IPType.TRADEMARK_CH, EreigIDat.min( from ),
        EreigIDat.max( to ) );
    return checkWdl( trademarkKeys, from, to, false );
  }

  /**
   * Checks the WDL of all given trademarks.
   * @param trademarkKeys The IP keys of the trademarks to check
   * @param from The start of the time period
   * @param to The end of the time period
   * @param checkForApplication If this is <code>true</code>, the applications are checked, otherwise the registrations
   * @return The log file content and the summary file content 
   */
  static String[] checkWdl( List<IPKeyVO> trademarkKeys, Date from, Date to, boolean checkForApplication )
  {
    WdlChecker wdlChecker = new WdlChecker();
    BatchCheckerResultVO totalResult = new BatchCheckerResultVO();
    StringBuilder log = new StringBuilder();
    BatchUtils.addLine( log, createHeader( from, to, checkForApplication ) );
    BatchUtils.addLine( log, "" );

    for ( IPKeyVO trademarkKey: trademarkKeys )
    {
      Trademark trademark = ServicesFactory.getInstance().findTrademark( trademarkKey );
      BatchUtils.addLine( log, BatchUtils.TRADEMARK_SEPARATOR );
      BatchUtils.addBaseInformation( log, trademark );
      BatchUtils.addLine( log, BatchUtils.DETAILS_SEPARATOR );

      TrademarkResultVO trademarkResult = null;
      try
      {
        trademarkResult = extractResult( wdlChecker.checkWdl( trademark, true ) );
      }
      catch ( WdlSyntaxException e )
      {
        trademarkResult = extractResult( null );
      }
      catch ( Exception e )
      {
        trademarkResult = extractResult( null );
        BatchUtils.addException( log, e );
      }
      logTrademarkResult( log, trademarkResult );
      totalResult.addResult( trademarkResult );

    }
    checkEmptyKeys( trademarkKeys, log );
    String summary = createSummary( totalResult, from, to, checkForApplication );
    String[] results = new String[] { log.toString(), summary };

    return results;
  }

  /**
   * Checks if the trademarkkeyslist is empty.
   * @param trademarkKeys to check
   * @param log to complete
   */
  private static void checkEmptyKeys( List<IPKeyVO> trademarkKeys, StringBuilder log )
  {
    if ( trademarkKeys.isEmpty() )
    {
      log.append( "Es wurden keine Marken gefunden." );
    }
  }

  /**
   * Gets the number of classes of the check result.
   * @param checkResult The check result to use
   * @return The number of classes within the check result
   */
  private static TrademarkResultVO extractResult( WdlCheckResultVO checkResult )
  {
    TrademarkResultVO result = new TrademarkResultVO();
    if ( checkResult == null )
    {
      result.isSyntacticallyCorrect = Boolean.FALSE;
    }
    else
    {
      result.isSyntacticallyCorrect = Boolean.TRUE;
      WdlVO wdl = checkResult.getWdl();
      result.checkedClassTexts = wdl.getClassBlocks().size();
      for ( ClassBlockVO classBlock: wdl.getClassBlocks() )
      {
        boolean totallyCorrectClassText = true;
        for ( ClassTextGroupVO group: classBlock.getClassTextGroups() )
        {
          CheckedClassTextGroupVO classTextGroup = (CheckedClassTextGroupVO) group;
          for ( ClassTextPartVO classTextPart: classTextGroup.getClassTextParts() )
          {
            result.checkedCharacters += classTextPart.getClassTextPart().length();
            if ( classTextPart.getCheckFlag().equals( CheckedClassTextPart.CORRECT ) )
            {
              result.correctCharacters += classTextPart.getClassTextPart().length();
            }
            else if ( totallyCorrectClassText )
            {
              totallyCorrectClassText = false;
            }
          }
        }
        if ( totallyCorrectClassText )
        {
          result.correctClassTexts++;
        }
      }
    }

    return result;
  }

  /**
   * Adds the result for one trademark to the log. 
   * @param log The log to add to
   * @param trademarkResult The result for one trademark
   */
  private static void logTrademarkResult( StringBuilder log, TrademarkResultVO trademarkResult )
  {
    String correct = "Ist die WDL syntaktisch korrekt? ";
    if ( trademarkResult.isSyntacticallyCorrect.booleanValue() )
    {
      correct += "Ja";
    }
    else
    {
      correct += "Nein";
    }
    BatchUtils.addLine( log, 1, correct );

    if ( trademarkResult.isSyntacticallyCorrect.booleanValue() )
    {
      BatchUtils.addLine( log, 1, "Anzahl Klassen: " + trademarkResult.checkedClassTexts );

      BatchUtils.addLine( log, 1, "Anzahl geprüfter Zeichen: " + trademarkResult.checkedCharacters );
      String percentage = BatchUtils.getPercentage( trademarkResult.correctCharacters, trademarkResult.checkedCharacters, false );
      BatchUtils.addLine( log, 1, "Anzahl grüner Zeichen: " + trademarkResult.correctCharacters + percentage );

      percentage = BatchUtils.getPercentage( trademarkResult.correctClassTexts, trademarkResult.checkedClassTexts, false );
      BatchUtils.addLine( log, 1, "Anzahl vollständig korrekter Klassen: " + trademarkResult.correctClassTexts + percentage );
    }
  }

  /**
   * Creates the formatted summary of the whole batch check.
   * @param result Contains the summary data
   * @param from The start of the time period
   * @param to The end of the time period
   * @param checkForApplication If this is <code>true</code>, the applications are checked, otherwise the registrations
   * @return The formatted summary of the whole batch check
   */
  private static String createSummary( BatchCheckerResultVO result, Date from, Date to, boolean checkForApplication )
  {
    StringBuilder summary = new StringBuilder();

    BatchUtils.addLine( summary, "ZUSAMMENFASSUNG FÜR " + createHeader( from, to, checkForApplication ) );
    BatchUtils.addCSVLine( summary, "Anzahl geprüfter Marken", result.checkedTrademarks.toString() );
    String percentage = BatchUtils.getPercentage( result.syntacticallyCorrectTrademarks, result.checkedTrademarks );
    BatchUtils.addCSVLine( summary, "Anzahl mit syntaktisch korrekter WDL", result.syntacticallyCorrectTrademarks.toString(), percentage );

    BatchUtils.addCSVLine( summary, "Anzahl geprüfter Zeichen", result.checkedCharacters.toString() );
    percentage = BatchUtils.getPercentage( result.correctCharacters, result.checkedCharacters );
    BatchUtils.addCSVLine( summary, "Anzahl grüner Zeichen", result.correctCharacters.toString(), percentage );

    BatchUtils.addCSVLine( summary, "Anzahl geprüfter Klassentexte", result.checkedClassTexts.toString() );
    percentage = BatchUtils.getPercentage( result.correctClassTexts, result.checkedClassTexts );
    BatchUtils.addCSVLine( summary, "Anzahl vollständig korrekter Klassentexte", result.correctClassTexts.toString(), percentage );

    return summary.toString();
  }

  /**
   * Returns the header used both in the log and the summary.
   * @param from The start of the time period
   * @param to The end of the time period
   * @param checkForApplication If this is <code>true</code>, the applications are checked, otherwise the registrations
   * @return The header
   */
  private static String createHeader( Date from, Date to, boolean checkForApplication )
  {
    String checkType = "";
    if ( checkForApplication )
    {
      checkType = "MIT ANMELDUNG";
    }
    else
    {
      checkType = "MIT EINTRAGUNG";
    }
    return "MARKEN " + checkType + " VON " + DateFormatter.shortFormat( from ) + " BIS " + DateFormatter.shortFormat( to );
  }

  /**
   * Contains the result of one trademark check.
   */
  private static class TrademarkResultVO
  {
    /**
     * If this is <code>true</code>, the trademark is syntactically correct; otherwise it has a syntax error.
     */
    public Boolean isSyntacticallyCorrect;

    /**
     * The total number of characters checked in this trademark
     */
    public Integer checkedCharacters = 0;

    /**
     * The number of correct characters in this trademark
     */
    public Integer correctCharacters = 0;

    /**
     * The number of checked class texts in this trademark
     */
    public Integer checkedClassTexts = 0;

    /**
     * The number of fully correct checked class texts in this trademark
     */
    public Integer correctClassTexts = 0;
  }

  /**
   * Contains the result of the whole batch check.
   */
  private static class BatchCheckerResultVO
  {
    /**
     * The total number of checked trademarks
     */
    public Integer checkedTrademarks = 0;

    /**
     * The number of checked trademarks containing a syntactically correct WDL
     */
    public Integer syntacticallyCorrectTrademarks = 0;

    /**
     * The total number of characters checked in all WDLs
     */
    public Integer checkedCharacters = 0;

    /**
     * The number of correct characters in all WDLs
     */
    public Integer correctCharacters = 0;

    /**
     * The total number of checked class texts in all WDLs
     */
    public Integer checkedClassTexts = 0;

    /**
     * The number of fully correct checked class texts in all WDLs
     */
    public Integer correctClassTexts = 0;

    /**
     * Adds the trademark result to this.
     * @param trademarkResult The trademark result
     */
    public void addResult( TrademarkResultVO trademarkResult )
    {
      this.checkedTrademarks++;
      if ( trademarkResult.isSyntacticallyCorrect.booleanValue() )
      {
        this.syntacticallyCorrectTrademarks++;
        this.checkedCharacters += trademarkResult.checkedCharacters;
        this.correctCharacters += trademarkResult.correctCharacters;
        this.checkedClassTexts += trademarkResult.checkedClassTexts;
        this.correctClassTexts += trademarkResult.correctClassTexts;
      }
    }
  }
}
