package ch.ige.wdltool.checker;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import junit.framework.TestCase;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.CheckedClassTextGroupVO;
import ch.ige.wdltool.vo.CheckedClassTextPart;
import ch.ige.wdltool.vo.ClassTextPartVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.StopwordVO;
import ch.ige.wdlutil.analyze.FormattedWdlTestCases;
import ch.ige.wdlutil.vo.ClassBlockVO;
import ch.ige.wdlutil.vo.ClassTextGroupVO;
import ch.ige.wdlutil.vo.WdlVO;

/**
 * Tests {@link ch.ige.wdltool.checker.WdlChecker}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Christian Heinzer
 * @author Eric Lewis
 * @version  $Id: WdlCheckerTest.java,v 1.59 2008/03/11 16:26:37 lewis_lee Exp $
 */
public class WdlCheckerTest extends TestCase
{
  static
  {
    BasicConfigurator.configure();
    ClientInitializer.initStandaloneClient();
    ServicesFactory.getStandaloneServices();
  }

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( WdlCheckerTest.class );

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlCheckerTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#parseAndInitialize(String, Locale, List, boolean, boolean, boolean, Map)}.
   * @throws Exception if the test fails
   */
  public void testParseAndInitialize() throws Exception
  {
    WdlVO wdl = WdlChecker.parseAndInitialize( FormattedWdlTestCases.NORMAL_WDL, Locale.GERMAN, Arrays.asList( new Integer[] { 1, 9, 25, 35 } ), false, false,
        true, null );
    checkNormalWdl( wdl, true );

    // Now parse without class numbers in WDL text
    wdl = WdlChecker.parseAndInitialize( "Wissenschaftliche Instrumente, Computer, Feuerlöschgeräte.", Locale.GERMAN, Arrays.asList( new Integer[] { 9 } ),
        true, false, true, null );
    assertEquals( "Checking number of class blocks", 1, wdl.getClassBlocks().size() );
    ClassBlockVO classBlock = wdl.getClassBlocks().get( 0 );
    assertEquals( "Checking defaulted class number", 9, classBlock.getClassNumber().intValue() );
    checkClassTextGroups( classBlock.getClassTextGroups(), true, "Wissenschaftliche Instrumente, Computer, Feuerlöschgeräte" );

    // Parse with replacement of multiple blanks
    String normalWdlWithMultipleBlanks = " 1 Gugustralala.\n" + " 9 Wissenschaftliche    Instrumente, Computer, Feuerlöschgeräte.\n"
        + "25 Bekleidungsstücke, Schuhwaren,  Kopfbedeckungen.\n" + "35 Werbung   ; Geschäftsführung.";
    wdl = WdlChecker
        .parseAndInitialize( normalWdlWithMultipleBlanks, Locale.GERMAN, Arrays.asList( new Integer[] { 1, 9, 25, 35 } ), false, false, false, null );
    checkNormalWdl( wdl, false );
  }

  /**
   * Checks the normal WDL after parsing.
   * @param keepBlanks If this is <code>true</code>, all blanks at the start and end are kept, otherwise they are trimmed
   * @param wdl The parsed WDL
   */
  private void checkNormalWdl( WdlVO wdl, boolean keepBlanks )
  {
    int checkedClassBlocks = 0;
    assertEquals( "Checking number of class blocks", 4, wdl.getClassBlocks().size() );
    for ( ClassBlockVO classBlock: wdl.getClassBlocks() )
    {
      List<ClassTextGroupVO> classTextGroups = classBlock.getClassTextGroups();
      switch( classBlock.getClassNumber().intValue() )
      {
        case 1:
          checkClassTextGroups( classTextGroups, keepBlanks, "Gugustralala" );
          checkedClassBlocks++;
          break;
        case 9:
          checkClassTextGroups( classTextGroups, keepBlanks, "Wissenschaftliche Instrumente, Computer, Feuerlöschgeräte" );
          checkedClassBlocks++;
          break;
        case 25:
          checkClassTextGroups( classTextGroups, keepBlanks, "Bekleidungsstücke, Schuhwaren, Kopfbedeckungen" );
          checkedClassBlocks++;
          break;
        case 35:
          checkClassTextGroups( classTextGroups, keepBlanks, "Werbung", " Geschäftsführung" );
          checkedClassBlocks++;
          break;
        default:
          fail( "Found unknown class number " + classBlock.getClassNumber() );
      }
    }
    assertEquals( "Checking found class blocks", 4, checkedClassBlocks );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkStopwords(List, CheckedClassTextGroupVO)} with only one word.
   */
  public void testCheckStopwordsWithOneWord()
  {
    List<StopwordVO> stopwords = createGermanStopwords( "ausgenommen" );

    // Contains no stopword
    List<ClassTextPartVO> expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );

    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer,", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Contains a stopword
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "ausgenommen", CheckedClassTextPart.STOPWORD );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "ausgenommen", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Contains a mixed-case stopword
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "ausGenoMMen", CheckedClassTextPart.STOPWORD );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "ausGenoMMen", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // A word that contains a stopword
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "ausgenommene", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "ausgenommene", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // A word that contains a stopword
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "vorausgenommen", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "vorausgenommen", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Contains a stopword, but the stopw
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "ausgenommen", CheckedClassTextPart.STOPWORD );
    stopwords = createGermanStopwords( "ausgenommen", "ausgenommen", "ausgenommen" );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "ausgenommen", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkStopwords(List, CheckedClassTextGroupVO)}.
   */
  public void testCheckStopwords()
  {
    List<StopwordVO> stopwords = createGermanStopwords( "die", "alle", "auf", "ausgenommen" );

    // Normal case
    List<ClassTextPartVO> expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " die", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " alle", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " auf", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " derselben Tischplatte liegen,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " ausgenommen", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " PCs", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer, die alle auf derselben Tischplatte liegen, ausgenommen PCs", '.' );

    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Upper- and lowercase characters
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " DIE", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " Alle", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " auF", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " derselben Tischplatte liegen,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " aUsGeNoMmEn", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " PCs", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer, DIE Alle auF derselben Tischplatte liegen, aUsGeNoMmEn PCs", '.' );

    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Stopword at the beginning should be recognised
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " Computer, welche", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "die Computer, welche", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Stopword after a comma should be recognised
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer,die", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Multiple occurences of the same stopword have to be detected
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );
    checkedClassTextGroup = new CheckedClassTextGroupVO( "die Computer,die", null );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // If a text part is invalid, the stopword is put into the next part, without blank
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "    ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );

    List<ClassTextPartVO> actualClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( actualClassTextParts, "die Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( actualClassTextParts, "    ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( actualClassTextParts, "die", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup = new CheckedClassTextGroupVO( "die Computer,    die", null );
    checkedClassTextGroup.setClassTextParts( actualClassTextParts );

    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Check special characters
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "dieäää Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "die", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, "; éééédie", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup = new CheckedClassTextGroupVO( "dieäää Computer,die; éééédie", ';' );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Special characters
    stopwords = createGermanStopwords( "dìe", "älle", "äuf", "ausgenömmen" );
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Cömputer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " dìe", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " älle", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " äuf", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " derselben Tischplatte liegen,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " ausgenömmen", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " PCs", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Cömputer, dìe älle äuf derselben Tischplatte liegen, ausgenömmen PCs", '.' );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // French stopwords that end with a '
    stopwords = createStopwords( Locale.FRENCH, "l'", "d'" );
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "l'", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, "age", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " d'", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, "or", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "l'age d'or", '.' );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // French stopwords that end with a ', but the WDL contains blanks in-between and some other stuff
    // that also contains the stopword, but not as a word.
    stopwords = createStopwords( Locale.FRENCH, "l'", "d'" );
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "l'", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "age", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " d'", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "or et bl'a", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "l' age d' or et bl'a", '.' );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // French stopwords that end with a ', followed by a special character.
    stopwords = createStopwords( Locale.FRENCH, "l'", "qu'" );
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "l'", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, "âge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " qu'", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts, " ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "épices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "l'âge qu' épices", '.' );
    WdlChecker.checkStopwords( stopwords, checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkMultipleBlanks(CheckedClassTextGroupVO)}.
   */
  public void testCheckMultipleBlanks()
  {
    // With leading blank, all mixed stuff
    List<ClassTextPartVO> expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, " Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "  ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "die alle auf", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "   ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "derselben Tischplatte liegen, ausgenommen PCs ", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( " Computer,  die alle auf   derselben Tischplatte liegen, ausgenommen PCs ",
        '.' );
    WdlChecker.checkMultipleBlanks( checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // No leading blank
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "  ", CheckedClassTextPart.INVALID );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer,  ", null );
    WdlChecker.checkMultipleBlanks( checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Two leading blanks
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "  ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "  Computer,", null );
    WdlChecker.checkMultipleBlanks( checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // No trailing blank
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer,", null );
    WdlChecker.checkMultipleBlanks( checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Two trailing blanks
    expectedClassTextParts.clear();
    addClassTextPartVO( expectedClassTextParts, "Computer,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "  ", CheckedClassTextPart.INVALID );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Computer,  ", null );
    WdlChecker.checkMultipleBlanks( checkedClassTextGroup );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkDuplicateEntries(List, Locale)}.
   */
  public void testCheckDuplicateEntries()
  {
    // Note that one of the duplicate entries starts with a blank!
    List<CheckedClassTextGroupVO> testedClassTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    testedClassTextGroups.add( new CheckedClassTextGroupVO( "Schokolade im Doppelpack", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " Süssigkeiten mit farbigem Aufdruck", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " Schokolade im Doppelpack", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " Milchschokolade, Schokoladegetränke", '.' ) );

    WdlChecker.checkDuplicateEntries( testedClassTextGroups, Locale.GERMAN );
    assertEquals( "First CheckedClassTextGroup must be duplicated: ", CheckedClassTextPart.DUPLICATE, testedClassTextGroups.get( 0 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Second CheckedClassTextGroup must not be duplicated: ", CheckedClassTextPart.UNKNOWN, testedClassTextGroups.get( 1 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Third CheckedClassTextGroup must be duplicated: ", CheckedClassTextPart.DUPLICATE, testedClassTextGroups.get( 2 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Fourth CheckedClassTextGroup must not be duplicated: ", CheckedClassTextPart.UNKNOWN, testedClassTextGroups.get( 3 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );

    // Check with upper and lower case
    testedClassTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    testedClassTextGroups.add( new CheckedClassTextGroupVO( "Überaus feine Schokolade im Doppelpack", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " Süssigkeiten mit farbigem Aufdruck", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " überaus feine Schokolade im Doppelpack", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " Milchschokolade, Schokoladegetränke", '.' ) );

    WdlChecker.checkDuplicateEntries( testedClassTextGroups, Locale.GERMAN );
    assertEquals( "First CheckedClassTextGroup must be duplicated: ", CheckedClassTextPart.DUPLICATE, testedClassTextGroups.get( 0 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Second CheckedClassTextGroup must not be duplicated: ", CheckedClassTextPart.UNKNOWN, testedClassTextGroups.get( 1 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Third CheckedClassTextGroup must be duplicated: ", CheckedClassTextPart.DUPLICATE, testedClassTextGroups.get( 2 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Fourth CheckedClassTextGroup must not be duplicated: ", CheckedClassTextPart.UNKNOWN, testedClassTextGroups.get( 3 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );

    // Check with upper and lower case
    testedClassTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    testedClassTextGroups.add( new CheckedClassTextGroupVO( "Eclairantes (matières -)", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " autres choses", ';' ) );
    testedClassTextGroups.add( new CheckedClassTextGroupVO( " éclairantes (matières -)", '.' ) );

    WdlChecker.checkDuplicateEntries( testedClassTextGroups, Locale.FRENCH );
    assertEquals( "First CheckedClassTextGroup must be duplicated: ", CheckedClassTextPart.DUPLICATE, testedClassTextGroups.get( 0 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Second CheckedClassTextGroup must not be duplicated: ", CheckedClassTextPart.UNKNOWN, testedClassTextGroups.get( 1 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
    assertEquals( "Third CheckedClassTextGroup must be duplicated: ", CheckedClassTextPart.DUPLICATE, testedClassTextGroups.get( 2 ).getClassTextParts()
        .get( 0 ).getCheckFlag() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkPunctuationMarks(List)}.
   */
  public void testCheckPunctuationMarks()
  {
    // Correct
    List<ClassTextPartVO> expectedClassTextParts1 = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ";", CheckedClassTextPart.UNKNOWN );
    CheckedClassTextGroupVO checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );

    List<ClassTextPartVO> expectedClassTextParts2 = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts2, "weitere Dinge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, ".", CheckedClassTextPart.CORRECT );
    CheckedClassTextGroupVO checkedClassTextGroup2 = new CheckedClassTextGroupVO( "weitere Dinge", '.' );

    List<CheckedClassTextGroupVO> classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );

    // Dot in-between
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ".", CheckedClassTextPart.INVALID );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", '.' );

    expectedClassTextParts2.clear();
    addClassTextPartVO( expectedClassTextParts2, "w", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts2, "eitere Dinge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup2 = new CheckedClassTextGroupVO( "weitere Dinge", '.' );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );

    // Dot missing at the end
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ";", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );

    expectedClassTextParts2.clear();
    addClassTextPartVO( expectedClassTextParts2, "weitere Ding", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, "e", CheckedClassTextPart.INVALID );
    checkedClassTextGroup2 = new CheckedClassTextGroupVO( "weitere Dinge", null );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );

    // Acronym somewhere in the text
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, W.C.-Reiniger, Alkaloide, Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ";", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, W.C.-Reiniger, Alkaloide, Sauerstoff, Automobile", ';' );

    expectedClassTextParts2.clear();
    addClassTextPartVO( expectedClassTextParts2, "weitere Dinge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup2 = new CheckedClassTextGroupVO( "weitere Dinge", '.' );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );

    // Acronym somewhere in the text
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, W.C.-Reiniger, Alkaloide, Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ";", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, W.C.-Reiniger, Alkaloide, Sauerstoff, Automobile", ';' );

    expectedClassTextParts2.clear();
    addClassTextPartVO( expectedClassTextParts2, "weitere Dinge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup2 = new CheckedClassTextGroupVO( "weitere Dinge", '.' );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );

    // Blank before punctuation mark, which can be in-between or at the end
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, W.C.-Reiniger", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, " ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts1, ", Alkaloide, Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, " ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts1, ";", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, W.C.-Reiniger , Alkaloide, Sauerstoff, Automobile ", ';' );

    expectedClassTextParts2.clear();
    addClassTextPartVO( expectedClassTextParts2, "weitere Dinge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, " ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts2, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup2 = new CheckedClassTextGroupVO( "weitere Dinge ", '.' );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );

    // Doesn't start with an uppercase character
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "d", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts1, "üngemittel, W.C.-Reiniger", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "düngemittel, W.C.-Reiniger", '.' );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );

    // Doesn't start with an uppercase character after a dot
    expectedClassTextParts1.clear();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, W.C.-Reiniger", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, ".", CheckedClassTextPart.INVALID );
    checkedClassTextGroup1 = new CheckedClassTextGroupVO( "Düngemittel, W.C.-Reiniger", '.' );

    expectedClassTextParts2.clear();
    addClassTextPartVO( expectedClassTextParts2, " w", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts2, "eitere Dinge", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup2 = new CheckedClassTextGroupVO( " weitere Dinge", '.' );

    classTextGroups = new ArrayList<CheckedClassTextGroupVO>();
    classTextGroups.add( checkedClassTextGroup1 );
    classTextGroups.add( checkedClassTextGroup2 );
    WdlChecker.checkPunctuationMarks( classTextGroups );
    assertEquals( expectedClassTextParts1, classTextGroups.get( 0 ).getClassTextParts() );
    assertEquals( expectedClassTextParts2, classTextGroups.get( 1 ).getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#createTermPattern(String, Locale)} for German terms.
   */
  public void testCreateTermPatternForGerman()
  {
    checkTermPattern( Locale.GERMAN, "aufblasbare Reifen", null );
    checkTermPattern( Locale.GERMAN, "Aufblasbare Reifen", null );
    checkTermPattern( Locale.GERMAN, "ätzende Reifen", null );
    checkTermPattern( Locale.GERMAN, "Ätzende Reifen", null );
    checkTermPattern( Locale.GERMAN, "essbare Reifen", null );
    checkTermPattern( Locale.GERMAN, "Essbare Reifen", null );
    checkTermPattern( Locale.GERMAN, "italienische Reifen", null );
    checkTermPattern( Locale.GERMAN, "Italienische Reifen", null );
    checkTermPattern( Locale.GERMAN, "orange Reifen", null );
    checkTermPattern( Locale.GERMAN, "Orange Reifen", null );
    checkTermPattern( Locale.GERMAN, "ölige Reifen", null );
    checkTermPattern( Locale.GERMAN, "Ölige Reifen", null );
    checkTermPattern( Locale.GERMAN, "ungepflegte Reifen", null );
    checkTermPattern( Locale.GERMAN, "Ungepflegte Reifen", null );
    checkTermPattern( Locale.GERMAN, "übergrosse Reifen", null );
    checkTermPattern( Locale.GERMAN, "Übergrosse Reifen", null );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#createTermPattern(String, Locale)} for Italian terms.
   */
  public void testCreateTermPatternForItalian()
  {
    // My Italian sucks...  ;-)
    checkTermPattern( Locale.ITALIAN, "aufblasbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Aufblasbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "àufblasbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Àufblasbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "áufblasbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Áufblasbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "essbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Essbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "èssbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Èssbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "éssbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Éssbare Reifen", null );
    checkTermPattern( Locale.ITALIAN, "italienische Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Italienische Reifen", null );
    checkTermPattern( Locale.ITALIAN, "ìtalienische Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Ìtalienische Reifen", null );
    checkTermPattern( Locale.ITALIAN, "ítalienische Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Ítalienische Reifen", null );
    checkTermPattern( Locale.ITALIAN, "orange Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Orange Reifen", null );
    checkTermPattern( Locale.ITALIAN, "òrange Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Òrange Reifen", null );
    checkTermPattern( Locale.ITALIAN, "órange Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Órange Reifen", null );
    checkTermPattern( Locale.ITALIAN, "ungepflegte Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Ungepflegte Reifen", null );
    checkTermPattern( Locale.ITALIAN, "ùngepflegte Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Ùngepflegte Reifen", null );
    checkTermPattern( Locale.ITALIAN, "úngepflegte Reifen", null );
    checkTermPattern( Locale.ITALIAN, "Úngepflegte Reifen", null );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#createTermPattern(String, Locale)} for French terms.
   */
  public void testCreateTermPatternForFrench()
  {
    checkTermPattern( Locale.FRENCH, "à plus tard", "[Aà] plus tard" );
    checkTermPattern( Locale.FRENCH, "ah oui", "ah oui" );

    checkTermPattern( Locale.FRENCH, "ça va", "[Cç]a va" );
    checkTermPattern( Locale.FRENCH, "canard", "canard" );

    checkTermPattern( Locale.FRENCH, "dingue", null );

    checkTermPattern( Locale.FRENCH, "épices", "[Eé]pices" );
    checkTermPattern( Locale.FRENCH, "et puis", "et puis" );

    checkTermPattern( Locale.FRENCH, "îîîîîîh", "[Iî]îîîîîh" );
    checkTermPattern( Locale.FRENCH, "iiiiiih", "iiiiiih" );

    checkTermPattern( Locale.FRENCH, "ô mes amis", "[Oô] mes amis" );
    checkTermPattern( Locale.FRENCH, "o mes amis", "o mes amis" );

    checkTermPattern( Locale.FRENCH, "üaaaa", "[Uü]aaaa" );
    checkTermPattern( Locale.FRENCH, "uaaaa", "uaaaa" );

    checkTermPattern( Locale.FRENCH, "ÿeah", "(?:Y|ÿ)eah" ); // This avoids a JDK bug!
    checkTermPattern( Locale.FRENCH, "yeah", "yeah" );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#createTermPattern(String, Locale)} for escaped brackets.
   */
  public void testCreateTermPatternWithBrackets()
  {
    checkTermPattern( Locale.GERMAN, "aufblasbare Reifen (in allen Grössen)", "aufblasbare Reifen \\(in allen Grössen\\)" );
    checkTermPattern( Locale.GERMAN, "aufblasbare Reifen [in allen Grössen]", "aufblasbare Reifen \\[in allen Grössen\\]" );
  }

  /**
   * Checks the term pattern.
   * @param language The language to use for the pattern
   * @param term The term
   * @param patternPart The expected pattern String, but only the term part of it;
   *        if this is <code>null</code>, then <code>term</code> is taken
   */
  private void checkTermPattern( Locale language, String term, String patternPart )
  {
    boolean termEndsWithBracket = term.endsWith( ")" ) || term.endsWith( "]" );
    Pattern checkerPattern = WdlChecker.createTermPattern( term, language );
    char firstCharacter = term.charAt( 0 );
    String expectedPattern = "( ?\\b" + ( patternPart == null ? term : patternPart ) + ")(?:\\" + ( termEndsWithBracket ? "W" : "b" ) + "|$)";
    assertEquals( "Checking pattern flags", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE, checkerPattern.flags() );
    assertEquals( "Checking term in " + language.getDisplayName( Locale.ENGLISH ) + " starting with '" + firstCharacter + "'", expectedPattern, checkerPattern
        .toString() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#createTermPattern(String, Locale)} with all kinds of
   * existing start and end characters which are non-word characters (in the meaning of regular expressions).
   */
  public void testCreateTermPatternForNonWordCharacters()
  {
    matches( Locale.GERMAN, "Gallerten (Gelees)" );
    matches( Locale.GERMAN, "Spirituosen mit der geschützten Ursprungsbezeichnung \"Tequila\"" );
    matches( Locale.GERMAN, "Spirituosen mit der geschützten Ursprungsbezeichnung 'Tequila'" );
    matches( Locale.GERMAN, "Vermieten von Zugriffszeit auf Computerdatenbanken (Informatikdienstleistung)\n" );
    matches( Locale.FRENCH, "vêtements*" );
    matches( Locale.FRENCH, "préparations pour nettoyer, polir, dégraisser et abraser," );
    matches( Locale.GERMAN, "Kontroll-" );
    matches( Locale.GERMAN, "Bier mit Tequila." );
    matches( Locale.ITALIAN, "banchi [mobili]" );
    matches( Locale.GERMAN, "(Hygiene-)Abfallsammler" );
    matches( Locale.GERMAN, "..für gesundheitliche Zwecke" );
  }

  /**
   * Tries to match a multitude of variations of <code>term</code>.
   * @param language The WDL's language
   * @param term The term to match
   */
  private void matches( Locale language, String term )
  {
    // Create different test cases where the pattern must always be found
    matches( language, term, term, true );
    matches( language, term, "bla " + term, true );
    matches( language, term, "bla," + term, true );
    matches( language, term, "bla, " + term, true );
    matches( language, term, "bla." + term, true );
    matches( language, term, "bla. " + term, true );
    matches( language, term, "bla;" + term, true );
    matches( language, term, "bla; " + term, true );
    matches( language, term, "    " + term, true );

    matches( language, term, term + " bla ", true );
    matches( language, term, term + ",bla", true );
    matches( language, term, term + ", bla", true );
    matches( language, term, term + ".bla", true );
    matches( language, term, term + ". bla", true );
    matches( language, term, term + ";bla", true );
    matches( language, term, term + "; bla", true );
    matches( language, term, term + "    ", true );
  }

  /**
   * Matches a term within a text, based on {@link WdlChecker#createTermPattern(String, Locale)}.
   * @param language The WDL's language
   * @param term The term to match
   * @param text The WDL text
   * @param expectedResult Shows whether the <code>term</code> should be matched or not
   */
  private void matches( Locale language, String term, String text, boolean expectedResult )
  {
    Pattern regexPattern = WdlChecker.createTermPattern( term, language );
    LOG.debug( "Checking for match of '" + term + "' in '" + text + "' (pattern '" + regexPattern.toString() + "')" );
    Matcher matcher = regexPattern.matcher( text );
    assertEquals( "Checking for match of '" + term + "' in '" + text + "' (pattern '" + regexPattern.toString() + "')", expectedResult, matcher.find() );
    // The Pattern also matches a blank before the term. Also, newline characters are deleted.
    assertEquals( "Checking match", term.replace( "\n", "" ), matcher.group( 1 ).trim() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkTerms(List, CheckedClassTextGroupVO, boolean, Locale)}.
   */
  public void testCheckTerms()
  {
    try
    {
      Map<Integer, List<AbstractTermVO>> terms = createGermanTerms( 1, "12345", "123456" );
      WdlChecker.checkTerms( terms.get( 1 ), new CheckedClassTextGroupVO( "blaasdfasdf", '.' ), true, Locale.GERMAN );
      fail( "Terms sorted by ascending length must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    Map<Integer, List<AbstractTermVO>> terms = createGermanTerms( 1, "Alkaloide, Sauerstoff", "Automobile", "Sauerstoff" );

    // Normal case
    List<ClassTextPartVO> expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Alkaloide, Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ",", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Check for partial match
    terms = createGermanTerms( 1, "Alkaloide Sauerstoff", "Automobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel, Alkaloide,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ",", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Check for non-word match
    terms = createGermanTerms( 1, "stoff, Automobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel, Alkaloide,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ", Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Check for lowercase search terms
    terms = createGermanTerms( 1, "chlorhaltige Waschmittel", "Automobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " chlorhaltige Waschmittel", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ", Alkaloide,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ",", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, chlorhaltige Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Check for lowercase search terms at the start of the class text group
    terms = createGermanTerms( 1, "chlorhaltige Waschmittel", "Automobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Chlorhaltige Waschmittel", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ", Alkaloide,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ",", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Chlorhaltige Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Check for search terms with special characters
    terms = createGermanTerms( 1, "Ölkölöidö, Sauerstoff", "Äutomobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Ölkölöidö, Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ",", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Äutomobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Ölkölöidö, Sauerstoff, Äutomobile", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // If a text part is invalid, the term is put into the next part, without blank
    terms = createGermanTerms( 1, "chlorhaltige Waschmittel", "Automobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Chlorhaltige Waschmittel", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ", Alkaloide,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, "      ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, "Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ",", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    List<ClassTextPartVO> actualClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( actualClassTextParts, "Chlorhaltige Waschmittel, Alkaloide,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( actualClassTextParts, "      ", CheckedClassTextPart.INVALID );
    addClassTextPartVO( actualClassTextParts, "Sauerstoff, Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( actualClassTextParts, ";", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Chlorhaltige Waschmittel, Alkaloide,      Sauerstoff, Automobile", ';' );
    checkedClassTextGroup.setClassTextParts( actualClassTextParts );

    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // If a text part is a duplicate (and thus invalid), it's ignored for matching
    terms = createGermanTerms( 1, "chlorhaltige Waschmittel", "Automobile", "Sauerstoff" );

    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Automobile", CheckedClassTextPart.INVALID );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.INVALID );

    actualClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( actualClassTextParts, "Automobile", CheckedClassTextPart.INVALID );
    addClassTextPartVO( actualClassTextParts, ";", CheckedClassTextPart.INVALID );
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Automobile", ';' );
    checkedClassTextGroup.setClassTextParts( actualClassTextParts );

    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkTerms(List, CheckedClassTextGroupVO, boolean, Locale)}
   * where the terms exist in mixed cases, and so do the class term texts.
   */
  public void testCheckTermsWithMixedCases()
  {
    Map<Integer, List<AbstractTermVO>> terms = createGermanTerms( 1, "Fahnen, Masten", "rote Fahnen" );

    // Normal case
    List<ClassTextPartVO> expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " rote Fahnen", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, rote Fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case, typo
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel, Lote Fahnen", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Lote Fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case, incorrect uppercase letter
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " ", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "R", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "ote Fahnen", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Rote Fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case, incorrect lowercase letter
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " rote ", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "f", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "ahnen", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, rote fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // NOW WITH UMLAUTS
    terms = createGermanTerms( 1, "äusserst rote Fahnen", "Fahnen, Masten" );

    // Normal case
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " äusserst rote Fahnen", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, äusserst rote Fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case, typo
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel, Öusserst rote Fahnen", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Öusserst rote Fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case, incorrect uppercase letter
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " ", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "Ä", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "usserst rote Fahnen", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Äusserst rote Fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case, incorrect lowercase letter
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, " äusserst rote ", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "f", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "ahnen", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ";", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, äusserst rote fahnen", ';' );
    WdlChecker.checkTerms( terms.get( 1 ), checkedClassTextGroup, true, Locale.GERMAN );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // NOW WITH FRENCH TERMS
    terms = createFrenchTerms( 5, "herbes médicinales" );

    // Mixed case
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "He", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "R", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "b", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "E", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "s ", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "M", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "é", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "D", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "i", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "C", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "i", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "N", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "a", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "L", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "e", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, "S", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "HeRbEs MéDiCiNaLeS", '.' );
    WdlChecker.checkTerms( terms.get( 5 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkTerms(List, CheckedClassTextGroupVO, boolean, Locale)}
   * indirectly by creating WDLs where the case switches
   * @throws Exception if the test fails
   */
  public void testCheckTermsWithMixedFrenchCases() throws Exception
  {
    // Set up the test
    // Note: This test contains duplicates, but only for testing. The WdlChecker would
    // later on mark them as duplicates.
    Map<Integer, List<AbstractTermVO>> terms = createFrenchTerms( 31, "épices" );

    // Correct, at the beginning of class text
    ArrayList<ClassTextPartVO> expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Epices", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "Epices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Correct, at the beginning of class text
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Épices", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Épices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Correct case, but incorrect character
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Èpices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Èpices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case for class text group that's not at the beginning
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Epices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Epices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, false, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Wrong case for class text group that's not at the beginning
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "É", CheckedClassTextPart.CORRECT_WRONG_CASE );
    addClassTextPartVO( expectedClassTextParts, "pices", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Épices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, false, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Incorrect case, incorrect character
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "Èpices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "Èpices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, false, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Incorrect in the middle of class text
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "epices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "epices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, false, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Correct in the middle of class text
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "épices", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "épices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, false, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Incorrect character
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "èpices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "èpices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, false, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Incorrect at the beginning of class text
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "epices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "epices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Incorrect at the beginning of class text
    // Note: The class text must start with an upper case character, but that's checked in another method
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "épices", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "épices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );

    // Incorrect character
    expectedClassTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts, "èpices", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts, ".", CheckedClassTextPart.UNKNOWN );

    checkedClassTextGroup = new CheckedClassTextGroupVO( "èpices", '.' );
    WdlChecker.checkTerms( terms.get( 31 ), checkedClassTextGroup, true, Locale.FRENCH );
    assertEquals( expectedClassTextParts, checkedClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#changeInlinePunctuationMarks(CheckedClassTextGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testChangeInlinePunctuationMarks() throws Exception
  {
    // Punctuation mark at the beginning - should never happen
    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "; Automobile", '.' );
    List<ClassTextPartVO> classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, ";", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeInlinePunctuationMarks( checkedClassTextGroup );
    assertEquals( "Checking non-digit/character at start", CheckedClassTextPart.INVALID, classTextParts.get( 0 ).getCheckFlag() );

    // Check for class text parts which are also 1 character long, they must be left as-is
    checkedClassTextGroup = new CheckedClassTextGroupVO( "aBcDé", null );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "a", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, "B", CheckedClassTextPart.INVALID );
    addClassTextPartVO( classTextParts, "c", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, "D", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( classTextParts, "é", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeInlinePunctuationMarks( checkedClassTextGroup );
    for ( ClassTextPartVO classTextPart: classTextParts )
    {
      assertEquals( "Checking length", 1, classTextPart.getClassTextPart().length() );
      assertTrue( "Checking check flag", !classTextPart.getCheckFlag().equals( CheckedClassTextPart.CORRECT ) );
    }

    // Check for class text parts with 1 character, mixed with punctuation marks with different check flags
    checkedClassTextGroup = new CheckedClassTextGroupVO( "aBcDé", null );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "a", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ";", CheckedClassTextPart.UNKNOWN ); // Is changed
    addClassTextPartVO( classTextParts, "b", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ";", CheckedClassTextPart.UNKNOWN ); // Not surrounded by correct parts
    addClassTextPartVO( classTextParts, "c", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.INVALID ); // Is invalid, leave it
    addClassTextPartVO( classTextParts, "é", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT ); // Is already correct, leave it
    addClassTextPartVO( classTextParts, "ö", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.UNKNOWN );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeInlinePunctuationMarks( checkedClassTextGroup );
    assertEquals( "Checking first ';'", CheckedClassTextPart.CORRECT, classTextParts.get( 1 ).getCheckFlag() );
    assertEquals( "Checking second ';'", CheckedClassTextPart.UNKNOWN, classTextParts.get( 3 ).getCheckFlag() );
    assertEquals( "Checking first '.'", CheckedClassTextPart.INVALID, classTextParts.get( 5 ).getCheckFlag() );
    assertEquals( "Checking second '.'", CheckedClassTextPart.CORRECT, classTextParts.get( 7 ).getCheckFlag() );
    assertEquals( "Checking third '.'", CheckedClassTextPart.UNKNOWN, classTextParts.get( 9 ).getCheckFlag() );

    // Check normal case with all kinds of combinations
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile", ';' );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "Düngemittel", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.UNKNOWN ); // Is left like that
    addClassTextPartVO( classTextParts, " Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, " Alkaloide", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.UNKNOWN ); // Is changed to correct
    addClassTextPartVO( classTextParts, " Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.INVALID ); // Is left like that
    addClassTextPartVO( classTextParts, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ";", CheckedClassTextPart.UNKNOWN ); // Is changed to correct
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeInlinePunctuationMarks( checkedClassTextGroup );
    assertEquals( "Checking class text part", CheckedClassTextPart.UNKNOWN, classTextParts.get( 0 ).getCheckFlag() );
    assertEquals( "Checking first ','", CheckedClassTextPart.UNKNOWN, classTextParts.get( 1 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.UNKNOWN, classTextParts.get( 2 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.CORRECT, classTextParts.get( 3 ).getCheckFlag() );
    assertEquals( "Checking second ','", CheckedClassTextPart.CORRECT, classTextParts.get( 4 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.CORRECT, classTextParts.get( 5 ).getCheckFlag() );
    assertEquals( "Checking third ','", CheckedClassTextPart.INVALID, classTextParts.get( 6 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.CORRECT, classTextParts.get( 7 ).getCheckFlag() );
    assertEquals( "Checking last ';'", CheckedClassTextPart.CORRECT, classTextParts.get( 8 ).getCheckFlag() );

    // Check normal case with stopwords
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Düngemittel, Waschmittel und, Sauerstoff, Automobile", '.' );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "Düngemittel", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.UNKNOWN ); // Is left like that
    addClassTextPartVO( classTextParts, " Waschmittel", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, " und", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.UNKNOWN ); // Is changed to correct
    addClassTextPartVO( classTextParts, " Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.UNKNOWN ); // Is left like that
    addClassTextPartVO( classTextParts, " Automobile", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT ); // Is left like that
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeInlinePunctuationMarks( checkedClassTextGroup );
    assertEquals( "Checking class text part", CheckedClassTextPart.CORRECT, classTextParts.get( 0 ).getCheckFlag() );
    assertEquals( "Checking first ','", CheckedClassTextPart.UNKNOWN, classTextParts.get( 1 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.UNKNOWN, classTextParts.get( 2 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.STOPWORD, classTextParts.get( 3 ).getCheckFlag() );
    assertEquals( "Checking second ','", CheckedClassTextPart.CORRECT, classTextParts.get( 4 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.CORRECT, classTextParts.get( 5 ).getCheckFlag() );
    assertEquals( "Checking third ','", CheckedClassTextPart.UNKNOWN, classTextParts.get( 6 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.UNKNOWN, classTextParts.get( 7 ).getCheckFlag() );
    assertEquals( "Checking last '.'", CheckedClassTextPart.CORRECT, classTextParts.get( 8 ).getCheckFlag() );

    // Check semicolon followed by comma, like "Véhicules;, appareils de locomotion pour terre, air et eau"
    checkedClassTextGroup = new CheckedClassTextGroupVO( ", appareils de locomotion pour terre, air et eau", ';' );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, ",", CheckedClassTextPart.UNKNOWN ); // Is changed to incorrect
    addClassTextPartVO( classTextParts, " appareils de locomotion pour terre, air et eau", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ";", CheckedClassTextPart.CORRECT ); // Is left like that
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeInlinePunctuationMarks( checkedClassTextGroup );
    assertEquals( "Checking first ','", CheckedClassTextPart.INVALID, classTextParts.get( 0 ).getCheckFlag() );
    assertEquals( "Checking class text part", CheckedClassTextPart.CORRECT, classTextParts.get( 1 ).getCheckFlag() );
    assertEquals( "Checking last ';'", CheckedClassTextPart.CORRECT, classTextParts.get( 2 ).getCheckFlag() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#changeBrackets(CheckedClassTextGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testChangeBrackets() throws Exception
  {
    // Correct brackets: ( )
    CheckedClassTextGroupVO checkedClassTextGroup = new CheckedClassTextGroupVO( "Automobile (und Flugzeuge)", '.' );
    List<ClassTextPartVO> classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "Automobile (und Flugzeuge)", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeBrackets( checkedClassTextGroup );
    assertEquals( "Checking fully correct brackets", 2, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking fully correct brackets", CheckedClassTextPart.CORRECT, classTextParts.get( 0 ).getCheckFlag() );

    // Correct brackets: [ ]
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Automobile [und Flugzeuge]", '.' );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "Automobile [und Flugzeuge]", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeBrackets( checkedClassTextGroup );
    assertEquals( "Checking fully correct brackets", 2, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking fully correct brackets", CheckedClassTextPart.CORRECT, classTextParts.get( 0 ).getCheckFlag() );

    // Brackets within an invalid text
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Automobile [und Flugzeuge] und so", '.' );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "Automobile ", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, "[und Flugzeuge] und so", CheckedClassTextPart.INVALID );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeBrackets( checkedClassTextGroup );
    assertEquals( "Checking invalid brackets", 6, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking before brackets", CheckedClassTextPart.CORRECT, classTextParts.get( 0 ).getCheckFlag() );
    assertEquals( "Checking before brackets", "Automobile ", classTextParts.get( 0 ).getClassTextPart() );
    assertEquals( "Checking opening bracket", CheckedClassTextPart.BRACKET, classTextParts.get( 1 ).getCheckFlag() );
    assertEquals( "Checking opening bracket", "[", classTextParts.get( 1 ).getClassTextPart() );
    assertEquals( "Checking in brackets", CheckedClassTextPart.INVALID, classTextParts.get( 2 ).getCheckFlag() );
    assertEquals( "Checking in brackets", "und Flugzeuge", classTextParts.get( 2 ).getClassTextPart() );
    assertEquals( "Checking closing bracket", CheckedClassTextPart.BRACKET, classTextParts.get( 3 ).getCheckFlag() );
    assertEquals( "Checking closing bracket", "]", classTextParts.get( 3 ).getClassTextPart() );
    assertEquals( "Checking after brackets", CheckedClassTextPart.INVALID, classTextParts.get( 4 ).getCheckFlag() );
    assertEquals( "Checking after brackets", " und so", classTextParts.get( 4 ).getClassTextPart() );
    assertEquals( "Checking punctuation mark", CheckedClassTextPart.CORRECT, classTextParts.get( 5 ).getCheckFlag() );
    assertEquals( "Checking punctuation mark", ".", classTextParts.get( 5 ).getClassTextPart() );

    // Brackets within an correct text
    checkedClassTextGroup = new CheckedClassTextGroupVO( "Automobile (und Flugzeuge) und so", '.' );
    classTextParts = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( classTextParts, "Automobile (", CheckedClassTextPart.INVALID );
    addClassTextPartVO( classTextParts, "und Flugzeuge", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( classTextParts, ") und so", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( classTextParts, ".", CheckedClassTextPart.CORRECT );
    checkedClassTextGroup.setClassTextParts( classTextParts );

    WdlChecker.changeBrackets( checkedClassTextGroup );
    assertEquals( "Checking invalid brackets", 6, checkedClassTextGroup.getClassTextParts().size() );
    assertEquals( "Checking before brackets", CheckedClassTextPart.INVALID, classTextParts.get( 0 ).getCheckFlag() );
    assertEquals( "Checking before brackets", "Automobile ", classTextParts.get( 0 ).getClassTextPart() );
    assertEquals( "Checking opening bracket", CheckedClassTextPart.BRACKET, classTextParts.get( 1 ).getCheckFlag() );
    assertEquals( "Checking opening bracket", "(", classTextParts.get( 1 ).getClassTextPart() );
    assertEquals( "Checking in brackets", CheckedClassTextPart.CORRECT, classTextParts.get( 2 ).getCheckFlag() );
    assertEquals( "Checking in brackets", "und Flugzeuge", classTextParts.get( 2 ).getClassTextPart() );
    assertEquals( "Checking closing bracket", CheckedClassTextPart.BRACKET, classTextParts.get( 3 ).getCheckFlag() );
    assertEquals( "Checking closing bracket", ")", classTextParts.get( 3 ).getClassTextPart() );
    assertEquals( "Checking after brackets", CheckedClassTextPart.STOPWORD, classTextParts.get( 4 ).getCheckFlag() );
    assertEquals( "Checking after brackets", " und so", classTextParts.get( 4 ).getClassTextPart() );
    assertEquals( "Checking punctuation mark", CheckedClassTextPart.CORRECT, classTextParts.get( 5 ).getCheckFlag() );
    assertEquals( "Checking punctuation mark", ".", classTextParts.get( 5 ).getClassTextPart() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkCharacters(CheckedClassTextGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testCheckCharacters() throws Exception
  {
    checkInvalidCharacter( "Hello ´", new char[] { '´' }, new int[] { 7 } );
    checkInvalidCharacter( "` goodbye", new char[] { '`' }, new int[] { 1 } );
    checkInvalidCharacter( "How are ¨ you?``", new char[] { '¨', '`', '`' }, new int[] { 9, 15, 16 } );
    checkInvalidCharacter( "I'm fine! ~~", new char[] { '~', '~' }, new int[] { 11, 12 } );
    checkInvalidCharacter( "I like ^ chinese", new char[] { '^' }, new int[] { 8 } );
    checkInvalidCharacter( "Waß, ich habe keinen ßprachfehler!", new char[] { 'ß', 'ß' }, new int[] { 3, 22 } );
    checkInvalidCharacter( "First hyphen \u2013", new char[] { '\u2013' }, new int[] { 14 } );
    checkInvalidCharacter( "Second hyphen \u2013", new char[] { '\u2013' }, new int[] { 15 } );
    checkInvalidCharacter( "Third hyphen \u2013", new char[] { '\u2013' }, new int[] { 14 } );
  }

  /**
   * Checks a WDL text for invalid characters.
   * @param wdlText The WDL text
   * @param invalidCharacters The expected invalid characters
   * @param positions The positions of the invalid characters within <code>wdlText</code> 
   * @throws Exception if the check fails
   */
  private void checkInvalidCharacter( String wdlText, char[] invalidCharacters, int[] positions ) throws Exception
  {
    assertEquals( "Checking parameters", invalidCharacters.length, positions.length );

    CheckedClassTextGroupVO classTextGroup = new CheckedClassTextGroupVO( wdlText, null );
    assertEquals( "Checking number of checked class text parts", 1, classTextGroup.getClassTextParts().size() );

    WdlChecker.checkCharacters( classTextGroup );
    for ( int i = 0; i < positions.length; i++ )
    {
      ClassTextPartVO classTextPart = classTextGroup.getPartAt( classTextGroup.new ModifiableInteger( positions[i] - 1 ) );
      assertEquals( "Checking length", 1, classTextPart.getClassTextPart().length() );
      assertEquals( "Checking status", CheckedClassTextPart.INVALID, classTextPart.getCheckFlag() );
      assertEquals( "Checking character", invalidCharacters[i], classTextPart.getClassTextPart().charAt( 0 ) );
    }
    // All other class text parts must be UNKNOWN
    for ( ClassTextPartVO classTextPart: classTextGroup.getClassTextParts() )
    {
      if ( classTextPart.getClassTextPart().length() > 1 )
      {
        assertEquals( "Checking status of other characters", CheckedClassTextPart.UNKNOWN, classTextPart.getCheckFlag() );
      }
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkWdl(String, Locale, boolean)}.
   * @throws Exception if the test fails
   */
  public void testCheckWdl() throws Exception
  {
    List<Integer> classNumbers = Arrays.asList( new Integer[] { 1 } );

    String wdlText = " 1 Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile;\n" + "   weitere Dinge";
    List<StopwordVO> stopwords = createGermanStopwords( "weitere" );
    Map<Integer, List<AbstractTermVO>> terms = createGermanTerms( 1, "Alkaloide, Sauerstoff", "Automobile", "Sauerstoff" );

    // Normal case
    List<ClassTextPartVO> expectedClassTextParts1 = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, " Alkaloide, Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts1, ",", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts1, " Automobile", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts1, ";", CheckedClassTextPart.CORRECT );

    List<ClassTextPartVO> expectedClassTextParts2 = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts2, " weitere", CheckedClassTextPart.STOPWORD );
    addClassTextPartVO( expectedClassTextParts2, " Ding", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts2, "e", CheckedClassTextPart.INVALID );

    WdlVO wdl = WdlChecker.parseAndInitialize( wdlText, Locale.GERMAN, classNumbers, false, false, true, null );
    wdl = WdlChecker.checkWdl( wdl, stopwords, terms );
    assertEquals( "Checking number of class blocks", 1, wdl.getClassBlocks().size() );

    ClassBlockVO classBlock = wdl.getClassBlocks().get( 0 );
    assertEquals( "Checking class number", 1, classBlock.getClassNumber().intValue() );

    assertEquals( "Checking number of class text groups", 2, classBlock.getClassTextGroups().size() );

    CheckedClassTextGroupVO firstClassTextGroup = (CheckedClassTextGroupVO) classBlock.getClassTextGroups().get( 0 );
    assertEquals( expectedClassTextParts1, firstClassTextGroup.getClassTextParts() );

    CheckedClassTextGroupVO secondClassTextGroup = (CheckedClassTextGroupVO) classBlock.getClassTextGroups().get( 1 );
    assertEquals( expectedClassTextParts2, secondClassTextGroup.getClassTextParts() );

    // Single class text group missing trailing point - does not work yet!
    wdlText = " 1 Düngemittel, Waschmittel, Alkaloide, Sauerstoff, Automobile";
    stopwords = createGermanStopwords( "weitere" );
    terms = createGermanTerms( 1, "Alkaloide, Sauerstoff", "Automobile", "Sauerstoff" );

    expectedClassTextParts1 = new ArrayList<ClassTextPartVO>();
    addClassTextPartVO( expectedClassTextParts1, "Düngemittel, Waschmittel,", CheckedClassTextPart.UNKNOWN );
    addClassTextPartVO( expectedClassTextParts1, " Alkaloide, Sauerstoff", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts1, ",", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts1, " Automobil", CheckedClassTextPart.CORRECT );
    addClassTextPartVO( expectedClassTextParts1, "e", CheckedClassTextPart.INVALID );

    wdl = WdlChecker.parseAndInitialize( wdlText, Locale.GERMAN, classNumbers, false, false, true, null );
    wdl = WdlChecker.checkWdl( wdl, stopwords, terms );
    assertEquals( "Checking number of class blocks", 1, wdl.getClassBlocks().size() );

    classBlock = wdl.getClassBlocks().get( 0 );
    assertEquals( "Checking class number", 1, classBlock.getClassNumber().intValue() );

    assertEquals( "Checking number of class text groups", 1, classBlock.getClassTextGroups().size() );

    firstClassTextGroup = (CheckedClassTextGroupVO) classBlock.getClassTextGroups().get( 0 );
    assertEquals( expectedClassTextParts1, firstClassTextGroup.getClassTextParts() );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#getAcronymPositions(String)}.
   */
  public void testGetAcronymPositions()
  {
    List<Integer> acronymPositions = WdlChecker.getAcronymPositions( null );
    assertTrue( "Checking for null String", acronymPositions.isEmpty() );

    acronymPositions = WdlChecker.getAcronymPositions( "" );
    assertTrue( "Checking for empty String", acronymPositions.isEmpty() );

    acronymPositions = WdlChecker.getAcronymPositions( "bla" );
    assertTrue( "Checking for String without dots", acronymPositions.isEmpty() );

    acronymPositions = WdlChecker.getAcronymPositions( "bla." );
    assertTrue( "Checking for String without acronyms", acronymPositions.isEmpty() );

    acronymPositions = WdlChecker.getAcronymPositions( "B.L.A." );
    assertEquals( "Checking for String with one acronym", "1,3,5", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( ".B.L.A." );
    assertEquals( "Checking for String with one acronym", "2,4,6", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( ".B.L.A" );
    assertEquals( "Checking for String with one acronym", "2,4", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( "   .B.L.A." );
    assertEquals( "Checking for String with one acronym", "5,7,9", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( "Das ist aber ein schönes W.C., Mr. Bond." );
    assertEquals( "Checking for String with one acronym", "26,28", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( "Mr. Bond, Sie haben ein schönes W.C.." );
    assertEquals( "Checking for String with one acronym", "33,35", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( "Mr. Bond, Sie haben ein schönes W.C." );
    assertEquals( "Checking for String with one acronym", "33,35", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( "bla bla etc. etc." );
    assertEquals( "Checking for String with etc.", "11,16", StringUtils.join( acronymPositions.iterator(), ',' ) );

    acronymPositions = WdlChecker.getAcronymPositions( "Die Ferrari S.p.A., z.B. die von Enzo, u.a. mit seinen Brüdern" );
    assertEquals( "Checking for String with well-known acronyms containing lower-case letter", "13,15,17,21,23,40,42", StringUtils.join( acronymPositions
        .iterator(), ',' ) );
  }

  /**
   * Checks two class text part Lists for equality.
   * @param expectedClassTextParts The class text parts as expected
   * @param actualClassTextParts The class text parts as a result
   */
  private void assertEquals( List<ClassTextPartVO> expectedClassTextParts, List<ClassTextPartVO> actualClassTextParts )
  {
    assertEquals( "Checking number of class text parts", expectedClassTextParts.size(), actualClassTextParts.size() );
    for ( int i = 0; i < expectedClassTextParts.size(); i++ )
    {
      ClassTextPartVO expectedClassTextPart = expectedClassTextParts.get( i );
      ClassTextPartVO actualClassTextPart = actualClassTextParts.get( i );
      assertEquals( "Checking text", expectedClassTextPart.getClassTextPart(), actualClassTextPart.getClassTextPart() );
      assertEquals( "Checking check flag", expectedClassTextPart.getCheckFlag(), actualClassTextPart.getCheckFlag() );
    }
  }

  /**
   * Adds a class text part.
   * @param classTextParts The existing class text parts
   * @param classTextPart The class text part String
   * @param checkedClassTextPart The status flag
   */
  private void addClassTextPartVO( List<ClassTextPartVO> classTextParts, String classTextPart, CheckedClassTextPart checkedClassTextPart )
  {
    classTextParts.add( new ClassTextPartVO( classTextPart, checkedClassTextPart ) );
  }

  /**
   * Creates a <code>List</code> of German stopwords.
   * @param stopwords The stopwords to use
   * @return The German stopwords in <code>stopwords</code>
   */
  private List<StopwordVO> createGermanStopwords( String... stopwords )
  {
    return createStopwords( Locale.GERMAN, stopwords );
  }

  /**
   * Creates a <code>List</code> of stopwords.
   * @param language The language of the stopwords
   * @param stopwords The stopwords to use
   * @return The stopwords in <code>stopwords</code>
   */
  private List<StopwordVO> createStopwords( Locale language, String... stopwords )
  {
    List<StopwordVO> stopwordVOs = new ArrayList<StopwordVO>( stopwords.length );
    for ( String stopword: stopwords )
    {
      stopwordVOs.add( new StopwordVO( language, stopword ) );
    }
    return stopwordVOs;
  }

  /**
   * Creates a <code>List</code> of German terms.
   * @param classNumber The class number of the terms
   * @param terms The terms to use
   * @return The German terms in <code>terms</code>
   */
  private Map<Integer, List<AbstractTermVO>> createGermanTerms( Integer classNumber, String... terms )
  {
    return createTerms( Locale.GERMAN, classNumber, terms );
  }

  /**
   * Creates a <code>List</code> of French terms.
   * @param classNumber The class number of the terms
   * @param terms The terms to use
   * @return The German terms in <code>terms</code>
   */
  private Map<Integer, List<AbstractTermVO>> createFrenchTerms( Integer classNumber, String... terms )
  {
    return createTerms( Locale.FRENCH, classNumber, terms );
  }

  /**
   * Creates a <code>List</code> of terms.
   * @param language The language to use
   * @param classNumber The class number of the terms
   * @param terms The terms to use
   * @return The German terms in <code>terms</code>
   */
  private Map<Integer, List<AbstractTermVO>> createTerms( Locale language, Integer classNumber, String... terms )
  {
    Map<Integer, List<AbstractTermVO>> termsMap = new HashMap<Integer, List<AbstractTermVO>>();
    List<AbstractTermVO> termVOs = new ArrayList<AbstractTermVO>( terms.length );
    for ( String term: terms )
    {
      NiceTermVO niceTermVO = new NiceTermVO();
      niceTermVO.setLanguage( language );
      niceTermVO.setTerm( term );
      termVOs.add( niceTermVO );
    }
    termsMap.put( classNumber, termVOs );
    return termsMap;
  }

  /**
   * Checks the texts of a number of class text groups.
   * @param classTextGroups The class text groups
   * @param keepBlanks If this is <code>true</code>, all blanks at the start and end are kept, otherwise they are trimmed
   * @param expectedTexts The exptected texts
   */
  private void checkClassTextGroups( List<ClassTextGroupVO> classTextGroups, boolean keepBlanks, String... expectedTexts )
  {
    assertEquals( "Checking number of class text group texts", classTextGroups.size(), expectedTexts.length );
    for ( int i = 0; i < classTextGroups.size(); i++ )
    {
      ClassTextGroupVO classTextGroup = classTextGroups.get( i );
      assertEquals( "Checking object class", CheckedClassTextGroupVO.class, classTextGroup.getClass() );
      CheckedClassTextGroupVO checkedClassTextGroup = (CheckedClassTextGroupVO) classTextGroup;

      String expectedText = expectedTexts[i];
      if ( !keepBlanks )
      {
        expectedText = expectedText.trim();
      }
      assertEquals( "Checking text", expectedText, checkedClassTextGroup.getClassTextGroup() );

      if ( i == classTextGroups.size() - 1 )
      {
        assertEquals( "Checking punctuation mark", Character.valueOf( '.' ), checkedClassTextGroup.getPunctuationMark() );
      }
      else
      {
        assertEquals( "Checking punctuation mark", Character.valueOf( ';' ), checkedClassTextGroup.getPunctuationMark() );
      }

      assertEquals( "Checking class text parts", 2, checkedClassTextGroup.getClassTextParts().size() );

      ClassTextPartVO classTextPart = checkedClassTextGroup.getClassTextParts().get( 0 );
      assertEquals( "Checking class text part text", expectedText, classTextPart.getClassTextPart() );
      assertEquals( "Checking check flag", CheckedClassTextPart.UNKNOWN, classTextPart.getCheckFlag() );

      Character punctuationMark = new Character( ';' );
      if ( i == classTextGroups.size() - 1 )
      {
        punctuationMark = new Character( '.' );
      }
      ClassTextPartVO punctuationMarkPart = checkedClassTextGroup.getClassTextParts().get( 1 );
      assertEquals( "Checking punctuation mark part", punctuationMark.toString(), punctuationMarkPart.getClassTextPart() );
      assertEquals( "Checking check flag", CheckedClassTextPart.UNKNOWN, punctuationMarkPart.getCheckFlag() );
    }
  }
}