package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.dao.CachingDAO;
import ch.ige.dbutils.handlers.TypedResultSetHandler;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.vo.IpiTermGroupVO;
import ch.ige.wdltool.vo.NiceTermGroupVO;
import ch.ige.wdltool.vo.TermGroupVO;

/**
 * Accesses the table <code>TERM_GROUP</code>.
 * 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @param <V> Type: A class extending <code>TermGroupVO</code>
 * @author   Esther Bandschapp
 * @version  $Id: BaseTermGroupDAO.java,v 1.16 2008/03/31 09:58:12 lewis_lee Exp $
 */
public abstract class BaseTermGroupDAO<V extends TermGroupVO> extends AbstractWdlDAO<V, Integer> implements CachingDAO
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( BaseTermGroupDAO.class );

  /**
   * The cache where the key is the ID and the value is the term group
   */
  private static Map<Class<? extends TermGroupVO>, Map<Integer, TermGroupVO>> termGroups;

  /**
   * The only constructor.
   */
  public BaseTermGroupDAO()
  {
    super();
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
  @Override
  protected final String getTableName()
  {
    return "term_group";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
  @Override
  protected final String getColumnNames()
  {
    return "name, nice_term_no, ipi_term_no";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getAliases()
  @Override
  protected final Map<String, String> getAliases()
  {
    return SQLStatementCreator.createAliases( "nice_term_no", "nice_term_number", "ipi_term_no", "ipi_term_number" );
  }

  // @see ch.ige.dbutils.dao.CachingDAO#clearCache()
  public void clearCache()
  {
    termGroups = null;
  }

  // @see ch.ige.dbutils.dao.CachingDAO#describeCachedContent()
  public CharSequence describeCachedContent()
  {
    return termGroups == null ? "" : termGroups.toString();
  }

  // @see ch.ige.dbutils.dao.CachingDAO#getCacheSize()
  public int getCacheSize()
  {
    return termGroups == null ? 0 : termGroups.size();
  }

  /**
   * Gets the term groups, which are cached after the first loading.
   * 
   * @return The term groups, where the first key is the term group VO class
   *         and this points to a Map where the key is the term group ID and the
   *         value is the corresponding term group
   * @throws SQLException if the query fails
   */
  private synchronized Map<Class<? extends TermGroupVO>, Map<Integer, TermGroupVO>> getTermGroups() throws SQLException
  {
    if ( termGroups == null )
    {
      termGroups = new HashMap<Class<? extends TermGroupVO>, Map<Integer, TermGroupVO>>();

      SQLStatementCreator<TermGroupVO> localSqlCreator = new SQLStatementCreator<TermGroupVO>( getTableName(), getIdColumnName(), getColumnNames(),
          getIdSequenceName(), getAliases() );
      List<TermGroupVO> allTermGroups = selectVOList( 15000, TermGroupVO.class, localSqlCreator.selectAllOrderBy( "nice_term_no, ipi_term_no" ) );

      for ( TermGroupVO termGroup: allTermGroups )
      {
        if ( !termGroups.containsKey( termGroup.getClass() ) )
        {
          termGroups.put( termGroup.getClass(), new LinkedHashMap<Integer, TermGroupVO>( 7500 ) );
        }
        Map<Integer, TermGroupVO> classTermGroups = termGroups.get( termGroup.getClass() );
        classTermGroups.put( termGroup.getIdentifier(), termGroup );
      }
    }
    return termGroups;
  }

  /**
   * Gets the term groups for a certain term group type.
   * @param termGroupClass The class of the term group type
   * @return The term groups for a certain term group type
   * @throws SQLException if the query fails
   */
  protected final Map<Integer, TermGroupVO> getTermGroups( Class<? extends TermGroupVO> termGroupClass ) throws SQLException
  {
    Map<Integer, TermGroupVO> allTermGroups = getTermGroups().get( termGroupClass );
    if ( allTermGroups == null )
    {
      return Collections.emptyMap();
    }
    else
    {
      return allTermGroups;
    }
  }

  /**
   * Gets the term group number for a given term group ID.
   * @param termGroupId The term group's unique ID
   * @return The term group number for the ID
   */
  public static Integer getTermGroupNumber( Integer termGroupId )
  {
    Check.notNegative( termGroupId, "term group ID" );
    if ( termGroups == null )
    {
      // Fill the cache
      try
      {
        new NiceTermGroupDAO().findAll();
      }
      catch ( SQLException e )
      {
        LOG.error( "Can't create the term group cache: " + e.getMessage(), e );
      }
    }

    Integer termGroupNumber = null;
    for ( Class<? extends TermGroupVO> termGroupClass: termGroups.keySet() )
    {
      Map<Integer, TermGroupVO> allTermGroup = termGroups.get( termGroupClass );
      if ( allTermGroup.containsKey( termGroupId ) )
      {
        TermGroupVO baseTermGroup = allTermGroup.get( termGroupId );
        if ( baseTermGroup != null )
        {
          if ( baseTermGroup instanceof NiceTermGroupVO )
          {
            NiceTermGroupVO niceTermGroup = (NiceTermGroupVO) baseTermGroup;
            termGroupNumber = niceTermGroup.getNiceTermNumber();
          }
          else if ( baseTermGroup instanceof IpiTermGroupVO )
          {
            IpiTermGroupVO ipiTermGroup = (IpiTermGroupVO) baseTermGroup;
            termGroupNumber = ipiTermGroup.getIpiTermNumber();
          }
        }
      }
    }
    return termGroupNumber;
  }

  // @see ch.ige.dbutils.dao.AbstractDAO#selectVOList(int, java.lang.Class, ch.ige.dbutils.sql.SQLStatement, java.lang.Object[])
  @Override
  protected <T extends AbstractVO> List<T> selectVOList( int estimateResultCount, Class<T> type, SQLStatement<T> query, Object... parameters )
      throws SQLException
  {
    return this.getQueryRunner().query( query, VOList( type, estimateResultCount ), parameters );
  }

  // @see ch.ige.dbutils.dao.AbstractDAO#VOList(Class, int)
  public static <T extends AbstractVO> TypedResultSetHandler<List<T>> VOList( Class<T> type, int expectedSize )
  {
    return new TermGroupVOListHandler<T>( type, AbstractVO.determineDefaultNullReplacements( type ), expectedSize );
  }
}
