package ch.ige.wdltool.dao;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.wdltool.util.Configuration;
import ch.ige.wdltool.vo.ConfigurationVO;

/**
 * Tests {@link ch.ige.wdltool.dao.ConfigurationDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ConfigurationDAOTest.java,v 1.6 2007/10/23 14:42:18 lewis_lee Exp $
 */
public class ConfigurationDAOTest extends BaseDAOTestTemplate<ConfigurationDAO, ConfigurationVO>
{
  /**
   * The key of a configuration entry that must exist in the DB.
   */
  private final static String EXISTING_KEY = Configuration.ANALYZER_LOG_DIRECTORY;

  /**
   * If a configuration has been inserted, it must be deleted during {@link #tearDown()}.
   */
  private String keyToDelete;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public ConfigurationDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.ConfigurationDAO#find(java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    try
    {
      getDAO().find( null );
      fail( "Null key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().find( "" );
      fail( "Empty key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    assertNull( "Checking unknown key", getDAO().find( "gugus" ) );
    assertNotNull( "Checking known key", getDAO().find( EXISTING_KEY ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.ConfigurationDAO#save(String, String)}.
   * @throws Exception if the test fails
   */
  public void testSave() throws Exception
  {
    try
    {
      getDAO().save( null, "value" );
      fail( "Null key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().save( "", "value" );
      fail( "Empty key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().save( "key", null );
      fail( "Null value must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().save( "key", "" );
      fail( "Empty value must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    // Insert
    ConfigurationVO configuration = getSerializableVO();
    Integer oldId = configuration.getIdentifier();
    configuration = getDAO().save( configuration.getKey(), configuration.getValue() );
    this.keyToDelete = configuration.getKey();
    ConfigurationVO loadedConfiguration = getDAO().find( configuration.getKey() );
    assertNotNull( "Checking that the configuration was saved", loadedConfiguration );
    assertTrue( "Checking that the ID changed", !oldId.equals( loadedConfiguration.getIdentifier() ) );
    compareVOs( configuration, loadedConfiguration );

    //Update
    configuration.setValue( "hey, a new value!" );
    oldId = configuration.getIdentifier();
    configuration = getDAO().save( configuration.getKey(), configuration.getValue() );
    loadedConfiguration = getDAO().find( configuration.getKey() );
    assertNotNull( "Checking that the configuration was saved", loadedConfiguration );
    assertTrue( "Checking that the ID didn't change", oldId.equals( loadedConfiguration.getIdentifier() ) );
    compareVOs( configuration, loadedConfiguration );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.ConfigurationDAO#delete(String)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    try
    {
      getDAO().delete( (String) null );
      fail( "Null key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().delete( "" );
      fail( "Empty key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    // An unknown key is ok
    getDAO().delete( "gugus33" );

    ConfigurationVO configuration = getSerializableVO();
    configuration.setKey( "delete this key" );
    configuration = getDAO().save( configuration.getKey(), configuration.getValue() );
    this.keyToDelete = configuration.getKey();

    getDAO().delete( configuration.getKey() );
    assertNull( "Checking key was deleted", getDAO().find( configuration.getKey() ) );
    this.keyToDelete = null;
  }

  /**
   * Compares the two configurations.
   * @param originalVO The original configuration
   * @param loadedVO The configuration loaded from the DB
   */
  private void compareVOs( ConfigurationVO originalVO, ConfigurationVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking key", originalVO.getKey(), loadedVO.getKey() );
    assertEquals( "Checking value", originalVO.getValue(), loadedVO.getValue() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<ConfigurationDAO> getDAOClass()
  {
    return ConfigurationDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected ConfigurationVO getSerializableVO()
  {
    ConfigurationVO configurationVO = new ConfigurationVO();
    configurationVO.setIdentifier( 33 );
    configurationVO.setKey( "theKey" );
    configurationVO.setValue( "theValue" );
    return configurationVO;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( ConfigurationVO originalVO, ConfigurationVO serializedVO )
  {
    return originalVO.getIdentifier().equals( serializedVO.getIdentifier() ) && originalVO.getKey().equals( serializedVO.getKey() )
        && originalVO.getValue().equals( serializedVO.getValue() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    if ( this.keyToDelete != null )
    {
      getDAO().delete( this.keyToDelete );
    }
    super.tearDown();
  }
}
