package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.dbutils.sql.SQLWhereStatementCreator;
import ch.ige.dbutils.vo.VOUtils;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.util.TermUtils;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.ClassVO;
import ch.ige.wdltool.vo.ExtendedSearchVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.TermReplicationVO;
import ch.ige.wdlutil.text.RegexUtils;
import ch.ige.wdlutil.text.SearchText;

/**
 * Accesses the table <code>TERM</code> for all kinds of terms.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: GeneralTermDAO.java,v 1.84 2008/12/09 07:31:25 heinzer Exp $
 */
public class GeneralTermDAO extends BaseTermDAO<AbstractTermVO, Integer>
{
  /**
   * Updates a number of terms to the same class number.
   */
  private static final SQLStatement<Integer> SET_TERMS_TO_CLASS_NUMBER = SQL.update( "UPDATE term SET class_id = ? WHERE term_id IN (?)" );

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( GeneralTermDAO.class );

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<AbstractTermVO> getVOType()
  {
    return AbstractTermVO.class;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermType()
  @Override
  protected TermType getTermType()
  {
    return null;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermTypeName()
  @Override
  protected String getTermTypeName()
  {
    return "term";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected Object[] getValues( AbstractTermVO term )
  {
    // Irrelevant, since it's only used for inserting and updating
    return null;
  }

  /**
   * Finds all terms <strong>except term candidates</strong>.
   * @return All terms except term candidates
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findAllTerms() throws SQLException
  {
    List<AbstractTermVO> allTerms = new ArrayList<AbstractTermVO>( BaseTermDAO.ESTIMATED_NUMBER_OF_TERMS );
    Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> languageTerms = getTerms();
    for ( Locale language: languageTerms.keySet() )
    {
      allTerms.addAll( findAllTerms( language ) );
    }
    return allTerms;
  }

  /**
   * Finds all terms <strong>except term candidates</strong> containing the given language.
   * @param language Only terms with this language are fetched
   * @return All terms except term candidates
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findAllTerms( Locale language ) throws SQLException
  {
    List<AbstractTermVO> allTerms = new ArrayList<AbstractTermVO>( BaseTermDAO.ESTIMATED_NUMBER_OF_TERMS / 3 );
    Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>> termTypeTerms = getTerms().get( language );
    for ( Class<? extends AbstractTermVO> termClass: termTypeTerms.keySet() )
    {
      List<AbstractTermVO> terms = termTypeTerms.get( termClass );
      allTerms.addAll( terms );
    }
    return allTerms;
  }

  /**
   * Sets all the terms to the given class number.
   * @param terms The terms to change
   * @param classNumber The new class number to set
   */
  @SuppressWarnings( "unchecked" )
  public void updateClassNumber( final List<AbstractTermVO> terms, final Integer classNumber )
  {
    Check.notNull( terms, "terms" );
    Check.notNegative( classNumber, "class number" );

    if ( !terms.isEmpty() )
    {
      // The whole thing is only necessary because IN (...) isn't supported in prepared statements
      Set<Integer> uniqueIds = VOUtils.extractIDs( terms );
      String sqlStatement = SET_TERMS_TO_CLASS_NUMBER.toString();
      sqlStatement = sqlStatement.replaceFirst( "\\?", classNumber.toString() );
      sqlStatement = sqlStatement.replaceFirst( "\\?", StringUtils.join( uniqueIds.iterator(), ", " ) );
      final SQLStatement<Integer> updateStatement = SQL.update( sqlStatement );

      this.execute( new SingleDAOUnitOfWork()
      {
        // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
        public void execute( IGEQueryRunner runner ) throws SQLException
        {
          int count = runner.update( updateStatement );
          verifyUpdateCount( count, terms.size(), "Must update " + terms.size() + " terms" );

          GeneralTermDAO.this.clearCache();
        }
      } );
    }
  }

  /**
   * Finds all terms that exist in term groups referenced by <code>terms</code> and fills them into the terms.
   * @param terms The terms whose term groups are fetched and inserted
   * @throws SQLException if the query fails
   */
  void fillTermGroupTerms( List<AbstractTermVO> terms ) throws SQLException
  {
    Check.notNull( terms, "terms" );

    Map<Integer, Map<Locale, List<AbstractTermVO>>> termGroupTerms = getTermGroupTerms();
    for ( AbstractTermVO term: terms )
    {
      Integer termGroupId = TermUtils.getTermGroupId( term );
      if ( termGroupId != null )
      {
        TermUtils.setTermGroupNumber( term );
        Map<Locale, List<AbstractTermVO>> languageTerms = termGroupTerms.get( termGroupId );
        if ( languageTerms == null )
        {
          throw new IllegalStateException( "Found term group ID " + termGroupId + " in term " + term + ", but no term group terms" );
        }

        // Since the current term is taken out of the Map, it has to be copied
        Map<Locale, List<AbstractTermVO>> newLanguageTerms = new HashMap<Locale, List<AbstractTermVO>>( languageTerms.size() );
        newLanguageTerms.putAll( languageTerms );
        List<AbstractTermVO> containingThisTerm = newLanguageTerms.get( term.getLanguage() );
        if ( containingThisTerm == null || containingThisTerm.isEmpty() )
        {
          throw new IllegalStateException( "Found term group ID " + termGroupId + " with language " + term.getLanguage() + " in term with ID "
              + term.getIdentifier() + ", but not in the loaded term group terms" );
        }

        // Make a copy of the list, except for the current term
        List<AbstractTermVO> notContainingThisTerm = new ArrayList<AbstractTermVO>( containingThisTerm.size() );
        for ( AbstractTermVO languageTerm: containingThisTerm )
        {
          if ( term.getIdentifier().equals( languageTerm.getIdentifier() ) )
          {
            // LOG.debug( "Found identical term with ID " + term.getIdentifier() + " to remove from the term group
            // terms: " + languageTerm );
          }
          else
          {
            notContainingThisTerm.add( languageTerm );
          }
        }
        newLanguageTerms.put( term.getLanguage(), notContainingThisTerm );
        if ( term instanceof NiceTermVO )
        {
          NiceTermVO niceTerm = (NiceTermVO) term;
          niceTerm.setTermGroupTerms( newLanguageTerms );
        }
        else if ( term instanceof IpiTermVO )
        {
          IpiTermVO ipiTerm = (IpiTermVO) term;
          ipiTerm.setTermGroupTerms( newLanguageTerms );
        }
      }
    }
  }

  /**
   * Finds all active positive terms (Nice and IPI terms) of a language and below an optional term length.
   * @param language Find only terms in this language
   * @param maximumLength Only terms up to this length are kept; set to <code>null</code> to ignore
   * @return The terms by class number, and then ordered by descending length
   * @throws SQLException if the query fails
   */
  public Map<Integer, List<AbstractTermVO>> findPositiveTerms( Locale language, final Integer maximumLength ) throws SQLException
  {
    Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> cachedTerms = getTerms();
    if ( cachedTerms.containsKey( language ) )
    {
      Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>> typeTerms = cachedTerms.get( language );
      // Only normal terms and IPI terms are used
      List<AbstractTermVO> niceTerms = typeTerms.get( NiceTermVO.class );
      LOG.debug( "Ignoring the maximum length, got " + niceTerms.size() + " Nice terms for " + language );
      List<AbstractTermVO> ipiTerms = typeTerms.get( IpiTermVO.class );
      LOG.debug( "Ignoring the maximum length, got " + ipiTerms.size() + " IPI terms for " + language );

      List<AbstractTermVO> positiveTerms = new ArrayList<AbstractTermVO>( niceTerms.size() + ipiTerms.size() );
      positiveTerms.addAll( niceTerms );
      positiveTerms.addAll( ipiTerms );
      // Here comes the garbage collector...
      niceTerms = null;
      ipiTerms = null;
      // And re-order by length
      Collections.sort( positiveTerms, new TermLengthComparator() );

      if ( positiveTerms.isEmpty() )
      {
        return Collections.emptyMap();
      }
      else
      {
        // Assume that about half of the terms will be kept
        Map<Integer, List<AbstractTermVO>> keptTerms = new HashMap<Integer, List<AbstractTermVO>>( positiveTerms.size() / 2 );
        for ( AbstractTermVO term: positiveTerms )
        {
          if ( maximumLength == null || term.getTerm().length() <= maximumLength )
          {
            Integer classNumber = term.getClassNumber();
            if ( !keptTerms.containsKey( classNumber ) )
            {
              // Again, assume that the terms are evenly distributed
              keptTerms.put( classNumber, new ArrayList<AbstractTermVO>( positiveTerms.size() / 2 / ClassVO.NUMBER_OF_CLASS_NUMBERS ) );
            }
            List<AbstractTermVO> classTerms = keptTerms.get( classNumber );

            boolean isActive = true; // Nice terms are always active
            if ( term instanceof IpiTermVO )
            {
              isActive = ( (IpiTermVO) term ).isActive();
            }

            if ( isActive )
            {
              classTerms.add( term );
            }
            else
            {
              LOG.debug( "The term " + term + " is not active and thus rejected" );
            }
          }
        }

        if ( LOG.isDebugEnabled() )
        {
          String displayMaximumLength = "";
          if ( maximumLength != null )
          {
            displayMaximumLength = " of maximum length " + maximumLength;
          }
          int totalTerms = 0;
          List<Integer> classNumbers = new ArrayList<Integer>( keptTerms.keySet() );
          Collections.sort( classNumbers );
          for ( Integer classNumber: classNumbers )
          {
            List<AbstractTermVO> classTerms = keptTerms.get( classNumber );
            totalTerms += classTerms.size();
            LOG.debug( "Got " + classTerms.size() + " terms" + displayMaximumLength + " for class number " + classNumber );
          }
          LOG.debug( "A total of " + totalTerms + " terms" + displayMaximumLength + " exist." );
        }

        return keptTerms;
      }
    }
    else
    {
      return Collections.emptyMap();
    }
  }

  /**
   * Finds all active terms that correspond to the search criteria defined in <code>searchDefinition</code>.
   * Afterwards, it fills the related term group terms into every term. <strong>Note that this is never cached!</strong>
   * (Except for term group terms) <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param searchDefinition The extended search parameters
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( ExtendedSearchVO searchDefinition ) throws SQLException
  {
    return findTermWithinActiveTerms( searchDefinition, true );
  }

  /**
   * Finds all active terms that correspond to the search criteria defined in <code>searchDefinition</code>.
   * Afterwards, it fills the related term group terms into every term. <strong>Note that this is never cached!</strong>
   * (Except for term group terms) <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param searchDefinition The extended search parameters
   * @param fillTermGroupTerms If this is <code>true</code>, term groups and terms within those term groups are
   *        also loaded; if it's <code>false</code>, only the bare term is used
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( ExtendedSearchVO searchDefinition, boolean fillTermGroupTerms ) throws SQLException
  {
    Check.notNull( searchDefinition, "search definition" );
    LOG.debug( "Searching with the following definition: " + searchDefinition );
    FullSqlStatement searchStatement = createSqlStatementForComplexFind( searchDefinition );
    LOG.debug( "Search statement: " + searchStatement.sqlStatement );
    LOG.debug( "Search parameters: " + StringUtils.join( searchStatement.whereStatement.getStatementParameters(), ", " ) );
    List<AbstractTermVO> terms = selectVOList( 50, AbstractTermVO.class, searchStatement.sqlStatement, searchStatement.whereStatement.getStatementParameters() );
    filterWholeWordOnly( searchDefinition, terms );
    if ( fillTermGroupTerms )
    {
      fillTermGroupTerms( terms );
    }
    return terms;
  }

  /**
   * Finds all active terms in the given language whose search term match exactly the given
   * <code>searchTerm</code> (after it has been transformed into a DB search term).
   * This does consider also 'checker only' terms.
   * @param term The term to search for (language and term), which is searched exactly as given, after having been
   *        transformed to a search term
   *        The term used as input is not returned.
   * @return All active terms whose search term matches <code>searchTerm</code>; may be empty.
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findSameSearchTermWithinActiveTerms( AbstractTermVO term ) throws SQLException
  {
    Check.notNull( term.getLanguage(), "language" );
    Check.notEmpty( term.getTerm(), "search term" );

    SQLStatementCreator<AbstractTermVO> creator = new SQLStatementCreator<AbstractTermVO>( getTableName(), getIdColumnName(), getColumnNames(),
        getIdSequenceName(), getAliases() );

    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    appendBaseActiveTermSelection( whereStatement, term.getLanguage(), null );
    whereStatement.appendFragmentWithParameters( " AND search_term = ?", SearchText.toSearchText( term.getTerm(), term.getLanguage() ) );

    SQLStatement<AbstractTermVO> completeQuery = creator.selectBy( whereStatement );
    LOG.debug( "Finding same search term with query: " + completeQuery.getStatement() );

    List<AbstractTermVO> terms = selectVOList( 50, AbstractTermVO.class, completeQuery, whereStatement.getStatementParameters() );
    // remove self from the list
    for ( AbstractTermVO termVO: terms )
    {
      if ( termVO.getIdentifier().equals( term.getIdentifier() ) )
      {
        terms.remove( termVO );
        break;
      }
    }
    return terms;
  }

  /**
   * Finds all active terms (in any language) that correspond to any word within <code>searchTerm</code>. This splits
   * up <code>searchTerm</code> into separate words and tries to find any of those words in the search terms. For this
   * to work, it simplifies the words in <code>searchTerm</code>. Afterwards, it fills the related term group terms
   * into every term. <strong>Note that this is never cached!</strong> (Except for term group terms) <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findAnyWordWithinActiveTerms( String searchTerm, Boolean isCheckerOnly ) throws SQLException
  {
    Check.notEmpty( searchTerm, "search term" );
    FullSqlStatement statement = createSqlStatementForSimpleFind( null, searchTerm, true, isCheckerOnly );
    List<AbstractTermVO> terms = selectVOList( 50, AbstractTermVO.class, statement.sqlStatement, statement.whereStatement.getStatementParameters() );
    fillTermGroupTerms( terms );
    return terms;
  }

  /**
   * Finds all active terms (in any language) that correspond to any word within <code>searchTerm</code>. This splits
   * up <code>searchTerm</code> into separate words and tries to find any of those words in the search terms. For this
   * to work, it simplifies the words in <code>searchTerm</code>. Afterwards, it fills the related term group terms
   * into every term. <strong>Note that this is never cached!</strong> (Except for term group terms) <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * This does not consider any checker only terms - use {@link #findAnyWordWithinActiveTerms(String, Boolean)} for different behaviour.
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findAnyWordWithinActiveTerms( String searchTerm ) throws SQLException
  {
    return findAnyWordWithinActiveTerms( searchTerm, false );
  }

  /**
   * Finds all active terms of a language that correspond to any word within <code>searchTerm</code>. This splits up
   * <code>searchTerm</code> into separate words and tries to find any of those words in the search terms. For this to
   * work, it simplifies the words in <code>searchTerm</code>. Afterwards, it fills the related term group terms into
   * every term. <strong>Note that this is never cached!</strong> (Except for term group terms) <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param language The language in which terms are searched for
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findAnyWordWithinActiveTerms( Locale language, String searchTerm, Boolean isCheckerOnly ) throws SQLException
  {
    Check.notNull( language, "language" );
    Check.notEmpty( searchTerm, "search term" );
    FullSqlStatement statement = createSqlStatementForSimpleFind( language, searchTerm, true, isCheckerOnly );
    List<AbstractTermVO> terms = selectVOList( 50, AbstractTermVO.class, statement.sqlStatement, statement.whereStatement.getStatementParameters() );
    fillTermGroupTerms( terms );
    return terms;
  }

  /**
   * Finds all active terms of a language that correspond to any word within <code>searchTerm</code>. This splits up
   * <code>searchTerm</code> into separate words and tries to find any of those words in the search terms. For this to
   * work, it simplifies the words in <code>searchTerm</code>. Afterwards, it fills the related term group terms into
   * every term. <strong>Note that this is never cached!</strong> (Except for term group terms) <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * This does not consider any checker only terms - use {@link #findAnyWordWithinActiveTerms(Locale, String, Boolean)} for different behaviour.
   * @param language The language in which terms are searched for
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findAnyWordWithinActiveTerms( Locale language, String searchTerm ) throws SQLException
  {
    return findAnyWordWithinActiveTerms( language, searchTerm, false );
  }

  /**
   * Finds all active terms of a language that correspond to the exact phrase in <code>searchTerm</code>.
   * <strong>Note that this is never cached!</strong>
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param term The exact term to search for
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( Locale language, Integer classNumber, String term, Boolean isCheckerOnly ) throws SQLException
  {
    Check.notNull( language, "language" );
    Check.notEmpty( term, "term" );

    SQLStatementCreator<AbstractTermVO> creator = new SQLStatementCreator<AbstractTermVO>( getTableName(), getIdColumnName(), getColumnNames(),
        getIdSequenceName(), getAliases() );
    SQLWhereStatementCreator whereStatement = createWhereStatementForFindTermWithinActiveTerms( language, classNumber, term, isCheckerOnly );
    appendDefaultOrderBy( whereStatement, language );
    SQLStatement<AbstractTermVO> completeQuery = creator.selectBy( whereStatement );
    return selectVOList( 50, AbstractTermVO.class, completeQuery, whereStatement.getStatementParameters() );
  }

  /**
   * Finds all active terms of a language that correspond to the exact phrase in <code>searchTerm</code>.
   * This does not consider any checker only terms - use {@link #findTermWithinActiveTerms(Locale, Integer, String, Boolean)} for different behaviour.
   * <strong>Note that this is never cached!</strong>
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param term The exact term to search for
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( Locale language, Integer classNumber, String term ) throws SQLException
  {
    return findTermWithinActiveTerms( language, classNumber, term, false );
  }

  /**
   * Deletes a number of terms. <strong>Note that this clears the cache first!</strong>
   * @param termIds The unique identifiers for the terms to delete
   */
  @SuppressWarnings( "unchecked" )
  public void delete( final Collection<Integer> termIds )
  {
    super.delete( termIds );
    clearCache();
  }

  /**
   * Creates there <code>WHERE</code> statement for {@link #findTermWithinActiveTerms(Locale, Integer, String)}.
   * Negative terms without class (= any class) are included.
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for. If this is <code>null</code> terms in all classes
   * (including any class) are looked for.
   * @param term The exact term to search for
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * @return The <code>WHERE</code> statement to execute
   */
  SQLWhereStatementCreator createWhereStatementForFindTermWithinActiveTerms( Locale language, Integer classNumber, String term, Boolean isCheckerOnly )
  {
    checkTermLength( term );
    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    appendBaseActiveTermSelection( whereStatement, language, isCheckerOnly );
    whereStatement.appendFragmentWithParameters( " AND term = ?", term );
    if ( classNumber != null ) // any class
    {
      whereStatement.appendFragmentWithParameters( " AND (class_id = ? OR (class_id IS NULL AND term_type = ?))", classNumber, TermType.NEGATIVE_IPI_TERM
          .toDB() );
    }
    return whereStatement;
  }

  /**
   * Creates the SQL statement used to simply find one or more words within <code>searchTerm</code>.
   * @param language The language in which terms are searched for
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @param splitWords If this is <code>true</code>, the <code>searchTerm</code> is split up into blank-separated
   * words; otherwise it's used as exact phrase
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   *
   * @return The SQL statement used to find the search term
   */
  FullSqlStatement createSqlStatementForSimpleFind( Locale language, String searchTerm, boolean splitWords, Boolean isCheckerOnly )
  {
    SQLStatementCreator<AbstractTermVO> creator = new SQLStatementCreator<AbstractTermVO>( getTableName(), getIdColumnName(), getColumnNames(),
        getIdSequenceName(), getAliases() );
    SQLWhereStatementCreator whereStatement = createWhereStatementForSimpleFind( language, searchTerm, splitWords, isCheckerOnly );
    SQLStatement<AbstractTermVO> completeQuery = creator.selectBy( whereStatement );
    LOG.debug( "simple search with query: " + completeQuery.getStatement() );
    return new FullSqlStatement( whereStatement, completeQuery );
  }

  /**
   * Creates the SQL <code>WHERE</code> statement used to simply find one or more words within <code>searchTerm</code>,
   * including the <code>ORDER BY</code> statement.
   * @param language The language in which terms are searched for
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @param splitWords If this is <code>true</code>, the <code>searchTerm</code> is split up into blank-separated
   * words; otherwise it's used as exact phrase
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * @return The SQL statement used to find the search term
   */
  SQLWhereStatementCreator createWhereStatementForSimpleFind( Locale language, String searchTerm, boolean splitWords, Boolean isCheckerOnly )
  {
    checkTermLength( searchTerm );
    String[] words = StringUtils.split( searchTerm ); // Use a light-weight split for whitespace
    if ( words.length == 0 )
    {
      throw new IllegalArgumentException( "Found no search term words in " + searchTerm );
    }

    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    appendBaseActiveTermSelection( whereStatement, language, isCheckerOnly );
    whereStatement.appendFragment( " AND ( " );

    if ( splitWords )
    {
      for ( int i = 0; i < words.length; i++ )
      {
        if ( i > 0 )
        {
          whereStatement.appendFragment( OR );
        }
        appendFullEqualityStatement( whereStatement, words[i], language, null, false, true );
      }
    }
    else
    {
      appendFullEqualityStatement( whereStatement, searchTerm.trim(), language, null, false, true );
    }
    whereStatement.appendFragment( " )" );
    appendDefaultOrderBy( whereStatement, language );
    return whereStatement;
  }

  /**
   * Constructs the query part for filtering on active terms (no candidates, active flag set) in a given language
   * @param search The search being created
   * @param language The language of the term to search in
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   */
  private void appendBaseActiveTermSelection( SQLWhereStatementCreator search, Locale language, Boolean isCheckerOnly )
  {
    // IPI term candidates must never appear
    search.appendFragmentWithParameters( " term_type != ?", TermType.IPI_TERM_CANDIDATE.toDB() );
    // Only active terms are taken
    search.appendFragmentWithParameters( " AND is_active_bool = ?", new Integer( 1 ) );
    if ( language != null )
    {
      search.appendFragmentWithParameters( " AND language_sn = ?", language.getLanguage() );
    }
    if ( isCheckerOnly != null )
    {
      if ( isCheckerOnly )
      {
        search.appendFragmentWithParameters( " AND is_checker_only_bool = ?", new Integer( 1 ) );
      }
      else
      {
        search.appendFragmentWithParameters( " AND is_checker_only_bool = ?", new Integer( 0 ) );
      }
    }
  }

  /**
   * Creates the SQL statement used to find one or more words within <code>searchTerm</code> with a lot of
   * possibilities.
   * @param searchDefinition The extended search parameters
   * @return The SQL statement used to find the search term
   */
  FullSqlStatement createSqlStatementForComplexFind( ExtendedSearchVO searchDefinition )
  {
    SQLStatementCreator<AbstractTermVO> creator = new SQLStatementCreator<AbstractTermVO>( getTableName(), getIdColumnName(), getColumnNames(),
        getIdSequenceName(), getAliases() );

    SQLWhereStatementCreator whereStatement = createWhereStatementForComplexFind( searchDefinition );
    SQLStatement<AbstractTermVO> completeQuery = creator.selectBy( whereStatement );
    if ( searchDefinition.termGroupNumber != null )
    {
      String joinedStatement = completeQuery.toString().replace( "FROM term",
          "FROM term trm " + "LEFT JOIN term_group tmg ON trm.term_group_id = tmg.term_group_id" ).replace( "term_group_id,", "trm.term_group_id," );
      completeQuery = SQL.select( joinedStatement );
    }
    LOG.debug( "extended search with query: " + completeQuery.getStatement() );
    return new FullSqlStatement( whereStatement, completeQuery );
  }

  /**
   * Creates the SQL <code>WHERE</code> statement used for the complex find, including the <code>ORDER BY</code>
   * statement.
   * @param searchDefinition Contains all search criteria
   * @return The SQL statement used to find the complex search
   */
  SQLWhereStatementCreator createWhereStatementForComplexFind( ExtendedSearchVO searchDefinition )
  {
    Check.notNull( searchDefinition, "search definition" );
    // Note: The search text is NOT checked for existence, because the extended search may be used without specifying one
    if ( StringUtils.isNotEmpty( searchDefinition.searchText ) )
    {
      checkTermLength( searchDefinition.searchText );
    }

    if ( searchDefinition.isGeneric != null && ( searchDefinition.dataSource == null || !searchDefinition.dataSource.equals( DataSource.NICE ) ) )
    {
      throw new IllegalArgumentException( "Boolean operator for generic terms is only valid if the data source is Nice" );
    }

    // Use a light-weight split for whitespace
    String[] words = StringUtils.split( searchDefinition.searchText );

    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    appendBaseActiveTermSelection( whereStatement, searchDefinition.language, searchDefinition.isCheckerOnly );

    if ( searchDefinition.classNumbers != null && !searchDefinition.classNumbers.isEmpty() )
    {
      if ( searchDefinition.classNumbers.size() == 1 && searchDefinition.classNumbers.contains( ExtendedSearchVO.NO_CLASS ) )
      {
        whereStatement.appendFragment( " AND class_id IS NULL" );
      }
      else
      {
        if ( searchDefinition.classNumbers.size() == 1 )
        {
          whereStatement.appendFragmentWithParameters( " AND class_id = ?", searchDefinition.classNumbers.iterator().next() );
        }
        else
        {
          whereStatement.appendFragment( " AND class_id IN (" + StringUtils.join( searchDefinition.classNumbers.iterator(), ", " ) + ")" );
        }
      }
    }

    if ( searchDefinition.dataSource != null )
    {
      whereStatement.appendFragmentWithParameters( " AND data_source_id = ?", searchDefinition.dataSource.toDB() );
      if ( searchDefinition.dataSource.equals( DataSource.NICE ) && searchDefinition.isGeneric != null )
      {
        whereStatement.appendFragmentWithParameters( " AND is_generic_bool = ?", getBooleanInteger( searchDefinition.isGeneric ) );
      }
    }

    if ( searchDefinition.isNegative != null )
    {
      if ( searchDefinition.isNegative )
      {
        whereStatement.appendFragmentWithParameters( " AND term_type = ?", TermType.NEGATIVE_IPI_TERM.toDB() );
      }
      else
      {
        whereStatement.appendFragmentWithParameters( " AND term_type != ?", TermType.NEGATIVE_IPI_TERM.toDB() );
      }
    }

    if ( searchDefinition.termGroupNumber != null )
    {
      whereStatement.appendFragmentWithParameters( " AND (tmg.nice_term_no = ?", searchDefinition.termGroupNumber.toString() );
      whereStatement.appendFragmentWithParameters( " OR tmg.ipi_term_no = ?)", searchDefinition.termGroupNumber.toString() );
    }

    if ( searchDefinition.isPublic != null )
    {
      whereStatement.appendFragmentWithParameters( " AND is_public_bool = ?", getBooleanInteger( searchDefinition.isPublic ) );
    }

    if ( searchDefinition.changedFromDate != null )
    {
      whereStatement.appendFragmentWithParameters( " AND trunc(changed_dat) >= to_date(?, 'dd.mm.yyyy')", COMPLEX_SEARCH_DATE_FORMAT
          .format( searchDefinition.changedFromDate ) );
    }

    if ( searchDefinition.changedToDate != null )
    {
      whereStatement.appendFragmentWithParameters( " AND trunc(changed_dat) <= to_date(?, 'dd.mm.yyyy')", COMPLEX_SEARCH_DATE_FORMAT
          .format( searchDefinition.changedToDate ) );
    }

    if ( searchDefinition.renewedFromDate != null )
    {
      whereStatement.appendFragmentWithParameters( " AND trunc(renewal_dat) >= to_date(?, 'dd.mm.yyyy')", COMPLEX_SEARCH_DATE_FORMAT
          .format( searchDefinition.renewedFromDate ) );
    }

    if ( searchDefinition.renewedToDate != null )
    {
      whereStatement.appendFragmentWithParameters( " AND trunc(renewal_dat) <= to_date(?, 'dd.mm.yyyy')", COMPLEX_SEARCH_DATE_FORMAT
          .format( searchDefinition.renewedToDate ) );
    }

    if ( searchDefinition.changedUser != null )
    {
      whereStatement.appendFragmentWithParameters( " AND changed_user = ?", searchDefinition.changedUser );
    }
    if ( searchDefinition.createdUser != null )
    {
      whereStatement.appendFragmentWithParameters( " AND created_user = ?", searchDefinition.createdUser );
    }

    if ( StringUtils.isNotEmpty( searchDefinition.searchText ) )
    {
      whereStatement.appendFragment( " AND ( " );

      if ( searchDefinition.splitWords && words != null )
      {
        for ( int i = 0; i < words.length; i++ )
        {
          if ( i > 0 )
          {
            if ( searchDefinition.compositionAnd )
            {
              whereStatement.appendFragment( AND );
            }
            else
            {
              whereStatement.appendFragment( OR );
            }
          }

          if ( searchDefinition.compositionAnd )
          {
            whereStatement.appendFragment( "(" );
          }
          appendFullEqualityStatement( whereStatement, words[i], searchDefinition.language, searchDefinition.searchComment, searchDefinition.exactSearch, true );

          if ( searchDefinition.compositionAnd )
          {
            whereStatement.appendFragment( ")" );
          }
        }
      }
      else
      {
        appendFullEqualityStatement( whereStatement, searchDefinition.searchText, searchDefinition.language, searchDefinition.searchComment,
            searchDefinition.exactSearch, true );
      }
      whereStatement.appendFragment( " )" );
    }

    Locale sortLanguage = searchDefinition.language;
    if ( searchDefinition.sortLanguage != null )
    {
      sortLanguage = searchDefinition.sortLanguage;
    }
    appendDefaultOrderBy( whereStatement, sortLanguage );

    return whereStatement;
  }

  /**
   * Filters the search results by {@link ExtendedSearchVO#wholeWordOnly}.
   * If this is <code>true</code>, then the search results in <code>terms</code> is only
   * kept if the search term in {@link ExtendedSearchVO#searchText} is contained as a whole
   * word or within word boundaries in the resulting term. If this isn't the case, the term
   * is removed from <code>terms</code>.
   * @param extendedSearch Defines the search
   * @param terms The search result
   */
  static void filterWholeWordOnly( ExtendedSearchVO extendedSearch, List<AbstractTermVO> terms )
  {
    if ( terms != null && terms.size() > 0 && extendedSearch != null && extendedSearch.wholeWordOnly && StringUtils.isNotEmpty( extendedSearch.searchText ) )
    {
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "Filtering search results for whole word(s) '" + extendedSearch.searchText + "', " + ( extendedSearch.splitWords ? "" : "not " )
            + "splitting words, with " + ( extendedSearch.exactSearch ? "" : "non " ) + "exact search" );
        if ( extendedSearch.splitWords )
        {
          LOG.debug( "Split words are combined by " + ( extendedSearch.compositionAnd ? "AND" : "OR" ) );
        }
      }

      // There must be a language for the term pattern.
      Locale language = Locale.GERMAN;
      if ( extendedSearch.language != null )
      {
        language = extendedSearch.language;
      }

      String[] searchTexts = { extendedSearch.searchText };
      if ( extendedSearch.splitWords )
      {
        searchTexts = StringUtils.split( extendedSearch.searchText );
      }
      // Contains the identifiers of all terms to keep with a counter how many times they were found.
      Map<Integer, Integer> termIdentifiersToKeep = new TreeMap<Integer, Integer>();

      for ( String searchText: searchTexts )
      {
        if ( !extendedSearch.exactSearch )
        {
          searchText = SearchText.toSearchText( searchText, language );
        }

        // It's roughly the same problem like when terms are matched
        Pattern termPattern = RegexUtils.createUnicodeWordPattern( searchText );
        // For comments, use the original search text.
        Pattern commentPattern = RegexUtils.createUnicodeWordPattern( extendedSearch.searchText );

        for ( AbstractTermVO term: terms )
        {
          Matcher matcher = null;
          if ( extendedSearch.exactSearch )
          {
            matcher = termPattern.matcher( term.getTerm() );
          }
          else
          {
            matcher = termPattern.matcher( term.getSearchTerm() );
          }
          // Yes, it's in the term
          if ( matcher.find() )
          {
            increment( termIdentifiersToKeep, term.getIdentifier() );
          }

          if ( StringUtils.isNotEmpty( term.getPrivateComment() ) )
          {
            matcher = commentPattern.matcher( term.getPrivateComment() );
            // Yes, it's in the private comment
            if ( matcher.find() )
            {
              increment( termIdentifiersToKeep, term.getIdentifier() );
            }
          }

          if ( StringUtils.isNotEmpty( term.getPublicComment() ) )
          {
            matcher = commentPattern.matcher( term.getPublicComment() );
            // Yes, it's in the public comment
            if ( matcher.find() )
            {
              increment( termIdentifiersToKeep, term.getIdentifier() );
            }
          }
        }
      }

      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "Potentially keeping " + termIdentifiersToKeep.size() + " terms of " + terms.size() + " terms." );
        if ( !termIdentifiersToKeep.isEmpty() )
        {
          String identifiers = StringUtils.join( termIdentifiersToKeep.keySet().iterator(), ", " );
          String counters = StringUtils.join( termIdentifiersToKeep.values().iterator(), ", " );
          LOG.debug( "Potentially keeping IDs " + identifiers + " with hit counters " + counters );
        }
      }
      for ( Iterator<AbstractTermVO> iterator = terms.iterator(); iterator.hasNext(); )
      {
        AbstractTermVO term = iterator.next();
        if ( termIdentifiersToKeep.keySet().contains( term.getIdentifier() ) )
        {
          // If the words are filtered using AND, the term must have been found
          // exactly the number of times that there are different words.
          int counter = termIdentifiersToKeep.get( term.getIdentifier() );
          if ( extendedSearch.compositionAnd && counter < searchTexts.length )
          {
            LOG.debug( "Term with ID " + term.getIdentifier() + " ('" + term.getTerm() + "') was found, but only " + counter + " words of "
                + searchTexts.length + " were found, removing..." );
            iterator.remove();
          }
        }
        else
        {
          LOG.debug( "Term with ID " + term.getIdentifier() + " ('" + term.getTerm() + "') wasn't found, removing..." );
          iterator.remove();
        }
      }
      LOG.debug( "Kept " + terms.size() + " terms." );
    }
  }

  /**
   * Increments the counter of term identifiers.
   * @param termIdentifiers The existing Map where the key is the term identifier and the value is a counter
   * @param termIdentifier The term identifier to increment
   */
  private static void increment( Map<Integer, Integer> termIdentifiers, Integer termIdentifier )
  {
    if ( termIdentifiers.containsKey( termIdentifier ) )
    {
      int counter = termIdentifiers.get( termIdentifier );
      termIdentifiers.put( termIdentifier, ++counter );
    }
    else
    {
      termIdentifiers.put( termIdentifier, 1 );
    }
  }

  /**
   * Gets the Integer value for a boolean.
   * @param booleanValue The boolean value
   * @return The Integer for the database
   */
  private static Integer getBooleanInteger( boolean booleanValue )
  {
    if ( booleanValue )
    {
      return ONE;
    }
    else
    {
      return ZERO;
    }
  }

  /**
   * Sorts terms by descending term length.
   */
  private class TermLengthComparator implements Comparator<AbstractTermVO>
  {
    // @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
    public int compare( AbstractTermVO term1, AbstractTermVO term2 )
    {
      int termLength1 = term1.getTerm().length();
      int termLength2 = term2.getTerm().length();
      if ( termLength1 > termLength2 )
      {
        return -1;
      }
      else if ( termLength1 < termLength2 )
      {
        return 1;
      }
      else
      {
        return 0;
      }
    }
  }

  @Override
  @SuppressWarnings( "unused" )
  public TermReplicationVO getReplicationTerms( String tableName )
  {
    throw new IllegalStateException( "Can't implement replication" );
  }

  /**
   * Contains the full SQL statement including the <code>WHERE</code> statement with its parameters.
   */
  class FullSqlStatement
  {
    /**
     * The <code>WHERE</code> statement and its parameters
     */
    public SQLWhereStatementCreator whereStatement;

    /**
     * The SQL statement
     */
    public SQLStatement<AbstractTermVO> sqlStatement;

    /**
     * Creates this with all data.
     * @param whereStatement The <code>WHERE</code> statement and its parameters
     * @param sqlStatement The SQL statement
     */
    public FullSqlStatement( SQLWhereStatementCreator whereStatement, SQLStatement<AbstractTermVO> sqlStatement )
    {
      this.whereStatement = whereStatement;
      this.sqlStatement = sqlStatement;
    }
  }
}
