package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.dbutils.sql.SQLWhereStatementCreator;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.TermReplicationVO;

/**
 * Accesses the table <code>TERM</code> for IPI term candidates.
 * <strong>Note that term candidates are never cached!</strong>
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: IpiTermCandidateDAO.java,v 1.39 2008/12/04 17:01:49 heinzer Exp $
 */
public class IpiTermCandidateDAO extends BaseTermDAO<IpiTermCandidateVO, Integer>
{
  /**
   * The SQL statement that activates a term
   */
  private static final SQLStatement<Integer> ACTIVATE_TERM = SQL.update( "UPDATE term SET term_type = ?, is_active_bool = ?, renewal_dat = ?,"
      + " changed_dat = ?, changed_user = ?, usage_count = ?, latest_occurrence_dat = ?, is_public_bool = ?, is_checker_only_bool = ? WHERE term_id = ?" );

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<IpiTermCandidateVO> getVOType()
  {
    return IpiTermCandidateVO.class;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermType()
  @Override
  protected TermType getTermType()
  {
    return TermType.IPI_TERM_CANDIDATE;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermTypeName()
  @Override
  protected String getTermTypeName()
  {
    return "IPI term candidate";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( IpiTermCandidateVO term )
  {
    return new Object[] { term.getDataSource(), term.getClassNumber(), null /* no term group */, TermType.IPI_TERM_CANDIDATE.toDB(),
        term.getLanguage().getLanguage(), term.getTerm(), term.getSearchTerm(), false /* is never generic */, term.isPublic(), term.getPrivateComment(),
        term.getPublicComment(), false /* never active */, null /* no renewal */, term.getUsageCount(), term.getLatestOccurrence(), term.getCreationUser(),
        term.getChangeUser(), term.getChangeDate(), term.isCheckerOnly() };
  }

  /**
   * Finds an IPI term candidate.
   * @param ipiTermCandidateId The unique identifier for an IPI term candidate
   * @return The IPI term candidate or <code>null</code> if it doesn't exist
   * @throws SQLException if the query fails
   */
  public IpiTermCandidateVO find( final Integer ipiTermCandidateId ) throws SQLException
  {
    return super.find( ipiTermCandidateId );
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public Integer insert( IpiTermCandidateVO term )
  {
    return super.insert( term, false );
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void update( IpiTermCandidateVO term )
  {
    super.update( term, false );
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#delete(java.lang.Integer)
  @Override
  public void delete( Integer termId )
  {
    super.delete( termId, false );
  }

  /**
   * Finds all term candidates of a language and class with term <code>searchTerm</code>.
   * <strong>Note that this is never cached!</strong>
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param searchTerm The exact term to search for
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<IpiTermCandidateVO> find( Locale language, Integer classNumber, String searchTerm ) throws SQLException
  {
    Check.notNull( language, "language" );
    Check.notNull( classNumber, "class number" );
    Check.notEmpty( searchTerm, "search term" );
    checkTermLength( searchTerm );

    SQLStatementCreator<IpiTermCandidateVO> creator = new SQLStatementCreator<IpiTermCandidateVO>( getTableName(), getIdColumnName(), getColumnNames(),
        getIdSequenceName(), getAliases() );
    SQLWhereStatementCreator whereStatement = createWhereStatementForFind( language, searchTerm, false );
    whereStatement.appendFragmentWithParameters( " AND class_id = ?", classNumber );
    SQLStatement<IpiTermCandidateVO> selectCandidates = creator.selectBy( whereStatement );
    return selectVOList( 50, IpiTermCandidateVO.class, selectCandidates, whereStatement.getStatementParameters() );
  }

  /**
   * Gets the terms candidates (uncached) for a given language ordered by descending usage count.
   * @param language The language, if <code>null</code> all languages are searched.
   * @return The term candidates that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<IpiTermCandidateVO> findAll( Locale language ) throws SQLException
  {
    SQLStatementCreator<IpiTermCandidateVO> creator = new SQLStatementCreator<IpiTermCandidateVO>( getTableName(), getIdColumnName(), getColumnNames(),
        getIdSequenceName(), getAliases() );

    SQLWhereStatementCreator whereStatement = createWhereStatementForFind( language, null, true );
    SQLStatement<IpiTermCandidateVO> selectCandidates = creator.selectBy( whereStatement );
    return selectVOList( 3000, IpiTermCandidateVO.class, selectCandidates, whereStatement.getStatementParameters() );
  }

  /**
   * Creates the <code>WHERE</code> statement for {@link #find(Locale, Integer, String)}.
   * @param language The language in which terms are searched for
   * @param searchTerm The exact term to search for
   * @param orderByUsageCount If this is <code>true</code>, the term candidates are ordered by descending usage count
   * @return The full <code>WHERE</code> statement
   */
  SQLWhereStatementCreator createWhereStatementForFind( Locale language, String searchTerm, boolean orderByUsageCount )
  {
    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    int numberOfParameters = 0;

    if ( language != null )
    {
      whereStatement.appendFragmentWithParameters( " language_sn = ?", language.getLanguage() );
      numberOfParameters++;
    }

    if ( StringUtils.isNotEmpty( searchTerm ) )
    {
      if ( numberOfParameters > 0 )
      {
        whereStatement.appendFragment( " AND" );
      }
      whereStatement.appendFragmentWithParameters( " term = ?", searchTerm );
      numberOfParameters++;
    }

    if ( numberOfParameters > 0 )
    {
      whereStatement.appendFragment( " AND" );
    }
    whereStatement.appendFragmentWithParameters( " term_type = ?", TermType.IPI_TERM_CANDIDATE.toDB() );

    if ( orderByUsageCount )
    {
      whereStatement.appendFragment( " ORDER BY usage_count DESC" );
    }

    return whereStatement;
  }

  /**
   * Activates a term candidate by transforming it into an IPI term.
   * @param candidate The candidate to set active. Only ID and change user is read from this.
   * @throws SQLException if the update fails
   */
  public void activate( final IpiTermCandidateVO candidate ) throws SQLException
  {
    Check.notNull( candidate, "IPI term candidate" );

    final Timestamp now = new Timestamp( System.currentTimeMillis() );
    SingleDAOUnitOfWork<Integer> singleDAOUnitOfWork = new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( ACTIVATE_TERM, TermType.IPI_TERM.toDB(), 1, now, now, candidate.getChangeUser(), null, null, 0, candidate.isCheckerOnly(),
            candidate.getIdentifier() );
        verifyUpdateCount( count, 1, "update failed" );
      }
    };
    singleDAOUnitOfWork.runWithException();
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getReplicationTerms(java.lang.String)
  @Override
  @SuppressWarnings( "unused" )
  public TermReplicationVO getReplicationTerms( String tableName )
  {
    throw new IllegalStateException( "Can't implement replication" );
  }
}
