package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.RandomStringUtils;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.dbutils.sql.SQLWhereStatementCreator;
import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.ParameterConstants;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.IpiTermCandidateVO;

/**
 * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO}. <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual
 * Property</p>
 * @author Eric Lewis
 * @version $Id: IpiTermCandidateDAOTest.java,v 1.26 2009/02/02 13:45:10 bandschapp Exp $
 */
public class IpiTermCandidateDAOTest extends BaseDAOWithCRUDTestTemplate<IpiTermCandidateDAO, IpiTermCandidateVO, Integer>
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public IpiTermCandidateDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<IpiTermCandidateDAO> getDAOClass()
  {
    return IpiTermCandidateDAO.class;
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#activate(IpiTermCandidateVO)}.
   * @throws Exception if the test fails
   */
  public void testActivate() throws Exception
  {
    IpiTermCandidateVO ipiTermCandidateVO = createAndInsert( true );
    setSavedId( null ); // we will take care of deletion

    // check precondition: corresponding term (term, language) does not already exist as active term
    GeneralTermDAO termDAO = DAOPool.get( GeneralTermDAO.class );
    List<AbstractTermVO> existingTerm = termDAO.findTermWithinActiveTerms( ipiTermCandidateVO.getLanguage(), 1, ipiTermCandidateVO.getTerm() );
    assertEquals( "Test term must not exist", 0, existingTerm.size() );

    IpiTermCandidateDAO candidateDAO = DAOPool.get( IpiTermCandidateDAO.class );
    candidateDAO.activate( ipiTermCandidateVO );
    existingTerm = termDAO.findTermWithinActiveTerms( ipiTermCandidateVO.getLanguage(), ipiTermCandidateVO.getClassNumber(), ipiTermCandidateVO.getTerm() );
    assertEquals( "Test term must be found as active", 1, existingTerm.size() );

    IpiTermDAO ipiTermDAO = DAOPool.get( IpiTermDAO.class );
    ipiTermDAO.delete( existingTerm.get( 0 ).getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    IpiTermCandidateVO ipiTermCandidateVO = createAndInsert( true );
    IpiTermCandidateVO expectedIpiTermCandidateVO = getSerializableVO();
    expectedIpiTermCandidateVO.setIdentifier( ipiTermCandidateVO.getIdentifier() );
    defaultTestRead( expectedIpiTermCandidateVO, -1, 999999999, ipiTermCandidateVO.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#findAll(java.util.Locale)}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    List<IpiTermCandidateVO> allCandidates = getDAO().findAll( null );
    int countBeforeInsert = allCandidates.size();
    IpiTermCandidateVO ipiTermCandidateVO = createAndInsert( true );
    int countAfterInsert = getDAO().findAll( null ).size();
    assertEquals( "count must have increased by one", countBeforeInsert + 1, countAfterInsert );

    allCandidates = getDAO().findAll( ipiTermCandidateVO.getLanguage() );
    assertTrue( "at least the inserted candidate must exist in the correct language", allCandidates.size() >= 1 );
    String insertedTerm = ipiTermCandidateVO.getTerm();
    Locale insertedLanguage = ipiTermCandidateVO.getLanguage();
    boolean found = false;

    int lastcount = Integer.MAX_VALUE;
    for ( IpiTermCandidateVO candidateVO: allCandidates )
    {
      if ( ( candidateVO.getTerm().equals( insertedTerm ) ) && ( candidateVO.getLanguage().equals( insertedLanguage ) ) )
      {
        found = true;
      }
      assertTrue( "usage count must not increase", candidateVO.getUsageCount() <= lastcount );
      lastcount = candidateVO.getUsageCount();
    }
    assertTrue( "inserted candidate must be found", found );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#find(Locale language, Integer classNumber, String searchTerm)}
   * with faults.
   * @throws Exception if the test fails
   */
  public void testFindByLanguageAndTermWithFaults() throws Exception
  {
    try
    {
      getDAO().find( null, 1, "hallo" );
      fail( "Null language must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().find( Locale.GERMAN, 1, null );
      fail( "Null search term must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().find( Locale.GERMAN, 1, "" );
      fail( "Empty search term must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().find( Locale.GERMAN, 1, RandomStringUtils.randomAlphabetic( ParameterConstants.EXCEEDED_LIMIT_FIELD_TERM ) );
      fail( "Too long search term must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#find(Locale language, Integer classNumber, String searchTerm)}.
   * @throws Exception if the test fails
   */
  public void testFindByLanguageAndTerm() throws Exception
  {
    IpiTermCandidateVO ipiTermCandidateVO = createAndInsert( true );
    // find in correct language - should hit
    List<IpiTermCandidateVO> candidates = getDAO()
        .find( getSerializableVO().getLanguage(), getSerializableVO().getClassNumber(), getSerializableVO().getTerm() );
    assertEquals( 1, candidates.size() );
    compareVOsAndValidateLoaded( ipiTermCandidateVO, candidates.get( 0 ) );

    // find in different language - should not hit
    Locale correctLanguage = getSerializableVO().getLanguage();
    Locale incorrectLanguage;

    if ( Locale.GERMAN == correctLanguage )
    {
      incorrectLanguage = Locale.FRENCH;
    }
    else
    {
      incorrectLanguage = Locale.GERMAN;
    }

    candidates = getDAO().find( incorrectLanguage, getSerializableVO().getClassNumber(), getSerializableVO().getTerm() );
    assertEquals( 0, candidates.size() );

    // find modified term - should not hit
    candidates = getDAO().find( getSerializableVO().getLanguage(), getSerializableVO().getClassNumber(), "bla" + getSerializableVO().getTerm() );
    assertEquals( 0, candidates.size() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#createWhereStatementForFind(Locale, String, boolean)}.
   * @throws Exception if the test fails
   */
  public void testCreateWhereStatementForFind() throws Exception
  {
    String expectedStatement = " WHERE language_sn = ? AND term = ? AND term_type = ?";
    SQLWhereStatementCreator actualStatement = getDAO().createWhereStatementForFind( Locale.ITALIAN, "hallo", false );
    GeneralTermDAOStatementTest.checkWhereStatement( actualStatement, expectedStatement, Locale.ITALIAN.getLanguage(), "hallo", TermType.IPI_TERM_CANDIDATE
        .toDB() );

    expectedStatement = " WHERE language_sn = ? AND term_type = ? ORDER BY usage_count DESC";
    actualStatement = getDAO().createWhereStatementForFind( Locale.ITALIAN, null, true );
    GeneralTermDAOStatementTest.checkWhereStatement( actualStatement, expectedStatement, Locale.ITALIAN.getLanguage(), TermType.IPI_TERM_CANDIDATE.toDB() );

    expectedStatement = " WHERE term_type = ? ORDER BY usage_count DESC";
    actualStatement = getDAO().createWhereStatementForFind( null, null, true );
    GeneralTermDAOStatementTest.checkWhereStatement( actualStatement, expectedStatement, TermType.IPI_TERM_CANDIDATE.toDB() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#insert(ch.ige.wdltool.vo.IpiTermCandidateVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#update(ch.ige.wdltool.vo.IpiTermCandidateVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermCandidateDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected IpiTermCandidateVO read( Integer ipiTermCandidateId ) throws SQLException
  {
    return getDAO().find( ipiTermCandidateId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO,
  // ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( IpiTermCandidateVO originalVO, IpiTermCandidateVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking class number", originalVO.getClassNumber(), loadedVO.getClassNumber() );
    assertEquals( "Checking language", originalVO.getLanguage(), loadedVO.getLanguage() );
    assertEquals( "Checking term", originalVO.getTerm(), loadedVO.getTerm() );
    assertEquals( "Checking private comment", originalVO.getPrivateComment(), loadedVO.getPrivateComment() );
    assertEquals( "Checking public comment", originalVO.getPublicComment(), loadedVO.getPublicComment() );
    assertEquals( "Checking change user", originalVO.getChangeUser(), loadedVO.getChangeUser() );
    assertEquals( "Checking creation user", originalVO.getCreationUser(), loadedVO.getCreationUser() );
    assertEquals( "Checking data source", originalVO.getDataSource(), loadedVO.getDataSource() );
    assertEquals( "Checking latest occurrence", originalVO.getLatestOccurrence(), loadedVO.getLatestOccurrence() );
    assertEquals( "Checking search term", originalVO.getSearchTerm(), loadedVO.getSearchTerm() );
    assertEquals( "Checking usage count", originalVO.getUsageCount(), loadedVO.getUsageCount() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( IpiTermCandidateVO originalVO, IpiTermCandidateVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#validateVO(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void validateVO( IpiTermCandidateVO loadedVO )
  {
    assertTrue( "Checking change date", loadedVO.getChangeDate().after( GeneralTermDAOTest.getLatestMigration() ) );
  }

  @Override
  public boolean assertVOEqual( IpiTermCandidateVO originalVO, IpiTermCandidateVO serializedVO )
  {
    return super.assertVOEqual( originalVO, serializedVO ) && ObjectUtils.equals( originalVO.getChangeDate(), serializedVO.getChangeDate() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( IpiTermCandidateVO vo )
  {
    // The insert must create this
    vo.setSearchTerm( null );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( IpiTermCandidateVO ipiTermCandidateVO )
  {
    return getDAO().insert( ipiTermCandidateVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOAfterInsert(ch.ige.dbutils.vo.AbstractVO,
  // ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void modifyVOAfterInsert( IpiTermCandidateVO ipiTermCandidateVO, IpiTermCandidateVO loadedIpiTermCandidateVO )
  {
    // This is the expected search term
    ipiTermCandidateVO.setSearchTerm( "dont use me" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void modifyVOBeforeUpdate( IpiTermCandidateVO ipiTermCandidateVO )
  {
    ipiTermCandidateVO.setClassNumber( 33 );
    ipiTermCandidateVO.setLanguage( Locale.ITALIAN );
    ipiTermCandidateVO.setTerm( "Use me" );
    ipiTermCandidateVO.setSearchTerm( null ); // The update must create this
    ipiTermCandidateVO.setPrivateComment( "The private comment, modified" );
    ipiTermCandidateVO.setPublicComment( "The public comment, modified" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( IpiTermCandidateVO ipiTermCandidateVO )
  {
    getDAO().update( ipiTermCandidateVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer ipiTermCandidateId )
  {
    getDAO().delete( ipiTermCandidateId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  public IpiTermCandidateVO getSerializableVO() throws SQLException
  {
    try
    {
      IpiTermCandidateVO ipiTermCandidate = new IpiTermCandidateVO();
      ipiTermCandidate.setIdentifier( 33 );
      ipiTermCandidate.setClassNumber( 42 );
      ipiTermCandidate.setLanguage( Locale.FRENCH );
      ipiTermCandidate.setTerm( "Dont use me" );
      ipiTermCandidate.setPrivateComment( "The private comment" );
      ipiTermCandidate.setPublicComment( "The public comment" );
      ipiTermCandidate.setChangeDate( DateConverter.stringToTimestamp( "03.02.2002 18:44:55.000" ) );
      ipiTermCandidate.setChangeUser( "billg" );
      ipiTermCandidate.setCreationUser( "steveb" );
      ipiTermCandidate.setLatestOccurrence( DateConverter.stringToTimestamp( "03.05.2005 13:22:11.000" ) );
      ipiTermCandidate.setSearchTerm( "dont use me" );
      ipiTermCandidate.setUsageCount( 4 );
      return ipiTermCandidate;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }
}
