package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.time.StopWatch;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.IpiTermVO;

/**
 * Tests {@link ch.ige.wdltool.dao.IpiTermDAO} for renewals and expirations.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: IpiTermDAOForRenewalTest.java,v 1.16 2008/04/29 16:38:12 heinzer Exp $
 */
public class IpiTermDAOForRenewalTest extends IpiTermDAOTest
{

  /**
   * DAO with methods that are only sensible for testing purpose.
   */
  @SuppressWarnings( "unchecked" )
  private class RenewalTestDAO extends AbstractWdlDAO
  {

    // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
    @Override
    protected String getColumnNames()
    {
      // not implemented
      return null;
    }

    // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
    @Override
    protected String getTableName()
    {
      // not implemented
      return null;
    }

    // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
    @Override
    protected Class getVOType()
    {
      // not implemented
      return null;
    }

    // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
    @Override
    @SuppressWarnings( "unused" )
    protected Object[] getValues( AbstractVO vo )
    {
      // not implemented
      return null;
    }

    /**
     * Sets the renewal date to an old date for all active IPI terms.
     * @return The number of terms affected
     */
    public Integer resetRenewalDate()
    {

      return this.execute( new SingleDAOUnitOfWork<Integer>()
      {
        // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
        public void execute( IGEQueryRunner runner ) throws SQLException
        {
          Integer count = runner.update( "update term set renewal_dat = to_date('2007-01-01','YYYY-MM-dd') where term_type = ? AND is_active_bool = 1",
              TermType.IPI_TERM.toDB() );
          this.setResult( count );
        }
      } );
    }

    /**
     * Sets the test db in a state ready for testing by deleting all terms with certain renewal date.
     * 
     * @param approximateTestdate Terms with renewal date near this date are deleted
     * @return The number of terms affected
     */
    public Integer cleanupTestDB( final Date approximateTestdate )
    {

      return this.execute( new SingleDAOUnitOfWork<Integer>()
      {
        // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
        public void execute( IGEQueryRunner runner ) throws SQLException
        {

          Calendar calendar = Calendar.getInstance();
          calendar.setTime( approximateTestdate );
          Integer count = runner.update( "delete from term where renewal_dat < to_date('" + ( calendar.get( Calendar.YEAR ) + 1 ) + "-01-01','YYYY-MM-dd')" );
          this.setResult( count );
        }
      } );
    }

    /**
     * Sets the renewal date for a term with given ID
     * @param renewalDate The renewal date to set
     * @param counter The amount of days to add to the <code>renewalDate</code>
     * @param insertedId The affected terms ID
     * @return The number of terms affected
     */
    public Integer setRenewalDate( final Date renewalDate, final int counter, final int insertedId )
    {

      return this.execute( new SingleDAOUnitOfWork<Integer>()
      {
        // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
        public void execute( IGEQueryRunner runner ) throws SQLException
        {
          SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd" );
          Calendar calendar = Calendar.getInstance();
          calendar.setTime( renewalDate );
          calendar.add( Calendar.DAY_OF_MONTH, counter - 2 );
          Integer count = runner.update( "update term set renewal_dat = to_date('" + sdf.format( calendar.getTime() ) + "','YYYY-MM-dd') where term_id = "
              + insertedId );
          if ( count != 1 )
          {
            fail( "Setup could not updated renewal of an inserted term" );
          }
          this.setResult( count );
        }
      } );

    }
  }

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( IpiTermDAOForRenewalTest.class );

  /**
   * The default expiration period (in days)
   */
  private static final int DEFAULT_VALIDITY_PERIOD = 10;

  /**
   * The default confirmation period (in days)
   */
  private static final int DEFAULT_CONFIRMATION_PERIOD = 20;

  /**
   * The IDs of the terms that were created for the test.
   * They are deleted during {@link #tearDown()}.
   */
  private List<Integer> createdTermIds = new ArrayList<Integer>();

  /**
   * The only constructor
   * @param name The test's name
   */
  public IpiTermDAOForRenewalTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    Timestamp antiqueDate = DateConverter.stringToTimestamp( "01.01.1977 03:00:34.512" ); //older than any real-live entries

    RenewalTestDAO renewalTestDAO = new RenewalTestDAO();
    int updatedTermsCount = renewalTestDAO.cleanupTestDB( antiqueDate );

    if ( updatedTermsCount > 0 )
    {
      LOG.debug( "had to clean up test-db: some existing terms with antique renewal date have been purged" );
    }

    for ( int i = 1; i <= ( DEFAULT_VALIDITY_PERIOD + DEFAULT_CONFIRMATION_PERIOD ); i++ )
    {
      IpiTermVO term = getSerializableVO();
      term.setTerm( term.getTerm() + " " + i );
      Integer insertedId = getDAO().insert( term );

      renewalTestDAO.setRenewalDate( antiqueDate, i, insertedId );
      this.createdTermIds.add( term.getIdentifier() );
    }

  }

  /**
   * Tests {@link IpiTermDAO#findInactiveTerms()}.
   * @throws Exception if the test fails
   */
  public void testFindInactiveTerms() throws Exception
  {
    List<IpiTermVO> inactiveTerms = getDAO().findInactiveTerms();
    int originalAmount = inactiveTerms.size();

    //insert one more inactive term - just to be sure
    IpiTermVO term = getSerializableVO();
    term.setTerm( term.getTerm() + " inactive" );
    term.setActive( false );
    getDAO().insert( term );
    this.createdTermIds.add( term.getIdentifier() );

    inactiveTerms = getDAO().findInactiveTerms();
    int changedAmount = inactiveTerms.size();

    assertEquals( originalAmount + 1, changedAmount );
  }

  /**
   * Tests {@link IpiTermDAO#findExpiredTerms()}.
   * @throws Exception if the test fails
   */
  public void testFindExpiredTerms() throws Exception
  {
    List<IpiTermVO> expiredTerms = getDAO().findExpiredTerms( DateConverter.stringToSqlDate( "01.01.1977" ), 1, 0 );
    assertTrue( "Checking before expiry", expiredTerms.isEmpty() );

    expiredTerms = getDAO().findExpiredTerms( DateConverter.stringToSqlDate( "02.01.1977" ), 1, 0 );
    checkTerms( expiredTerms, "Renewal 1" );

    expiredTerms = getDAO().findExpiredTerms( DateConverter.stringToSqlDate( "03.01.1977" ), 1, 0 );
    checkTerms( expiredTerms, "Renewal 1", "Renewal 2" );

    expiredTerms = getDAO().findExpiredTerms( DateConverter.stringToSqlDate( "03.01.1977" ), 2, 0 );
    checkTerms( expiredTerms, "Renewal 1" );

    expiredTerms = getDAO().findExpiredTerms( DateConverter.stringToSqlDate( "09.01.1977" ), 4, 1 );
    checkTerms( expiredTerms, "Renewal 1", "Renewal 2", "Renewal 3", "Renewal 4" );

    Calendar counterCalendar = Calendar.getInstance();
    counterCalendar.setTime( DateConverter.stringToDate( "01.01.1977" ) );

    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "No term should be expired on this date", expiredTerms.isEmpty() );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 11 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "No term should be expired after 11 days", expiredTerms.isEmpty() );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 11 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "No term should be expired after 23 days", expiredTerms.isEmpty() );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 7 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "No term should be expired after 30 days", expiredTerms.isEmpty() );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "1 term should be expired after 31 days, not " + expiredTerms.size(), expiredTerms.size() == 1 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 3 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "4 terms should be expired after 34 days, not " + expiredTerms.size(), expiredTerms.size() == 4 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 11 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "15 terms should be expired after 45 days, not " + expiredTerms.size(), expiredTerms.size() == 15 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 9 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "24 terms should be expired after 54 days, not " + expiredTerms.size(), expiredTerms.size() == 24 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 5 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "29 terms should be expired after 59 days, not " + expiredTerms.size(), expiredTerms.size() == 29 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "30 terms should be expired after 60 days, not " + expiredTerms.size(), expiredTerms.size() == 30 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 100 );
    expiredTerms = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "30 terms should be expired after 160 days, not " + expiredTerms.size(), expiredTerms.size() == 30 );

  }

  /**
   * Tests {@link IpiTermDAO#findExpiringTerms()}.
   * @throws Exception if the test fails
   */
  public void testfindExpiringTerms() throws Exception
  {
    List<IpiTermVO> expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "03.01.1977" ), 3, 1 );
    assertTrue( "None should have reached the confirmation start", expiringTerms.isEmpty() );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "08.01.1977" ), 3, 1 );
    checkTerms( expiringTerms, "Renewal 5" );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "08.01.1977" ), 3, 2 );
    checkTerms( expiringTerms, "Renewal 4", "Renewal 5" );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "02.01.1975" ), 50, 25 );
    assertTrue( "The terms do not exist yet, ergo the result must be empty", expiringTerms.isEmpty() );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "03.01.1977" ), 1, 0 );
    assertTrue( "Giving zero confirmation time, so should have no result", expiringTerms.isEmpty() );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "03.01.1977" ), 0, 1 );
    checkTerms( expiringTerms, "Renewal 3" );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "04.01.1977" ), 1, 2 );
    checkTerms( expiringTerms, "Renewal 2", "Renewal 3" );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "05.01.1977" ), 1, 2 );
    checkTerms( expiringTerms, "Renewal 3", "Renewal 4" );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "05.01.1977" ), 1, 10 );
    checkTerms( expiringTerms, "Renewal 1", "Renewal 2", "Renewal 3", "Renewal 4" );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.stringToSqlDate( "10.01.2100" ), 10, 10 );
    assertTrue( "All terms should be expired so far away in the future", expiringTerms.isEmpty() );

    Calendar counterCalendar = Calendar.getInstance();
    counterCalendar.setTime( DateConverter.stringToDate( "01.01.1977" ) );

    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "No term should be going to expire on this date", expiringTerms.isEmpty() );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 9 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "No term should be going to expire on this date", expiringTerms.isEmpty() );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "1 term should be going to expire after 10 days, not " + expiringTerms.size(), expiringTerms.size() == 1 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "2 terms should be going to expire after 11 days, not " + expiringTerms.size(), expiringTerms.size() == 2 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 18 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "20 terms should be going to expire after 29 days, not " + expiringTerms.size(), expiringTerms.size() == 20 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "20 terms should be going to expire after 30 days, not " + expiringTerms.size(), expiringTerms.size() == 20 );

    //Always 20 between these two

    counterCalendar.add( Calendar.DAY_OF_MONTH, 9 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "20 terms should be going to expire after 39 days, not " + expiringTerms.size(), expiringTerms.size() == 20 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "19 terms should be going to expire after 40 days, not " + expiringTerms.size(), expiringTerms.size() == 19 );

    //and counting backwards now between these two

    counterCalendar.add( Calendar.DAY_OF_MONTH, 18 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "1 term should be going to expire after 58 days, not " + expiringTerms.size(), expiringTerms.size() == 1 );

    counterCalendar.add( Calendar.DAY_OF_MONTH, 1 );
    expiringTerms = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( counterCalendar.getTime() ), DEFAULT_VALIDITY_PERIOD, DEFAULT_CONFIRMATION_PERIOD );
    assertTrue( "All terms should be expired by now, none should be expiring", expiringTerms.isEmpty() );

  }

  /**
   * Tests {@link IpiTermDAO#renewTerms( java.util.List )} with expiring terms.
   * @throws Exception if the test fails
   */
  public void testRenewTerms() throws Exception
  {
    java.sql.Date oldTestDate = DateConverter.stringToSqlDate( "05.01.1977" );
    Calendar renewalCal = Calendar.getInstance();
    renewalCal.setTime( DateConverter.stringToDate( "01.01.1980" ) );

    //The first list
    List<IpiTermVO> expiringTerms1 = getDAO().findExpiringTerms( oldTestDate, 1, 2 );
    checkTerms( expiringTerms1, "Renewal 3", "Renewal 4" );
    //renew the terms from the first list
    getDAO().renewTerms( renewalCal.getTime(), expiringTerms1 );
    //Same finder as for the first list in the second list, should be empty now.
    List<IpiTermVO> expringTerms2 = getDAO().findExpiringTerms( oldTestDate, 1, 2 );
    assertTrue( "List of expiring terms must be empty after update, No of terms: " + expringTerms2.size(), expringTerms2.isEmpty() );
    //We check the results from the first old list to the changed values, whether correct
    verifyUpdatedTerms( renewalCal.getTime(), expiringTerms1 );

    //Now we add 10 days
    renewalCal.add( Calendar.DAY_OF_MONTH, 10 );
    //Now for the third list we try to find the updated results
    List<IpiTermVO> expiringTerms3 = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( renewalCal.getTime() ), 1, 10 );
    checkTerms( expiringTerms3, "Renewal 3", "Renewal 4" );
    //we renew the same terms again
    getDAO().renewTerms( renewalCal.getTime(), expiringTerms3 );
    //Same finder as for the third list in the forth list, should be empty now.
    List<IpiTermVO> expiringTerms4 = getDAO().findExpiringTerms( DateConverter.dateToSqlDate( renewalCal.getTime() ), 1, 10 );
    assertTrue( "List of expring terms must be empty after update, No of terms: " + expiringTerms4.size(), expiringTerms4.isEmpty() );
    //We check the results from the third old list to the changed values, whether correct
    verifyUpdatedTerms( renewalCal.getTime(), expiringTerms3 );
  }

  /**
   * Tests {@link IpiTermDAO#renewTerms( java.util.List )} with expiring terms with a collision.
   * @throws Exception if the test fails
   */
  public void testRenewTermsWithCollision() throws Exception
  {
    java.sql.Date oldTestDate = DateConverter.stringToSqlDate( "05.01.1977" );
    Calendar renewalCal = Calendar.getInstance();
    renewalCal.setTime( DateConverter.stringToDate( "01.01.1980" ) );

    //Client 1: The first list - received before the update
    List<IpiTermVO> expiringTerms1 = getDAO().findExpiringTerms( oldTestDate, 1, 2 );
    checkTerms( expiringTerms1, "Renewal 3", "Renewal 4" );

    //Client 2: updating a term from the list in the meantime
    IpiTermVO renewal3Term = expiringTerms1.get( 0 );
    IpiTermVO client2Term = new IpiTermVO( renewal3Term ); //client 2 works with its own copy of the term
    client2Term.setTerm( "Renewal 3 changed" );
    getDAO().update( client2Term );

    //Client 1: tries to renew the terms from the first list. This should provoke a collision because its local copy of the term has still the old change date
    List<Integer> collisions = getDAO().renewTerms( renewalCal.getTime(), expiringTerms1 );
    assertEquals( "One Collision must be detected", 1, collisions.size() );
    assertEquals( "The ID of the collided term must be correct", renewal3Term.getIdentifier(), collisions.get( 0 ) );
  }

  /**
   * Tests the performance of {@link IpiTermDAO#renewTerms( java.util.List )}.
   * <strong>Warning: this resets the renewal date of all existing IPI terms!</strong>
   * @throws Exception
   */
  public void checkRenewTermsPerformance() throws Exception
  {
    RenewalTestDAO renewalTestDAO = new RenewalTestDAO();

    int updatedTermsCount = renewalTestDAO.resetRenewalDate();

    java.sql.Date newTestDate = DateConverter.stringToSqlDate( "01.05.2007" );
    List<IpiTermVO> expiredTerms = getDAO().findExpiredTerms( newTestDate, 1, 2 );
    assertEquals( "All terms should be expired ", updatedTermsCount, expiredTerms.size() );

    StopWatch stopWatch = new StopWatch();
    stopWatch.reset();
    stopWatch.start();
    getDAO().renewTerms( expiredTerms );
    stopWatch.stop();

    float millisPerTerm = ( (float) stopWatch.getTime() ) / updatedTermsCount;
    //print out details about time consumption 
    System.out.println( "renewal time was " + stopWatch.getTime() / 1000 + " seconds. That is " + millisPerTerm + " milliseconds per term" );
    assertTrue( "renewal time per term should be below 10ms", millisPerTerm < 10 );

    expiredTerms = getDAO().findExpiredTerms( newTestDate, 1, 2 );
    assertEquals( "All terms should have been renewed ", 0, expiredTerms.size() );
  }

  /**
   * Tests {@link IpiTermDAO#renewTerms( java.util.List )} with terms which are expired (reactivate).
   * @throws Exception if the test fails
   */
  public void testReactivateExpiredTerms() throws Exception
  {
    Calendar calendar1 = Calendar.getInstance();
    calendar1.setTime( DateConverter.stringToDate( "07.01.1977" ) );

    Calendar calendar2 = Calendar.getInstance();
    calendar2.setTime( DateConverter.stringToDate( "08.07.1977" ) );

    //The first list
    List<IpiTermVO> expiredTerms1 = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( calendar1.getTime() ), 5, 0 );
    checkTerms( expiredTerms1, "Renewal 1", "Renewal 2" );
    //renew the terms from the first list with the date of the first calendar
    getDAO().renewTerms( calendar1.getTime(), expiredTerms1 );
    //Same finder as for the first list in the second list, should be empty now.
    List<IpiTermVO> expiredTerms2 = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( calendar1.getTime() ), 5, 0 );
    assertTrue( "List of expired terms must be empty after update, No of terms: " + expiredTerms2.size(), expiredTerms2.isEmpty() );
    //We check the results from the first old list to the changed values, whether correct
    verifyUpdatedTerms( calendar1.getTime(), expiredTerms1 );

    //Now we add 2 days on the first calendar
    calendar1.add( Calendar.DAY_OF_MONTH, 2 );
    //Now for the third list we try to find the updated results, others should not be expired by then
    List<IpiTermVO> expiredTerms3 = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( calendar1.getTime() ), 5, 0 );
    checkTerms( expiredTerms3, "Renewal 3", "Renewal 4" );
    //we renew the same terms again
    getDAO().renewTerms( calendar2.getTime(), expiredTerms3 );
    //Same finder as for the third list in the forth list, should be empty now.
    List<IpiTermVO> expiredTerms4 = getDAO().findExpiredTerms( DateConverter.dateToSqlDate( calendar1.getTime() ), 5, 0 );
    assertTrue( "List of expired terms must be empty after update, No of terms: " + expiredTerms4.size(), expiredTerms4.isEmpty() );
    //We check the results from the third old list to the changed values, whether correct
    verifyUpdatedTerms( calendar2.getTime(), expiredTerms3 );
  }

  /**
   * Verifies the data in the database and compares it with the terms before the update.
   * @param dateOfRenewal The renewal date
   * @param termsBeforeUpdate The terms before the update happened
   * @throws SQLException if the query fails
   */
  private void verifyUpdatedTerms( Date dateOfRenewal, List<IpiTermVO> termsBeforeUpdate ) throws SQLException
  {
    Calendar expectedDateCal = Calendar.getInstance();
    expectedDateCal.setTime( dateOfRenewal );

    Calendar actualDate = Calendar.getInstance();
    actualDate.setTime( new java.util.Date() );

    Calendar renewalDateCal = Calendar.getInstance();
    Calendar changeDateCal = Calendar.getInstance();

    for ( IpiTermVO termVO: termsBeforeUpdate )
    {
      IpiTermVO ipiTermVOToTest = read( termVO.getIdentifier() );
      renewalDateCal.setTime( new java.util.Date( ipiTermVOToTest.getRenewal().getTime() ) );
      changeDateCal.setTime( new java.util.Date( ipiTermVOToTest.getChangeDate().getTime() ) );

      //These values should have changed and are known 
      assertEquals( expectedDateCal.get( Calendar.DAY_OF_MONTH ), renewalDateCal.get( Calendar.DAY_OF_MONTH ) );
      assertEquals( expectedDateCal.get( Calendar.MONTH ), renewalDateCal.get( Calendar.MONTH ) );
      assertEquals( expectedDateCal.get( Calendar.YEAR ), renewalDateCal.get( Calendar.YEAR ) );
      assertEquals( true, ipiTermVOToTest.isActive() );
      assertEquals( actualDate.get( Calendar.DAY_OF_MONTH ), changeDateCal.get( Calendar.DAY_OF_MONTH ) );
      assertEquals( actualDate.get( Calendar.MONTH ), changeDateCal.get( Calendar.MONTH ) );
      assertEquals( actualDate.get( Calendar.YEAR ), changeDateCal.get( Calendar.YEAR ) );
      assertEquals( "test", ipiTermVOToTest.getChangeUser() );
      assertNull( "Expiration date should now be null", ipiTermVOToTest.getExpirationDate() );

      //And the rest should still be the same
      assertEquals( "Identifier should match", termVO.getIdentifier(), ipiTermVOToTest.getIdentifier() );
      assertEquals( "ClassNumber should match", termVO.getClassNumber(), ipiTermVOToTest.getClassNumber() );
      assertEquals( "isPublic should match", termVO.isPublic(), ipiTermVOToTest.isPublic() );
      assertEquals( "CreationUser should match", termVO.getCreationUser(), ipiTermVOToTest.getCreationUser() );
      assertEquals( "DataSource should match", termVO.getDataSource(), ipiTermVOToTest.getDataSource() );
      assertEquals( "IpiTermGroupNumber should match", termVO.getIpiTermGroupNumber(), ipiTermVOToTest.getIpiTermGroupNumber() );
      assertEquals( "Language should match", termVO.getLanguage(), ipiTermVOToTest.getLanguage() );
      assertEquals( "LatestOccurrence should match", termVO.getLatestOccurrence(), ipiTermVOToTest.getLatestOccurrence() );
      assertEquals( "PrivateComment should match", termVO.getPrivateComment(), ipiTermVOToTest.getPrivateComment() );
      assertEquals( "PublicComment should match", termVO.getPublicComment(), ipiTermVOToTest.getPublicComment() );
      assertEquals( "SearchTerm should match", termVO.getSearchTerm(), ipiTermVOToTest.getSearchTerm() );
      assertEquals( "Term should match", termVO.getTerm(), ipiTermVOToTest.getTerm() );
      assertEquals( "TermGroupId should match", termVO.getTermGroupId(), ipiTermVOToTest.getTermGroupId() );
      assertEquals( "TermGroupTerms should match", termVO.getTermGroupTerms(), ipiTermVOToTest.getTermGroupTerms() );
      assertEquals( "UsageCount should match", termVO.getUsageCount(), ipiTermVOToTest.getUsageCount() );
    }
  }

  // @see ch.ige.wdltool.dao.AbstractGeneralTermDAOTestTemplate#getSerializableVO()
  @Override
  public IpiTermVO getSerializableVO() throws SQLException
  {
    try
    {
      Timestamp renewalDate = DateConverter.stringToTimestamp( "01.01.1977 03:00:34.512" );

      IpiTermVO ipiTerm = new IpiTermVO();
      ipiTerm.setIdentifier( 33 );
      ipiTerm.setActive( true );
      ipiTerm.setChangeDate( renewalDate );
      ipiTerm.setChangeUser( "test" );
      ipiTerm.setClassNumber( 33 );
      ipiTerm.setCreationUser( "test" );
      ipiTerm.setLanguage( Locale.GERMAN );
      ipiTerm.setLatestOccurrence( renewalDate );
      ipiTerm.setPrivateComment( "Private" );
      ipiTerm.setPublicComment( "Public" );
      ipiTerm.setPublic( false );
      ipiTerm.setRenewal( renewalDate );
      ipiTerm.setTerm( "Renewal" );
      ipiTerm.setSearchTerm( "renewal" );
      ipiTerm.setUsageCount( 5 );
      return ipiTerm;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  /**
   * Checks the values of terms.
   * @param terms The received terms
   * @param expectedTerms The expected term values
   */
  private void checkTerms( List<IpiTermVO> terms, String... expectedTerms )
  {
    Collections.sort( terms, new TermTextComparator() );
    assertEquals( "Checking the number of terms", expectedTerms.length, terms.size() );
    for ( int i = 0; i < expectedTerms.length; i++ )
    {
      String expectedTerm = expectedTerms[i];
      IpiTermVO expiredTerm = terms.get( i );
      assertEquals( "Checking term", expectedTerm, expiredTerm.getTerm() );
      assertNotNull( "Checking expiration date", expiredTerm.getExpirationDate() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    DAOPool.get( GeneralTermDAO.class ).delete( this.createdTermIds );
    super.tearDown();
  }

  /**
   * Sorts terms by descending term length.
   */
  class TermTextComparator implements Comparator<AbstractTermVO>
  {
    // @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
    public int compare( AbstractTermVO term1, AbstractTermVO term2 )
    {
      return term1.getTerm().compareTo( term2.getTerm() );
    }
  }
}
