package ch.ige.wdltool.dao;

import java.io.ObjectStreamException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.dbutils.handlers.VOListHandler;
import ch.ige.dbutils.handlers.assigners.FieldAssigner;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.wdltool.enums.TermType;

/**
 * Converts the result of a query to a <code>List</code> of term value objects.
 * This is required because the results can be different VO types.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @param <T> The VO type
 * @author   Eric Lewis
 * @version  $Id: TermVOListHandler.java,v 1.5 2007/09/14 06:56:09 lewis_lee Exp $
 */
public class TermVOListHandler<T extends AbstractVO> extends VOListHandler<T>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( TermVOListHandler.class );

  /**
   * Creates the handler.
   * 
   * @param type The class of the returned value objects
   * @param expectedSize The number of value objects to expect from the query
   */
  public TermVOListHandler( Class<T> type, int expectedSize )
  {
    super( type, expectedSize );
  }

  /**
   * Creates the handler.
   * 
   * @param type The class of the returned value objects
   * @param nullReplacementsByFieldName The <code>Map</code> to replace <code>null</code> values in the VO. The key
   *          is the field name (a String) and the value is an object to set instead of <code>null</code>
   * @param expectedSize The number of value objects to expect from the query
   */
  public TermVOListHandler( Class<T> type, Map<String, Object> nullReplacementsByFieldName, int expectedSize )
  {
    super( type, nullReplacementsByFieldName, expectedSize );
  }

  /**
   * Creates the VO and fills it using the result set and the field assigners.
   * 
   * @param resultSet The result of the query
   * @param fieldAssigners A <code>List</code> of {@link ch.ige.dbutils.handlers.assigners.FieldAssigner} objects to
   *          use for transferring column values into VO fields
   * @return The new VO with its fields set from the result set
   * @throws SQLException if the result set can't be read
   */
  @SuppressWarnings( "unchecked" )
  protected T createVO( ResultSet resultSet, List<? extends FieldAssigner<T, ?>> fieldAssigners ) throws SQLException
  {
    TermType termType = TermType.parse( resultSet.getInt( "term_type" ) );
    T vo = this.newInstance( termType );
    for ( FieldAssigner assigner: fieldAssigners )
    {
      assigner.assign( vo, resultSet );
    }
    try
    {
      // this is unchecked. Is this signature "Object readResolve" needed by the Serialization system, or would
      // "AbstractVO readResolve" work too???
      return (T) vo.readResolve();
    }
    catch ( ObjectStreamException e )
    {
      SQLException sex = new SQLException( "exception when calling readResolve of " + vo );
      sex.initCause( e );
      throw sex;
    }
  }

  /**
   * Creates a new instance of the value object.
   * 
   * @param termType The term type to create
   * @throws SQLException if the VO can't be created
   * @return a value object of the class given when this was constructed
   */
  @SuppressWarnings( "unchecked" )
  protected T newInstance( TermType termType ) throws SQLException
  {
    try
    {
      return (T) termType.getTermClass().newInstance();
    }
    catch ( InstantiationException e )
    {
      LOG.error( e );
      SQLException sex = new SQLException( "VO InstantiationException: " + e.getMessage() );
      sex.initCause( e );
      throw sex;
    }
    catch ( IllegalAccessException e )
    {
      LOG.error( e );
      SQLException sqlException = new SQLException( e.getMessage() );
      sqlException.initCause( e );
      throw sqlException;
    }
  }

}
