package ch.ige.wdltool.ejb;

import java.io.Serializable;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Locale;
import java.util.Map;

import org.hibernate.validator.InvalidValue;

import ch.ige.dbutils.dao.DAO;
import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WdlRemoveException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedIllegalArgumentException;
import ch.ige.wdltool.validator.ValidatorTestUtils;
import ch.ige.wdltool.vo.AbstractTermVO;

/**
 * The base class for a service that implements the normal Create, Read, Update and Delete (CRUD) methods.
 * This service must be based on a DAO.
 * 
 * @param <D> The DAO type
 * @param <V> The VO type
 * @param <I> The ID type, which identifies a VO
 * @param <S> The service type
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: BaseServiceTestTemplate.java,v 1.17 2008/04/29 16:38:12 heinzer Exp $
 */
public abstract class BaseServiceTestTemplate<D extends DAO, V extends AbstractVO & IdentifiableVO<I>, I extends Serializable, S extends Object> extends
    BaseDAOWithCRUDTestTemplate<D, V, I>
{
  /**
   * The normal constructor.
   * 
   * @param name The test's name
   */
  public BaseServiceTestTemplate( String name )
  {
    super( name );
  }

  /**
   * Gets the Services containing the business logic.
   * @return The Services which are tested
   */
  protected abstract S getServices();

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected final V read( I id ) throws SQLException
  {
    try
    {
      return readWithService( id );
    }
    catch ( Exception e )
    {
      throw wrapException( e );
    }
  }

  /**
   * Reads the value object from the database.
   * 
   * @param id The value object's unique ID used for reading
   * @return The value object from the database
   * @throws Exception if the read fails
   */
  @SuppressWarnings( "unused" )
  public V readWithService( I id ) throws Exception
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  protected final Collection<V> read( Object... searchParameters ) throws SQLException
  {
    try
    {
      return readWithService( searchParameters );
    }
    catch ( Exception e )
    {
      throw wrapException( e );
    }
  }

  /**
   * Reads value objects from the database.
   * 
   * @param searchParameters The parameters for searching a collection of value objects
   * @return The value objects from the database
   * @throws Exception if the read fails
   */
  @SuppressWarnings( "unused" )
  public Collection<V> readWithService( Object... searchParameters ) throws Exception
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  protected final I insert( V vo ) throws SQLException
  {
    try
    {
      return insertWithService( vo );
    }
    catch ( Exception e )
    {
      throw wrapException( e );
    }
  }

  /**
   * Inserts the value object into the database.
   * 
   * @param vo The value object to insert
   * @return The unique ID of the inserted value object
   * @throws Exception if the insert fails
   */
  @SuppressWarnings( "unused" )
  public I insertWithService( V vo ) throws Exception
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void modifyVOBeforeUpdate( V vo )
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  protected final void update( V vo ) throws SQLException
  {
    try
    {
      updateWithService( vo );
    }
    catch ( Exception e )
    {
      throw wrapException( e );
    }
  }

  /**
   * Updates the value object.
   * 
   * @param vo The value object to update
   * @throws Exception if the update fails
   */
  @SuppressWarnings( "unused" )
  public void updateWithService( V vo ) throws Exception
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  protected final void delete( I id ) throws SQLException
  {
    try
    {
      deleteWithService( id );
    }
    catch ( Exception e )
    {
      throw wrapException( e );
    }
  }

  /**
   * Deleted the value object from the database.
   * 
   * @param id The value object's unique ID used for reading
   * @throws Exception if the delete fails
   */
  @SuppressWarnings( "unused" )
  public void deleteWithService( I id ) throws Exception
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  /**
   * Tries to read using an invalid ID, tries to read an unknown value object, then reads
   * a known value object and compares it to <code>expectedVO</code> using {@link #compareVOsAndValidateLoaded(AbstractVO, AbstractVO)}.
   * 
   * @param expectedVO The value object as it should be
   * @param invalidId An ID that's considered to be invalid, for instance a negative ID
   * @param knownId An ID that's found in the database
   * @param unknownId An ID that's not found in the database
   * @throws SQLException if the read fails
   * @see #read(Serializable)
   * @see #compareVOsAndValidateLoaded(AbstractVO, AbstractVO)
   */
  protected void defaultTestRead( V expectedVO, I invalidId, I unknownId, I knownId ) throws SQLException
  {
    try
    {
      read( (I) null );
      fail( "Reading with null ID must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      read( invalidId );
      fail( "Reading with invalid ID " + invalidId + " must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      read( unknownId );
      fail( "Reading with unknown ID " + unknownId + " must fail" );
    }
    catch ( SQLException e )
    {
      WdlObjectNotFoundException causingException = checkCausingException( e, WdlObjectNotFoundException.class );
      assertEquals( "Checking ID", unknownId, causingException.getIdentifier() );
    }

    V loadedVO = read( knownId );
    compareVOsAndValidateLoaded( expectedVO, loadedVO );
  }

  /**
   * Tries to insert a <code>null</code> value object, then inserts a valid value object, reloads it from the
   * database, lets the test modify it and then compares the original value object with the reloaded value object.
   * 
   * @param checkForInsertedId If this is <code>true</code>, the DAO must have inserted the new ID into the VO,
   *        and this is checked; if this is <code>false</code>, the ID returned from {@link #insert(AbstractVO)}
   *        is inserted into the VO using {@link ch.ige.dbutils.vo.IdentifiableVO#setIdentifier(Serializable)} 
   * @throws SQLException if the query fails
   * @see #insert(AbstractVO)
   * @see #createAndInsert()
   * @see #read(Serializable)
   * @see #modifyVOAfterInsert(AbstractVO, AbstractVO)
   * @see #compareVOsAndValidateLoaded(AbstractVO, AbstractVO)
   */
  protected void defaultTestCreate( boolean checkForInsertedId ) throws SQLException
  {
    try
    {
      insert( null );
      fail( "Null value object must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      V vo = getSerializableVO();
      modifyVOBeforeInsert( vo );
      checkInvalidMemberVariables( vo, false );
    }
    catch ( Exception e )
    {
      checkCausingException( e, WdlValidationException.class );
    }

    V vo = createAndInsert( checkForInsertedId );
    V loadedVO = read( getSavedId() );
    modifyVOAfterInsert( vo, loadedVO );
    compareVOsAndValidateLoaded( vo, loadedVO );
  }

  /**
   * Creates, modifies and inserts a value object, sets or checks for its ID and returns it.
   * 
   * @param checkForInsertedId If this is <code>true</code>, the DAO must have inserted the new ID into the VO,
   *        and this is checked; if this is <code>false</code>, the ID returned from {@link #insert(AbstractVO)}
   *        is inserted into the VO using {@link ch.ige.dbutils.vo.IdentifiableVO#setIdentifier(Serializable)} 
   * @return The created value object, reloaded from the database
   * @throws SQLException if the insert fails
   * @see #getSerializableVO()
   * @see #modifyVOBeforeInsert(AbstractVO)
   * @see #insert(AbstractVO)
   * @see ch.ige.dbutils.vo.IdentifiableVO#setIdentifier(Serializable)
   */
  protected V createAndInsert( boolean checkForInsertedId ) throws SQLException
  {
    V vo = getSerializableVO();
    modifyVOBeforeInsert( vo );
    if ( checkForInsertedId )
    {
      vo.setIdentifier( null );
    }
    setSavedId( insert( vo ) );
    if ( checkForInsertedId )
    {
      assertNotNull( "Checking that the ID has been set in the VO", vo.getIdentifier() );
      assertEquals( "Checking that the ID in the VO is the same as the returned ID", getSavedId(), vo.getIdentifier() );
    }
    else
    {
      vo.setIdentifier( getSavedId() );
    }
    return vo;
  }

  /**
   * Tries to insert a term when a stopword containing the same text already exists.
   * @param language The language for the existing stopword and active term; since the value object is
   *        created by default, it may contain a different language, so this overrides the default
   * @param existingStopword The text for an already existing stopword
   * @throws SQLException if the query fails
   */
  protected void defaultTestCreateTermWithExistingStopword( Locale language, String existingStopword ) throws SQLException
  {
    V vo = getSerializableVO();
    AbstractTermVO term = (AbstractTermVO) vo;
    String termBackup = term.getTerm();

    try
    {
      modifyVOBeforeInsert( vo );
      term.setTerm( existingStopword );
      term.setLanguage( language );
      insert( vo );
      fail( "Stopword as term must fail" );
    }
    catch ( Exception e )
    {
      checkExistingTermException( e );
    }
    finally
    {
      term.setTerm( termBackup );
    }
  }

  /**
   * Tries to update a <code>null</code> value object, then inserts a valid value object, reloads it from the
   * database, lets the test modify it and then compares the original value object with the reloaded value object.
   * 
   * @throws SQLException if the query fails
   * @see #update(AbstractVO)
   * @see #createAndInsert()
   * @see #read(Serializable)
   * @see #modifyVOAfterInsert(AbstractVO, AbstractVO)
   * @see #modifyVOBeforeUpdate(AbstractVO)
   * @see #update(AbstractVO)
   * @see #modifyVOAfterUpdate(AbstractVO, AbstractVO)
   * @see #compareVOsAndValidateLoaded(AbstractVO, AbstractVO)
   */
  protected void defaultTestUpdate() throws SQLException
  {
    try
    {
      update( null );
      fail( "Null value object must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      V vo = getSerializableVO();
      modifyVOBeforeUpdate( vo );
      checkInvalidMemberVariables( vo, true );
    }
    catch ( Exception e )
    {
      checkCausingException( e, WdlValidationException.class );
    }

    V vo = createAndInsert( true );
    V loadedVO = read( getSavedId() );
    modifyVOAfterInsert( vo, loadedVO );
    modifyVOBeforeUpdate( vo );
    update( vo );
    V reloadedVO = read( getSavedId() );
    modifyVOAfterUpdate( vo, reloadedVO );
    compareVOsAndValidateLoaded( vo, reloadedVO );
  }

  /**
   * Tries to update a term when a stopword containing the same text already exists.
   * @param language The language for the existing stopword; since updating usually also changes
   *        the term's language, it must be reset
   * @param existingStopword The text for an already existing stopword
   * @throws SQLException if the query fails
   */
  protected void defaultTestUpdateTermWithExistingStopword( Locale language, String existingStopword ) throws SQLException
  {
    V vo = createAndInsert( true );
    V loadedVO = read( getSavedId() );
    AbstractTermVO term = (AbstractTermVO) vo;
    String termBackup = term.getTerm();

    try
    {
      modifyVOAfterInsert( vo, loadedVO );
      modifyVOBeforeUpdate( vo );
      term.setTerm( existingStopword );
      term.setLanguage( language );
      update( vo );
      fail( "Stopword as term must fail" );
    }
    catch ( Exception e )
    {
      checkExistingTermException( e );
    }
    finally
    {
      term.setTerm( termBackup );
    }
  }

  /**
   * Tries to update a <code>null</code> value object, then updates an existing value object, reloads it from the
   * database and then compares the original value object with the reloaded value object.
   * @param vo An existing value object, which <strong>is not deleted after the test</strong>
   * 
   * @throws SQLException if the query fails
   * @see #update(AbstractVO)
   * @see #createAndInsert()
   * @see #read(Serializable)
   * @see #modifyVOAfterInsert(AbstractVO, AbstractVO)
   * @see #modifyVOBeforeUpdate(AbstractVO)
   * @see #update(AbstractVO)
   * @see #modifyVOAfterUpdate(AbstractVO, AbstractVO)
   * @see #compareVOsAndValidateLoaded(AbstractVO, AbstractVO)
   */
  protected void defaultTestUpdate( V vo ) throws SQLException
  {
    try
    {
      update( null );
      fail( "Null value object must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      modifyVOBeforeUpdate( vo );
      checkInvalidMemberVariables( vo, true );
    }
    catch ( Exception e )
    {
      SQLException sqlException = new SQLException();
      sqlException.initCause( e );
    }

    modifyVOBeforeUpdate( vo );
    update( vo );
    V reloadedVO = read( vo.getIdentifier() );
    modifyVOAfterUpdate( vo, reloadedVO );
    compareVOsAndValidateLoaded( vo, reloadedVO );
  }

  /**
   * Tries to delete an unknown ID, then creates and inserts a value objects and deletes it.
   * 
   * @param invalidId An ID that's considered to be invalid, for instance a negative ID
   * @param unknownId An ID that's not found in the database
   * @throws SQLException if the delete fails
   * @see #delete(Serializable)
   * @see #createAndInsert()
   * @see #read(Serializable)
   */
  protected void defaultTestDelete( I invalidId, I unknownId ) throws SQLException
  {
    try
    {
      delete( (I) null );
      fail( "Null ID must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      delete( invalidId );
      fail( "Invalid ID must fail" );
    }
    catch ( SQLException e )
    {
      checkCausingException( e, WrappedIllegalArgumentException.class );
    }

    try
    {
      delete( unknownId );
      fail( "Unknown ID must fail" );
    }
    catch ( SQLException e )
    {
      WdlRemoveException causingException = checkCausingException( e, WdlRemoveException.class );
      assertEquals( "Checking ID", unknownId, causingException.getIdentifier() );
    }

    createAndInsert( true );

    delete( getSavedId() );

    try
    {
      read( getSavedId() );
      fail( "Reading deleted ID must fail" );
    }
    catch ( SQLException e )
    {
      WdlObjectNotFoundException causingException = checkCausingException( e, WdlObjectNotFoundException.class );
      assertEquals( "Checking ID", getSavedId(), causingException.getIdentifier() );
    }

    clearSavedId();
  }

  /**
   * Checks all member variables using the Validator.
   * @param valueObject The value object which will be inserted or updated
   * @param isUpdate Shows whether this is an update or an insert
   * @return The Map of all of the value object's property names with their invalid values
   * @throws IllegalStateException if the check fails 
   */
  @SuppressWarnings( "unused" )
  protected Map<String, Object[]> getInvalidPropertyValues( V valueObject, boolean isUpdate ) throws IllegalStateException
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  /**
   * Checks all member variables using the Validator.
   * @param vo The stopword to check
   * @param isUpdate if this is <code>true</code>, the identifier must be set
   * @throws IllegalStateException if the check fails 
   */
  private void checkInvalidMemberVariables( V vo, boolean isUpdate ) throws IllegalStateException
  {
    Map<String, Object[]> invalidPropertyValues = getInvalidPropertyValues( vo, isUpdate );

    if ( isUpdate )
    {
      ValidatorTestUtils.checkValidatedValuesForUpdate( this, vo, invalidPropertyValues );
    }
    else
    {
      ValidatorTestUtils.checkValidatedValuesForInsert( this, vo, invalidPropertyValues );
    }
  }

  /**
   * Since the original CRUD methods only throw an {@link SQLException}, the real Exception
   * is wrapped in it.
   * @param <E> The type of Exception to expect
   * @param exception The Exception coming from the CRUD method
   * @param causingExceptionClass The real Exception to look for, buried in the stack trace
   * @return The causing Exception
   */
  @SuppressWarnings( "unchecked" )
  protected <E extends Exception> E checkCausingException( Exception exception, Class<E> causingExceptionClass )
  {
    assertNotNull( "Checking for causing exception", exception.getCause() );
    if ( exception.getCause() != null && !exception.getCause().getClass().equals( causingExceptionClass ) )
    {
      exception.printStackTrace();
    }

    assertEquals( "Checking exception", causingExceptionClass, exception.getCause().getClass() );
    return (E) exception.getCause();
  }

  /**
   * Checks for the real Exception when faced with an SQLException thrown by the check for an existing active term or stopword.
   * @param exception The base Exception
   */
  private void checkExistingTermException( Exception exception )
  {
    assertEquals( "Checking exception", SQLException.class, exception.getClass() );
    SQLException sqlException = (SQLException) exception;
    WdlValidationException realException = checkCausingException( sqlException, WdlValidationException.class );
    assertTrue( "Checking for Validator invalid values", realException.getInvalidValues().size() == 1 );
    InvalidValue firstInvalidValue = realException.getInvalidValues().get( 0 );
    assertEquals( "Checking for Validator message", "validator.existingStopword", firstInvalidValue.getMessage() );
  }

  /**
   * Wraps an Exception into an SQLException.
   * @param e The Exception to wrap
   * @return The Exception wrapped into an SQLException
   */
  private SQLException wrapException( Exception e )
  {
    SQLException sqlException = new SQLException();
    sqlException.initCause( e );
    return sqlException;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void compareVOs( V originalVO, V loadedVO )
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  @SuppressWarnings( "unused" )
  protected V getSerializableVO() throws SQLException
  {
    throw new IllegalStateException( "This must be implemented in the test class!" );
  }

  /**
   * This is not used, since the DAO is not tested directly.
   */
  @Override
  protected Class<D> getDAOClass()
  {
    return null;
  }

  /**
   * This is not used, since serialization is not within the test scope.
   */
  @Override
  public void testSerialization()
  {
    // Nothing to do
  }

  /**
   * This is not used, since serialization is not within the test scope.
   */
  @Override
  @SuppressWarnings( "unused" )
  protected boolean assertVOEqual( V originalVO, V serializedVO )
  {
    return true;
  }
}
