package ch.ige.wdltool.ejb;

import java.io.File;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.exception.WdlCreateException;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WdlRemoveException;
import ch.ige.wdltool.exception.WdlUpdateException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.ClassVO;
import ch.ige.wdltool.vo.DataSourceVO;
import ch.ige.wdltool.vo.ExtendedSearchVO;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.IpiTermGroupVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.NiceTermGroupVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.StopwordVO;
import ch.ige.wdltool.vo.TermCleanupCandidateStatusVO;
import ch.ige.wdltool.vo.TermGroupVO;
import ch.ige.wdltool.vo.WdlCheckResultVO;
import ch.ige.wdlutil.analyze.WdlSyntaxConfiguration;
import ch.ige.wdlutil.exception.WdlSyntaxException;

/**
 * Contains the services offered to external callers.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlServices.java,v 1.94 2008/12/09 16:08:56 bandschapp Exp $
 */
public interface WdlServices
{
  // ------------------------------------------------------------------------------------------
  // WDL STUFF
  // ------------------------------------------------------------------------------------------

  /**
   * Gets the trademark's latest WDL text.
   * @param trademarkIpKey The key of the trademark to get the WDL text for 
   * @return The WDL text of the given trademark at the given date
   * @throws RemoteException if a system error occurs
   */
  public String getWdlText( Integer trademarkIpKey ) throws RemoteException;

  /**
   * Gets the trademark's basic data, its latest WDL and checks
   * the WDL based on the terms in the WDL DB.
   * @param trademarkIpKey The trademark's unique IP key
   * @param doStrictSyntaxCheck If this is <code>true</code>, the WDL text
   *        must stick to the given syntax exactly, otherwise the parsing is more tolerant
   * @param wdlSyntaxConfiguration Configures how this reacts to WDL syntax errors; ignored if <code>null</code>
   * @return The checked WDL where each class block in
   *         {@link ch.ige.wdlutil.vo.WdlVO#getClassBlocks()}
   *         contains the validated class text in
   *         {@link ch.ige.wdlutil.vo.ClassBlockVO#getClassTextGroups()}.
   * @throws WdlSyntaxException if the WDL has a wrong syntax
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public WdlCheckResultVO checkWdl( Integer trademarkIpKey, boolean doStrictSyntaxCheck, Map<WdlSyntaxConfiguration, Boolean> wdlSyntaxConfiguration )
      throws WdlSyntaxException, WrappedRuntimeException, RemoteException;

  /**
   * Checks the WDL without referring to the trademark.
   * @param wdlText The WDL text
   * @param language The WDL text's language
   * @param doStrictSyntaxCheck If this is <code>true</code>, the WDL text
   *        must stick to the given syntax exactly, otherwise the parsing is more tolerant
   * @param wdlSyntaxConfiguration Configures how this reacts to WDL syntax errors; ignored if <code>null</code>
   * @return The checked WDL where each class block in
   *         {@link ch.ige.wdlutil.vo.WdlVO#getClassBlocks()}
   *         contains the validated class text in
   *         {@link ch.ige.wdlutil.vo.ClassBlockVO#getClassTextGroups()}.
   * @throws WdlSyntaxException if the WDL has a wrong syntax
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public WdlCheckResultVO checkWdl( String wdlText, Locale language, boolean doStrictSyntaxCheck, Map<WdlSyntaxConfiguration, Boolean> wdlSyntaxConfiguration )
      throws WdlSyntaxException, WrappedRuntimeException, RemoteException;

  /**
   * Checks the WDLs of all trademarks whose application falls into the time period
   * limited by <code>from</code> and <code>to</code>. 
   * @param from The start date
   * @param to The end date
   * @return Two file contents: The log file content (text file) and the summary file content (tab-separated file)
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException
   */
  public String[] checkWdlForApplication( Date from, Date to ) throws WrappedRuntimeException, RemoteException;

  /**
   * Checks the WDLs of all trademarks whose registration falls into the time period
   * limited by <code>from</code> and <code>to</code>. 
   * @param from The start date
   * @param to The end date
   * @return Two file contents: The log file content (text file) and the summary file content (tab-separated file)
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException
   */
  public String[] checkWdlForRegistration( Date from, Date to ) throws WrappedRuntimeException, RemoteException;

  /**
   * Analyzes the WDLs of trademarks entered in the range from...to
   * @param from Start of range to analyze - only the day is relevant, time is ignored
   * @param to End of range to analyze - only the day is relevant, time is ignored
   * @return Logging information with details for all analyzed WDLs. 
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public String analyzeWdls( Date from, Date to ) throws WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // STOPWORD
  // ------------------------------------------------------------------------------------------

  /**
   * Finds a stopword.
   * @param stopwordId The unique ID of the stopword
   * @return The stopword
   * @throws WdlObjectNotFoundException if the stopword can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public StopwordVO findStopword( Integer stopwordId ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Finds all stopwords.
   * @return All stopwords, ordered by language
   * @throws WdlObjectNotFoundException if the stopwords can't be found
   * @throws RemoteException if a system error occurs
   */
  public List<StopwordVO> findAllStopwords() throws WdlObjectNotFoundException, RemoteException;

  /**
   * Finds all stopwords of a language that correspond to any word within <code>searchStopword</code>.
   * This splits up <code>searchStopword</code> into separate words and tries to find
   * any of those words in the search stopwords.
   * @param language The language in which stopwords are searched for; if this is <code>null</code>, all languages are taken
   * @param searchStopword The stopword to search for, which can contain any number of blank-separated words
   * @return The stopwords that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the stopword can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<StopwordVO> findAnyWordWithinStopwords( Locale language, String searchStopword ) throws WdlObjectNotFoundException, WrappedRuntimeException,
      RemoteException;

  /**
   * Inserts a stopword.
   * @param stopword The stopword to insert
   * @return The inserted stopword, including its ID
   * @throws WdlValidationException if the stopword's data is faulty
   * @throws WdlCreateException if the stopword can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public StopwordVO insertStopword( StopwordVO stopword ) throws WdlValidationException, WdlCreateException, WrappedRuntimeException, RemoteException;

  /**
   * Updates a stopword.
   * @param stopword The stopword to update
   * @throws WdlValidationException if the stopword's data is faulty
   * @throws WdlUpdateException if the stopword can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void updateStopword( StopwordVO stopword ) throws WdlValidationException, WdlUpdateException, WrappedRuntimeException, RemoteException;

  /**
   * Deletes a stopword.
   * @param stopwordId The unique ID of the stopword
   * @throws WdlRemoveException if the stopword can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteStopword( Integer stopwordId ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  /**
   * Deletes a number of stopwords.
   * @param stopwordIds The unique IDs of the stopwords
   * @throws WdlRemoveException if the stopwords can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteStopwords( Collection<Integer> stopwordIds ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // CLASS
  // ------------------------------------------------------------------------------------------

  /**
   * Finds all classes.
   * @return All classes ordered by class number
   * @throws WdlObjectNotFoundException if the classes can't be found
   * @throws RemoteException if a system error occurs
   */
  public List<ClassVO> findAllClasses() throws WdlObjectNotFoundException, RemoteException;

  /**
   * Updates the class number for a number of terms.
   * @param terms The terms to update
   * @param classNumber The new class number
   * @throws WdlUpdateException if the terms can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws WdlValidationException if the new class number doesn't match the existing term group number
   * @throws RemoteException if a system error occurs
   */
  public void updateClassNumber( final List<AbstractTermVO> terms, final Integer classNumber ) throws WdlUpdateException, WrappedRuntimeException,
      WdlValidationException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // DATA SOURCE
  // ------------------------------------------------------------------------------------------

  /**
   * Finds all data sources.
   * @return All data sources
   * @throws WdlObjectNotFoundException if the data sources can't be found
   * @throws RemoteException if a system error occurs
   */
  public List<DataSourceVO> findAllDataSources() throws WdlObjectNotFoundException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // TERM GROUP IN GENERAL
  // ------------------------------------------------------------------------------------------

  /**
   * Finds a term group.
   * @param termGroupId The unique ID of the term group
   * @return The term group
   * @throws WdlObjectNotFoundException if the term group can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public TermGroupVO findTermGroup( Integer termGroupId ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // NICE TERM GROUP
  // ------------------------------------------------------------------------------------------

  /**
   * Finds all Nice term groups.
   * @return All Nice term groups, ordered by {@link NiceTermGroupVO#getNiceTermNumber()}
   * @throws WdlObjectNotFoundException if the Nice term groups can't be found
   * @throws RemoteException if a system error occurs
   */
  public List<NiceTermGroupVO> findAllNiceTermGroups() throws WdlObjectNotFoundException, RemoteException;

  /**
   * Finds a Nice term group.
   * @param niceTermNumber The term group's number (not the ID!)
   * @return The Nice term group containing the Nice term number
   * @throws WdlObjectNotFoundException if the Nice term group can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public NiceTermGroupVO findByNiceTermNumber( Integer niceTermNumber ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Inserts a Nice term group.
   * @param vo The Nice term group to insert
   * @return The same Nice term group, including its ID   
   * @throws WdlCreateException
   * @throws RemoteException
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws WdlValidationException
   */
  public NiceTermGroupVO insertNiceTermGroup( NiceTermGroupVO vo ) throws WdlCreateException, WdlValidationException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // IPI TERM GROUP
  // ------------------------------------------------------------------------------------------

  /**
   * Finds all IPI term groups.
   * @return All IPI term groups, ordered by {@link IpiTermGroupVO#getIpiTermNumber()}
   * @throws WdlObjectNotFoundException if the IPI term groups can't be found
   * @throws RemoteException if a system error occurs
   */
  public List<IpiTermGroupVO> findAllIpiTermGroups() throws WdlObjectNotFoundException, RemoteException;

  /**
   * Finds an IPI term group.
   * @param ipiTermNumber The term group's number (not the ID!)
   * @return The IPI term group containing the IPI term number
   * @throws WdlObjectNotFoundException if the IPI term group can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public IpiTermGroupVO findByIpiTermNumber( Integer ipiTermNumber ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Inserts an IPI term group.
   * @param vo The IPI term group to insert
   * @return The same IPI term group, including its ID   
   * @throws WdlCreateException if the IPI term can't be inserted
   * @throws WdlValidationException if the IPI term is invalid
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public IpiTermGroupVO insertIpiTermGroup( IpiTermGroupVO vo ) throws WdlCreateException, WdlValidationException, WrappedRuntimeException, RemoteException;

  /**
   * Gets the next free IPI term group number and inserts a
   * IPI term group with it.
   * @return The inserted IPI term group with the next free IPI term group number
   * @throws WdlCreateException if the IPI term can't be inserted
   * @throws RemoteException if a system error occurs
   */
  public IpiTermGroupVO createNextFreeIpiTermGroup() throws WdlCreateException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // TERMS IN GENERAL
  // ------------------------------------------------------------------------------------------

  /**
   * Finds all active terms of a language that correspond to any word within <code>searchTerm</code>.
   * This splits up <code>searchTerm</code> into separate words and tries to find
   * any of those words in the search terms. For this to work, it simplifies the words in <code>searchTerm</code>.
   * Note that an active term is either a {@link NiceTermVO}, a {@link NegativeIpiTermVO} or a
   * {@link IpiTermVO} where {@link IpiTermVO#isActive()} is <code>true</code>.
   * <br>Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param language The language in which terms are searched for; if this is <code>null</code>, all languages are taken
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the word can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<AbstractTermVO> findAnyWordWithinActiveTerms( Locale language, String searchTerm ) throws WdlObjectNotFoundException, WrappedRuntimeException,
      RemoteException;

  /**
   * Finds all active terms of a language that correspond to the exact <code>term</code>.
   * Note that an active term is either a {@link NiceTermVO}, a {@link NegativeIpiTermVO} or a
   * {@link IpiTermVO} where {@link IpiTermVO#isActive()} is <code>true</code>.
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param term The term to search for
   * This does not consider any checker only terms - use {@link #findTermWithinActiveTerms(Locale, Integer, String, Boolean)} for different behaviour.
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the term candidates can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( Locale language, Integer classNumber, String term ) throws WdlObjectNotFoundException,
      WrappedRuntimeException, RemoteException;

  /**
   * Finds all active terms of a language that correspond to the exact <code>term</code>.
   * Note that an active term is either a {@link NiceTermVO}, a {@link NegativeIpiTermVO} or a
   * {@link IpiTermVO} where {@link IpiTermVO#isActive()} is <code>true</code>.
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param term The term to search for
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the term candidates can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( Locale language, Integer classNumber, String term, Boolean isCheckerOnly )
      throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Finds all inactive terms of a language that correspond to the exact phrase in <code>term</code> within class <code>classNumber</code>.
   * <strong>Note that this is never cached!</strong>
   * @param language The language in which terms are searched for
   * @param classNumber The class in which the terms are searched for
   * @param term The exact term to search for
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the terms can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<IpiTermVO> findTermWithinInactiveTerms( Locale language, Integer classNumber, String term ) throws WdlObjectNotFoundException,
      WrappedRuntimeException, RemoteException;

  /**
   * Finds all active terms that correspond to the search criteria defined in <code>searchDefinition</code>.
   * <br>Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param searchDefinition The extended search parameters
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the terms can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( ExtendedSearchVO searchDefinition ) throws WdlObjectNotFoundException, WrappedRuntimeException,
      RemoteException;

  /**
   * Finds all active terms in the given language whose search term match exactly the given term
   * (after it has been transformed into a search term).
   * This does consider also 'checker only' terms.
   * @param term The term to search for (language and term), which is searched exactly as given, after having been
   *        transformed to a search term
   * @return All active terms whose search term matches <code>term</code> or an empty <code>List</code> if none were found. The term used as input is not included.
   * @throws WdlObjectNotFoundException if the terms can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<AbstractTermVO> findSameSearchTermWithinActiveTerms( AbstractTermVO term ) throws WdlObjectNotFoundException, WrappedRuntimeException,
      RemoteException;

  /**
   * Deletes a number of generic terms.
   * @param termIds The unique IDs of the terms
   * @throws WdlRemoveException if the term can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteTerms( Collection<Integer> termIds ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  /**
   * Finds all terms which have expired.
   * This means that the latest renewal plus the term validity period is later than the current date/time.
   * <br>Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getExpirationDate()}</li>
   * </ul>
   * @return All terms that have expired
   * @throws RemoteException if a system error occurs
   */
  public List<IpiTermVO> findExpiredTerms() throws RemoteException;

  /**
   * Finds all terms which are going to expire soon.
   * This means that the latest renewal plus the term confirmation period is later
   * than the current date/time, but hasn't expired yet.
   * <br>Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getExpirationDate()}</li>
   * </ul>
   * @return All terms that are going to expire
   * @throws RemoteException if a system error occurs
   */
  public List<IpiTermVO> findExpiringTerms() throws RemoteException;

  /**
   * Finds all inactive terms.
   * @return All inactive terms
   * @throws RemoteException if a system error occurs
   */
  public List<IpiTermVO> findInactiveTerms() throws RemoteException;

  /**
   * Renews the given terms by setting their renewal date to the current date.
   * @param terms The terms to renew.
   * @return The IDs of the terms that could not be renewed because of a collision
   *  (another person updated a term contained in <code>terms</code>)
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<Integer> renewTerms( List<IpiTermVO> terms ) throws WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // NICE TERM
  // ------------------------------------------------------------------------------------------

  /**
   * Finds a Nice term.
   * @param niceTermId The unique ID of the Nice term
   * @return The Nice term
   * @throws WdlObjectNotFoundException if the Nice term can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public NiceTermVO findNiceTerm( Integer niceTermId ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Finds the generic Nice terms.
   * @return The generic Nice terms.
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<NiceTermVO> findGenericTerms() throws WrappedRuntimeException, RemoteException;

  /**
   * Inserts a Nice term and removes a corresponding candidate - if it exists.
   * Note that for security reasons, this checks whether the term already exists, either as term
   * or as stopword.
   * @param niceTerm The Nice term to insert
   * @return The same Nice term, including its ID with a status flag determining whether a candidate has been deleted because of the operation
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlCreateException if the IPI term can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public TermCleanupCandidateStatusVO<NiceTermVO> insertNiceTermAndRemoveIpiTermCandidate( NiceTermVO niceTerm ) throws WdlValidationException,
      WdlCreateException, WrappedRuntimeException, RemoteException;

  /**
   * Updates a Nice term and removes a corresponding candidate - if it exists.
   * Note that for security reasons, this checks whether the term already exists, either as term
   * or as stopword.
   * @param niceTerm The Nice term to update
   * @return a status flag determining whether a candidate has been deleted because of the operation
   * @throws WdlValidationException if the Nice term's data is faulty
   * @throws WdlUpdateException if the Nice term can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public boolean updateNiceTermAndRemoveIpiTermCandidate( NiceTermVO niceTerm ) throws WdlValidationException, WdlUpdateException, WrappedRuntimeException,
      RemoteException;

  /**
   * Deletes a Nice term.
   * @param niceTermId The unique ID of the Nice term
   * @throws WdlRemoveException if the Nice term can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteNiceTerm( Integer niceTermId ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // IPI TERM
  // ------------------------------------------------------------------------------------------

  /**
   * Finds an IPI term.
   * @param ipiTermId The unique ID of the IPI term
   * @return The IPI term
   * @throws WdlObjectNotFoundException if the IPI term can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public IpiTermVO findIpiTerm( Integer ipiTermId ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Inserts an IPI term and removes a corresponding IPI term candidate - if it exists.
   * Note that for security reasons, this checks whether the term already exists, either as term
   * or as stopword.
   * @param ipiTerm The IPI term to insert
   * @return The same IPI term, including its ID with a status flag determining whether a candidate has been deleted because of the operation
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlCreateException if the IPI term can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public TermCleanupCandidateStatusVO<IpiTermVO> insertIpiTermAndRemoveIpiTermCandidate( IpiTermVO ipiTerm ) throws WdlValidationException, WdlCreateException,
      WrappedRuntimeException, RemoteException;

  /**
   * Updates an IPI term and removes a corresponding IPI term candidate - if it exists.
   * Note that for security reasons, this checks whether the term already exists, either as term
   * or as stopword.
   * @param ipiTerm The IPI term to update
   * @return a status flag determining whether a candidate has been deleted because of the operation
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlUpdateException if the IPI term can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public boolean updateIpiTermAndRemoveIpiTermCandidate( IpiTermVO ipiTerm ) throws WdlValidationException, WdlUpdateException, WrappedRuntimeException,
      RemoteException;

  /**
   * Converts a negative IPI term into an IPI term, updates it and removes a corresponding IPI term candidate - if it exists.
   * Note that for security reasons, this checks whether the term already exists, either as term
   * or as stopword.
   * @param ipiTerm The IPI term to update. The term with this <code>ID</code> is still of type negative ipi term in the DB. 
   * @return a status flag determining whether a candidate has been deleted because of the operation
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlUpdateException if the IPI term can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public boolean convertNegativeIpiTermToIpiTermAndRemoveIpiTermCandidate( IpiTermVO ipiTerm ) throws WdlValidationException, WdlUpdateException,
      WrappedRuntimeException, RemoteException;

  /**
   * Deletes an IPI term.
   * @param ipiTermId The unique ID of the IPI term
   * @throws WdlRemoveException if the IPI term can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteIpiTerm( Integer ipiTermId ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // IPI TERM CANDIDATE
  // ------------------------------------------------------------------------------------------

  /**
   * Finds an IPI term candidate.
   * @param ipiTermCandidateId The unique ID of the IPI term candidate
   * @return The IPI term candidate
   * @throws WdlObjectNotFoundException if the IPI term candidate can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public IpiTermCandidateVO findIpiTermCandidate( Integer ipiTermCandidateId ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Finds an IPI term candidate.
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param searchTerm The term to look for
   * @return The IPI term candidate if it exists, <code>null</code> otherwise.
   * @throws WdlObjectNotFoundException if the IPI term candidate can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public IpiTermCandidateVO findIpiTermCandidate( Locale language, Integer classNumber, String searchTerm ) throws WdlObjectNotFoundException,
      WrappedRuntimeException, RemoteException;

  /**
   * Gets the IPI term candidates for a given language ordered by usage count.
   * @param language The language, if <code>null</code> all languages are searched.
   * @return The term candidates that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the term candidates can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException
   */
  public List<IpiTermCandidateVO> findIpiTermCandidates( Locale language ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Inserts an IPI term candidate.
   * @param ipiTermCandidate The IPI term candidate to insert
   * @return The same IPI term candidate, including its ID
   * @throws WdlValidationException if the IPI term candidate's data is faulty
   * @throws WdlCreateException if the IPI term candidate can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public IpiTermCandidateVO insertIpiTermCandidate( IpiTermCandidateVO ipiTermCandidate ) throws WdlValidationException, WdlCreateException,
      WrappedRuntimeException, RemoteException;

  /**
   * Updates an IPI term candidate.
   * @param ipiTermCandidate The IPI term candidate to update
   * @throws WdlValidationException if the IPI term candidate's data is faulty
   * @throws WdlUpdateException if the IPI term candidate can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void updateIpiTermCandidate( IpiTermCandidateVO ipiTermCandidate ) throws WdlValidationException, WdlUpdateException, WrappedRuntimeException,
      RemoteException;

  /**
   * Converts an inactive IPI term into an IPI term candidate and updates it.
   * @param ipiTermCandidate The IPI term candidate to update. The term with this <code>ID</code> is still of type IPI term in the DB.
   * @throws WdlValidationException if the IPI term candidate's data is faulty
   * @throws WdlUpdateException if the IPI term candidate can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void convertInactiveTermToIpiTermCandidate( IpiTermCandidateVO ipiTermCandidate ) throws WdlValidationException, WdlUpdateException,
      WrappedRuntimeException, RemoteException;

  /**
   * Activates an IPI term candidate.
   * @param ipiTermCandidate The IPI term candidate to set active
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws WdlValidationException if the term candidate's data is invalid
   * @throws WdlUpdateException if the IPI term candidate can't be activated
   * @throws RemoteException if a system error occurs
   */
  public void activateIpiTermCandidate( IpiTermCandidateVO ipiTermCandidate ) throws WrappedRuntimeException, WdlValidationException, WdlUpdateException,
      RemoteException;

  /**
   * Deletes an IPI term candidate.
   * @param ipiTermCandidateId The unique ID of the IPI term candidate
   * @throws WdlRemoveException if the IPI term candidate can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteIpiTermCandidate( Integer ipiTermCandidateId ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  /**
   * Gets the variants that can be extracted from the IPI term candidate's term.
   * This means splitting it up into its class text sub-groups, ignoring all of them
   * that correspond to an existing stopword or an existing active term. 
   * @param ipiTermCandidate The IPI term candidate containing the term
   * @return The term variants, which can be empty
   * @throws WdlObjectNotFoundException if the term candidates can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public List<String> extractTermVariants( IpiTermCandidateVO ipiTermCandidate ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // NEGATIVE IPI TERM
  // ------------------------------------------------------------------------------------------

  /**
   * Finds a negative IPI term.
   * @param negativeIpiTermId The unique ID of the IPI term
   * @return The IPI term
   * @throws WdlObjectNotFoundException if the IPI term can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public NegativeIpiTermVO findNegativeIpiTerm( Integer negativeIpiTermId ) throws WdlObjectNotFoundException, WrappedRuntimeException, RemoteException;

  /**
   * Inserts a negative IPI term.
   * @param negativeIpiTerm The IPI term to insert
   * @return The same IPI term, including its ID
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlCreateException if the IPI term can't be inserted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public NegativeIpiTermVO insertNegativeIpiTerm( NegativeIpiTermVO negativeIpiTerm ) throws WdlValidationException, WdlCreateException,
      WrappedRuntimeException, RemoteException;

  /**
   * Updates a negative IPI term.
   * @param negativeIpiTerm The negative IPI term to update
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlUpdateException if the IPI term can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void updateNegativeIpiTerm( NegativeIpiTermVO negativeIpiTerm ) throws WdlValidationException, WdlUpdateException, WrappedRuntimeException,
      RemoteException;

  /**
   * Converts an IPI term into a negative IPI term and updates it.
   * @param negativeIpiTerm The negative IPI term to update to. The term with this <code>ID</code> is still of type ipi term in the DB. 
   * @throws WdlValidationException if the IPI term's data is faulty
   * @throws WdlObjectNotFoundException If the negative IPI term can't be found
   * @throws WdlUpdateException if the IPI term can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void convertIpiTermToNegativeIpiTerm( NegativeIpiTermVO negativeIpiTerm ) throws WdlValidationException, WdlObjectNotFoundException,
      WdlUpdateException, WrappedRuntimeException, RemoteException;

  /**
   * Deletes a negative IPI term.
   * @param negativeIpiTermId The unique ID of the IPI term
   * @throws WdlRemoveException if the IPI term can't be deleted
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void deleteNegativeIpiTerm( Integer negativeIpiTermId ) throws WdlRemoveException, WrappedRuntimeException, RemoteException;

  // ------------------------------------------------------------------------------------------
  // CONFIGURATION
  // ------------------------------------------------------------------------------------------

  /**
   * Clears the whole cache, including terms, classes etc.
   * @throws RemoteException if a system error occurs
   */
  public void clearCache() throws RemoteException;

  /**
   * Returns the directory containing analyzer log files.
   * @return The directory (including server prefix) of the log files
   * @throws RemoteException if a system error occurs
   */
  public File getAnalyzerLogDirectory() throws RemoteException;

  /**
   * Returns the directory containing replication log files.
   * @return The directory (including server prefix) of the log files
   * @throws RemoteException if a system error occurs
   */
  public File getReplicationLogDirectory() throws RemoteException;

  /**
   * Sets the directory where analyzer log files are saved.
   * @param logDirectory The directory (including server prefix) of the log files
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void setAnalyzerLogDirectory( File logDirectory ) throws WrappedRuntimeException, RemoteException;

  /**
   * Returns the file name (with the full path) of the analyzer log file to write.
   * @return The file name, which includes the current date
   * @throws RemoteException if a system error occurs 
   */
  public File getNewAnalyzerLogFileName() throws RemoteException;

  /**
   * Returns the directory containing checker log files.
   * @return The directory (including server prefix) of the log files
   * @throws RemoteException if a system error occurs
   */
  public File getCheckerLogDirectory() throws RemoteException;

  /**
   * Sets the directory where checker log files are saved.
   * @param logDirectory The directory (including server prefix) of the log files
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void setCheckerLogDirectory( File logDirectory ) throws WrappedRuntimeException, RemoteException;

  /**
   * Returns the file name (with the full path) of the checker log file to write.
   * @return The file name, which includes the current date and time
   * @throws RemoteException if a system error occurs 
   */
  public File getNewCheckerLogFileName() throws RemoteException;

  /**
   * Returns the file name (with the full path) of the checker summary file to write.
   * Note that unlike other log files, this is a CSV file.
   * @return The file name, which includes the current date and time
   * @throws RemoteException if a system error occurs 
   */
  public File getNewCheckerSummaryFileName() throws RemoteException;

  /**
   * Returns the directory containing cleaner log files.
   * @return The directory (including server prefix) of the log files
   * @throws RemoteException if a system error occurs
   */
  public File getCleanerLogDirectory() throws RemoteException;

  /**
   * Sets the directory where cleaner log files are saved.
   * @param logDirectory The directory (including server prefix) of the log files
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void setCleanerLogDirectory( File logDirectory ) throws WrappedRuntimeException, RemoteException;

  /**
   * Returns the file name (with the full path) of the cleaner log file to write.
   * @return The file name, which includes the current date and time
   * @throws RemoteException if a system error occurs 
   */
  public File getNewCleanerLogFileName() throws RemoteException;

  /**
   * Shows whether correct words in the checker results must also be search links.
   * @return <code>true</code> if correct words are search links, otherwise <code>false</code>
   * @throws RemoteException if a system error occurs
   */
  public boolean createLinksForCorrectWords() throws RemoteException;

  /**
   * Shows whether correct words in the checker results must also be search links.
   * @param createLinksForCorrectWords <code>true</code> if correct words are search links, otherwise <code>false</code>
   * @throws RemoteException if a system error occurs
   */
  public void setCreateLinksForCorrectWords( boolean createLinksForCorrectWords ) throws RemoteException;

  /**
   * Gets the confirmation period for terms, usually IPI terms, in days.
   * After this period a term must be reconfirmed to still be active.
   * @param dataSource The data source whose configuration is fetched
   * @return The confirmation period for terms within the data source in days
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public Integer getTermConfirmationPeriod( DataSource dataSource ) throws WrappedRuntimeException, RemoteException;

  /**
   * Sets the confirmation period for terms, usually IPI terms, in days.
   * After this period a term must be reconfirmed to still be active.
   * @param dataSource The data source whose configuration is set
   * @param days The confirmation period for terms within the data source in days
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void setTermConfirmationPeriod( DataSource dataSource, Integer days ) throws WrappedRuntimeException, RemoteException;

  /**
   * Gets the validity period for terms, usually IPI terms, in days.
   * After this period a term becomes invalid.
   * @param dataSource The data source whose configuration is fetched
   * @return The validity period for terms within the data source in days
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public Integer getTermValidityPeriod( DataSource dataSource ) throws WrappedRuntimeException, RemoteException;

  /**
   * Sets the validity period for terms, usually IPI terms, in days.
   * After this period a term becomes invalid.
   * @param dataSource The data source whose configuration is set
   * @param days The validity period for terms within the data source in days
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws RemoteException if a system error occurs
   */
  public void setTermValidityPeriod( DataSource dataSource, Integer days ) throws WrappedRuntimeException, RemoteException;

  /**
   * Inactivates the terms which are expired to the given configuration. Can be used after 
   * updating the configuration when a term should expire.
   * @return The list of updated terms
   * @throws RemoteException
   */
  public List<IpiTermVO> inactivateExpiredTerms() throws RemoteException;

  /**
   * Gets the JDBC URL to the WDL DB
   * @return String the URL
   * @throws RemoteException if a system error occurs
   */
  public String getDbUrl() throws RemoteException;

  /**
   * Gets the Version of the WDL on the Server
   * @return String the Version
   * @throws RemoteException if a system error occurs
   */
  public String getServerVersion() throws RemoteException;

}