package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.wdltool.dao.IpiTermDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.TermCleanupCandidateStatusVO;
import ch.ige.wdltool.vo.TermIdCleanupCandidateStatusVO;

/**
 * Implements CRUD services for IPI terms.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: IpiTermImpl.java,v 1.23 2008/12/11 13:39:45 bandschapp Exp $
 */
public class IpiTermImpl extends BaseTermServicesImpl<IpiTermDAO, IpiTermVO, Integer>
{

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "IPI term";
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<IpiTermDAO> getDAOClass()
  {
    return IpiTermDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#find(ch.ige.dbutils.dao.DAO, java.io.Serializable)
  @Override
  protected IpiTermVO find( IpiTermDAO dao, Integer id ) throws SQLException
  {
    return dao.find( id );
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#insert(ch.ige.dbutils.dao.DAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected IpiTermVO insert( IpiTermDAO dao, IpiTermVO vo )
  {
    dao.insert( vo );
    return vo;
  }

  /**
   * Inserts an IPI term when no corresponding stopword already exists. During the insert operation a
   * corresponding term candidate is removed if it exists.
   * 
   * @param vo The VO to insert
   * @return The new VO an the flag if a term candidate had been removed
   * @throws SQLException
   * @throws WdlValidationException
   * @throws WrappedRuntimeException 
   */
  public TermCleanupCandidateStatusVO<IpiTermVO> insertAndRemoveIpiTermCandidate( IpiTermVO vo ) throws SQLException, WdlValidationException,
      WrappedRuntimeException
  {
    try
    {
      checkValueObject( vo, false );
      TermIdCleanupCandidateStatusVO termIdCleanupCandidateStatusVO = DAOPool.get( getDAOClass() ).insertAndRemoveIpiTermCandidate( vo );
      return new TermCleanupCandidateStatusVO<IpiTermVO>( vo, termIdCleanupCandidateStatusVO.isCandidateRemoved() );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( "Couldn't insert term and remove IPI term candidate " + vo, e );
    }
    return null;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#update(ch.ige.dbutils.dao.DAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( IpiTermDAO dao, IpiTermVO vo )
  {
    dao.update( vo );
  }

  /**
   * Updates an IpiTerm and simultaneously removes a corresponding term candidate if it exists.
   * 
   * @param vo The VO to insert
   * @return the flag if a term candidate had been removed
   * @throws SQLException
   * @throws WdlValidationException
   * @throws WrappedRuntimeException 
   */
  public boolean updateAndRemoveIpiTermCandidate( IpiTermVO vo ) throws SQLException, WdlValidationException, WrappedRuntimeException
  {
    try
    {
      checkValueObject( vo, true );
      return DAOPool.get( getDAOClass() ).updateAndRemoveIpiTermCandidate( vo );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return false;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#delete(ch.ige.dbutils.dao.DAO, java.io.Serializable)
  @Override
  protected void delete( IpiTermDAO dao, Integer id ) throws IllegalArgumentException, UnitOfWorkException
  {
    dao.delete( id );
  }

  /**
   * Finds inactive terms.
   * 
   * @return The inactive terms
   * @throws SQLException if the lookup fails
   * @see ch.ige.wdltool.ejb.WdlServices#findInactiveTerms()
   */
  public List<IpiTermVO> findInactiveTerms() throws SQLException
  {
    return DAOPool.get( getDAOClass() ).findInactiveTerms();
  }

  /**
   * Finds expired terms.
   * 
   * @return The expired terms
   * @throws SQLException if the query fails 
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiredTerms()
   */
  public List<IpiTermVO> findExpiredTerms() throws SQLException
  {
    return DAOPool.get( getDAOClass() ).findExpiredTerms();
  }

  /**
   * Inactivates expired terms.
   * 
   * @return The list of inactivated terms
   * @throws UnitOfWorkException if the inactivation fails
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiredTerms()
   */
  public List<IpiTermVO> inactivateExpiredTerms() throws UnitOfWorkException
  {
    return DAOPool.get( getDAOClass() ).inactivateExpiredTerms();
  }

  /**
   * Finds terms which are going to to expire.
   * 
   * @return The terms which are going to expire
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiringTerms()
   */
  public List<IpiTermVO> findExpiringTerms() throws SQLException
  {
    return DAOPool.get( getDAOClass() ).findExpiringTerms();
  }

  /**
   * Finds all inactive terms of a language that correspond to the exact phrase in <code>term</code> within class <code>classNumber</code>.
   * <strong>Note that this is never cached!</strong>
   * @param language The language in which terms are searched for
   * @param classNumber The class in which the terms are searched for
   * @param term The exact term to search for
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException 
   * @throws WrappedRuntimeException 
   */
  public List<IpiTermVO> findTermWithinInactiveTerms( Locale language, Integer classNumber, String term ) throws WdlObjectNotFoundException,
      WrappedRuntimeException
  {
    List<IpiTermVO> terms = new ArrayList<IpiTermVO>();
    try
    {
      terms = DAOPool.get( IpiTermDAO.class ).findTermWithinInactiveTerms( language, classNumber, term );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search for inactive term in: " + language + " with search term: " + term + " and class: "
          + classNumber, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return terms;
  }

  /**
   * Renews the given terms by setting their renewal date to the current date.
   * 
   * @param terms The terms to renew.
   * @return The IDs of the terms that could not be renewed because of a collision (another person updated a term
   *         contained in <code>terms</code>)
   * @throws WrappedRuntimeException 
   * @see ch.ige.wdltool.ejb.WdlServices#renewTerms(List)
   */
  public List<Integer> renewTerms( List<IpiTermVO> terms ) throws WrappedRuntimeException
  {
    try
    {
      return DAOPool.get( getDAOClass() ).renewTerms( terms );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return null;
  }

}
