package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.Date;
import java.util.Locale;
import java.util.Map;

import ch.ige.wdltool.checker.BatchChecker;
import ch.ige.wdltool.checker.WdlChecker;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.WdlCheckResultVO;
import ch.ige.wdlutil.analyze.WdlSyntaxConfiguration;
import ch.ige.wdlutil.exception.WdlSyntaxException;

/**
 * Implements services for checking WDLs.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: WdlCheckerImpl.java,v 1.3 2008/02/05 13:55:05 lewis_lee Exp $
 */
public class WdlCheckerImpl
{
  /**
   * Gets the trademark's basic data, its latest WDL and checks
   * the WDL based on the terms in the WDL DB.
   * @param trademarkIpKey The trademark's unique IP key
   * @param doStrictSyntaxCheck If this is <code>true</code>, the WDL text
   *        must stick to the given syntax exactly, otherwise the parsing is more tolerant
   * @param wdlSyntaxConfiguration Configures how this reacts to WDL syntax errors; ignored if <code>null</code>
   * @return The checked WDL where each class block in
   *         {@link ch.ige.wdlutil.vo.WdlVO#getClassBlocks()}
   *         contains the validated class text in
   *         {@link ch.ige.wdlutil.vo.ClassBlockVO#getClassTextGroups()}.
   * @throws WdlSyntaxException if the WDL has a wrong syntax
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#checkWdl(Integer, boolean, Map)
   */
  public WdlCheckResultVO checkWdl( Integer trademarkIpKey, boolean doStrictSyntaxCheck, Map<WdlSyntaxConfiguration, Boolean> wdlSyntaxConfiguration )
      throws WdlSyntaxException, WrappedRuntimeException
  {
    try
    {
      return new WdlChecker( wdlSyntaxConfiguration ).checkWdl( trademarkIpKey, doStrictSyntaxCheck );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwSQLException( "Can't load stopwords or terms", e );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return null;
  }

  /**
   * Checks the WDL without referring to the trademark.
   * @param wdlText The WDL text
   * @param language The WDL text's language
   * @param doStrictSyntaxCheck If this is <code>true</code>, the WDL text
   *        must stick to the given syntax exactly, otherwise the parsing is more tolerant
   * @param wdlSyntaxConfiguration Configures how this reacts to WDL syntax errors; ignored if <code>null</code>
   * @return The checked WDL where each class block in
   *         {@link ch.ige.wdlutil.vo.WdlVO#getClassBlocks()}
   *         contains the validated class text in
   *         {@link ch.ige.wdlutil.vo.ClassBlockVO#getClassTextGroups()}.
   * @throws WdlSyntaxException if the WDL has a wrong syntax
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#checkWdl(String, Locale, boolean, Map)
   */
  public WdlCheckResultVO checkWdl( String wdlText, Locale language, boolean doStrictSyntaxCheck, Map<WdlSyntaxConfiguration, Boolean> wdlSyntaxConfiguration )
      throws WdlSyntaxException, WrappedRuntimeException
  {
    try
    {
      return new WdlChecker( wdlSyntaxConfiguration ).checkWdl( wdlText, language, doStrictSyntaxCheck );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwSQLException( "Can't load stopwords or terms", e );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return null;
  }

  /**
   * Checks the WDLs of all trademarks whose application falls into the time period
   * limited by <code>from</code> and <code>to</code>. 
   * @param from The start date
   * @param to The end date
   * @return Two file contents: The log file content (text file) and the summary file content (tab-separated file)
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#checkWdlForApplication(java.util.Date, java.util.Date)
   */
  public String[] checkWdlForApplication( Date from, Date to ) throws WrappedRuntimeException
  {
    String[] result = null;
    try
    {
      result = BatchChecker.checkWdlForApplication( from, to );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwSQLException( "Couldn't search for trademarks with application", e );
    }
    return result;
  }

  /**
   * Checks the WDLs of all trademarks whose registration falls into the time period
   * limited by <code>from</code> and <code>to</code>. 
   * @param from The start date
   * @param to The end date
   * @return Two file contents: The log file content (text file) and the summary file content (tab-separated file)
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#checkWdlForRegistration(java.util.Date, java.util.Date)
   */
  public String[] checkWdlForRegistration( Date from, Date to ) throws WrappedRuntimeException
  {
    String[] result = null;
    try
    {
      result = BatchChecker.checkWdlForRegistration( from, to );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwSQLException( "Couldn't find the trademarks with registration", e );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return result;
  }
}
