package ch.ige.wdltool.vo;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import ch.ige.util.validation.Check;
import ch.ige.wdlutil.vo.ClassTextGroupVO;

/**
 * Contains a class text group that has been checked by the WDL checker
 * and thus contains the result of the check.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   heinzer
 * @version  $Id: CheckedClassTextGroupVO.java,v 1.12 2008/03/31 09:58:42 lewis_lee Exp $
 */
public class CheckedClassTextGroupVO extends ClassTextGroupVO
{
  /**
   * The checked parts of the class text
   */
  private List<ClassTextPartVO> classTextParts = new ArrayList<ClassTextPartVO>();

  /**
   * Creates this with a full class text group. Two corresponding
   * class text part are also created, both are {@link ch.ige.wdltool.vo.CheckedClassTextPart#UNKNOWN}.
   * @param classTextGroup The full class text group
   * @param punctuationMark The punctuation mark after the class text group
   */
  public CheckedClassTextGroupVO( String classTextGroup, Character punctuationMark )
  {
    super( classTextGroup, punctuationMark );

    if ( classTextGroup != null )
    {
      ClassTextPartVO classTextPart = new ClassTextPartVO( classTextGroup, CheckedClassTextPart.UNKNOWN );
      this.classTextParts.add( classTextPart );
    }
    if ( punctuationMark != null )
    {
      ClassTextPartVO punctuationMarkPart = new ClassTextPartVO( punctuationMark.toString(), CheckedClassTextPart.UNKNOWN );
      this.classTextParts.add( punctuationMarkPart );
    }
  }

  /**
   * Gets the checked parts of the class text.
   * @return the checked parts of the class text
   */
  public List<ClassTextPartVO> getClassTextParts()
  {
    return this.classTextParts;
  }

  /**
   * Sets the checked parts of the class text.
   * @param classTextParts The checked parts of the class text
   */
  public void setClassTextParts( List<ClassTextPartVO> classTextParts )
  {
    this.classTextParts = classTextParts;
  }

  /**
   * Gets the total length of all class text parts.
   * @return The total class text parts' length
   */
  public int getClassTextPartsLength()
  {
    int length = 0;
    if ( this.classTextParts != null )
    {
      for ( ClassTextPartVO classTextPart: this.classTextParts )
      {
        if ( classTextPart.getClassTextPart() != null )
        {
          length += classTextPart.getClassTextPart().length();
        }
      }
    }
    return length;
  }

  /**
   * Adds a class text part.
   * @param classTextPart The class text part to add
   * @param index An optional index, added to the end of the class text parts <code>List</code>
   *        when <code>null</code>
   */
  public void addClassTextPart( ClassTextPartVO classTextPart, Integer index )
  {
    if ( index == null )
    {
      this.classTextParts.add( classTextPart );
    }
    else
    {
      this.classTextParts.add( index, classTextPart );
    }
  }

  /**
   * Removes a class text part.
   * @param classTextPartToRemove The class text part to remove
   */
  public void removeClassTextPart( ClassTextPartVO classTextPartToRemove )
  {
    this.classTextParts.remove( classTextPartToRemove );
  }

  /**
   * Replaces an existing class text part with other class text parts.
   * This means that if there were 3 parts and this is called with 7 parts to add,
   * then in the end there are 9 parts.
   * @param classTextPartToRemove This is removed from the class text parts
   * @param classTextPartsToAdd These are inserted where <code>classTextPartToRemove</code> was
   */
  public void replaceClassTextPart( ClassTextPartVO classTextPartToRemove, List<ClassTextPartVO> classTextPartsToAdd )
  {
    int oldIndex = this.classTextParts.lastIndexOf( classTextPartToRemove );
    if ( oldIndex > -1 )
    {
      this.classTextParts.remove( classTextPartToRemove );
      this.classTextParts.addAll( oldIndex, classTextPartsToAdd );
    }
  }

  /**
   * Gets the class text part at the position in the whole class text group.
   * @param position Position in whole text to find, <strong>gets changed</strong>
   *        to the position within the returned <code>ClassTextPartVO</code> 
   * @return The part containing the requested position of the whole text or <code>null</code> if the position
   *         is out of bounds
   */
  public ClassTextPartVO getPartAt( ModifiableInteger position )
  {
    Check.notNegative( position.getValue(), "position" );

    int parsePosition = 0;

    List<ClassTextPartVO> parts = getClassTextParts();
    ClassTextPartVO parsePart = null;

    Iterator<ClassTextPartVO> iter = parts.iterator();
    while ( iter.hasNext() && parsePosition <= position.getValue() )
    {
      parsePart = iter.next();
      parsePosition = parsePosition + parsePart.getClassTextPart().length();
    }
    // The position was bigger than the existing class text parts
    if ( position.getValue() >= parsePosition )
    {
      return null;
    }
    if ( parsePart != null )
    {
      parsePosition = parsePosition - parsePart.getClassTextPart().length();
      position.setValue( position.getValue() - parsePosition );
    }
    else
    {
      parsePart = parts.get( 0 );
    }
    return parsePart;
  }

  /**
   * Finds the class text part at <code>position</code> and splits
   * it into three possible parts:
   * <ol>
   * <li>The part before <code>position</code></li>
   * <li>The part at <code>position</code>, which is replaced by <code>newText</code> in
   *     the length of <code>newText</code></li>
   * <li>The part after <code>position</code></li>
   * </ol> 
   * @param position The position to split
   * @param newText Replaces the existing text part at <code>position</code> 
   * @param newCheckFlag The new text part will have this flag
   */
  public void splitClassTextParts( int position, String newText, CheckedClassTextPart newCheckFlag )
  {
    Check.notNegative( position, "position" );
    Check.notEmpty( newText, "replacement text" );
    Check.notNull( newCheckFlag, "check flag" );
    int totalLength = getClassTextPartsLength();
    if ( position > totalLength - 1 )
    {
      throw new IllegalArgumentException( "Position out of bounds, total length: " + totalLength );
    }

    ClassTextPartVO existingClassTextPart = getPartAt( new ModifiableInteger( position ) );
    if ( existingClassTextPart == null )
    {
      throw new IllegalArgumentException( "Found no class text part at position " + position );
    }

    // Calculate the position within the existing class text part
    int lengthUpToExistingPart = 0;
    for ( ClassTextPartVO classTextPart: this.classTextParts )
    {
      if ( classTextPart.equals( existingClassTextPart ) )
      {
        break;
      }
      if ( classTextPart.getClassTextPart() != null )
      {
        lengthUpToExistingPart += classTextPart.getClassTextPart().length();
      }
    }
    String classTextPartString = existingClassTextPart.getClassTextPart();
    if ( newText.length() > classTextPartString.length() )
    {
      throw new IllegalArgumentException( "Trying to insert " + newText.length() + " characters into a class text part with " + classTextPartString.length()
          + " characters" );
    }
    int indexWithinExistingTextPart = position - lengthUpToExistingPart;

    String preText = classTextPartString.substring( 0, indexWithinExistingTextPart );
    ClassTextPartVO preTextPart = new ClassTextPartVO( preText, existingClassTextPart.getCheckFlag() );

    ClassTextPartVO newTextPart = new ClassTextPartVO( newText, newCheckFlag );

    String postText = classTextPartString.substring( indexWithinExistingTextPart + newText.length() );
    ClassTextPartVO postTextPart = new ClassTextPartVO( postText, existingClassTextPart.getCheckFlag() );

    List<ClassTextPartVO> replacingParts = packageNonEmptyParts( preTextPart, newTextPart, postTextPart );
    replaceClassTextPart( existingClassTextPart, replacingParts );
  }

  /**
   * Packages non-empty parts into a new <code>List</code>.
   * @param newClassTextParts The class text parts to maybe put into the <code>List</code> 
   * @return A <code>List</code> containing non-empty objects from <code>classTextParts</code>
   */
  private List<ClassTextPartVO> packageNonEmptyParts( ClassTextPartVO... newClassTextParts )
  {
    List<ClassTextPartVO> classTextPartList = new ArrayList<ClassTextPartVO>( newClassTextParts.length );
    for ( ClassTextPartVO classTextPart: newClassTextParts )
    {
      if ( StringUtils.isNotEmpty( classTextPart.getClassTextPart() ) )
      {
        classTextPartList.add( classTextPart );
      }
    }
    return classTextPartList;
  }

  /**
   * Wraps an integer value that can be changed when used as method parameter.
   */
  public class ModifiableInteger
  {
    /**
     * The integer
     */
    private int i = 0;

    /**
     * Only constructor.
     * @param i initial value
     */
    public ModifiableInteger( int i )
    {
      this.i = i;
    }

    /**
     * Sets the integer value.
     * @param i The value
     */
    public void setValue( int i )
    {
      this.i = i;
    }

    /**
     * Gets the integer value.
     * @return The integer value
     */
    public int getValue()
    {
      return this.i;
    }
  }

}
